%% Read variables from OPAC data file:
filename = [fileref,'_OPAC.h5'];

VARSPC  = h5read(filename,'/VARSPC')*1000; % Wavelength grid (nm)
ANGF    = h5read(filename,'/ANGF');        % Scatering angle grid (deg)
RH      = h5read(filename,'/RH');          % Relative humidity grid [%]
SPEC    = h5read(filename,'/SPEC'); 

auxF    = h5read(filename,'/F');       % Phase Function
auxEXTC = squeeze(SPEC(:,:,1,:));      % Extinction coef [km^-1]
auxABSC = squeeze(SPEC(:,:,2,:));      % Absorption coef [km^-1]
auxASYM = squeeze(SPEC(:,:,3,:));      % Asymmetry parameter (-)

% Interpolate data for a RH corresponding to mid-latitude summer (74.4%):
for i=1:size(auxEXTC,3)
    EXTC(:,i) = interp1(RH,auxEXTC(:,:,i)',74.4);
    ABSC(:,i) = interp1(RH,auxABSC(:,:,i)',74.4);
    ASYM(:,i) = interp1(RH,auxASYM(:,:,i)',74.4);
    for j=1:size(auxF,1)
        F(j,:,i)  = interp1(RH,squeeze(auxF(j,:,:,i))',74.4);
    end
end

% Normalize phase function:
for i=1:size(F,2)
    for j=1:size(F,3)
        f = 2*pi*trapz(ANGF*pi/180,F(:,i,j).*sind(ANGF));
        F(:,i,j) = squeeze(F(:,i,j))./f;
    end
end


%% Fit and plot:
aernames = {'Cont+Mar #2','Cont+Mar #1','Cont+Des #2','Cont+Des #1',...
			'Mar. (clean)','Desert','Urban','Cont. (avg.)'};
C = {[0 1 0],[0 0.65 0],[0 0 0],[1 0 0],[1 0.5 0],...
    'c',[0 0 1],[0 0 0.65],[0.5 0.5 0.5],[0.75 0.75 0.75]};

% Extinction coefficients ==> Amstrong exponent
figure(1)
hold on
leg = cell(1,size(EXTC,2));
for i =1:size(EXTC,2)
    fo = fitoptions('Method','NonlinearLeastSquares',...
        'Lower',0,'Upper',10,'StartPoint',1.5);
    ft = fittype('(550/x)^a','options',fo);
    [cf,gof] = fit(VARSPC,EXTC(:,i),ft);
    alpha(i) = cf.a; rsquare(i) = gof.rsquare;
    plot(VARSPC,cf(VARSPC),'--','color',C{i},'linewidth',1.5)  
    leg{i} = [aernames{i},': ',sprintf('%.2f',alpha(i))];
end
for i=1:size(EXTC,2)
    plot(VARSPC,EXTC(:,i),'o','markersize',10,'color',C{i})
end
sprintf('Angstrom exponent - Mean r^2: %.4f',mean(rsquare))
title('OPAC''s normalized extinction coefficient')
xlabel('Wavelength (nm)'), ylabel('Normalized extinction (km^{-1})')
grid on, xlim([400 1000]), ylim([0.4 1.5])
set(gcf,'position',[200 600 550 300])

% Phase function ==> Henyey-Greenstein parameter
figure(2); hold on
leg = cell(1,size(F,3));
for i =1:size(F,3)
    semilogy(ANGF,F(:,3,i),'-','color',C{i},'linewidth',1.5)
end
for i =1:size(F,3)
    for j=1:size(F,2)
        fo = fitoptions('Method','NonlinearLeastSquares',...
            'Lower',0,'Upper',1,'StartPoint',0.7);
        ft = fittype('(1/(4*pi))*(1-g^2)/(1+g^2-2*g*cosd(x))^(3/2)','options',fo);
        [cf,gof] = fit(ANGF,F(:,j,i),ft);
        G(j) = cf.g; rsquare(j) = gof.rsquare;
        if j==3
%             semilogy(ANGF,F(:,j,i),'-','color',C{i})
            semilogy(ANGF,cf(ANGF),'--','color',C{i},'linewidth',1.5) 
        end
        leg{i} = [aernames{i},': ',sprintf('%.3f',cf.g)];
	end
    Rsqr(i) = mean(rsquare);
    g(i) = mean(G); 
    g(i) = G(3); dg(i) = 100*abs(G(end)-G(2))/g(i);
end
sprintf('HG asymmetry parameter - Mean r^2: %.4f',mean(Rsqr))
set(gca,'YScale','log','XScale','log')
set(gca,'xtick',[0.1 1 10 100])
grid on, xlim([0 180]), ylim([1e-3,1e2])
title('OPAC''s phase function')
xlabel('Scatering angle (deg)'), ylabel('Phase function (-)')
% legend(leg,'fontsize',20,'location','northeast')
set(gcf,'position',[200 600 550 300])

% Phase function asymmetry parameter
figure(3); hold on
for i =1:size(F,3)
    plot(VARSPC,ASYM(:,i),'-','color',C{i},'linewidth',1.5)
end
title('OPAC''s asymmetry parameter')
xlabel('Wavelength (nm)'), ylabel('Asymmetry param. (-)')
grid on, xlim([400 1000]), ylim([0.62 0.8])
% legend(leg,'fontsize',20,'location','southwest')
set(gcf,'position',[200 600 550 300])



% Single scattering albedo
figure(4); hold on
leg = cell(1,size(EXTC,2));
for i =1:size(F,3)
    aux = (EXTC(:,i)-ABSC(:,i))./EXTC(:,i);
    semilogy(VARSPC,aux,'-','color',C{i},'linewidth',1.5)
    ssa(i) = mean(aux(2:end)); dssa(i) = 100*abs(aux(end)-aux(2))/ssa(i);
    leg{i} = [aernames{i},': ',sprintf('%.3f',ssa(i))];
end
title('OPAC''s single scattering albedo')
xlabel('Wavelength (nm)'), ylabel('SSA (-)')
grid on, xlim([400 1000]), ylim([0.7 1])
% legend(leg,'fontsize',20,'location','southwest')
set(gcf,'position',[200 600 550 300])




