classdef ALG
    %ALG Main class for the Atmospheric Look-up table Generator (ALG) toolbox
    %   Detailed explanation goes here
    
    properties
        RTMfilepath % Path to RTM executable file
        RTMconf     % RTM variables information
        GenConf     % Generic LUT configuration
        AdvConf     % Advanced LUT configuration
        SpeConf     % Spectral LUT configuration
        ParamConf   % LUT input parameters configuration
        LUTheader   % LUT header (i.e., LUT nodes)
        LUTinputs   % Input values of the LUT
        inputs0     % Values of static RTM input variables
        varnames    % RTM input variable names (IDs)
        AER         % Structure containing OPAC or user-defined aerosol optical properties
        ATM         % Structure containing user-defined atmospheric profiles
    end
    
    methods
        function this=ALG(LUTconffile,RTMconffile)
            % Read Generic, Spectral and Input Parameters configuration:
            this.GenConf   = read_LUTconfig(LUTconffile,'GenConf');
            this.SpeConf   = read_LUTconfig(LUTconffile,'SpeConf');
            this.ParamConf = read_LUTconfig(LUTconffile,'ParamConf');
            % Read software configuration:
            [RTMconf,~,RTMconfdata] = read_RTMconfig(RTMconffile,this.GenConf.RTMname);
            this.RTMconf = RTMconfdata;
            % Extract Advanced configuration:
            this.AdvConf = this.extractAdvConf(LUTconffile,RTMconfdata);
            % Determine executable RTM file:            
            idx = find(strcmp({RTMconf(:).name},this.GenConf.RTMname));
            this.RTMfilepath = RTMconf(idx).filepath;
        end
    end
    
    methods % Functions for extracting data from model and configuration
        function LUTinputs=determineLUTinputs(this)
            groups = fieldnames(this.ParamConf);
            % Predefine fields:
            LUTinputs.dynamic.discrete = []; LUTinputs.dynamic.continuous = [];

            for i=1:numel(groups) % For every LUT group (atm, aer, geo)
                varnames   = fieldnames(this.RTMconf.(groups{i}));   % Default (all) inputs
                if ~isempty(this.ParamConf.(groups{i}))
                    % There are user-input parameters in this group
                    varnamesIN = fieldnames(this.ParamConf.(groups{i})); 

                    for j=1:numel(varnames) % For every variable in the group
                        idx = find(strcmp(varnamesIN(:),varnames{j}));
                        RTMparam = this.RTMconf.(groups{i}).(varnames{j}); % Current parameter
                        if isempty(idx)     % Not input by user ==> take default value
                            LUTinputs.static.(RTMparam.name) = str2num(RTMparam.default);
                        else                % Input by user ==> take input values
                            value = str2num(this.ParamConf.(groups{i}).(varnames{j}).values);
                            % Check if input value is "static" (only one input value)
                            % or "dynamic" (2 or more):
                            if isempty(value) % Input is a string
                                LUTinputs.static.(RTMparam.name) = this.ParamConf.(groups{i}).(varnames{j}).values;
                            elseif length(value)==1 % Only one input value
                                LUTinputs.static.(RTMparam.name) = value;
                            else
                                % Check if variable is "discrete" or "continuous":
                                if strcmp(RTMparam.paramtype,'continuous')
                                    LUTinputs.dynamic.continuous.(RTMparam.name) = value;
                                else
                                    % Check if there are various user-input atmospheric
                                    % models in MODTRAN5 or MODTRAN6:
                                    if strcmp(this.GenConf.RTMname,'MODTRAN5') || strcmp(this.GenConf.RTMname,'MODTRAN6')
                                        if strcmp(RTMparam.name,'MODEL') && any(value<=-1)
                                            aux = value(value<=-1); aux = -(1:abs(aux));
                                            value = [value(value>=0),aux];
                                        end
                                    end
                                    LUTinputs.dynamic.discrete.(RTMparam.name) = value;
                                end
                            end
                        end
                    end
                else
                    % There are no user-input variables in this group ==>
                    % take default values
                    for j=1:numel(varnames)
                        RTMparam = this.RTMconf.(groups{i}).(varnames{j});
                        LUTinputs.static.(RTMparam.name) = str2num(RTMparam.default);
                    end
                end
            end
        end
        
        function Ncomb=numberCombinations(this)
            % Extract number of combinations in LUT
            if isempty(this.LUTheader)
                Ncomb = 1;
            else
                Ncomb = size(this.LUTheader,2);
            end
        end

        function this=findInputVariables(this)
            % Find static RTM input parameters (default or user-input)
            if isempty(this.RTMconf.AdvParam)
                fn_adv  = [];
                this.inputs0 = struct2cell(this.LUTinputs.static);
            else
                fn_adv  = fieldnames(this.RTMconf.AdvParam);   
                this.inputs0 = [struct2cell(this.AdvConf);struct2cell(this.LUTinputs.static)];
            end
            fn_static   = fieldnames(this.LUTinputs.static);
            if ~isempty(this.LUTinputs.dynamic.discrete)          
                fn_discrete = fieldnames(this.LUTinputs.dynamic.discrete);
            else
                fn_discrete = [];
            end
            if ~isempty(this.LUTinputs.dynamic.continuous)
                fn_cont     = fieldnames(this.LUTinputs.dynamic.continuous);
            else
                fn_cont = [];
            end
            this.varnames   = [fn_adv;fn_static;fn_discrete;fn_cont];            
        end
    
        function this=readAerosols(this)
            % Read OPAC or user-input aerosol optical properties (if input
            % by the user)
            this.AER = [];
            % 1) Detect Aerosol model variable name:
            fn = fieldnames(this.RTMconf.AerParam);
            for i=1:numel(fn)
                if  contains(this.RTMconf.AerParam.(fn{i}).description,'Aerosol model')
                    AerModelVarName = this.RTMconf.AerParam.(fn{i}).name;
                end
            end
            % 2) Index identifying the RTM aerosol variable:
            ind = find(strcmp([this.varnames],AerModelVarName));
            % 3) Identifying if OPAC or user-defined aerosols are
            % included in the LUT:
            n = numel(this.inputs0);
            try % OPAC (<0) or user-defined (>10)
                   aux = any(unique(this.LUTheader(ind-n,:))<0) | ...
                         any(unique(this.LUTheader(ind-n,:))>10);
            catch, aux = false;
            end
            aux = (ind<=n && (this.inputs0{ind}<0 || this.inputs0{ind}>10)) || aux;
            if aux
                % 4) Create AER structure with optical properties
                this.AER.ind = ind;
                % LUT includes OPAC or user-defined (ATK) aerosol models
                try % More than one aerosol ==> LUTheader
                    aerIDs = this.LUTheader(this.AER.ind-n,:);
                catch % Only one aerosol ==> static
                    aerIDs = this.inputs0{this.AER.ind};
                end
                aerIDs = aerIDs(aerIDs<0 | aerIDs>10); auxaerIDs = unique(aerIDs);
                this.AER.aerIDs = auxaerIDs;
                if any(aerIDs<0)  % Read common data for OPAC aerosols
                    SPEC = h5read([this.GenConf.filename,'_OPAC.h5'],'/SPEC');
                    F    = h5read([this.GenConf.filename,'_OPAC.h5'],'/F');
                    VARSPC  = h5read([this.GenConf.filename,'_OPAC.h5'],'/VARSPC');
                    RH      = h5read([this.GenConf.filename,'_OPAC.h5'],'/RH');
                    ANGF    = h5read([this.GenConf.filename,'_OPAC.h5'],'/ANGF');
                end
                if any(aerIDs>10) % Read info data from user-defined aerosol database
                    aerdbfile = ['..',filesep,'Userdata',filesep,'aerdb.h5'];
                    info = h5info(aerdbfile,['/',this.GenConf.RTMname]);
                end
                for i=1:numel(auxaerIDs)
                    if auxaerIDs(i)<0
                        % Aerosol model from OPAC ==> read OPAC LUT .h5 file:
                        this.AER.opticalProp(i).type = 'OPAC';
                        this.AER.opticalProp(i).ID = abs(auxaerIDs(i));
                        this.AER.opticalProp(i).VARSPC = VARSPC;
                        this.AER.opticalProp(i).RH   = RH;
                        this.AER.opticalProp(i).ANGF = ANGF;
                        this.AER.opticalProp(i).EXTC = SPEC(:,:,1,i);
                        this.AER.opticalProp(i).ABSC = SPEC(:,:,2,i);
                        this.AER.opticalProp(i).ASYM = SPEC(:,:,3,i);
                        % MODTRAN5 needs to define phase function at all aerosol layers but
                        % OPAC only provides phase function for the tropospheric aerosols
                        % (IREG2) ==> better use only asymmetry parameter + Mie
                        this.AER.opticalProp(i).F    = [];%F(:,:,:,i);
                    elseif auxaerIDs(i)>10
                        % User-defined aerosol model from ATK ==> read aerdb.h5 file:
                        this.AER.opticalProp(i).type = 'ATK';
                        aername = info.Datasets(aerIDs(i)-10).Name;
                        auxdata = h5read(aerdbfile,['/',this.GenConf.RTMname,'/',aername]);
                        this.AER.opticalProp(i).modeltype = h5readatt(aerdbfile,['/',this.GenConf.RTMname,'/',aername],'modeltype');
                        this.AER.opticalProp(i).data = auxdata;
                        this.AER.opticalProp(i).aername = aername;
%                         this.AER.opticalProp(i).VARSPC = auxdata(:,1);
%                         this.AER.opticalProp(i).EXTC = auxdata(:,2);
%                         this.AER.opticalProp(i).ASYM = auxdata(:,4);
%                         this.AER.opticalProp(i).ABSC = (1-auxdata(:,3)).*auxdata(:,2); 
%                         this.AER.opticalProp(i).RH   = [];
%                         this.AER.opticalProp(i).ANGF = []; 
%                         this.AER.opticalProp(i).F    = [];
                    end
                end
            end
        end
        
        function this=readAtmospheres(this)
            % Read user-input atmospheric profile (if input by the user)
            this.ATM = [];
            % 1) Detect Atmosphere model variable name:
            fn = fieldnames(this.RTMconf.AtmParam);
            for i=1:numel(fn)
                if  strcmp(this.RTMconf.AtmParam.(fn{i}).description,'Atmospheric profile model')
                    AtmModelVarName = this.RTMconf.AtmParam.(fn{i}).name;
                end
            end
            % 2) Index identifying the RTM atmosphere variable:
            ind = find(strcmp([this.varnames],AtmModelVarName));
            % 3) Identifying if user-defined aerosols are included in the LUT:
            n = numel(this.inputs0);
            try
                   aux = any(unique(this.LUTheader(ind-n,:))<0) | ...
                         any(unique(this.LUTheader(ind-n,:))>10);
            catch, aux = false;
            end
            aux = (ind<=n && this.inputs0{ind}<0) || aux;
            if aux
                % 4) Create ATM structure with optical properties
                this.ATM.ind = ind;
                % LUT includes user-defined atmospheric profiles
                try % More than one atmosphere ==> LUTheader
                    atmIDs = this.LUTheader(this.ATM.ind-n,:);
                catch % Only one atmosphere ==> static
                    atmIDs = this.inputs0{this.ATM.ind};
                end
                atmIDs = atmIDs(atmIDs<0 | atmIDs>10); auxatmIDs = unique(atmIDs);
                this.ATM.atmIDs = auxatmIDs;
                if any(atmIDs>10) % Read info data from user-defined atmosphere database
                    infoDB = h5info(['..',filesep,'Userdata',filesep,'atmdb.h5'],['/',this.GenConf.RTMname]);
                end
                if any(atmIDs<0) % Read info data from user-defined atmosphere database
                    infoAtm = h5info([this.GenConf.filename,'_UserAtm.h5']);
                end
                for i=1:numel(auxatmIDs)
                    if auxatmIDs(i)<0
                        % Atmospheric profile from doy/latitude==> read Atm LUT .h5 file:
                        this.ATM.profiles(i).type = 'user-defined';
                        for j=1:numel(infoAtm.Datasets)
                            fn = infoAtm.Datasets(j).Name;
                            if ~strcmp(fn,'ZM')
                                sz = infoAtm.Datasets(j).Dataspace.Size;
                                this.ATM.profiles(i).(fn)  = h5read([this.GenConf.filename,'_UserAtm.h5'],['/',fn],[1 i],[sz(1) 1]);
                            else
                                this.ATM.profiles(i).(fn)  = h5read([this.GenConf.filename,'_UserAtm.h5'],['/',fn]);
                            end
                        end
                    elseif auxatmIDs(i)>10
                        % User-defined atmospheric profile from ATK ==> read atmdb.h5 file:
                        % TO DO!!
                    end
                end
            end
        end
        
        this = calcLUTgrid(this,iter,rndseed,errors)
        
        writeLUThdr(this)
        
        updateLUThdr(this,wvl,I0,outvarnames,LUTheader)
        
        generateRTMinputs(this)

    end
    
    methods(Static)
        function AdvConf=extractAdvConf(LUTconffile,RTMconfdata)
            % Determine advanced configuration: If defined in the selected 
            % RTM, read default (from RTMconf) and override by those 
            % parameters defined in LUTconffile
            AdvConf = read_LUTconfig(LUTconffile,'AdvConf');
            if isempty(RTMconfdata.AdvParam)
               AdvConfIN = [];
            else
                fn_adv     = fieldnames(RTMconfdata.AdvParam);   % Default (all) inputs
                varnamesIN = fieldnames(AdvConf);           % User-input parameters
                for i=1:numel(fn_adv)
%                     idx = find(cellfun(cellfind(fn_adv{i}),varnamesIN),1);
                    idx = find(strcmp(varnamesIN(:), fn_adv{i}));
                    RTMparam = RTMconfdata.AdvParam.(fn_adv{i});
                    if isempty(idx)     % Not input by user ==> take default value
                    	if isempty(str2num(RTMparam.default))
                        	AdvConfIN.(RTMparam.name) = RTMparam.default;
                        else
                        	AdvConfIN.(RTMparam.name) = str2num(RTMparam.default);
                        end
                    else                % Input by user ==> take input values
                    	if isempty(str2num(AdvConf.(RTMparam.name).values))
                        	AdvConfIN.(RTMparam.name) = AdvConf.(RTMparam.name).values;
                        else
                        	AdvConfIN.(RTMparam.name) = str2num(AdvConf.(RTMparam.name).values);
                        end
                    end
                end
            end
            AdvConf = AdvConfIN;
        end
    end

end

