function alg = calcLUTgrid(alg,iter,rndseed,errors)
%CALCLUTGRID Calculate Look-Up table grid points
%   [LUTINPUTS,LUTHEADER] = CALCLUTGRID(RTMCONF,LUTCONFFILE) calculates the
%   Look-Up Table (LUT) grid points based on the default RTM configuration 
%   (RTMCONF) and user-input LUT configuration stored in the ALG class
%   (ALG). The additional input parameters ITER, RNDSEED and ERRORS (only
%   used when the number of LUT grid points is calculated automatically)
%   indicate: (1) the iteration of sequential introduction of input grid 
%   points through the stop condition defined in LUTCONFFILE, (2) the
%   random seed based on Matlab's function rgn for the repetition of 
%   results, and (3) a structure with the interpolation errors based on the
%   "leave-on-out" cross-validation.
%   As results, CALCLUTGRID produces two outputs:
%
%   LUTINPUTS: A structure containing all the RTM input parameters
%              separated into "static" and "dynamic". For the "dynamic" 
%              parameters, these are further identified as "discrete" or
%              "continuous".
%   LUTHEADER: A table of dimensions P x N being P the number of "dynamic"
%              parameters and N the number of combinations.
%
%   See also: INTERPERROR, LUTGEN_INPUTS, WRITELUTHDR

%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.04 $  $Date: 2018/12/17 $


% All dynamic or static variables in the LUTconf file override the default
% values from the RTMconf file. The variable LUTinputs contain all the
% values of the parameters needed to run the selected RTM separated in
% "static" (fixed for all the LUT) and "dynamic" (variable in LUT and thus
% included in the header file). In addition, dynamic variables are
% separated in "discrete" (not valid for interpolation) or "continuous"


cellfind = @(string)(@(cell_contents)(strcmp(string,cell_contents)));

% Determine the LUT input values (default or user-input) both for static 
% and dynamic parameters:
LUTinputs = alg.determineLUTinputs();
   
% The dynamic key input variables in the LUTconf file define the grid 
% points of the LUT. Create LUTheader variable with all the combination of
% parameters as input to the selected RTM.
switch alg.GenConf.inputmode
	case 'Manual (gridded) sampling'
        LUTheader = griddedsampling(LUTinputs);
    case {'Latin hypercube','Sobol sequence','Halton sequence'}
        LUTheader = quasirandomsampling(LUTinputs,alg.GenConf,iter,rndseed);
    case 'Automatic (gradient)'
        T = 3;
        if isempty(iter) || iter==0 %|| iter~=round(iter)
            LUTheader = quasirandomsampling(LUTinputs,alg.GenConf,iter,rndseed);
        elseif iter~=round(iter) || round(iter/T)==iter/T
            LUTheader = homogeneussampling(alg.GenConf,iter);
        elseif iter==round(iter)
            LUTheader = automaticsampling(alg.GenConf,iter,errors);
        end
end
alg.LUTheader = LUTheader;
alg.LUTinputs = LUTinputs;

end % END of calcLUTgrid function



%% SAMPLING FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%
function LUTheader = griddedsampling(LUTinputs)
% Determine number of input values (siz) and number of combinations (ncomb):
if ~isempty(LUTinputs.dynamic.discrete)
	fn_discrete = fieldnames(LUTinputs.dynamic.discrete);
else
	fn_discrete = [];
end

if ~isempty(LUTinputs.dynamic.continuous)
	fn_cont = fieldnames(LUTinputs.dynamic.continuous);
else
	fn_cont = [];
end

siz = zeros(1,numel(fn_discrete)+numel(fn_cont));
if isempty(siz) % Only static values (i.e. one simulation) ==> no header
	LUTheader = [];
else            % One or more dynamic values ==> header
	for i=1:numel(fn_discrete)
        LUTaux.(fn_discrete{i}) = LUTinputs.dynamic.discrete.(fn_discrete{i});
        siz(i) = numel(LUTinputs.dynamic.discrete.(fn_discrete{i}));
    end
    for i=1:numel(fn_cont)
    	LUTaux.(fn_cont{i}) = LUTinputs.dynamic.continuous.(fn_cont{i});
        siz(i+numel(fn_discrete)) = numel(LUTinputs.dynamic.continuous.(fn_cont{i}));
    end
    
    % Fill LUTheader, starting from discrete variables then continuous:
    if numel(fieldnames(LUTaux))==1
        aux = fieldnames(LUTaux);
        LUTheader = LUTaux.(aux{1});
    else
    	aux    = struct2cell(LUTaux);	C = cell(1,numel(aux));
        [C{:}] = ndgrid(aux{:});        LUTheader = [];
        for i=1:numel(aux)
        	aux = C{i}; aux = aux(:); LUTheader = [LUTheader,aux];
        end
        LUTheader = LUTheader';
	end    
end

end % END of griddedsampling function
%%%%%%%%

%%%%%%%%
function LUTheader = quasirandomsampling(LUTinputs,GenConf,iter,rndseed)
fn_cont = fieldnames(LUTinputs.dynamic.continuous);  p = numel(fn_cont);

lb = zeros(1,p); ub = lb; aux = cell(p,1);
for i=1:p
	lb(i) = min(LUTinputs.dynamic.continuous.(fn_cont{i}));
    ub(i) = max(LUTinputs.dynamic.continuous.(fn_cont{i}));
    aux{i} = [lb(i),ub(i)];
end
if ~isempty(iter)
    % Add 2% margins on lb and ub to avoid border effects on interpolation: 
    margin = 2*abs(ub-lb)/100;
    lb = lb - margin; ub = ub + margin;
end

if isempty(iter) || iter==0
    if isempty(iter), N=2; elseif iter==0, N=3; end
    % Generate default samples corresponding to all corners of the 
    % hypercube based on the lower and upper bounds from user-input 
    % continuous values:
    aux = {linspace(0,1,N)}; aux = repmat(aux,[1,p]); [aux{:}] = ndgrid(aux{:});
    A = zeros(N^p,p); for i=1:p, A(:,i) = aux{i}(:); end
    A1 = A(sum(A==0 | A==1,2)==p,:); A2 = A(sum(A==0 | A==1,2)==p-1,:);
    A = [A1;A2];
    LUTheader_cont = bsxfun(@plus,lb,bsxfun(@times,A,(ub-lb)))'; clear A
else
    LUTheader_cont = [];
end


% Add n continuous samples, not counting the number of combinations from 
% discrete variables:
ndisc = 1;
if ~isempty(LUTinputs.dynamic.discrete)
	fn_discrete = fieldnames(LUTinputs.dynamic.discrete);
    for i=1:numel(fn_discrete)
    	ndisc  = ndisc*numel(LUTinputs.dynamic.discrete.(fn_discrete{i}));
        siz(i) = numel(LUTinputs.dynamic.discrete.(fn_discrete{i}));
    end
end
    
if isempty(iter)
    n      = fix(str2num(GenConf.numberpoints)/ndisc);
    counts = 1:n;
elseif iter>0 && iter~=round(iter)
    n      = 5*(2^p); % Fixed value REVISAR!!
else
    % dn: Increase of dn points in LUT at each iteration
    dn     = 5*2^p; % Fixed value REVISAR!!!
    counts = iter*dn+1:(iter+1)*dn; n = numel(counts);
end

% Distribute n samples of continuous variables according to selected 
% sampling distribution in GenConf.inputmode:
if ~isempty(rndseed), rng(rndseed), end
if strcmp(GenConf.inputmode,'Latin hypercube') || strcmp(GenConf.inputmode,'Automatic (gradient)')
	xn = lhsdesign(n,p); % generate normalized design
elseif strcmp(GenConf.inputmode,'Sobol sequence')
	xn = sobolset(p);  xn = scramble(xn,'MatousekAffineOwen'); xn  = xn(counts,:); % generate normalized design
elseif strcmp(GenConf.inputmode,'Halton sequence')
	xn = haltonset(p); xn = scramble(xn,'RR2'); xn  = xn(counts,:); % generate normalized design
end
LUTheader_cont = [LUTheader_cont,bsxfun(@plus,lb,bsxfun(@times,xn,(ub-lb)))'];
    
% Combine discrete with continuous variables:
if ~isempty(LUTinputs.dynamic.discrete)
	subindex      = cell(1,numel(fn_discrete));  aux = zeros(1,numel(fn_discrete));
    for i=1:ndisc
    	[subindex{:}] = ind2sub(siz(1:numel(fn_discrete)),i); % Determine subindexes
        for j=1:numel(siz), aux(j) = LUTinputs.dynamic.discrete.(fn_discrete{j})(subindex{j}); end
        LUTheader_disc(:,i) = aux';
	end
    for i=1:n
    	ind = (i-1)*ndisc+1;
        LUTheader(:,ind:ind+ndisc-1) = [LUTheader_disc;repmat(LUTheader_cont(:,i),[1,ndisc])];
    end
else
	LUTheader = LUTheader_cont;
end

% if ~isempty(iter) && iter>0
%     [~,~,errormap,~]   = interperror(GenConf.filename,1,'matlablinear',[]);% BORRAR
%     plotresults(GenConf.filename,errormap,LUTheader,iter) % BORRAR
% end
end % END of quasirandomsampling function
%%%%%%%%

%%%%%%%%
function LUTheader = automaticsampling(GenConf,iter,errors)

% Calculate error map
X = errors.X; IND = errors.IND; errormap = errors.errormap;
clear errors
errormap(1:2^size(X,1)) = myinterpn(X(:,2^size(X,1)+1:end),errormap(2^size(X,1)+1:end),X(:,1:2^size(X,1)),'nearest');

% Delaunay triangulation of LUT grid points (due to different units of 
% input dimensions, normalize them to min=0 and max=1):
minX = min(X,[],2);     maxX = max(X,[],2);
M    = 1./(maxX-minX);  N    = -minX.*M;
X    = bsxfun(@plus,bsxfun(@times,X,M),N);
tri  = delaunayn(X'); % Might be problematic at dimensions >6

% For very large files it would be better to access to each point instead 
% of loading the whole LUT in RAM memory
Y = h5read([GenConf.filename,'.h5'],'/LUTdata');

% LUTheader = zeros(size(X,1),sum(errormap>str2num(GenConf.errorthrs))); k=0;
LUTheader = [];
ind = 1:size(X,2);
% Add a new node around each LUT node following the gradients to each 
% surrounding vertex of a Delaunay triangulation:
for i=2^size(X,1)+1:size(X,2) % For each node
    if errormap(i)>str2num(GenConf.errorthrs)
%         k=k+1;
        % Find vertex of the Delaunay triangulation:
        [indT,~] = find(tri==i);
%         indT     = unique(reshape(tri(indT,:),1,numel(tri(indT,:)))); indT(indT==i)=[];
        E = zeros(1,numel(indT));
        for j=1:numel(indT)
            indTj = indT(j); triJ = tri(indTj,:);
            DY = abs(bsxfun(@minus,Y(IND(i),triJ),Y(IND(i),i)));
            E(j) = sqrt(sum(DY.^2)/numel(DY));
%             if E(j) > str2num(GenConf.errorthrs)
%                 r = sum(X(:,triJ),2)/(size(X,1)+1); % Based on centroid of triangle
%                 LUTheader = [LUTheader,r];                
%             end
        end
        [maxE,aux] = max(E);
        if maxE>str2num(GenConf.errorthrs)
            % Add one in the triangle with maximum error
            r = sum(X(:,tri(indT(aux),:)),2)/(size(X,1)+1); % Based on centroid of triangle
        else
            % Add one in the triangle with maximum volume
            V = zeros(1,numel(indT));
            for j=1:numel(indT)
                V(j) = det(bsxfun(@minus,X(:,tri(indT(j),2:end)),X(:,tri(indT(j),1))));
            end
            V = abs(V)/factorial(size(X,1)); [~,aux] = max(V);
            r = sum(X(:,tri(indT(aux),:)),2)/(size(X,1)+1); % Based on centroid of triangle
        end
        LUTheader = [LUTheader,r];
    end
end

% Remove repeated nodes:
LUTheader = unique(LUTheader','rows')';
% Convert back into variable units:
LUTheader = bsxfun(@rdivide,bsxfun(@minus,LUTheader,N),M);

% plotresults(GenConf.filename,errormap,LUTheader,iter) % BORRAR
end % END of automaticsampling function
%%%%%%%%

%%%%%%%%
function LUTheader = homogeneussampling(GenConf,iter)

hdr = readLUThdr([GenConf.filename,'.h5']); X = hdr.LUTheader;

% Delaunay triangulation of LUT grid points (due to different units of 
% input dimensions, normalize them to min=0 and max=1):
minX = min(X,[],2);     maxX = max(X,[],2);
M    = 1./(maxX-minX);  N    = -minX.*M;
X    = bsxfun(@plus,bsxfun(@times,X,M),N);
tri  = delaunayn(X'); % Might be problematic at dimensions >6
p    = size(X,1);

% Calculate volume of each simplex:
V = zeros(1,size(tri,1));
for i=1:numel(V)
    V(i) = det(bsxfun(@minus,X(:,tri(i,2:end)),X(:,tri(i,1))));
end
V = abs(V)/factorial(p); [~,idx] = sort(V,'descend');

% Calculate centroid of the n simplex with highest volume:
n = 2*(2^p); % Fixed value REVISAR!!
LUTheader = zeros(p,n);
for i=1:n, LUTheader(:,i) = sum(X(:,tri(idx(i),:)),2)/(p+1); end

% Convert back into variable units:
LUTheader = bsxfun(@rdivide,bsxfun(@minus,LUTheader,N),M);

% [~,~,errormap,~]   = interperror(GenConf.filename,1,'matlablinear',[]);% BORRAR
% plotresults(GenConf.filename,errormap,LUTheader,iter) % BORRAR
end % END of homogeneussampling function
%%%%%%%%
