function writeLUThdr(alg)
%WRITELUTHDR Write Look-Up table header file
%   WRITELUTHDR(ALG,LUTINPUTS) creates the header of the LUT file 
%   containing a summary of the input values to run an atmospheric RTM.
%   This includes the default values for the key atmospheric parameters 
%   given in the RTM configuration structure ALG.RTMCONF and the dynamic 
%   user-input values given by the LUTINPUTS structure that create the 
%   input table stored in ALG.LUTHEADER variable.
%
%   See also: ALG, LUTGEN_INPUTS, CALCLUTGRID, READLUTHDR

%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.03 $  $Date: 2018/12/22 $

cellfind = @(string)(@(cell_contents)(strcmp(string,cell_contents)));

filename = [alg.GenConf.filename,'.h5'];

groups = fieldnames(alg.ParamConf);

if ~isempty(alg.LUTinputs.dynamic.discrete)          
    fn_discrete = fieldnames(alg.LUTinputs.dynamic.discrete);
else
    fn_discrete = [];
end
if ~isempty(alg.LUTinputs.dynamic.continuous)
    fn_cont     = fieldnames(alg.LUTinputs.dynamic.continuous);
else
    fn_cont = [];
end

% File ID open:
fid = H5F.create(filename);

%% RTM name
% Write attributes to main h5 group
h5writeatt(filename,'/','RTMname',alg.GenConf.RTMname)
h5writeatt(filename,'/','opmode',alg.GenConf.op_mode)
h5writeatt(filename,'/','inputmode',alg.GenConf.inputmode)

%% Print static parameters: "* Description (ID): value (desc if available)"
fn = fieldnames(alg.LUTinputs.static);
for i=1:numel(fn)
    for j=1:numel(groups)
        varnames   = fieldnames(alg.RTMconf.(groups{j}));
        idx = find(cellfun(cellfind(fn{i}),varnames),1);
        if ~isempty(idx)
            aux1 = alg.RTMconf.(groups{j}).(varnames{idx}).description;
            aux2 = alg.RTMconf.(groups{j}).(varnames{idx}).name;
%             aux3 = RTMconf.(groups{j}).(varnames{idx}).default;
            aux3 = num2str(alg.LUTinputs.static.(varnames{idx}));
            desc = alg.RTMconf.(groups{j}).(varnames{idx}).desc;
            if ~strcmp(alg.RTMconf.(groups{j}).(varnames{idx}).datatype,'STRING')
            	h5create(filename,['/static/',aux2],[1 1])
                h5write(filename,['/static/',aux2], alg.LUTinputs.static.(varnames{idx}))
            else
            	h5writestr(fid,['/static/',aux2],{' '})
            end
            h5writeatt(filename,['/static/',aux2],'unit',alg.RTMconf.(groups{j}).(varnames{idx}).unit)
            
            if strcmp(alg.RTMconf.(groups{j}).(varnames{idx}).paramtype,'discrete')
                desc = str2cell(desc(2:end-1));
                values = str2num(alg.RTMconf.(groups{j}).(varnames{idx}).values);
                ind = find(str2num(aux3)==values);
                h5writeatt(filename,['/static/',aux2],'description',[aux1,': ',desc{ind}])
            elseif isempty(desc)
                h5writeatt(filename,['/static/',aux2],'description',aux1)
            else
                desc = alg.RTMconf.(groups{j}).(varnames{idx}).desc;
                h5writeatt(filename,['/static/',aux2],'description',[aux1,': ',desc(strfind(desc,'=')+1:end)])
            end
        end
    end
end

%% Print dynamic parameters: "* Description (ID, continuous/discrete)"
% for each combination"
auxvarnames = [];

fn = [fn_discrete;fn_cont];
for i=1:numel(fn)
    for j=1:numel(groups)
    	varnames   = fieldnames(alg.RTMconf.(groups{j}));
        idx = find(cellfun(cellfind(fn{i}),varnames),1);
        if ~isempty(idx)
        	aux1 = alg.RTMconf.(groups{j}).(varnames{idx}).description;
            aux2 = alg.RTMconf.(groups{j}).(varnames{idx}).name;
            auxvarnames = [auxvarnames,',',alg.RTMconf.(groups{j}).(varnames{idx}).name];
            aux3 = alg.RTMconf.(groups{j}).(varnames{idx}).paramtype;
            aux4 = alg.RTMconf.(groups{j}).(varnames{idx}).unit;
            if contains(alg.GenConf.inputmode,'grid') || strcmp(aux3,'discrete')
            	h5create(filename,['/dynamic/',aux2],[1 numel(unique(alg.LUTheader(i,:)))])
                h5write(filename,['/dynamic/',aux2], unique(alg.LUTheader(i,:)))
            else
            	h5create(filename,['/dynamic/',aux2],[1 2])
                h5write(filename,['/dynamic/',aux2], [min(alg.LUTheader(i,:)),max(alg.LUTheader(i,:))])
            end
            h5writeatt(filename,['/dynamic/',aux2],'description',aux1)
            h5writeatt(filename,['/dynamic/',aux2],'paramtype',aux3)
            h5writeatt(filename,['/dynamic/',aux2],'unit',aux4)
        end
    end
end

sz = size(alg.LUTheader);
if ~strcmp(alg.GenConf.stopmethod,'Input max. # of samples')
	h5create(filename,'/LUTheader',[sz(1) Inf],'ChunkSize',sz);
else
	h5create(filename,'/LUTheader',sz);
end
h5write(filename,'/LUTheader',alg.LUTheader,[1 1],sz);
auxvarnames(1)=[];
h5writeatt(filename,'/LUTheader','varnames',auxvarnames)

%% Close editing header file:
H5F.close(fid)
end

function h5writestr(fid,datasetname,str)
    plist = 'H5P_DEFAULT';
    type_id = H5T.copy('H5T_C_S1');
    H5T.set_size(type_id,'H5T_VARIABLE');
    dims = [1 numel(str)]; h5_dims = fliplr(dims); h5_maxdims = h5_dims;
    space_id = H5S.create_simple(2,h5_dims,h5_maxdims);
    dcpl = 'H5P_DEFAULT';
    dset_id = H5D.create(fid,datasetname,type_id,space_id,dcpl);
    H5D.write(dset_id,'H5ML_DEFAULT','H5S_ALL','H5S_ALL',plist,str);
    H5S.close(space_id); H5T.close(type_id); H5D.close(dset_id);
end