function LUTGen_inputs(RTMconffile,LUTconffile)
%LUTGEN_INPUTS Look-Up table input files generator
%   LUTGEN_INPUT(RTMconf,LUTconf) generates all the temporary input
%   files needed for the execution of a user-selected Radiative Transfer
%   Model (RMT) over a set of combination of key atmospheric parameters
%   defining the input grid space. It also runs the atmospheric RTMs and
%   store all the temporary RTM output files needed for the generation of 
%   an atmospheric Look-Up Table (LUT).
%   The input information (key input parameters, selected RTM, LUT file
%   name...) is stored in the LUT configuration file LUTconf.
%   Default values for the key input parameters are stored in the RTM
%   configuration file RMTconf.
%
%   See also: LUTGEN_PROCESS

%   Copyright 2019 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.06 $  $Date: 2019/01/24 $


%%
cellfind = @(string)(@(cell_contents)(strcmp(string,cell_contents)));

% Read LUTconf and RTMconf files:
GenConf = read_LUTconfig(LUTconffile,'GenConf');
% alg = ALG(LUTconffile,RTMconffile);
eval(['alg = ',GenConf.RTMname,'(LUTconffile,RTMconffile);'])

%% 1.2) Run OPAC (only if option selected in LUTconf file):
% If OPAC is selected in the LUTconf file (or by default in the RTMconf
% file), then OPAC is run based on the key input parameters defined in the
% LUTconf file and/or the default values stored in the OPACconf file.

% Detect if OPAC aerosols are input in the LUT:
OpacConf = [];
try OpacConf = read_LUTconfig(LUTconffile,'OpacConf');
    fn = fieldnames(OpacConf);
    if any(cellfun(cellfind('OpacConf'),fn))
        if isempty(OpacConf.OpacConf), OpacConf = []; end
    end
end

if ~isempty(OpacConf)
    [OPACconf,~,data] = read_OPACconfig(['..',filesep,'Conf',filesep,'OPACconf.xml']);
    [opacpathstr,~,~] = fileparts(OPACconf.filepath);
    inputpath         = [opacpathstr,filesep,'input',filesep];
    outputpath        = [opacpathstr,filesep,'result',filesep];
    varnames          = {'AERTYPE','PARTYPE','NDENS','WVLAER','RH'};
    auxWVL            = str2num(data.MainParam.WVLAER.values);

    AERTYPE           = str2num(OpacConf.AERTYPE.values);
    RH                = [];                                     % Include all RH available in OPAC
    minWVL            = str2num(alg.SpeConf.MinLambda.values)/1000; % In [um]
    maxWVL            = str2num(alg.SpeConf.MaxLambda.values)/1000; % In [um]
    WVLAER            = [];
    for i=1:numel(minWVL)
        liminf = find(auxWVL<minWVL(i)); if isempty(liminf), liminf=1; else liminf = liminf(end);end
        limsup = find(auxWVL>maxWVL(i)); if isempty(limsup), limsup=numel(auxWVL); else limsup = limsup(1);end
        WVLAER = [WVLAER,auxWVL(liminf:limsup)];
    end
    WVLAER = unique(sort([0.55,WVLAER])); % 550nm value must be present to scale for MODTRAN
    for i=1:numel(AERTYPE) % For each OPAC aerosol in LUT
        filename = [inputpath,'Aer',num2str(AERTYPE(i)),'.inp'];
        if AERTYPE(i)<=10
            % Value is predefined mixture from OPAC 
            inputval = {AERTYPE(i),zeros(1,5),zeros(1,5),WVLAER,RH};
        else
            % Value is user-input mixture to OPAC
            fn          = fieldnames(OpacConf);
            composition = [];
            for j=1:numel(fn)
                if strfind(fn{j},'NDENS')
                    aux         = str2num(OpacConf.(['NDENS',num2str(j)]).values);
                    composition = [composition,aux(AERTYPE(i)-10)];
                end
            end
            ind      = find(composition~=0);
            inputval = {0,ind,composition(ind),WVLAER,RH};
        end
        % Write OPAC input file:
        Write_opacinp(filename,varnames,inputval)
        % Run OPAC based on created input file:
        fid = fopen([alg.GenConf.filename,'_runOPAC.bat'],'w');
        if ~strcmp(alg.GenConf.filename(1),OPACconf.filepath(1))
            % Change between disks to run OPAC if necessary
            fprintf(fid,'%-100s\n',[OPACconf.filepath(1),':']);
        end
        fprintf(fid,'%-100s\n',['cd ',fileparts(OPACconf.filepath)]);
        [~,name,~] = fileparts(filename);
        fprintf(fid,'%-100s\n',['echo ',name,' | opac']); fclose(fid);
        evalin('base',['!',alg.GenConf.filename,'_','runOPAC.bat',' &'])
        delete([alg.GenConf.filename,'_','runOPAC.bat'])
        % Read OPAC each output file and store variables in OPAC LUT .h5
        % file:
        filename = [outputpath,'Aer',num2str(AERTYPE(i)),'.out'];
        [VARSPC,RH,EXTC,ABSC,ASYM,ANGF,F] = Read_opacout(filename);
        if i==1 % First OPAC file ==> create opac LUT file
            % Delete old .h5 file if has the same name
            [pathstr,~,~] = fileparts(alg.GenConf.filename); d = dir(pathstr);
            for n=1:length(d)
                if ~d(n).isdir && ~strcmp(d(n).name,[alg.GenConf.filename,'_OPAC.h5'])
                    delete([pathstr,'/input/',d(n).name]) % Delete old files  
                end
            end
            h5create([alg.GenConf.filename,'_OPAC.h5'],'/AERTYPE',[numel(AERTYPE) 1]);
            h5create([alg.GenConf.filename,'_OPAC.h5'],'/VARSPC',[numel(VARSPC) 1]);
            count = [numel(VARSPC) 1];
            h5write([alg.GenConf.filename,'_OPAC.h5'], '/VARSPC',VARSPC',[1 1],count);
            h5create([alg.GenConf.filename,'_OPAC.h5'],'/RH',[numel(RH) 1]);
            count = [numel(RH) 1];
            h5write([alg.GenConf.filename,'_OPAC.h5'], '/RH',RH',[1 1],count);
            h5create([alg.GenConf.filename,'_OPAC.h5'],'/ANGF',[numel(ANGF) 1]);
            count = [numel(ANGF) 1];
            h5write([alg.GenConf.filename,'_OPAC.h5'], '/ANGF',ANGF',[1 1],count);
            h5create([alg.GenConf.filename,'_OPAC.h5'],'/SPEC',[numel(VARSPC) numel(RH) 3 numel(AERTYPE)]);
            h5create([alg.GenConf.filename,'_OPAC.h5'],'/F',[numel(ANGF) numel(VARSPC) numel(RH) numel(AERTYPE)]);
        end
        start = [i 1];
        h5write([alg.GenConf.filename,'_OPAC.h5'], '/AERTYPE',AERTYPE(i),start,[1 1]);
        start = [1,1,1,i]; count = [numel(VARSPC) numel(RH) 3 1];
        h5write([alg.GenConf.filename,'_OPAC.h5'], '/SPEC',cat(3,EXTC,ABSC,ASYM),[1 1 1 i],count);
        start = [1,1,1,i]; count = [numel(ANGF) numel(VARSPC) numel(RH) 1];
        h5write([alg.GenConf.filename,'_OPAC.h5'], '/F',F,start,count);
    end
end

%% The execution of the following steps depends on the selected stop method
% 1.4) Determine LUT grid points
% 2)   Create atmospheric RTM input file(s)
% 3)   Run atmospheric RTM
% 4)   Read & process atmospheric RTM output file(s) and create LUT file

if strcmp(alg.GenConf.stopmethod,'Input max. # of samples')
    % In this case, the LUT grid points depend on the user-input number of
    % samples (or the input values of gridded variables). The steps 1.4),
    % 2) and 3) are therefore run sequentially in one single iteration.
    % Step 4) is run manually by the user once all the RTM output files
    % have been generated.
    
    % 1.4) Determine LUT grid points:
    % Calculate LUT grid points based on default RTM and user-input LUT 
    % configuration files and create LUT header file:
    rndseed               = randi(2^32);       % Fixed random seed
    alg  = alg.calcLUTgrid([],rndseed,[]);
    alg.writeLUThdr();
    
    % 2) Create atmospheric RTM input file(s):
    % A set of files are generated for every combination of the key input
    % atmospheric parameters so that they can be read and run by the
    % selected atmospheric RTM. This step considers the possibility of 
    % multithreading.
    alg.generateRTMinputs()
    
    % 3) Run atmospheric RTM:
    alg.runRTM([])
    
elseif strcmp(alg.GenConf.stopmethod,'Cross-validation error')
    % In this case, the number of LUT grid points depends on the evaluation 
    % of the interpolation error against the user-input error threshold
    % value and through the method of "Cross-validation error".
    % The steps 1.4), 2), 3) and 4) are therefore run sequentially in 
    % several iterations until the stop condition is met.
    msgh = findobj(findobj('Tag','AtmLutGen'),'Tag','msgtextbox');
    
    rndseed = rng('shuffle');%(2^32); % Fixed random seed
    iter    = 0; Ntrue = 0;
    
    % 1.4) Determine LUT grid points: Calculate the initial LUT grid points
    % based on default RTM and user-input LUT configuration files and 
    % create the initial LUT header file:
    alg  = alg.calcLUTgrid(iter,rndseed,[]);
    alg.writeLUThdr();
    % 2) Create atmospheric RTM input file(s) for iter=0:
    alg.generateRTMinputs()
    % 3) Run atmospheric RTM and create LUT file for iter=0:
    msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter)],'Running RTM. Please wait...'};
    set(msgh,'String',msgtext,'Visible','on')
    tRTM = 0; tic, alg.runRTM('spmd'), delete(gcp('nocreate')), tRTM = tRTM + toc;
    msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter)],['Running RTM completed. Elapsed time: ',num2str(toc/60),' min']};
    set(msgh,'String',msgtext)
    % 4) Read & process atmospheric RTM output file(s) and create LUT file
    alg.createLUTfile(iter)
    
    % Evaluation of the interpolation error:
    method     = 'matlablinear'; % Interpolation method (see help myintepn)
    Kfold      = 1;              % Leave-one-out (Kfold=1) or K-fold validation (Kfold>1)
    P          = 95;             % Percentile value
    msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter)],'Calculating interpolation error. Please wait...'};
    set(msgh,'String',msgtext)
    [errors,errorstats] = interperror(alg.GenConf.filename,Kfold,method,P); kk=1; F(kk) = getframe(gcf);
    error      = errorstats.maxerror;
    msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter)],['Interpolation error: ',sprintf('%4.1f',error),' %'],['Goal: ',alg.GenConf.errorthrs,' %']};
    set(msgh,'String',msgtext)
    if error<=str2num(alg.GenConf.errorthrs)
        Ntrue = Ntrue+1;
        if strcmp(alg.GenConf.inputmode,'Automatic (gradient)')
            % Repeat
            alg  = alg.calcLUTgrid(iter+0.5,[],[]);
            alg.updateLUThdr([],[],[],alg.LUTheader)
            alg.generateRTMinputs()
            msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],'Running RTM. Please wait...'};
            set(msgh,'String',msgtext)
            tic, alg.runRTM('spmd'), delete(gcp('nocreate')), tRTM = tRTM + toc;
            msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],['Running RTM completed. Elapsed time: ',num2str(toc/60),' min']};
            set(msgh,'String',msgtext)
            alg.createLUTfile(iter+0.5)
            msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],'Calculating interpolation error. Please wait...'};
            set(msgh,'String',msgtext)          
            [errors,errorstats] = interperror(alg.GenConf.filename,Kfold,method,P); kk=kk+1; F(kk) = getframe(gcf);
            error      = errorstats.maxerror;
            msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],['Interpolation error: ',sprintf('%4.1f',error),' %'],['Goal: ',alg.GenConf.errorthrs,' %']};
            set(msgh,'String',msgtext)
            if error<=str2num(alg.GenConf.errorthrs), Ntrue = Ntrue+1; end
        end
    end
    
    % Report results:
    f   = dir([alg.GenConf.filename,'.h5']);
    hdr = readLUThdr([alg.GenConf.filename,'.h5']); X = hdr.LUTheader;
    fid = fopen([alg.GenConf.filename,'_report.txt'],'w+');
    fprintf(fid,['Stop method: ',alg.GenConf.stopmethod,'\n']);
    fprintf(fid,['Interpolation method: ',method,'\n\n']);
    fprintf(fid,['iter \t N_samples \t t_RTM [h] \t t_interp [s/spectra] \t ',...
                 'File Size [MB] \t P',num2str(P),'(max_error) [%%] \t P',num2str(P),'(RRMSE) [%%]\n']);
    fprintf(fid,'---- \t --------- \t --------- \t -------------------- \t -------------- \t ------------------- \t ---------------\n');
    fprintf(fid,'%1.1f \t\t %i \t\t %2.2f \t\t\t %2.5f \t\t\t %2.2f \t\t\t\t %2.2f \t\t\t\t\t %2.2f\n',...
           iter,size(X,2),tRTM/3600,errorstats.t_interp,f.bytes/1048576,errorstats.maxerror,errorstats.rrmse);
    
    % Iterative process until interpolation error is below user-input error
    % threshold value:
    while error>str2num(alg.GenConf.errorthrs) && Ntrue<2
        iter = iter+1; % Iteration #1 to...
        
        % Calculate the next iterations of the LUT grid points and update
        % LUT header file:
        alg  = alg.calcLUTgrid(iter,rndseed,errors);
        alg.updateLUThdr([],[],[],alg.LUTheader)
        % 2) Create atmospheric RTM input file(s):
        alg.generateRTMinputs()
        % 3) Run atmospheric RTM and create LUT file for iter:
        msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],'Running RTM. Please wait...'};
        set(msgh,'String',msgtext)
        tic, alg.runRTM('spmd'), delete(gcp('nocreate')), tRTM = tRTM + toc;
        msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],['Running RTM completed. Elapsed time: ',num2str(toc/60),' min']};
        set(msgh,'String',msgtext)
        % 4) Read & process atmospheric RTM output file(s) and create LUT file
        alg.createLUTfile(iter)

        % Evaluation of the interpolation error:
        msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter)],'Calculating interpolation error. Please wait...'};
        set(msgh,'String',msgtext)
        [errors,errorstats] = interperror(alg.GenConf.filename,Kfold,method,P); kk=kk+1; F(kk) = getframe(gcf);
        error      = errorstats.maxerror;
        msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter)],['Interpolation error: ',sprintf('%4.1f',error),' %'],['Goal: ',alg.GenConf.errorthrs,' %']};
        set(msgh,'String',msgtext)
       
        % Report results:
        hdr = readLUThdr([alg.GenConf.filename,'.h5']); X = hdr.LUTheader;
        fprintf(fid,'%1.1f \t\t %i \t\t %2.2f \t\t\t %2.5f \t\t\t %2.2f \t\t\t\t %2.2f \t\t\t\t\t %2.2f\n',...
                iter,size(X,2),tRTM/3600,errorstats.t_interp,f.bytes/1048576,errorstats.maxerror,errorstats.rrmse);
        
        if error<=str2num(alg.GenConf.errorthrs)
            Ntrue = Ntrue+1;
            if strcmp(alg.GenConf.inputmode,'Automatic (gradient)')
                % Repeat
                alg  = alg.calcLUTgrid(iter+0.5,[],[]);
                alg.updateLUThdr([],[],[],alg.LUTheader)
                alg.generateRTMinputs()
                msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],'Running RTM. Please wait...'};
                set(msgh,'String',msgtext)
                tic, alg.runRTM('spmd'), delete(gcp('nocreate')), tRTM = tRTM + toc;
                msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],['Running RTM completed. Elapsed time: ',num2str(toc/60),' min']};
                set(msgh,'String',msgtext)
                alg.createLUTfile(iter+0.5)
                msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],'Calculating interpolation error. Please wait...'};
                set(msgh,'String',msgtext)
                [errors,errorstats] = interperror(alg.GenConf.filename,Kfold,method,P); kk=kk+1; F(kk) = getframe(gcf);
                error      = errorstats.maxerror;
                msgtext = {[alg.GenConf.inputmode,'. Iter: ',num2str(iter+0.5)],['Interpolation error: ',sprintf('%4.1f',error),' %'],['Goal: ',alg.GenConf.errorthrs,' %']};
                set(msgh,'String',msgtext)
       
                hdr = readLUThdr([alg.GenConf.filename,'.h5']); X = hdr.LUTheader;
                fprintf(fid,'%1.1f \t\t %i \t\t %2.2f \t\t\t %2.5f \t\t\t %2.2f \t\t\t\t %2.2f \t\t\t\t\t %2.2f\n',...
                        iter+0.5,size(X,2),tRTM/3600,errorstats.t_interp,f.bytes/1048576,errorstats.maxerror,errorstats.rrmse);

                if error<=str2num(alg.GenConf.errorthrs), Ntrue = Ntrue+1;
                else                                  Ntrue = Ntrue-1;
                end
            end
        end
        
    end
    fclose(fid);
    set(msgh,'Visible','off')
%     save([alg.GenConf.filename,'_movie.mat'],'F')
%     movie2avi(F, [alg.GenConf.filename,'_movie.avi'],'fps',1.25)
    
elseif strcmp(alg.GenConf.stopmethod,'Comparison of consecutive interp.')
    % NOT YET WRITTEN
end

end % END of LUTGen_inputs function
