function [VARSPC,RH,EXTC,ABSC,ASYM,ANGF,F] = Read_opacout(outfile)
%READ_OPACOUT Read OPAC output file (.out)
%   [WVL,TDIR,DRCT_RFLT,GRND_RFLT,SOL_SCAT] = READ_OPACOUT(OUTFILE) reads
%   OPAC output file (OUTFILE) and generates the following output
%   variables:
%   VARSPC      - Wavelength vector (in [um]) at which the output data is 
%                 given
%   RH          - Relative himidity (in [%]) at which the output data is
%                 given
%   EXTC        - Extinction coefficient (in [km-1]) evaluated at VARSPC 
%                 wavelengths and RH humidities and normalized to 1 km-1 at
%                 550 nm
%   ABSC        - Absorption coefficient (in [km-1]) evaluated at VARSPC 
%                 wavelengths and RH humidities and normalized so that EXTC
%                 is 1 km-1 at 550 nm
%   ASYM        - Asymmetry parameter (in [-]) evaluated at VARSPC 
%                 wavelengths and RH humidities
%   ANGF        - Scattering angle vector (in [deg]) at which the output 
%                 phase function is given and compatible with MODTRAN5
%   F           - Unit-normalized phase function (in [sr-1]. F is a 3D 
%                 matrix with dimensions IxJxK with I=numel(AGNF),
%                 J=numel(VARSPC) and K=numel(RH)
%
%   This function reads only OPAC data generated at one single relative
%   humidity value and containing only

%   Copyright 2016 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.00 $  $Date: 2016/10/14 $

%% Open File:
fid   = fopen(outfile);

%% Determine number of wavelengths, relative humidities and scattering
% angle values in header:

% Move to line 5, extract number of relative humidities and store it in 
% variable numRH:
for i=1:5
    tline = fgetl(fid);
end
aux = textscan(tline,'%s %f %s'); numRH = aux{2};

% Move to line 6, extract number of wavelengths and store it in variable
% numWVL:
tline = fgetl(fid);
aux = textscan(tline,'%s %f %s'); numWVL = aux{2};

% Detect line where phase function scattering angles are given:
while ischar(tline)
    if strfind(tline,'scattering angles')
        break
    end
    tline = fgetl(fid);
end
% The next 14 lines contain the scattering angles in 8 columns:
ANGF = zeros(1,14*8);
for i=1:14
    tline = fgetl(fid);
    aux = textscan(tline,'#\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f');
    ANGF(8*(i-1)+1:8*(i-1)+8) = cell2mat(aux);
end
tline = fgetl(fid);

%% Extract data for the variables VARSPC, RH, EXTC, ABSC, ASYM and F
VARSPC = zeros(1,numWVL);
RH     = zeros(1,numRH);
EXTC   = zeros(numWVL,numRH); ABSC = EXTC; ASYM = EXTC;
F      = zeros(numel(ANGF),numWVL,numRH);

for k=1:numRH
    tline = fgetl(fid);
    aux = textscan(tline,'#\t%f%s %s %s'); RH(k) = aux{1};
    for j=1:numWVL        
        tline = fgetl(fid); tline = fgetl(fid); tline = fgetl(fid);
        % Extract VARSPC, EXPC, ABSC and ASYM:
        aux = cell2mat(textscan(tline,'%f'));
        VARSPC(j) = aux(1);
        EXTC(j,k) = aux(2); ABSC(j,k) = aux(3); ASYM(j,k) = aux(4);
        tline = fgetl(fid);
        % Extract F(:,j,k):
        for i=1:14
            tline = fgetl(fid);
            aux = cell2mat(textscan(tline,'%f')); n = numel(aux);
            F(n*(i-1)+1:n*(i-1)+n,j,k) = aux;
        end
    end
    tline = fgetl(fid);
end
fclose(fid);


%% Re-scale data according to MODTRAN5:
% Re-scale EXTC and ABSC so that EXTC at 550 nm is 1 km-1:
% NOTE: Extinction = Absorption + Scattering
scale = interp1(VARSPC,EXTC,0.55);
EXTC = bsxfun(@rdivide,EXTC,scale);
ABSC = bsxfun(@rdivide,ABSC,scale);

% Re-scale F from [km-1] to Unit-normalized phase function:



% END of function
end
