function Write_opacinp(file,varnames,inputval)
% WRITE_OPACINP Create OPAC v3.1 input (.inp) file
%   WRITE_OPACINP(FILE,VARNAMES,INPUTVAL) creates, according to OPAC 
%   documentation, an OPAC's input file (FILE) for the configuration is 
%   given in the input variables and associated values stored in the cell 
%   arrays VARNAMES (strings) and INPUTVAL (numeric values).
%
%   Copyright 2016 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.00 $  $Date: 2016/09/20 $

%% Input variables to be used by OPAC:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%             INPUT VARIABLES TO BE USED BY OPAC                %%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if iscell(varnames)
    if numel(varnames)~=numel(inputval)
        error('Number of variable names and input values are not coincident.')
    end
end

% Associates input values to each variable name:
if iscell(varnames)
    for i=1:numel(varnames)
        eval([varnames{i},'=inputval{',num2str(i),'};'])
    end
else
    eval([varnames,'=inputval;'])
end

%--------------------------------------------------------------------------
% NOTE: Read OPAC Documentation for further information on the variables
% and input file formats.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%-------------------- PART 1: Cloud or Aerosol Type -----------------------
%--------------------------------------------------------------------------
% AERTYPE: aerosol or cloud model to be used: user-defined (0), default 
% aerosol types (1-10) or default cloud types (11-19)
AERTYPE = AERTYPE;
% PARTYPE: particule types used to create new aerosol and cloud mixtures:
% insoluble (1), water-soluble (2), soot (3), sea-salt accummulation mode
% (4), sea-salt coarse mode (5), mineral nuclei mode (6), mineral
% accummulation mode (7), mineral coarse mode (8), mineral transported (9)
% and sulfate (10). Only to be read when AERTYPE=0 (user-defined)
if AERTYPE==0
    % PARTYPE should be a vector of 5 elements. If input combination
    % contains less, then repeat last particle type
    PARTYPE = PARTYPE; 
    if numel(PARTYPE)<5
        PARTYPE(end+1:5) = PARTYPE(end);
    end
else
    PARTYPE = [1 2 3 4 5];
end
% NDENS: Number density of particles in [particles/cm3] of a user-define 
% aerosol mixture. Only to be read when AERTYPE=0 (user-defined)
if AERTYPE==0
    % NDENS should be a vector of 5 elements. If input combination
    % contains less, then complete with 0 particles/cm3 for the last elements
    NDENS = NDENS; 
    if numel(NDENS)<5
        NDENS(end+1:5) = 0;
    end
else
    NDENS = [0 0 0 0 0];
end
% AERNAME: new aerosol name (max. 30 characters)
AERNAME = 'new_aerosol_name';

%--------------------------------------------------------------------------
%--------------------     PART 2: Height profile    -----------------------
%--------------------------------------------------------------------------
% Hmin: Lower boundary or aerosol layer above ground in [km]. OPAC defines
% 5 layers being: 1) mixing layer, 2) mineral transported, 3) free
% troposphere, 4) stratosphere, and 5) cloud. Hmin therefore is a vector of
% 5 elements.
Hmin = zeros(1,5);
% Hmax: Upper boundary or aerosol layer above ground in [km]. OPAC defines
% 5 layers (see Hmin). Hmax therefore is a vector of 5 elements.
Hmax = zeros(1,5);
% Z: scale height in [km] of the particle density vertical profile for each
% aerosol layer. The vertical profile follows a decreasing exponential 
% scaled by Z: ~exp(-h/Z). OPAC defines 5 layers (see Hmin). Z therefore is
% a vector of 5 elements.
Z = zeros(1,5);


%--------------------------------------------------------------------------
%--------------------      PART 3: Wavelengths      -----------------------
%--------------------------------------------------------------------------
% WVLAER define the selected wavelengths in which the OPAC aerosol and 
% water clouds output data should be provided. The wavelength grid (61) is 
% predefined by OPAC and should not be changed.
refWVL_aer = [0.25 0.3 0.35 0.4 0.45 0.5 0.55 0.6 0.65 0.7 0.75 0.8 0.9 ...
              1 1.25 1.5 1.75 2 2.5 3 3.2 3.39 3.5 3.75 4 4.5 5 5.5 6 ...
              6.2 6.5 7.2 7.9 8.2 8.5 8.7 9 9.2 9.5 9.8 10 10.6 11 11.5 ...
              12.5 13 14 14.8 15 16.4 17.2 18 18.5 20 21.3 22.5 25 27.9 ...
              30 35 40];
if isempty(WVLAER), WVLAER=refWVL_aer; end
auxWVL_aer = WVLAER; % User input values
WVLAER = zeros(size(refWVL_aer));
[~,ia,~]=intersect(refWVL_aer,auxWVL_aer);
WVLAER(ia) = 1;
% WVL_aer = ones(1,61);
% WVL_cir define the selected wavelengths in which the OPAC cirrus output 
% data should be provided. The wavelength grid (32) is predefined by OPAC 
% and should not be changed.
WVL_cir = zeros(1,32);

%--------------------------------------------------------------------------
%--------------------   PART 4: Relative humidity   -----------------------
%--------------------------------------------------------------------------
% RH: Selection of relative humidity value of the atmosphere. RH should be 
% a vector of 0 or 1 with the selected value of relative humidity among the
% default OPAC options (0,50,70,80,90,95,98,99).
refRH = [0 50 70 80 90 95 98 99]; % Reference RH values in OPAC
if isempty(RH), RH=refRH; end
auxRH = RH; % User input values
RH = zeros(size(refRH));
[~,ia,~]=intersect(refRH,auxRH);
RH(ia) = 1;

%--------------------------------------------------------------------------
%--------------------   PART 5: Optical parameters  -----------------------
%--------------------------------------------------------------------------
% SELECT selects the output OPAC variables among 16 defined optical
% parameters (see OPAC manual for further information)
SELECT = [1 0 1 0 1 0 0 0 1 0 0 0 0 0 0 0];





%% Write variables to to generate OPAC input file (.inp):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%                    WRITE VARIABLES ON .INP FILE                     %%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Open file with write permission
fid = fopen(file,'w');

%----------------- PART 1: Cloud and Aerosol Type
fprintf(fid,'# PART 1: Cloud and Aerosol Type\n');
fprintf(fid,'%-2i\t: give here the selected number\n',AERTYPE);

fprintf(fid,'%-30s\t: name of new type (max. 30 characters)\n',AERNAME);

for i=1:5
    fprintf(fid,['%-2i, %-9.2e\t: number of ',num2str(i),'. component, number density\n'],PARTYPE(i),NDENS(i));
end

%----------------- PART 2: Height profile
fprintf(fid,'# PART 2: Height profile\n');
str = {'mixing layer','mineral transported','free troposphere','stratosphere','cloud'};
for i=1:5
    fprintf(fid,['%-3.1f, %-3.1f, %-3.1f\t: ',str{i},': Hmin, Hmax, Z\n'],Hmin(i),Hmax(i),Z(i));
end

%----------------- PART 3: Wavelengths
fprintf(fid,'# PART 3: Wavelengths\n');
fprintf(fid,'61         : number of wavelengths for A. (DONT CHANGE!)\n');
                        
auxWVL = [0.25,0.3,0.35,0.4,0.45,0.5,0.55,0.6,0.65,0.7,0.75,0.8,0.9,1.0,...
          1.25,1.5,1.75,2.0,2.5,3.0,3.2,3.39,3.5,3.75,4.0,4.5,5.0,5.5,...
          6.0,6.2,6.5,7.2,7.9,8.2,8.5,8.7,9.0,9.2,9.5,9.8,10.0,10.6,11.0,...
          11.5,12.5,13.0,14.0,14.8,15.0,16.4,17.2,18.0,18.5,20.0,21.3,22.5,...
          25.0,27.9,30.0,35.0,40.0];
for i=1:numel(auxWVL)
    fprintf(fid,'%i                               : %-3.2f\n',WVLAER(i),auxWVL(i));
end

fprintf(fid,'32         : number of wavelengths for B. (DONT CHANGE!)\n');

auxWVL = [0.280,0.300,0.320,0.350,0.355,0.400,0.423,0.532,0.535,0.550,...
          0.635,0.780,0.830,0.873,1.015,1.064,1.100,1.200,1.400,1.449,...
          1.504,1.555,1.615,1.850,1.905,2.000,2.190,2.600,3.077,3.413,...
          3.700,10.00];
for i=1:numel(auxWVL)
    fprintf(fid,'%i                               : %-5.3f\n',WVL_cir(i),auxWVL(i));
end

%----------------- PART 4: Relative humidity
fprintf(fid,'# PART 4: Relative humidity\n');
fprintf(fid,'8		 	: number of humidity classes (DONT CHANGE!)\n');

for i=1:numel(refRH)
    fprintf(fid,'%i 			: %2i%%\n',RH(i),refRH(i));
end

%----------------- PART 5: Relative humidity
fprintf(fid,'# PART 4: Optical parameters\n');
fprintf(fid,'18	 	: number of optical parameters (DONT CHANGE!)\n');

str = {'extinction coefficient (extco)',...
       'scattering      ''''      (scaco)',...
       'absorption      ''''      (absco)',...
       'single-scattering albedo  (sisca)',...
       'asymmetry factor          (asymf)',...
       'optical depth             (odepe)'};

for i=1:numel(str)
    fprintf(fid,['%i 			: ',str{i},'\n'],SELECT(i));
end
auxi=i;
fprintf(fid,'0                               :           -----------\n');

str = {'spectral turbidity factor (turbf)',...
       'lidar ratio               (lirat)',...
       'phase function',...
       'mass ext. cross section   (extra)',...
       'mass abs. cross section   (absra)'};

for i=1:numel(str)
    fprintf(fid,['%i 			: ',str{i},'\n'],SELECT(i+auxi));
end
auxi=i+auxi;
fprintf(fid,'0                               :           -----------\n');

str = {'norm. ext. coefficient    (exnor)',...
       'spectrally weighted coefficients',...
       'Angstrom coefficients',...
       'visibility',...
       'refractive index     (refre,refim)'};

for i=1:numel(str)
    fprintf(fid,['%i 			: ',str{i},'\n'],SELECT(i+auxi));
end



% Close .inp file:
fclose(fid);
% disp('File written. END')

end