function [errors,errorstats] = interperror(filename,Kfold,interpmethod,P)
%INTERPERROR Calculate interpolation errror on Look-Up Table through 
% cross-validation
%   [ERRORS,ERRORSTATS] = INTERPERROR(FILENAME,KFOLD,INTERPMETHOD,P)
%   calculates the interpolation error of a Look-Up Table (LUT) indicated 
%   in FILENAME and based on the cross-validation method given in KFOLD 
%   (>1 for Kfold; else for "leave-one-out"). The interpolation method is 
%   indicated in INTERPMETHOD, choosing among those methods implemented in 
%   the function MYINTERPN.
%   As results, INTERPERROR produces four outputs:
%
%   ERRORS: 	A structure containing: (1) X, the input variables matrix 
%               read from the LUT header file FILENAME; (2) IND, the index
%               of the spectral channels where the interpolation error 
%               (spectral) is maximum; and (3) ERRORMAP, the interpolation 
%               error calculated over each LUT node (only for the 
%               "leave-one-out" cross-validation method.
%   ERRORSTATS: The Pth percentile and RMSE of ERRORMAP.
%
%   See also: LUT2TOA, MYINTERPN, READLURHDR

%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.01 $  $Date: 2018/03/03 $
hdr = readLUThdr([filename,'.h5']); X = hdr.LUTheader;

GenConf = read_LUTconfig([filename,'.xml'],'GenConf');
Nparallel = str2num(GenConf.parallel);

refldb  = str2num(GenConf.refldb); clear GenConf
rho     = csvread(['..',filesep,'Others',filesep,'refldb.txt']);
rho     = spline(rho(:,1),rho(:,refldb),hdr.wvl);
Y       = h5read([filename,'.h5'],'/LUTdata');

p = size(X,1); sz = size(X,2); ini = 2^p;

% Recover lower/upper bounds of the user-defined input space where the
% interpolation error should be evaluated:
lb = min(X(:,1:ini),[],2); ub = max(X(:,1:ini),[],2);
margin = (ub - lb)/1.04;   lb = lb + 2*margin/100; ub = ub - 2*margin/100;

if Kfold>1 % KFold Validation % REVISAR EL CODIGO PARA KFOLD!!
    ind = [zeros(ini,1);crossvalind('Kfold', ini+1:sz, Kfold)]; n = 1:Kfold;
else       % Leave-One-Out Validation
    ind = 1:sz; n = ini+1:sz; n = true;
    for i=1:p, n = n & X(i,:)>=lb(i) & X(i,:)<=ub(i); end
    in = find(n); out = find(~n);
end
auxerror = zeros(numel(hdr.wvl),size(Y,2)); auxind = ini;
% auxerror2 = zeros(numel(hdr.wvl),size(Y,2)); 

if p>=3 % Altenative method fsor dimensions >3
    % NOTE: Triangulation won't be exactly the same as the method for
    % dimensions <3
    minX = min(X,[],2); maxX = max(X,[],2);
    M  = 1./(maxX-minX); N = -minX.*M;
    Xn  = bsxfun(@plus,bsxfun(@times,X,M),N);
    tri  = delaunayn(Xn');
end

tic
if p>=3
    parpool(Nparallel)
    parfor i=ind
        if any(in==i)
            test = (ind == i);
            % We assume that, when removing a point from the original
            % triangulation (tri), the new delaunay triangle should be among
            % the original vertices of the simplex containing i=in
            [train,~] = find(tri==i); 
            train     = unique(reshape(tri(train,:),1,numel(tri(train,:))));
            train(train==i)=[]; 
            
            % Interpolate atmospheric transfer functions:
            YI  = myinterpn(X(:,train),Y(:,train),X(:,test),interpmethod);
            if sum(isnan(YI))==size(YI,1), YI = myinterpn(X(:,train),Y(:,train),X(:,test),'nearest');end
        %     auxind = auxind(end)+(1:sum(test));  REVISAR EL CODIGO PARA KFOLD!!
            LTOAI = LUT2TOA(filename,YI,rho,test);
            LTOA  = LUT2TOA(filename,Y(:,test),rho,test);
            % Calculate error in TOA radiance:
            auxerror(:,i) = 100*abs(LTOAI - LTOA)./LTOA; % REVISAR INDICE PARA KFOLD!!
        end
    end
    poolobj = gcp('nocreate'); delete(poolobj);
else
    for i=ind
        if any(in==i)
            test = (ind == i);
            train = ~test;
            
            % Interpolate atmospheric transfer functions:
            YI  = myinterpn(X(:,train),Y(:,train),X(:,test),interpmethod);
            if sum(isnan(YI))==size(YI,1), YI = myinterpn(X(:,train),Y(:,train),X(:,test),'nearest');end
        %     auxind = auxind(end)+(1:sum(test));  REVISAR EL CODIGO PARA KFOLD!!
            LTOAI = LUT2TOA(filename,YI,rho,test);
            LTOA  = LUT2TOA(filename,Y(:,test),rho,test);
            % Calculate error in TOA radiance:
            auxerror(:,i) = 100*abs(LTOAI - LTOA)./LTOA; % REVISAR INDICE PARA KFOLD!!
        end
    end
end

t_interp = toc; errorstats.t_interp = t_interp/numel(in);

TF = myisoutlier(auxerror); auxerror(TF) = nan;  % Remove outliers from auxerror
[errormap,IND] = max(auxerror);

errormap(out) = myinterpn(X(:,in),errormap(in),X(:,out),'nearest'); % Just for doing nice 2D error maps

errors.errormap = errormap;
errors.X        = X;
errors.IND      = IND;

if ~isempty(P)
    errorstats.maxerror = prctile(errormap(in),P);
    errorstats.rrmse    = prctile(sqrt(sum(auxerror(:,in).^2)/size(auxerror(:,in),2)),P);
end

end

