function LUThdr=readLUThdr(filename)
%READLUTHDR Reads Look-Up table header file
%   LUTHDR = READLUTHDR(FILENAME) reads the header file FILENAME and
%   provides in the structure LUTHDR with following data:
%   - RTM name used to generate the LUT in LUTHDR.RTMname
%   - LUT dynamic key atmospheric input variable names in LUTHDR.varnames
%   - Names of output atmospheric transfer functions in LUTHDR.outnames
%   - LUT header table in LUTHDR.LUTheader
%   - Wavelengths of the output atmospheric transfer functions in LUTHDR.wvl
%   - Solar zenith angle of the output observations in LUTHDR.SZA.
%   The field LUTHEADER is a matrix of P x N dimensions where P is the 
%   number of key atmospheric input variables (same as NUMEL(VARNAMES)) and
%   N is the number of combinations.
%
%   See also: LUTGEN_INPUTS, WRITELUTHDR

%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.03 $  $Date: 2018/03/03 $

% Pre-allocate LUThdr fields:
LUThdr.RTMname   = []; % RTM name
LUThdr.varnames  = []; % Names of RTM input variables
LUThdr.outnames  = []; % Names of RTM output atmospheric transfer functions
LUThdr.LUTheader = []; % Values of input variables
LUThdr.wvl       = []; % Wavelength of output variables (in [nm])
LUThdr.SZA       = []; % Solar zenith angle (in [deg])
LUThdr.I0        = []; % Extraterrestrial solar irradiance (in [mW/m2/nm])

[~,~,ext] = fileparts(filename);

% File ID open:
if strcmp(ext,'.hdr') % OPEN .HDR FILE
    fid = fopen(filename);
    tline = fgetl(fid);

    % Scan the header file to detect the dynamic parameters:
    while ischar(tline) && ~strcmp(tline,'-9999.')
        if strfind(tline,'RTM name')
            ind = strfind(tline,':');
            LUThdr.RTMname = tline(ind+2:end);
        end
        if strfind(tline,'Solar zenith angle')
            LUThdr.SZA = str2num(tline(findstr(tline,':')+2:end));
        end
        if strfind(tline,'Dynamic')
            tline = fgetl(fid); n=1;
            while ischar(tline) && ~isempty(strfind(tline,'('))
                aux = textscan(tline,'%s','delimiter','');  aux = aux{1}; aux = aux{1};
                LUThdr.varnames{n} = aux(strfind(aux,'(')+1:strfind(aux,',')-1);
                if strfind(tline,'Solar zenith angle'), LUThdr.SZA = num2str(n); end
                tline = fgetl(fid); n=n+1;
            end
            break
        end    
        tline = fgetl(fid);
    end

    % Scan the header file from the headerlines as many lines as variables and
    % save output data in the LUTheader matrix:
    tline = fgetl(fid); 
    for i=1:numel(LUThdr.varnames)
        tline = fgetl(fid);
        LUThdr.LUTheader(i,:) = cell2mat(textscan(tline,'%.3f','delimiter','\t'));
        if ischar(LUThdr.SZA) && i==str2num(LUThdr.SZA)
            LUThdr.SZA = LUThdr.LUTheader(i,:);
        end
    end

    while ischar(tline) && ~strcmp(tline,'-9999.')
        if strfind(tline,'Output')
                tline = fgetl(fid); n=1;
                while ischar(tline) && ~isempty(strfind(tline,'('))
                    aux = textscan(tline,'%s','delimiter','');  aux = aux{1}; aux = aux{1};
                    LUThdr.outnames{n} = aux(strfind(aux,'(')+1:strfind(aux,')')-1);
                    tline = fgetl(fid); n=n+1;
                end
        break
        end
        tline = fgetl(fid);
    end

    % Scan the file and obtain the wavelengths:
    wvl = [];
    while ischar(tline) && isempty(strfind(tline,'-9999.'))
        tline = fgetl(fid);
        if strfind(tline,'Wavelengths')
            tline = fgetl(fid);
            wvl = textscan(tline,'%f','delimiter',' '); LUThdr.wvl = wvl{1};
            break
        end
    end

    % Scan the file and obtain the Solar Irradiance:
    I0 = [];
    while ischar(tline) && isempty(strfind(tline,'-9999.'))
        tline = fgetl(fid);
        if strfind(tline,'Solar irradiance (mW/m2/nm)')
            tline = fgetl(fid);
            I0 = textscan(tline,'%f','delimiter',' '); LUThdr.I0 = I0{1};
            break
        end
    end
else % OPEN .H5 FILE
    cellfind = @(string)(@(cell_contents)(strcmp(string,cell_contents)));

    LUThdr.RTMname   = h5readatt(filename,'/','RTMname');
    LUThdr.LUTheader = h5read(filename,'/LUTheader');
    LUThdr.inputmode = h5readatt(filename,'/','inputmode');
    try
        % Old LUT format (remove in future versions!!)
        LUThdr.varnames  = auxstr2cell(h5readatt(filename,'/LUTheader','ID'));
        desc = auxstr2cell(h5readatt(filename,'/LUTheader','description'));
        idx = [];
        if isstr(desc)
            if strfind(desc,'Solar zenith angle'), idx=1; end
        else
            for i=1:numel(desc)
                if strfind(desc{i},'Solar zenith angle'), idx=i; end
            end
        end
        % Read Solar Zenith Angle (SZA):
        if ~isempty(idx)
            LUThdr.SZA   = LUThdr.LUTheader(idx,:);
        end
    catch
        % New LUT format
        LUThdr.varnames = auxstr2cell(h5readatt(filename,'/LUTheader','varnames'));
        dynamic = h5info(filename,'/dynamic'); dynamic = dynamic.Datasets;
        for i=1:numel(dynamic)
            LUThdr.dynamic.(dynamic(i).Name).value = h5read(filename,['/dynamic/',dynamic(i).Name]);
            for j=1:numel(dynamic(i).Attributes)
                LUThdr.dynamic.(dynamic(i).Name).((dynamic(i).Attributes(j).Name)) = dynamic(i).Attributes(j).Value;
            end
            desc = h5readatt(filename,['/dynamic/',dynamic(i).Name],'description');
            if strfind(desc,'Solar zenith angle')
                idx = strcmp(LUThdr.varnames,dynamic(i).Name);
                LUThdr.SZA   = LUThdr.LUTheader(idx,:);
            end
        end
    end
    try LUThdr.outnames  = auxstr2cell(h5readatt(filename,'/','outnames')); end
    try LUThdr.wvl       = h5read(filename,'/wvl'); end
    try LUThdr.I0        = h5read(filename,'/I0'); end
    % Read static parameters:
    static = h5info(filename,'/static'); static = static.Datasets;
    for i=1:numel(static)
        LUThdr.static.(static(i).Name).value = h5read(filename,['/static/',static(i).Name]);
        for j=1:numel(static(i).Attributes)
            LUThdr.static.(static(i).Name).((static(i).Attributes(j).Name)) = static(i).Attributes(j).Value;
        end
        desc = h5readatt(filename,['/static/',static(i).Name],'description');
        if strfind(desc,'Solar zenith angle')
            LUThdr.SZA   = h5read(filename,['/static/',static(i).Name]);
        end
    end
end

%% Close reading header file (.hdr):
if strcmp(ext,'.hdr'), fclose(fid); end

end

function outcell = auxstr2cell(instr)
%   OUTPUT_CELL = STR2CELL(INPUT_STRING) converts the input string of N
%   comma-separated elements into an output cell of N elements
%
%   STR2CELL will return the INPUT_STRING when there are no commas.
%
%   Examples
%       instr = 'text1,text2';
%       strfind(instr)    returns a cell with two elements: 'text1'  'text2' 
%       instr = 'text1';
%       strfind(instr)    returns the same input string 'text1' 
%
%   See also: STRFIND.

idx = strfind(instr,',');
if ~isempty(idx)
    outcell{1} = instr(1:idx(1)-1);
    if length(idx)>1
        for i=1:length(idx)-1
            outcell{i+1} = instr(idx(i)+1:idx(i+1)-1);
        end
    end
    outcell{end+1} = instr(idx(end)+1:end);
else
    outcell = instr;
end
end
