function confdata = read_LUTconfig(ruta,conftype)
%READ_LUTCONFIG Read Look-Up Table (LUT) configuration file.
%   [] = READ_LUTCONFIG(RUTA,CONFTYPE) reads set of parameters of type
%   CONFTYPE in the LUT configuration file pointed in RUTA and store them
%   in the output structure CONFDATA.
%
%   CONFTYPE set of LUT configuration parameters are:
%
%     'GenConf'   - Generic configuration
%     'ParamConf' - Key input parameters configuration
%     'OpacConf'  - OPAC configuration (if option available by RTM and
%                   selected within ParamConf)
%     'SpeConf'   - Spectral configuration
%     'AdvConf'   - Advanced configuration
%
%   See also: UPDATE_LUTCONFIG

%   Copyright 2016 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.00 $  $Date: 2016/09/12 $

%% Opens XML file from "ruta":
xDoc = xmlread(ruta);
nodos=xDoc.getChildNodes;    
LUTconf = nodos.item(0); % Node 0 is LUTconf

switch conftype
    case 'GenConf'
        GenConf = LUTconf.getElementsByTagName('GenConf').item(0);
        confdata.filename  = char(GenConf.getElementsByTagName('filename').item(0).getTextContent);
        confdata.parallel  = char(GenConf.getElementsByTagName('parallel').item(0).getTextContent);
        confdata.RTMname   = char(GenConf.getElementsByTagName('RTMname').item(0).getTextContent);
        confdata.op_mode   = char(GenConf.getElementsByTagName('op_mode').item(0).getTextContent);
        confdata.inputmode = char(GenConf.getElementsByTagName('inputmode').item(0).getTextContent);
        confdata.stopmethod = char(GenConf.getElementsByTagName('stopmethod').item(0).getTextContent);
        confdata.userlevel = char(GenConf.getElementsByTagName('userlevel').item(0).getTextContent);
        
        attributesInputmode = GenConf.getElementsByTagName('stopmethod').item(0).getAttributes;
        for count = 1:attributesInputmode .getLength
            attrib                      = attributesInputmode .item(count-1);
            confdata.(char(attrib.getName))  = char(attrib.getValue);
        end
        
    case 'ParamConf'
        ParamConf   = LUTconf.getElementsByTagName('ParamConf').item(0);
        nodos       = ParamConf.getChildNodes;
        length      = (nodos.getLength-1)/2;
        Groupnames  = cell(1,length);	auxidx = 1;	confdata = [];
        
        for i=1:2:nodos.getLength-1
            % 2) Read Parameters Group names:
            Group   = nodos.item(i);
            theAttributes   = Group.getAttributes;
            for count = 1:theAttributes.getLength
                attrib                      = theAttributes.item(count-1);
                tmpgoup.(char(attrib.getName))  = char(attrib.getValue);
            end
            Groupnames{auxidx}  = tmpgoup.name;
            auxidx   = auxidx+1;
            
            % 3) Read attributes of each parameter within Group:
            parameters  = Group.getChildNodes;
            auxidx2     = 1;
            asd         = Group.getElementsByTagName('parameter');
            
            if asd.getLength>0
                for j = 0:asd.getLength-1
                    param           = asd.item(j);
                    theAttributes   = param.getAttributes;
                    confdata        = leernodo(param,confdata,char(parameters.getNodeName));
                    auxidx2         = auxidx2+1;
                end
            else
                confdata.(char(parameters.getNodeName)) = [];
            end
        end
        
    case 'OpacConf'
        OpacConf	= LUTconf.getElementsByTagName('OpacConf').item(0);
        nodos       = OpacConf.getChildNodes;
        length      = (nodos.getLength-1)/2;
        Groupnames  = cell(1,length);	auxidx = 1;	confdata = [];
        
        parameters  = OpacConf.getChildNodes;
        auxidx2     = 1;
        asd         = OpacConf.getElementsByTagName('parameter');
        
        % 2) Read attributes of each parameter within Group:
        parameters  = OpacConf.getChildNodes;
        auxidx2     = 1;
        asd         = OpacConf.getElementsByTagName('parameter');
        
        if asd.getLength>0
            for j = 0:asd.getLength-1
                param           = asd.item(j);
                theAttributes   = param.getAttributes;
                confdata        = leernodo(param,confdata,[]);
                auxidx2         = auxidx2+1;
            end
        else
            confdata.(char(parameters.getNodeName)) = [];
        end
        
    case 'SpeConf'
        SpeConf     = LUTconf.getElementsByTagName('SpeConf').item(0);
        nodos       = SpeConf.getChildNodes;
        length      = (nodos.getLength-1)/2;
        Groupnames  = cell(1,length);	auxidx = 1;	confdata = [];
        
        parameters  = SpeConf.getChildNodes;
        auxidx2     = 1;
        asd         = SpeConf.getElementsByTagName('parameter');
        
        % 2) Read attributes of each parameter within Group:
        parameters  = SpeConf.getChildNodes;
        auxidx2     = 1;
        asd         = SpeConf.getElementsByTagName('parameter');
        
        if asd.getLength>0
            for j = 0:asd.getLength-1
                param           = asd.item(j);
                theAttributes   = param.getAttributes;
                confdata        = leernodo(param,confdata,[]);
                auxidx2         = auxidx2+1;
            end
        else
            confdata.(char(parameters.getNodeName)) = [];
        end
        
    case 'AdvConf'
        AdvConf = LUTconf.getElementsByTagName('AdvConf').item(0);
        nodos       = AdvConf.getChildNodes;
        length      = (nodos.getLength-1)/2;
        Groupnames  = cell(1,length);	auxidx = 1;	confdata = [];
        
        parameters  = AdvConf.getChildNodes;
        auxidx2     = 1;
        asd         = AdvConf.getElementsByTagName('parameter');
        
        % 2) Read attributes of each parameter within Group:
        parameters  = AdvConf.getChildNodes;
        auxidx2     = 1;
        asd         = AdvConf.getElementsByTagName('parameter');
        
        if asd.getLength>0
            for j = 0:asd.getLength-1
                param           = asd.item(j);
                theAttributes   = param.getAttributes;
                confdata        = leernodo(param,confdata,[]);
                auxidx2         = auxidx2+1;
            end
        else
            confdata.(char(parameters.getNodeName)) = [];
        end
        
end


function [datos]=leernodo(nodo,datos,str)
if nodo.hasAttributes
    theAttributes = nodo.getAttributes;
    for count = 1:theAttributes.getLength
        attrib = theAttributes.item(count-1);
        tmp.(char(attrib.getName))= char(attrib.getValue);
    end
    
    name=tmp.name;
    if ~isempty(str)
        datos.(str).(name)=tmp;
    else
        datos.(name)=tmp;
    end
% else
%     if nodo.hasChildNodes
%         name=char(nodo.getNodeName);
%         hijos=nodo.getChildNodes;
%         j=1;
%         datos1=[];
%         for count=1:hijos.getLength
%             datos1=leernodo(hijos.item(count-1),datos1);            
%         end
%         if ~isempty(datos1)
%             datos.(name)=datos1;
%         end
%     end
end