function update_LUTconfig(ruta,conftype,confdata)
%UPDATE_LUTCONFIG Creates or modifies a Look-Up Table (LUT) configuration
%file.
%   [] = UPDATE_LUTCONFIG(RUTA,CONFTYPE,CONFDATA) creates or modifies the
%   LUT configuration file pointed in RUTA for the set of parameters of
%   type indicated by CONFTYPE and input in CONFDATA structure.
%
%   CONFTYPE set of LUT configuration parameters are:
%
%     'GenConf'   - Generic configuration
%     'ParamConf' - Key input parameters configuration
%     'OpacConf'  - OPAC configuration (if option available by RTM and
%                   selected within ParamConf)
%     'SpeConf'   - Spectral configuration
%     'AdvConf'   - Advanced configuration
%
%   CONFDATA is a structure whose fields and stored elements depend on the
%   CONFTYPE. For further information read AtmLutGen User Manual.
%
%   See also: READ_LUTCONFIG

%   Copyright 2016 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.00 $  $Date: 2016/09/15 $

%% Opens or Creates XML file from "ruta":
if exist(ruta,'file')~= 2               % File doesn't exist ==> create new
    xDoc = com.mathworks.xml.XMLUtils.createDocument('LUTconf');
else                                    % File exist ==> read file
    xDoc = xmlread(ruta);
end

nodos   = xDoc.getChildNodes;
LUTconf = xDoc.item(0);
LUTconf.setAttribute('date',date);

%% Creates or updates LUT configuration (conftype) with confdata:
switch conftype
    case 'GenConf'
        fn = fieldnames(confdata);
        GenConf = LUTconf.getElementsByTagName('GenConf').item(0);
        if isempty(GenConf)
            product = xDoc.createElement('GenConf');
            LUTconf.appendChild(product);
            GenConf = LUTconf.getElementsByTagName('GenConf').item(0);
            for i=1:length(fn)
                if ~strcmp(fn{i},'errorthrs') && ~strcmp(fn{i},'refldb')
                    product = xDoc.createElement(fn{i});
                    if strcmp(fn{i},'stopmethod')
                        if strcmp(confdata.(fn{i}),'Input max. # of samples')
                            product.setAttribute('numberpoints','');
                        else
                            product.setAttribute('errorthrs','');
                            product.setAttribute('refldb','');
                        end
                    end
                    GenConf.appendChild(product);
                end
            end
        end
        
        for i=1:length(fn)
            if ~strcmp(fn{i},'errorthrs') && ~strcmp(fn{i},'refldb')
                GenConf.getElementsByTagName(fn(i)).item(0).setTextContent(num2str(confdata.(fn{i})))
                if strcmp(fn{i},'stopmethod') && strcmp(confdata.(fn{i}),'Input max. # of samples')
                    try
                        GenConf.getElementsByTagName(fn(i)).item(0).removeAttribute('errorthrs');
                    end
                    GenConf.getElementsByTagName(fn(i)).item(0).setAttribute('numberpoints','');
                elseif strcmp(fn{i},'stopmethod') % Automatic stop condition by comparison against error threshold
                    try
                        GenConf.getElementsByTagName(fn(i)).item(0).removeAttribute('numberpoints');
                    end
                    GenConf.getElementsByTagName(fn(i)).item(0).setAttribute('errorthrs',confdata.('errorthrs'));
                    GenConf.getElementsByTagName(fn(i)).item(0).setAttribute('refldb',confdata.('refldb'));
                end
            end
        end
        
    case 'ParamConf'
        ParamConf = LUTconf.getElementsByTagName('ParamConf').item(0);
        aux1 = {'AtmParam','AerParam','GeoParam'};
        aux2 = {'Atmospheric','Aerosols','Geometric'};
        
        if ~isempty(ParamConf) % Remove ParamConf to update with new values
            LUTconf.removeChild(LUTconf.getElementsByTagName('ParamConf').item(0));
        end
        
        % Add Groups to update with new values
        product = xDoc.createElement('ParamConf');
        LUTconf.appendChild(product);
        ParamConf = LUTconf.getElementsByTagName('ParamConf').item(0);
        for i=1:length(aux1)
            product = xDoc.createElement(aux1{i});
            product.setAttribute('name',aux2{i});
            ParamConf.appendChild(product);
        end
        
        if ~isempty(confdata)
            fn = fieldnames(confdata);
            inputNpoints = true;
            for i=1:length(fn)                                             % For every group...
                if ~strcmp(fn{i},'Npoints')                                %...as far as it's not "Npoints"...
                    Group = LUTconf.getElementsByTagName(fn{i}).item(0);
                    data = confdata.(fn{i}); fn2 = fieldnames(data);
                    for j=1:length(fn2)                                    % ...then for each parameter...
                        product = xDoc.createElement('parameter');
                        fn3 = fieldnames(data.(fn2{j}));
                        for k=1:length(fn3)                                % ...we set attributes...
                            product.setAttribute(fn3{k},data.(fn2{j}).(fn3{k}));
                        end
                        Group.appendChild(product);                        % ...and append new/updated parameter
                    end
                else % In case "Npoints" modify attribute to stopmethod in GenConf
                    GenConf = LUTconf.getElementsByTagName('GenConf').item(0);
                    GenConf.getElementsByTagName('stopmethod').item(0).setAttribute('numberpoints',num2str(confdata.Npoints));
                    inputNpoints = false;
                end
            end
            if inputNpoints
                GenConf = LUTconf.getElementsByTagName('GenConf').item(0);
                GenConf.getElementsByTagName('stopmethod').item(0).removeAttribute('numberpoints');
            end
        end
        
    case 'OpacConf'
        OpacConf = LUTconf.getElementsByTagName('OpacConf').item(0);
        
        if ~isempty(OpacConf) % Remove OpacConf to update with new values
            LUTconf.removeChild(LUTconf.getElementsByTagName('OpacConf').item(0));
        end
        
        % Add Groups to update with new values
        product = xDoc.createElement('OpacConf');
%         if ~isempty(confdata)
%             product.setAttribute('AERTYPE','');
%         else
%             product.setAttribute('AERTYPE','Spectral');
%         end
        LUTconf.appendChild(product);
        OpacConf = LUTconf.getElementsByTagName('OpacConf').item(0);
        
        if ~isempty(confdata)
            fn = fieldnames(confdata);
            for i=1:numel(fn)                                              % For each parameter...
                product = xDoc.createElement('parameter');
                data = confdata.(fn{i}); fn2 = fieldnames(data);
                for j=1:length(fn2)                                        % ...we set attributes...
                    product.setAttribute(fn2{j},data.(fn2{j}));
                end
                OpacConf.appendChild(product);                             % ...and append new/updated parameter
            end
        end

    case 'SpeConf'
        SpeConf = LUTconf.getElementsByTagName('SpeConf').item(0);
        
        if ~isempty(SpeConf) % Remove SpeConf to update with new values
            LUTconf.removeChild(LUTconf.getElementsByTagName('SpeConf').item(0));
        end
        
        % Add Groups to update with new values
        product = xDoc.createElement('SpeConf');
        product.setAttribute('name','Spectral');
        LUTconf.appendChild(product);
        SpeConf = LUTconf.getElementsByTagName('SpeConf').item(0);

        fn = fieldnames(confdata);
        for i=1:numel(fn)
            product = xDoc.createElement('parameter');
            product.setAttribute('name',fn{i});
            product.setAttribute('datatype',confdata.(fn{i}).datatype);
            product.setAttribute('description',confdata.(fn{i}).description);
            product.setAttribute('unit',confdata.(fn{i}).unit);
            product.setAttribute('values',confdata.(fn{i}).values);
            SpeConf.appendChild(product);
        end
        
    case 'AdvConf'
        AdvConf = LUTconf.getElementsByTagName('AdvConf').item(0);
        
        if ~isempty(AdvConf) % Remove AdvConf to update with new values
            LUTconf.removeChild(LUTconf.getElementsByTagName('AdvConf').item(0));
        end
        
        % Add Groups to update with new values
        product = xDoc.createElement('AdvConf');
        product.setAttribute('name','Advanced');
        LUTconf.appendChild(product);
        AdvConf = LUTconf.getElementsByTagName('AdvConf').item(0);

        if ~isempty(confdata)
            fn = fieldnames(confdata);
            for i=1:numel(fn)
                product = xDoc.createElement('parameter');
                product.setAttribute('name',fn{i});
                product.setAttribute('datatype',confdata.(fn{i}).datatype);
                product.setAttribute('description',confdata.(fn{i}).description);
                product.setAttribute('values',confdata.(fn{i}).values);
                AdvConf.appendChild(product);
            end
        end
end


%% Update/write XML file:
XML_string = xmlwrite(xDoc); %XML as string
XML_string = regexprep(XML_string,'\n\s*\n','\n'); %removes tabs and extra lines

%Write to file
fid = fopen(ruta,'w');
fprintf(fid,'%s\n',XML_string);
fclose(fid); 
