function YI = convolve_ISRF(wvl,Y,C,FWHM,model,P,ksize)
% CONVOLVE_ISRF Convolution by Instrument Spectral Response Function (ISRF) 
%   kernel (i.e. spectral resampling).
%   YI = CONVOLVE_ISRF(WVL,Y,C,FWHM) resamples the high spectral resolution
%   data Y, defined at the high resolution spectral grid WVL, by 
%   convolution with a parametric ISRF to produce lower spectral resolution 
%   data YI at the spectral grid C. This convolution is applied at each
%   spectral channel defined by the ISRF full-width at half maximum (FWHM).
%   Y is of dimension k-by-n, representing n spectral data at k high 
%   resolution spectral channels. WVL is therefore of dimension k-by-1. 
%   C should be of dimension l-by-1 where l represents the number of output
%   spectral channels, thus the output YI will be of dimension l-by-n. FWHM
%   can be constant or of dimension l-by-1.
%   
%   YI = CONVOLVE_ISRF(WVL,Y,CWVL,FWHM,MODEL,P) specifies alternate 
%   parametric ISRF models (normalized to the area) among the following
%   (notice that additional parameter P might be required):
%       'trap'      - trapezoid with a slope parameter P (constant or 
%                     dimension l-by-1). Default: P=0 (i.e. step function).
%       'dsigmf'    - diference between two sigmoids, symmetric with
%                     respect to barycenter, with a slope parameter P 
%                     (constant or dimension l-by-1). Default: P=5.
%       'gaussmf'   - gaussian (default)
%       'erf'       - difference between two error functions with a
%                     shape/kurtosis parameter P(:,1) (default 0.7) and 
%                     asymmetry parameter P(:,2) (default 0, i.e. 
%                     symmetric). P should be of size l-by-2 (for
%                     wavelength dependency) or 1-by-2 (constant for all 
%                     wavelengths). 
%       'sGaussian' - generic Gaussian function with a shape parameter
%                     P(:,1) (default 4) and two asymmetry parameters 
%                     (width-asymmetry P(:,2), default 0, i.e. symmetric) 
%                     and shape asymmetry P(:,3), default 0, i.e. 
%                     symmetric):
%                     f = exp(-abs((lambda - C)/(w-P(2))).^(P(1)-P(3)))
%                     where w is related with the FWHM.
%       'sCauchy'   - generic Cauchy function, with a shape parameter and 
%                     two asymmetry parameters (see 'sGauss'):
%                     f = 1./(1+abs((lambda - C)./(w-P(2))).^(P(1)-P(3)))
%                     where w is related with the FWHM.
%       'sHS'       - generic Hyperbolic Secant function, with a shape 
%                     parameter and two asymmetry parameters (see 'sGauss'):
%                     f = sech((pi/2)*abs(lambda - C)./(w-P(2))).^(P(1)-P(3)))
%                     where w is related with the FWHM.
%   If MODEL is [] or not specified, then the default 'gaussian' model  will
%   be used. When the parameter(s) P is [] or not specified, the default
%   values will be applied.
%
%   YI = CONVOLVE_ISRF(...,KSIZE) specifies the size of the ISRF kernel
%   function as a percentage (in %) of the FWHM. KSIZE can be constant or 
%   dimension l-by-1. If is [] or not specified, KSIZE will be 200% by 
%   default.
%
%   Example:
%      wvl = [700:0.01:780]';
%      Y = 1 -  sin(pi*(wvl-640)/180) + 0.25*rand(size(wvl));
%      C = 701:0.1:779;  FWHM = 0.3;
%      YI_gauss = convolve_ISRF(wvl,Y,C,FWHM);
%      P(wvl<740) = 2; P(wvl>=740) = 10;
%      YI_dsigm = convolve_ISRF(wvl,Y,C,FWHM,'dsigmf',P,250);
%      
%      plot(wvl,Y), hold on, plot(C,YI_gauss,'r'), plot(C,YI_dsigm,'y')
%      legend('hires data','convolved with gaussian','convolved with dsigmf')
%   
%   See also TRAPMF, DSIGMF, GAUSSMF, ERF.
%
%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.02 $  $Date: 2018/07/09 $


% Definition of data dimensions:
n = size(Y,2); % n (number of spectra)
l = numel(C);  % l (output wavelengths)
szC = size(C);


% Memory allocation:
YI = zeros(l,n);

if ( nargin < 5 || isempty(model) ), model = 'gaussmf'; P=[]; end

if ( nargin < 6 || isempty(P) )
    % NOTE: maybe we could calculate automatically the parameter(s) P so 
    % that the ISRF fits as much as possible a gaussian
    switch model
        case 'trap'
            P = zeros(szC);       % Kurtosis (shape/slope) parameter
        case 'gaussmf'
            P = [];
        case 'dsigmf'
            P = 5*ones(szC);      % Kurtosis (shape/slope) parameter
        case 'erf'
            P(:,1) = 0.7*ones(szC); % Kurtosis (shape/slope) parameter
            P(:,2) = zeros(szC);    % Asymmetry parameter
        case {'sGaussian','sCauchy','sHS'}
            P(:,1) = 4*ones(szC); % Kurtosis (shape/slope) parameter
            P(:,2) = zeros(szC);  % Width-asymmetry parameter
            P(:,3) = zeros(szC);  % Shape-asymmetry parameter
    end
end

if nargin == 6
    szP = size(P);
    switch model
        case 'erf'
            if szP(2)<2
                error(['MATLAB:convolve_ISRF:',model,':parameters'],...
                      'Selected ISRF model requires 2 parameters')
            end
        case {'sGaussian','sCauchy','sHS'}
            if szP(2)<3
                error(['MATLAB:convolve_ISRF:',model,':parameters'],...
                      'Selected ISRF model requires 3 parameters')
            end
    end
    if szP(1)==1, P = repmat(P,[l,1]);
    elseif szP(1)~=l
        error(['MATLAB:convolve_ISRF:',model,':parameters'],...
              ['Please introduce a constant vector of parameters P (1x',...
              num2str(szP(2)),' or a vector of (',num2str(l),'x',...
              num2str(szP(2)),')'])
    end
end
if numel(FWHM)==1, FWHM = FWHM*ones(size(C)); end

if ( nargin < 7 || isempty(ksize) ), ksize = 200*ones(size(C)); end
if numel(ksize)==1, ksize = ksize*ones(size(C)); end


if strcmp(model,'file')
    load('isrf_data.mat')
end

% Definition of data kernel size, i.e. wavelength limits here to "cut" the ISRF:
Bmin = C - ksize.*FWHM/100;
Bmax = C + ksize.*FWHM/100;          

for i = 1:l
    % Locate wavelenghts of the hires spectra to perform the convolution
    IwlB   = wvl >= Bmin(i) & wvl <= Bmax(i);
    auxwvl = wvl(IwlB); auxY = Y(IwlB,:);
    % Calculate ISRF:
    if isempty(P)
        ISRF = isrfCalculation(auxwvl,C(i),FWHM(i),[],model); 
    else
        ISRF = isrfCalculation(auxwvl,C(i),FWHM(i),P(i,:),model);
    end
    % Convolve data with ISRF:
    YI(i,:) = convolution(auxwvl,ISRF,auxY);
end

end


%%%% ADITIONAL SUB-FUNCTIONS:
function isrf = isrfCalculation(wvl,C,FWHM,P,model)
    switch model
        case 'trap'
            % Trapezoidal ISRF. Same as:
            % trapmf(wvl,[C-(P+FWHM)/2 C-(FWHM-P)/2 C+(FWHM-P)/2 C+(FWHM+P)/2])
            % WLV is the high-resolution spectral grid where the ISRF is 
            % defined C is the barycenter of the spectral channel, FWHM is 
            % the full-width at half maximum, P specifies the distance in 
            % the spectral domain for a decrease of the ISRF between 1 and 
            % 0, thus is related with the slope of the ISRF:
            y   = 0.5 + (FWHM - 2 *abs(wvl - C))./(2*P);
            isrf = max(min(y,1),0);
        case 'dsigmf'
            A1 = C - FWHM/2;  A2 = C + FWHM/2;
            % Calculate ISRF:
            sigm1 = 1./(1 + exp(-P*(wvl-A1)));
            sigm2 = 1./(1 + exp(-P*(wvl-A2)));
            isrf = abs(sigm1 - sigm2);
%             isrf = dsigmf(auxwvl,[P A1 P A2]);
        case 'gaussmf'%--------------------------------------------------------
            sig = FWHM/2.3548;
            % Calculate ISRF:
            isrf = exp(-(wvl - C).^2/(2*sig^2));
%             isrf = gaussmf(auxwvl,[sig C]);
        case 'erf'%--------------------------------------------------------
            % Difference between two error functions (similar to dsigmf) ISRF. 
            % WLV is the high-resolution spectral grid where the ISRF is 
            % defined S1 and S2 specify the slopes of the two error functions
            % A1 and A2 specify the wavelength position at which the two 
            % error functions equal 0.5.
            slope = P(1); skew  = P(2);
            S1 = slope.*(1 - abs(skew).*(0<skew));
            S2 = slope.*(1 - abs(skew).*(0>skew));
            A1 = C + FWHM/2;    A2 = C - FWHM/2;
            isrf = 0.5*( erf(-S1*(wvl - A1)) - erf(-S2*(wvl - A2)) );
            % Determine modified FWHM:
            [~,aux]  = max(isrf);
            [~,aux1] = sort(abs(isrf(1:aux)   - max(isrf)/2));
            [~,aux2] = sort(abs(isrf(aux:end) - max(isrf)/2));
            l2 = interp1(isrf(aux1(1:2)),wvl(aux1(1:2)),max(isrf)/2,'linear','extrap');
            l1 = interp1(isrf(aux-1+aux2(1:3)),wvl(aux-1+aux2(1:3)),max(isrf)/2,'linear','extrap');
            aux = abs(l1-l2);
            % Re-center and reshape ISRF to match input barycenter C and FWHM:
            m    = trapz(wvl,isrf.*wvl)/trapz(wvl,isrf);
            isrf = interp1((wvl-m)*FWHM/aux,isrf,wvl-C,'linear','extrap');
        case 'sGaussian'%-----------------------------------------------
            k  = P(1); % Shape parameter
            Aw = P(2); % Asymmetric width
            Ak = P(3); % Asymmetric shape
            ln2 = log(2);
            % Calculation of the Width parameter for an input FWHM:
            w = (FWHM - Aw.*(ln2.^(1./(k+Ak))-ln2.^(1./(k-Ak)))) ./ (ln2.^(1./(k+Ak))+ln2.^(1./(k-Ak)));
            if Aw~=0 || Ak~=0
                isrf = ones(size(wvl));
                x1 = wvl<=C; x2 = wvl>C;
                isrf(x1) = exp(-abs((wvl(x1) - C)/(w-Aw)).^(k-Ak));
                isrf(x2) = exp(-abs((wvl(x2) - C)/(w+Aw)).^(k+Ak));
                % Re-center and reshape ISRF to match input barycenter C and FWHM:
                m    = trapz(wvl,isrf.*(wvl-C))/trapz(wvl,isrf);
                x1 = wvl<=C-m; x2 = wvl>C-m;
                isrf(x1) = exp(-abs((wvl(x1) - C+m)/(w-Aw)).^(k-Ak));
                isrf(x2) = exp(-abs((wvl(x2) - C+m)/(w+Aw)).^(k+Ak));
            else % Symmetric
            	isrf =  exp(-abs((wvl - C)/w).^k);
            end
        case 'sCauchy'%------------------------------------------------
            k  = P(1); % Shape parameter
            Aw = P(2); % Asymmetric width
            Ak = P(3); % Asymmetric shape
            % Calculation of the Width parameter for an input FWHM:
            w = 0.5*FWHM;
            if Aw~=0 || Ak~=0
                isrf = ones(size(wvl));
                x1 = wvl<=C; x2 = wvl>C;
                isrf(x1) = 1./(1+abs((wvl(x1) - C)./(w-Aw)).^(k-Ak));
                isrf(x2) = 1./(1+abs((wvl(x2) - C)./(w+Aw)).^(k+Ak));
                % Re-center and reshape ISRF to match input barycenter C(i) and FWHM(i):
                m    = trapz(wvl,isrf.*(wvl-C))/trapz(wvl,isrf);
                x1 = wvl<=C-m; x2 = wvl>C-m;
                isrf(x1) = 1./(1+abs((wvl(x1) - C+m)./(w-Aw)).^(k-Ak));
                isrf(x2) = 1./(1+abs((wvl(x2) - C+m)./(w+Aw)).^(k+Ak));
            else % Symmetric
                isrf = 1./(1+abs((wvl - C)./w).^k);
            end
        case 'sHS'%--------------------------------------
            k  = P(1); % Shape parameter
            Aw = P(2); % Asymmetric width
            Ak = P(3); % Asymmetric shape
            % Calculation of the Width parameter for an input FWHM:
            cte = 2*log(2+sqrt(3))/pi;
            w = (FWHM - Aw.*(cte.^(1./(k+Ak))-cte.^(1./(k-Ak)))) ./ (cte.^(1./(k+Ak))+cte.^(1./(k-Ak)));
            if Aw~=0 || Ak~=0
                isrf = ones(size(wvl));
                x1 = wvl<=C; x2 = wvl>C;
                isrf(x1) = sech((pi/2)*abs((wvl(x1) - C)./(w-Aw)).^(k-Ak));
                isrf(x2) = sech((pi/2)*abs((wvl(x2) - C)./(w+Aw)).^(k+Ak));
                % Re-center and reshape ISRF to match input barycenter C and FWHM:
                m    = trapz(wvl,isrf.*(wvl-C))/trapz(wvl,isrf);
                x1 = wvl<=C-m; x2 = wvl>C-m;
                isrf(x1) = sech((pi/2)*abs((wvl(x1) - C+m)./(w-Aw)).^(k-Ak));
                isrf(x2) = sech((pi/2)*abs((wvl(x2) - C+m)./(w+Aw)).^(k+Ak));
            else % Symmetric
                isrf = sech((pi/2)*abs((wvl - C)./w).^k);
            end  
        case 'file' %------------------------------------------------------
            % NOT YET IMPLEMENTED   
    end
end


function YI = convolution(wvl,ISRF,Y)
if any(ISRF~=0) 
        if length(ISRF)~=1            
            norm  = Sint(ISRF,wvl);    % Area below ISRF
%             tel   = Sint(bsxfun(@times,Y,ISRF),wvl); % Convolution with ISRFs
            tel   = Sint(Y.*ISRF,wvl); % Convolution with ISRFs
            YI     = tel/norm;         % Normalize ISRF by area
        else
        	mean(wvl) % Revisar!!!
        end
else
    YI = 0;
end
end

function int = Sint(y,x)
% Simpson (trapezoid) integration
% x must be a row vector. y must be a matrix of one or more column vectors 
% with the same length of x.  x must be a monotonically increasing series
% Modified by Jorge Vicent (jorge.vicent@uv.es) based on original from% WV Jan. 2013, for SCOPE 1.40
    
    nx   = numel(x);
    if size(x,1) ~= 1
        x = x';
    end
    if size(y,1) ~= size(x,2)
        y = y';
    end
    step = x(2:nx) - x(1:nx-1);
    mean = 0.5 * (y(1:nx-1,:) + y(2:nx,:));
    int  = step * mean; 
end

