function YI = myinterpn(varargin)%X,Y,XI,method)
% MYINTERPN N-D interpolation (table lookup).
%   YI = MYINTERPN(X,Y,XI) interpolates a hyper-surface Look-Up Table (LUT)
%   of the form Y = F(X), defined in the n-D input space given in the X 
%   matrix, at the points specified by XI to produce YI.
%
%   X is of dimension n-by-m, representing m points in n-D space. Y is of
%   dimension k-by-m, representing m values of the hyper-surface F(X) 
%   defined in the k-D output space. XI is a vector of size n-by-p, 
%   representing p points in the n-D input space at which the function
%   F(XI) is to be evaluated. YI is a vector of size k-by-p approximating 
%   the values F(XI). The hyper-surface always goes through the data points
%   (X,Y).
%
%   Alternatively, Y can be a string specifying a HDF5 file containing the
%   m-by-k Y data in a dataset named "/LUTdata". It is recommended to use
%   this alternate version if the LUT sizes is larger than the computer
%   RAM. However, this comes at expenses larger computation time due to 
%   access to disk when reading individual elements of the LUT file.
%
%   The interpolation works both on gridded and scattered data. The only
%   requirement when interpolating gridded data is that the input data 
%   (X,Y) is shorted according to Matlab's NDGRID function. E.g.:
%   
%   [x1,x2] = NDGRID(-1:0.02:1,0:0.5:10);
%   X = [x1(:), x2(:)]';
%
%   YI = MYINTERPN(X,Y,XI,METHOD) specifies alternate interpolation methods
%   among the following:
%       'nearest'         - nearest neighbor interpolation. For scattered
%                           data, this method is based on finding the 
%                           minimum relative distance between the grid 
%                           points (X) and the query points (XI)
%       'linear'          - linear interpolation (default)
%       'spline'          - spline interpolation (only for gridded data)
%       'inverse'         - inverse distance weighting (or  Shepard's 
%                           Method) with a default power parameter P=2. 
%                           Change the power parameter by input its value 
%                           MYINTERPN(...,'inverse',P)
%       'inverse2'        - Modified Shepard's Method with a default power 
%                           parameter P=1.5 and default R-sphere of Nw=10
%                           neightbors. Change the power parameter and size
%                           of R-sphere by input its values 
%                           MYINTERPN(...,'inverse2',P,Nw). If P and/or Nw
%                           are empty [], the default values is userd
%       'matlabnearest'   - Matlab's nearest neighbor interpolation using 
%                           griddedInterpolant (for gridded data) and 
%                           GRIDDATAN (for scattered data). For scattered
%                           data, interpolation is based on triangulation.
%                           If interpolation should be called multiple
%                           times, it is advised to pre-calculate the
%                           triangulation (TRI) through DELAUNAYN and give
%                           it as input MYINTERPN(...,TRI)
%       'matlablinear'    - Matlab's linear interpolation using 
%                           griddedInterpolant (for gridded data) and 
%                           GRIDDATAN (for scattered data). For scattered
%                           data, interpolation is based on triangulation.
%                           If interpolation should be called multiple
%                           times, it is advised to pre-calculate the
%                           triangulation (TRI) through DELAUNAYN and give
%                           it as input MYINTERPN(...,TRI)
%       'matlabspline'    - Matlab's spline interpolation using
%                           griddedInterpolant. Valid only for 
%                           gridded data
%   If METHOD is [] or not specified, then the default 'linear' method will
%   be used.
%
%   Example:
%      X = 2*rand(5000,3)-1; Y = [sum(X.^2,2),10*sum(X.^2,2)];
%      d = -0.8:0.05:0.8; [x0,y0,z0] = meshgrid(d,d,d);
%      XI = [x0(:) y0(:) z0(:)];
%      YI = myinterpn(X,Y,XI);
%   
%   See also DELAUNAYN, GRIDDATAN, GRIDDEDINTERPOLANT, H5DISP, H5READ, 
%   INTERPN, NDGRID
%
%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.03 $  $Date: 2017/08/01 $

% IDEAS FOR FUTURE UPDATES:
% - Hardy's method. See http://journals.ametsoc.org/doi/full/10.1175/2008JAS2626.1
% - Natural Neighbor. See S. Park et al (2006), Discrete Sibson
% Interpolation, IEEE Transactions on Visualization and Computer Graphics,
% Vol. 12, No. 2


X  = varargin{1};   Y  = varargin{2};   XI = varargin{3};

% Check interpolation method:
if nargin < 4
    method = 'linear';
else
    method = varargin{4};
    if ~ischar(method)
        error('MATLAB:myinterpn:method','Interpolation method should be a string')
    end
end


% Definition of data dimensions
n = size(X,1);  % n-D input space
if ischar(Y)
    aux = h5info(Y,'/LUTdata'); aux = aux.Dataspace.Size;
    m = aux(2); % input m points
    k = aux(1); % k-D output space
else
    [k,m] = size(Y);
end
p = size(XI,2); % output p points


% Check parameters for inverse weighting and Modified Shepard's interpolation:
if strcmp(method,'inverse')
    if nargin == 4, P = 2; else P = varargin{5}; end
    if isempty(P), P = 2; end
elseif strcmp(method,'inverse2')
    if nargin == 4, P = 1.5; Nw = 10;%2^n;
    elseif nargin ==5, P = varargin{5}; Nw = 10;
    else P = varargin{5}; Nw = varargin{6};
    end
    if isempty(P), P = 1.5; end
    if isempty(Nw), Nw = 10; end
    if Nw>m, Nw=m; end
end


% Output memory allocation:
YI = NaN*zeros(k,p);

% Determine unique values defining the input parameter space:
auxX = cell(1,n); auxI = auxX;
for j=n:-1:1, [auxX{j},auxI{j}] = unique(X(j,:)); end
% Determine whether input space is gridded:
auxdim = zeros(1,n);
for i=1:n, auxdim(i) = numel(auxX{i}); end, aux = prod(auxdim);
if aux==m, gridded=true; else gridded=false; end

% Check if selected method is compatible with input (X,Y) data:
if ~isempty(strfind(method,'spline')) && ~gridded
    error('MATLAB:myinterpn:method', [method,' not available on scattered data'])
end


if isempty(strfind(method,'matlab')) 
    %% My interpolation method
    switch method
        case 'nearest' %---------------------------------------------------
            %%  Find index (indx) of grid-points in X around interpolation point XI:
            if gridded % Gridded data
                indx = 1;   cprod = [1 cumprod(auxdim(1:end-1))];
                if p~=1
                    for j=1:n
                       aux = abs(bsxfun(@minus,repmat(auxX{j},[p,1]),XI(j,:)')); 
                       [~,asd] = min(aux,[],2);
                       indx = indx + (asd-1)*cprod(j);
                    end
                else
                    for j=1:n
                        aux = abs(auxX{j}-XI(j)); [~,asd] = min(aux);
                        indx = indx + (asd-1)*cprod(j);
                    end
                end
            else % Scattered data
                % Based on finding the minimum relative distance between 
                % the grid points (X) and the query points (XI)
                % Definition of "distance": 
%                 aux = abs(bsxfun(@minus,repmat(X,[1,1,p]),reshape(XI,[n,1,p]))); % May have memory problems when p>>
                aux = abs(X-reshape(XI,[n,1,p])); % May have memory problems when p>> 
                aux = bsxfun(@rdivide,aux,max(X,[],2)-min(X,[],2)); % Normalize by min/max distance
                aux = sum(aux.^2);
                % Find minimum distance
                [~,indx] = min(aux,[],2); indx = squeeze(indx);
            end
            
            %--- Interpolation of LUT output variables:
            if ischar(Y)
                for i=1:p % For each p point
                    YI(:,i) = h5read(Y,'/LUTdata',[1,indx(i)],[k,1]);
                end
            else
                YI = Y(:,indx);	
            end
        case 'linear' %----------------------------------------------------
            %%  Find index (indx) of grid-points in X around interpolation point XI:
            if gridded % Gridded data
%             prevasd = zeros(p,1);  cprod = [1 cumprod(auxdim(1:end-1))]; 
%             for j=1:n
%             	aux = abs(bsxfun(@minus,repmat(auxX{j},[p,1]),XI(j,:)')); 
%                 [aux,asd] = sort(aux,2); 
%                 if any(aux==0), asd = asd(1); x_c{j} = aux(1);
%                 else asd = asd(:,1:2); x_c{j} = aux(1:2); end
%                 asd = (asd-1)*cprod(j);
%                 for i=1:2, indx = [prevasd bsxfun(@plus,prevasd,asd(:,i))]; end
%                 prevasd = asd;
%                 indx = indx + 1;
%             end
                for i=1:p
                    indx = 1;
                    x_c = cell(1,n);
                    for j=1:n % For each dimension of the input space
                        % Find the 2 values closer to the interpolation point (IX):
                        IX = [find(auxX{j}<= XI(j,i),1,'last'), find(auxX{j}>= XI(j,i),1,'first')];
                        % Calculate the "distance" (x_c) to the point:
                        if numel(IX)==1, IX(2) = IX(1); end
                        if IX(1)~=IX(2)
                            x_c{j} = [auxX{j}(IX(2))-XI(j,i) ; XI(j,i)-auxX{j}(IX(1))];
                            if auxI{j}(IX(2))-auxI{j}(IX(1))<0
                                x_c{j} = [x_c{j}(2),x_c{j}(1)];
                            end
                        else x_c{j} = 1;
                        end
                        % Find those elements in X that are equal to the 2
                        % values closer to interpolatin point
                        aux1 = X(j,:)==auxX{j}(IX(1)); aux2 = X(j,:)==auxX{j}(IX(2));
                        indx = indx & (aux1 | aux2);
                    end
                    indx = find(indx);

                    %--- Interpolation of LUT output variables:
                    % Extract grid points of LUT in the Y matrix or stored in HDF5 file:
                    Y_subgrid = zeros(k,numel(indx));
                    for j=1:numel(indx)
                        if ischar(Y), Y_subgrid(:,j) = h5read(Y,'/LUTdata',[1,indx(j)],[k,1]);
                        else          Y_subgrid(:,j) = Y(:,indx(j));	end
                    end

                    % Interpolation:
                    aux = cell(n,1); [aux{:}] = ndgrid(x_c{:});
                    V = prod(cat(numel(aux)+1,aux{:}),numel(aux)+1);
                    V = reshape(V,1,numel(V)); V_tot = sum(V(:));
                    %             V = prod(V,1); V_tot = sum(V);
                    YI(:,i) = sum(bsxfun(@times,Y_subgrid,V/V_tot),2);
                end
            else % Scattered data
                error('MATLAB:myinterpn:linear:scattered',[method,...
                    ' interpolation on scattered data not yet implemented'])
            end
            
        case 'spline' %----------------------------------------------------
            %% Find index (indx) of grid-points in X around interpolation point XI:
                error('MATLAB:myinterpn:spline',[method,' interpolation not yet implemented'])
        
        case 'inverse'
            %%  Find index (indx) of grid-points in X around interpolation point XI:
            % Definition of "distance":
            aux = abs(bsxfun(@minus,repmat(X,[1,1,p]),reshape(XI,[n,1,p]))); % This line has memory problems when p>>
            % Normalize distance by min/max distance so that all dimensions
            % axis are equal:
            aux = bsxfun(@rdivide,aux,max(X,[],2)-min(X,[],2));
            % Calculate normalized inverse distance weights:
            aux = 1./(sum(aux.^2).^P); aux = bsxfun(@times,aux,1./sum(aux,2));            
            
            %--- Interpolation of LUT output variables:
            if ischar(Y)
                error('MATLAB:myinterpn:inverse',...
                      ['Inverse weighting interpolation using LUT data ',...
                      'file not yet implemented'])
            else
                YI = squeeze(sum(bsxfun(@times,Y,aux),2));
            end
            
        case 'inverse2'
            %%  Find index (indx) of grid-points in X around interpolation point XI:
            % Definition of "distance":
%             aux = abs(bsxfun(@minus,repmat(X,[1,1,p]),reshape(XI,[n,1,p]))); % This line has memory problems when p>>
            aux = abs(X-reshape(XI,[n,1,p])); % This line has memory problems when p>>
            % Normalize distance by min/max distance so that all dimensions
            % axis are equal:
            aux = bsxfun(@rdivide,aux,max(X,[],2)-min(X,[],2));
            % Calculate distance:
            aux = sqrt(sum(aux.^2));
            % Select R-sphere of Nw neighbors:
%             [aux,indx] = sort(aux,2); indx = indx(1,1:Nw,:); aux = aux(1,1:Nw,:);
           [aux,indx] =  mink(aux,Nw,2); % Valid after Matlab R2017b
            % Calculate normalized weights:
            R = aux(1,end,:);
%             aux = ((bsxfun(@minus,R,aux))./bsxfun(@times,R,aux)).^P;
            aux = ((R-aux)./(R.*aux)).^P;
            aux = bsxfun(@times,aux,1./sum(aux,2));
            
            %--- Interpolation of LUT output variables:
            if ischar(Y)
                error('MATLAB:myinterpn:inverse',...
                      ['Inverse weighting interpolation using LUT data ',...
                      'file not yet implemented'])
            else
                for i=1:p
%                     YI(:,i) = sum(bsxfun(@times,Y(:,indx(1,:,i)),aux(1,:,i)),2);
                    YI(:,i) = sum(Y(:,indx(1,:,i)).*aux(1,:,i),2);
                end
            end
            
    end
    
else
    %% Method is Matlab interpolation:
    if gridded 
        %% Interpolation of gridded data based on Matlab's griddedInterpolant
        % Create Matlab interpolant for gridded data
        if k~=1
            Y = reshape(Y,[k,auxdim]);
            F = griddedInterpolant([1:k,auxX],Y,method(7:end));
        else
            Y = reshape(Y,auxdim);
            F = griddedInterpolant(auxX,Y,method(7:end));
        end
        
        % Interpolate using the calculated griddedInterpolant:
        if k~=1
            if k>=p
               for i=1:p
                   % Loop for each output point <--- How to make an 
                   % efficient interpolation for multi-output?
                   aux = [(1:k)',repmat(XI(:,i)',k,1)];
                   YI(:,i) = F(aux);
               end 
            else
                aux = ones(size(XI,2),1); XI = XI';
                for i=1:k
                    % Loop for each output dimension <--- How to make an 
                    % efficient interpolation for multi-output?
                    YI(i,:) = F([i*aux,XI]);
                end
            end
        else
            YI = F(XI');
        end
    else
        %% Interpolation of scattered data based on Matlab's GRIDDATAN
        % Due to different units of input dimensions, normalize them to 
        % min=0 and max=1
        minX = min(X,[],2); maxX = max(X,[],2);
        M  = 1./(maxX-minX); N = -minX.*M;
        X  = bsxfun(@plus,bsxfun(@times,X,M),N);
        XI = bsxfun(@plus,bsxfun(@times,XI,M),N);
                
        % Triangulation and interpolation:
        if nargin==4, tri  = delaunayn(X');
        else          tri  = varargin{5};
        end
        switch method
            case 'matlabnearest' 
                %%  Find index (indx) of grid-points in X around interpolation point XI:
                indx = dsearchn(X',tri,XI');

                %--- Interpolation of LUT output variables:
                if ischar(Y)
                    for i=1:p % For each p point
                        YI(:,i) = h5read(Y,'/LUTdata',[1,indx(i)],[k,1]);
                    end
                else
                    YI = Y(:,indx);	
                end
                
            case 'matlablinear' % Based on Matlab's GRIDDATAN (does not extrapolate!)
                %%  Find index (indx) of grid-points in X around interpolation point XI:
                % Find the enclosing triangle (t)
                [indx,B] = mytsearchn(X',tri,XI');
                
                %--- Interpolation of LUT output variables:
                if ischar(Y)
                    for i=1:p % For each p point
                        if ~isnan(indx(i)) % Point in convex hull ==> Linear interpolation
                            for j=1:size(B,2)
                                auxY(:,j) = h5read(Y,'/LUTdata',[1,tri(indx(i),j)],[k,1]);
                            end
                            YI(:,i) = sum(bsxfun(@times,B(i,:),auxY),2);
                        end
                    end
                else
                    for i=1:p % For each p point
                        if ~isnan(indx(i)) % Point in convex hull ==> Linear interpolation
                            YI(:,i) = sum(bsxfun(@times,B(i,:),Y(:,tri(indx(i),:))),2);
                        end
                    end
                end
        end
    end
    
end



%%%%% SUPPORT FUNCTIONS %%%%%%%%%
function [t, p] = mytsearchn(x,tri,xi)
%MYTSEARCHN N-D closest simplex search.
%   T = MYTSEARCHN(X,TRI,XI) returns the indices T of the enclosing simplex
%   of the Delaunay triangulation TRI for each point in XI. X is 
%   an m-by-n matrix, representing m points in n-D space. XI is 
%   a p-by-n matrix, representing p points in n-D space.  MYTSEARCHN returns 
%   NaN for all points outside the convex hull of X. MYTSEARCHN requires a 
%   triangulation TRI of the points X obtained from DELAUNAYN.
% 
%   [T,P] = MYTSEARCHN(X,TRI,XI) also returns the barycentric coordinate P
%   of XI in the simplex TRI. P is an p-by-n+1 matrix. Each row of P is the
%   barycentric coordinate of the corresponding point in XI. It is useful
%   for interpolation.
%
%   The function is based on Matlab's TSEARCHN with some modifications to
%   speed-up in our application
%
%   See also delaunayTriangulation, DSEARCHN, QHULL, GRIDDATAN, DELAUNAYN.

%   Relies on the MEX file tsrchnmx to do most of the work.

%   Copyright 1984-2013 The MathWorks, Inc.

myeps = -sqrt(eps);
[npt, ndim] = size(xi); % Number of points
ntri = size(tri,1); % Number of simplexes

if size(x,2) ~= ndim
    error(message('MATLAB:tsearchn:InvalidDimensions'))
elseif size(tri,2) ~= ndim+1
    error(message('MATLAB:tsearchn:InvalidTessellation'))
end

% Heuristic for determining whether to use MEX-file
if npt > 100
    mult = 1;
else
    mult = npt/100;
end
use_m_file = ntri < (mult*10^ndim);

if use_m_file
    t = nan(npt,1); % Simplex containing corresponding input point
    p = nan(npt,ndim+1); % Barycentric coordinates for corresponding input point
    X = [ones(size(x,1),1) x]; % Append 1s to vertex matrix
    b = [ones(npt,1) xi]; % Append 1s to point matrix
    aux = false(npt,1); Ii = aux;
    for i = 1:ntri
        % For each triangle
        q = b * inv(X(tri(i,:),:)); % Compute barycentric coordinate of each point (MI VERSION. PARECE MAS RAPIDA)
        I = all(q > myeps,2); % Find simplex where all coordinates are positive
        if sum(I)%~aux
            Ii(~aux)=I;Ii(aux)=false; aux(~aux)=I;
            t(Ii) = i; % Set simplex
            p(Ii,:) = q(I,:); % Set barycentric coordinates
            b(I,:) = []; % Remove points that have been already found
            %if isempty(b), break,end 
        end
    end
else
  if nargout == 2
    [t, p] = tsearchn(x,tri,xi);
  else
    t = tsearchn(x,tri,xi);
  end
end


