function [lambda,Lp0,Edir0,Edif0,S,tdir,tdif,I0] = tp7_to_LUT(alg,file1,file2,rho1,rho2,SZA)
% TP7_TO_LUT processes the MODTRAN output tape7 files (same atmosphere and
% two different contant reflectivities) in order to obtain the atmospheric
% variables independent from surface properties.
%
% INPUTs: file1 and file2 = input tape7 files
%         RHO1 and RHO2 = constant reflectivities associated with the
%                         simulations on file1 and file2
%         SZA = Sun Zenith Angle
%
% OUTPUTs: lambda = Wavelength in [nm]
%          Lp0  : Atmospheric path radiance (in [mW/m2/sr/nm])
%          Edir : At-surface direct solar irradiance (in [mW/m2/nm])
%          Edif : At-surface diffuse solar irradiance (in [mW/m2/nm])
%          S    : Spherical Albedo (in [-])
%          tdir : Target-to-sensor direct transmittance (in [-])
%          tdif : Target-to-sensor diffuse transmittance (in [-])
%          I0   : Extraterrestrial solar irradiance (in mW/m2/nm)
%
%   Based on L. Guanter et al., On the application of the MODTRAN4 
%   atmospheric radiative transfer code to optical remote sensing, 
%   International Journal of Remote Sensing, vol. 30, no. 6, pp. 14071424,
%   2009.
%
%   See also: LUTGEN_PROCESS, READ_TAPE7

%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.01 $  $Date: 2018/02/28 $


% Initial values:
u_s = cosd(SZA); % Cosinus of the solar zenit angle in [deg]

% Recover parameters from the rho1 and rho2 files:
[lambda,tdir,Dir_Ref_Rad1,Tot_Ref_Rad1,Path_Rad1,I0] = alg.readRTMoutput(file1);
[lambda,tdir,Dir_Ref_Rad2,Tot_Ref_Rad2,Path_Rad2,I0] = alg.readRTMoutput(file2);
tdir(isnan(tdir) | isinf(tdir) | tdir<0) = 0;
I0(I0<0 | isnan(I0) | isinf(I0)) = 0;

% Direct fluxes (Edir1 = Edir2 i.e. independent of surface reflectance):
Edir1 = pi*Dir_Ref_Rad1./(u_s*tdir.*rho1); Edir1(isnan(Edir1) | isinf(Edir1) | Edir1<0) = 0;
Edir2 = pi*Dir_Ref_Rad2./(u_s*tdir.*rho2); Edir2(isnan(Edir2) | isinf(Edir2) | Edir2<0) = 0;

% Difuse fluxes (over an horizontal surface):
Edif1 = pi*Tot_Ref_Rad1./(tdir.*rho1) - u_s*Edir1; Edif1(isnan(Edif1) | isinf(Edif1) | Edif1<0) = 0;
Edif2 = pi*Tot_Ref_Rad2./(tdir.*rho2) - u_s*Edir2; Edif2(isnan(Edif2) | isinf(Edif2) | Edif2<0) = 0;

% Calculate Eg(rho1) and Eg(rho2):
Eg1 = Edir1*u_s + Edif1;
Eg2 = Edir2*u_s + Edif2;

% Calculate Lp0, tdif and S:
Lp0 = (rho1*Path_Rad2.*Eg1-rho2*Path_Rad1.*Eg2)./(rho1*Eg1-rho2*Eg2); Lp0(isnan(Lp0) | isinf(Lp0) | Lp0<0) = 0;
tdif = pi*(Path_Rad1-Lp0)./(rho1*Eg1); tdif(Eg1<10*eps | isnan(tdif) | isinf(tdif) | tdif<0) = 0;
S = (Eg2-Eg1)./(rho2*Eg2-rho1*Eg1); S(isnan(S) | isinf(S) | S<0) = 0;

Edir0 = Edir1;
Edif0 = (1-S*rho1).*(Edir0*u_s+Edif1)-Edir0*u_s;

% END of function
end