function writeRTMinput(file,rho,varnames,inputval,SpeConf,flag)
% WRITERTMINPUT Create MODTRAN5 input tape5 file
%   WRITERTMINPUT(FILE,RHO,VARNAMES,INPUTVAL,SPECONF,FLAG) creates, 
%   according to MODTRAN documentation, a MODTRAN's tape5 file (FILE) with
%   a spectrally constant reflectance RHO in the spectral range and 
%   configuration defined in the input SPECONF structure. MODTRAN 
%   configuration is based on the input variables and associated values
%   stored in the cell arrays VARNAMES (strings) and INPUTVAL (numeric 
%   values). An optional flag FLAG is set to determine whether simulations
%   are carried out at sensor level (FLAG='toa' or FLAG=[]), 
%   bottom-of-atmosphere (FLAG='boa'). FLAG  is mainly used to perform the
%   MODTRAN interrogation technique and proposed by Verhoef et al. 2012
%   (DISALB='W') and to derive atmospheric transfer functions.
%
%   Copyright 2019 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 2.06 $  $Date: 2019/05/19 $

%% Input variables to be used by MODTRAN:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%            INPUT VARIABLES TO BE USED BY MODTRAN              %%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numel(varnames)~=numel(inputval)
    error('Number of variable names and input values are not coincident.')
end

% Associates input values to each variable name:
for i=1:numel(varnames)
    eval([varnames{i},'=inputval{',num2str(i),'};'])
end

%--------------------------------------------------------------------------
% NOTE: Read MODTRAN5 Documentation for further information on the variables
% and CARD formats.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%-------------------- CARD 1 variables (obligatory) -----------------------
%--------------------------------------------------------------------------
% MODTRN model to be used: 'T' or 'M' (band model) or 'K' (correlated-k, 
% only for radiance modes)
% SPEED of the correlated-k option: 'S' (slow, 33 k values) or 'M' (medium,
% 17 k values)
if numel(MODTRN)>1, SPEED = MODTRN(2);
else                SPEED = 'S'; % Actually not used when Correlated-k is selected ('KS' or 'KM')
end
MODTRN = MODTRN(1);
% BINARY: Selection of the output files: 'F' (output in ASCII) or 'T' (tp7,
% tp8, plt, flx and acd files generated in binary
BINARY = 'F';
% LYMOLC: Includes auxiliary trace species ('+') or doesn't include them
% (blank)
LYMOLC = LYMOLC; if ~strcmp(LYMOLC,'+'), LYMOLC=' '; end
% MODEL: selects on geographical-seasonal atmospheres or specify to use a
% user-defined or radiosonde data (read MODTRAN documentation): 1 to 8
MODEL = MODEL; auxMODEL = MODEL;
if isstruct(MODEL) % MODEL contains user-defined atmospheric profile
    MODEL = 7;
elseif MODEL==0
    % No Gaseous Absorption atmosphere (In MODTRAN5 user manual, MODEL=0 is
    % for single-altitude meteorological data but we don't use this option
    % in ALG). We will scale gas concentration to 0 to remove absorptions.
    % For Rayleigh and temperature emission, we consider the Mid-Latitude
    % Summer atmosphere (review if other atmospheres might be needed)
    MODEL = 2; C_PROF = 7;
end 
% H: boundary layer aerosols top height [km]. User-defined variable, not MODTRAN's
% Z: boundary layer aerosols scale height [km]. User-defined variable, not MODTRAN's
H = H; Z = Z;
if H>0 || Z>0, MODEL=7; H = abs(H); Z=abs(Z);end % User-defined aerosol vertical distribution
% ITYPE indicates the line-of-sight path: in our case the option will be 3 
%(vertical or slant path to space or ground)
ITYPE = 3;
% IEMSCT selects the radiation transport mode of execution (see MODTRAN
% documentation): 0-4
IEMSCT = IEMSCT;
% IMULT adds Multiple Scattering (MS): 0 (no MS) or +1 plane, -1 Space 
% (with MS)
IMULT = -1;
% M1, M2, M3, M4, M5, M6 and MDEF modify user-specified altitude profiles
% for temperature, pressure and default molecular gases and heavy
% molecules: 1-6 (for M1, M2, M3, M4, M5 and M6) and 0 or 1 for MDEF
% If MODEL is 1 to 6, then M1 to M6 and MDEF are all overwritten (i.e. the
% selected MODEL is used for all the atmospheric species, temperature and
% pressure)
M1 = MODEL; M2 = MODEL; M3 = MODEL; M4 = MODEL; M5 = MODEL; M6 = MODEL;
MDEF = 0;
if MODEL==7 && isstruct(auxMODEL)
    % User-defined atmospheric model. Atmospheric species, temperature and 
    % pressure are provided (JCHAR in CARD 2C1 must be specified)
    M1 = 0; M2 = 0; M3 = 0; M4 = 0; M5 = 0; M6 = 0;
    MDEF = -1;
elseif MODEL==7 && auxMODEL>0
    % Only user-defined aerosol vertical profiles are introduced.
    % Atmospheric species, temperature and pressure are the same as the 
    % default atmospheric MODEL (set JCHAR blank in CARD 2C1)
    M1 = auxMODEL; M2 = auxMODEL; M3 = auxMODEL;
    M4 = auxMODEL; M5 = auxMODEL; M6 = auxMODEL;
    MDEF = 0;
end
% I_RD2C selects if normal operation of MODTRAN is used or input
% atmospheric profiles are to be read: 0 normal operation (reuse of last
% read in atmosphere) or 1 (inputs are to be read)
I_RD2C = 0; if MODEL==7, I_RD2C=1; end
% NOPRNT: printing of output files (see MODTRAN documentation)
NOPRNT = NOPRNT;
% TPTEMP: boundary temperature, in [K], of image pixel (i.e. H2) for paths
% intersecting the Earth's surface. For no surface emission if H2 above
% ground, then TPTEM<=0. If left blank, MODTRAN takes the temperature of he
% 1st atmospheric level
TPTEMP = 0;
% SURREF selects to use a BRDF or Lambertian surfaces, constant albedo or
% pre-stored spectral surface albedos:
% 'B' for BRDF, 'L' for Lambertian, >=0 for constant albedo or <0 for
% pre-stored spectral surface albedos:
SURREF = rho; % Note: Put single quotes for numerical negative values. 

%--------------------------------------------------------------------------
%-------------------- CARD 1A variables (obligatory) -----------------------
%--------------------------------------------------------------------------
% DIS choose among the different multiple-scattering algorithms: 'T' for
% using DISORT, 'F' for faster Isaac 2-stream algorithm or 'S' for scaling
% Isaac 2-stream by running DISORT at selected spectral windows
DIS = 'T';
if     NSTR==0,  DIS='F'; 
elseif NSTR==-1, NSTR=8; DIS='S';
end
% DISAZM is a flag for the azimut dependence using DISORT ('T' for
% considering the azimut dependence and 'F' to not consider it). Adding the
% azimut dependence increase the computation time so consider DISAZM='F'
% when vertical fluxes are needed, solar/viewing angles are near vertical 
% or solar multiple scattering is a small radiance component (e.g. in LWIR
% range).
DISAZM = DISAZM;
% DISALB='T' will calculate the spectral spherical albedo of the atmosphere
% and diffuse transmittance for the LOS and Sun-to-Ground paths. DISALB
% option is only available with DISORT (DIS='T').
DISALB = DISALB;
if strcmp(DISALB,'T')
    DIS = 'T';
elseif strcmp(DISALB,'W')
    % Interrogation technique of Verhoef et al. 2012
    DISALB ='F';
end
% NSTR selects the number of streams used by DISORT (2, 4, 8, 16, 32).
% Usually take NSTR=8 but set to 32 if more precission is requiered
NSTR = NSTR;
% SFWHM selects the FWHM for the TOA solar irradiance used. =0 for using
% default TOA solar irradiance files, >0 for degrading data and <0 for new
% solar FWHM
SFWHM = 0;
% CO2MX introduces the CO2 mix in ppmv. Set to 0 for using the default 330
% ppmv. In 1999, the CO2 mix is close to 365 ppmv.
CO2MX = CO2MX;
if ~isstruct(auxMODEL) && auxMODEL==0 % No Gaseous Absorption
    CO2MX = 0.001;% In practice it's nearly no CO2
end
% H2OSTR vertical water vapor column. This variable is separated in two for
% input in MATLAB. The first selects the character for selecting the units
% ('g' for g/cm^2 or 'a' for ATM-cm). Then the value is introduced. If the
% first charecter is blank ' ', the value is a scale factor on the default
% water vapor column.
if H2OSTR<0
    H2OSTRunit = ' '; % Scale factor
elseif H2OSTR==0
    H2OSTRunit = 'g'; % g/cm2
    H2OSTRvalue = 0.00001; % In practice it's nearly no H2O
else
    H2OSTRunit = 'g'; % g/cm2
end
H2OSTRvalue = abs(H2OSTR);
if ~isstruct(auxMODEL) && auxMODEL==0 % No Gaseous Absorption
    H2OSTRunit = 'g';
    H2OSTRvalue = 0.00001; % In practice it's nearly no H2O
end  
% O3STR vertical ozone column. Similar to H2OSTR.
if O3STR<0
	O3STRunit = ' '; % Scale factor
	O3STR = 1; % Automatically set to default value
elseif O3STR==0
    O3STRunit = ' '; % Scale factor
else
    O3STRunit = 'a'; % atm-cm
end
O3STRvalue = abs(O3STR);
if ~isstruct(auxMODEL) && auxMODEL==0 % No Gaseous Absorption
    O3STRunit = 'a';
    O3STRvalue = 0.000001; % In practice it's nearly no O3
end 
% C_PROF selects the scale factors for the molecular species (uniformely
% mixed, cross-section and trace gases) (see MODTRAN5 documentation)
C_PROF = C_PROF;
% LSUNFL select the TOA solar irradiance file to be read (see MODTRAN
% documentation)
LSUNFL = num2str(LSUNFL); % Must be string!!
% LBMNAM select the band model database file (see MODTRAN documentation)
LBMNAM = 'T';
% LFLTNM allows to define the instrument filter function file to be read
% from CARD 1A3.
LFLTNM = 'F';
% H2OAER modify, 'T', (or not, 'F') the aerosol optical properties to 
% reflect changes on the H2OSTR water vapor column.
H2OAER = H2OAER;
if ~isstruct(auxMODEL) && auxMODEL==0 % No Gaseous Absorption
    H2OAER = 'F'; % Try not to affect aerosol properties when removing H2O (check!) 
end
% CDTDIR allows to select from which directory the input tape5 files are to
% be read. 'T' allows to use CARD 1A4 to select the directory. 'F' assume
% that the tape5 files are in the directory DATA/
CDTDIR = 'F';
% SOLCON sets the scale factor for the TOA Solar irradiance on the input
% file. <0 selects the scale factor, 0 uses the default TOA solar
% irradiance and >0 assigns the input W/m^2.
SOLCON = 0;
% CDASTM perturbs the boundary layer and tropospheric aerosol reference
% spectral extinction data (see MODTRAN documentation)
CDASTM = ' ';
% ASTMC Amstrong law coefficient
ASTMC = 1;
% ASTMX Amstrong law offset used in perturbation method
ASTMX = ASTMX;
% ASTMO Amstrong law offset not used with perturbation method
ASTMO = 0;
% AERRH relative humidity for the boundary layer aerosol (>0). If <=0 then
% the value is taken from the input atmosphere boundary layer
AERRH = 0;
% NSSALB selects the number of aerosol single scattering albedo spectral 
% grid points (>0). If =0 then use reference value and if <0 then aerosol
% single scattering coalbedo is read on CARD 1B.
NSSALB = 0;

% SSA Spectrally independent single scattering albedo (between 0 and 1). 
% Only valid with the the definition of the Henyen-Greenstein phase 
% function coefficient AND the Angstrom law exponent. If -1 default model 
% spectrally dependent single scattering albedo will be used.
SSA = SSA;
if SSA>=0
    if G==10 || ASTMX==0
        error('WriteTape5:SSA',...
        ['Single scattering albedo option only available with the use of',...
        'Henyey-Greenstein phase function and Angstrom law. Review input values'])
    else
        IHAZE = 7;
    end
end

%--------------------------------------------------------------------------
%-------------------- CARD 1A1 variables (optional) -----------------------
%--------------------------------------------------------------------------
% USRSUN selects the TOA solar irradiance file to be used. Card read if
% LSUNFL='T'. See MODTRAN documentation.
if LSUNFL=='T'
    USRSUN = 1;
end

%--------------------------------------------------------------------------
%-------------------- CARD 1A2 variables (optional) -----------------------
%--------------------------------------------------------------------------
% BMNAME selects the name of the binary, direct-access version of the band
% model parameter data file. See MODTRAN documentation
if LBMNAM=='T' || LBMNAM=='4' || LBMNAM=='2'
    aux = str2cell(SpeConf.BMNAME.values(2:end-1));
    if iscell(aux), BMNAME = fliplr(aux); else BMNAME = aux; end
end

%--------------------------------------------------------------------------
%-------------------- CARD 1A3 variables (optional) -----------------------
%--------------------------------------------------------------------------
% FILTNM selects the user-defined instrument filter response function
if LFLTNM=='T'
    FILTNM = 'DATA/modis399_2176nm.flt';
end

%--------------------------------------------------------------------------
%-------------------- CARD 1A4 variables (optional) -----------------------
%--------------------------------------------------------------------------
% DATDIR indicates the path for the MODTRAN data files
if CDTDIR=='T'
    DATDIR = '';
end

%--------------------------------------------------------------------------
%-------------------- CARD 1A5 variables (optional) -----------------------
%--------------------------------------------------------------------------
% S_UMIX: Scale factors for the default profiles of the 9 uniformly mixed
% species and N2
if C_PROF==1 || C_PROF==3 || C_PROF==5 || C_PROF==7
    S_UMIX1 = 1; S_UMIX2 = 1; S_UMIX3 = 1; S_UMIX4 = 0; S_UMIX5 = 1;
    S_UMIX6 = 1; S_UMIX7 = 1; S_UMIX8 = 1; S_UMIX9 = 1; S_UMIX10 = 1;
    if auxMODEL==0
        S_UMIX1 = 0; S_UMIX2 = 0; S_UMIX3 = 0; S_UMIX4 = 0; S_UMIX5 = 0;
        S_UMIX6 = 0; S_UMIX7 = 0; S_UMIX8 = 0; S_UMIX9 = 0; S_UMIX10 = 0;
    end
end

%--------------------------------------------------------------------------
%-------------------- CARD 1A6 variables (optional) -----------------------
%--------------------------------------------------------------------------
% S_XSEC: Scale factors for the 13 cross-section molecular species
if C_PROF==2 || C_PROF==3 || C_PROF==6 || C_PROF==7
    S_XSEC1  = 1.85; S_XSEC2  = 2.25; S_XSEC3  = 1;    S_XSEC4  = 1;
    S_XSEC5  = 2.75; S_XSEC6  = 4;    S_XSEC7  = 1;    S_XSEC8  = 1;
    S_XSEC9  = 1;    S_XSEC10 = 1;    S_XSEC11 = 0.73; S_XSEC12 = 1;  
    S_XSEC13 = 0;
    if auxMODEL==0
        S_XSEC1  = 0; S_XSEC2  = 0; S_XSEC3  = 0;	S_XSEC4  = 0;
        S_XSEC5  = 0; S_XSEC6  = 0; S_XSEC7  = 0;	S_XSEC8  = 0;
        S_XSEC9  = 0; S_XSEC10 = 0; S_XSEC11 = 0;   S_XSEC12 = 0;  
        S_XSEC13 = 0;
    end
end

%--------------------------------------------------------------------------
%-------------------- CARD 1A7 variables (optional) -----------------------
%--------------------------------------------------------------------------
% S_TRAC: Scale factors for the 16 trace molecular species
if C_PROF==4 || C_PROF==5 || C_PROF==6 || C_PROF==7
    S_TRAC1  = 1; S_TRAC2  = 1; S_TRAC3  = 1; S_TRAC4  = 1; S_TRAC5  = 1; 
    S_TRAC6  = 1; S_TRAC7  = 1; S_TRAC8  = 1; S_TRAC9  = 1; S_TRAC10 = 1; 
    S_TRAC11 = 1; S_TRAC12 = 1; S_TRAC13 = 1; S_TRAC14 = 1;
    S_TRAC15 = 1; S_TRAC16 = 1;
    if auxMODEL==0
        S_TRAC1  = 0; S_TRAC2  = 0; S_TRAC3  = 0; S_TRAC4  = 0; S_TRAC5  = 0; 
        S_TRAC6  = 0; S_TRAC7  = 0; S_TRAC8  = 0; S_TRAC9  = 0; S_TRAC10 = 0; 
        S_TRAC11 = 0; S_TRAC12 = 0; S_TRAC13 = 0; S_TRAC14 = 0;
        S_TRAC15 = 0; S_TRAC16 = 0;
    end
end

%--------------------------------------------------------------------------
%-------------------- CARD 1B variables (optional) -----------------------
%--------------------------------------------------------------------------
% AWAVLN wavelenght grid (in um) and ASSALB single scattering albedo values
% ACOALB is the aerosol single scattering co-albedo
% RHASYM is the relative humidity used to define the aerosol asymmetry

% factor [%]
if NSSALB > 0
    AWAVLN = 0.2; ASSALB = 0.5; % Repeat NSSALB times (REVIEW implementation!!!)
elseif NSSALB < 0 && ASTMX > 0
    ACOALB = 0.5;
    RHASYM = 70;
end


%--------------------------------------------------------------------------
%-------------------- CARD 2 variables (obligatory) -----------------------
%--------------------------------------------------------------------------
% APLUS allows to vertically translate or scale user-defined aerosol
% optical profiles. ' ' = defaul, 'A+' = introduce vertical profiles in
% CARD 2A+
APLUS = ' ';
% IHAZE specifies the aerosol model used for the boundary layer (0-2 km)
% and a default, surface meteorological range. See MODTRAN documentation
IHAZE = IHAZE;
if isstruct(IHAZE) % IHAZE contains data from OPAC or user-defined aerosol:
    aeraux = IHAZE;
    IHAZE   = 7;
    if strcmp(aeraux.type,'OPAC') % OPAC aerosol
        % RH: average relative humidity of the boundary and troposphere layers
        % for the selected atmosphere model and input columnar water vapor,
        % weighted by exp(-h[km]/8[km]):
        RH_boundary = [74.4 66.7 71.5 72 73.7 48.9 67.8]; % Default average RH for boundary layer and MODEL
        RH_troposphere = [41.8 38.1 44.5 52.7 52.3 47.9 46.2]; % Default average RH for troposphere and MODEL
        if strcmp(H2OSTRunit,'g')
            auxCWV = [4.115, 2.922, 0.852, 2.081, 0.416, 1.416]; % Default CWV [g/cm2] for MODEL
            auxCWV = [auxCWV,mean(auxCWV)];
            scale = auxCWV(MODEL)/H2OSTRvalue;
        else
            scale = H2OSTRvalue;
        end
        RH = scale*[RH_boundary(MODEL), RH_troposphere(MODEL)]; RH(RH>100)=100;
    elseif strcmp(aeraux.type,'ATK') % ATK aerosol
        aeraux.VARSPC = aeraux.data(:,1);
        aeraux.EXTC = aeraux.data(:,2);
        aeraux.ASYM = aeraux.data(:,4);
        aeraux.ABSC = (1-aeraux.data(:,3)).*aeraux.data(:,2); 
        aeraux.RH   = [];
        aeraux.ANGF = []; 
        aeraux.F    = [];
    end
elseif IHAZE>=3.1 && IHAZE<=3.9 % Navy maritime aerosols
    ICSTL = num2str(IHAZE); ICSTL = str2num(ICSTL(3));
    IHAZE = 3;
end
% CNOVAM = 'N' allows introducing the Navy Ocean Vertical Aerosol Model
CNOVAM = ' ';
% ISEASN selects the seasonal aerosol profile for the tropospheric and 
% stratospheric aerosols. Only the tropospheric aerosol 
% extinction coefficients are used with the 2 to 10 km profiles
ISEASN = 0;
% ARUSS user-defined aerosol optical properties (ARUSS='USS'). ARUSS=' '
% default
ARUSS = ' '; if IHAZE==7, ARUSS = 'USS'; end
% IVULCN selects the aerosol profile and type of extinction for
% stratospheric aerosols. See MODTRAN documentation
IVULCN = IVULCN;
if IVULCN==0
    % In ALG, IVULCN=0 means no stratospheric aerosols while in MODTRAN,
    % IVULCN=0 is background stratospheric (same as IVULCN=1). To remove
    % background stratospheric...
end
% ICSTL is the air mass character (1 to 10). Default value is 3. See
% MODTRAN documentation
if IHAZE~=3, ICSTL = 3; end
% ICLD specifies the cloud/rain models to be used. See MODTRAN
% documentation
ICLD = 0;
% IVSA selects the use of Army Vertical Structure Algorithm for aerosols in
% the boundary layer. Not used (IVSA=0), VSA (IVSA=1).
IVSA = 0;
% VIS defines the surface meteorological range. >0 value in [km], =0 using
% the default range set by IHAZE or <0 for the negative of the 550nm
% vertical aerosol optical depth
VIS = -1*VIS;
if VIS==0, IHAZE=-1; end % No aerosols
% WSS specifies the wind speedm in [m/s], for use with the Navy Maritime
% (IHAZE=3) and Desert (IHAZE=10) aerosol models.
WSS = 0;
% WHH specifies the 24-hours average wind speed for use with the Navy
% Maritime aerosol model (IHAZE=3)
WHH = 0;
% Note: for Navy Aerosol Maritime model, if WSS=WHH=0, the default wind
% speed are set according to the atmospheric model (MODEL). See MODTRAN
% documentation (table 4).
% RAINRT is the rain rate in [mm/hr]. The default value is 0 for no rain.
% Used to top of cloud when cloud is present, otherwise rain rate used to 6
% km.
RAINRT = 0;
% GNDALT: Altitude, in [km] of surface relative to sea level. Negative
% values can be used but it cannot be larger than 6 km. From 0 to 6 km, the
% aerosol profiles are compressed or stretched. GNDALT is set to the 1st
% profile altitude when radiosone data is used (MODEL=7)
GNDALT = GNDALT;

%--------------------------------------------------------------------------
%-------------------- CARD 2A+ variables (optional) -----------------------
%--------------------------------------------------------------------------
% This card allows to move MODTRAN's built-in aerosols from their original
% altitudes to user-defined altitudes which may overlap. CARD 2A+ cannot be
% used with NOVAM.
% ZAERi1 bottom of aerosol i in [km]
% ZAERi2 top of aerosol i in [km]
% SCALEi scales the aerosol column density (if SCALEi=0 then the column
% density is preserved)
if strcmp(APLUS,'A+')
    ZAER11 = 0; ZAER12  = 3;  SCALE1 = 0; ZAER21 = 2;  ZAER22 = 11;  SCALE2 = 0;
    ZAER31 = 10; ZAER32 = 35; SCALE3 = 0; ZAER41 = 30; ZAER42 = 100; SCALE4 = 0;
end

%--------------------------------------------------------------------------
%--------------------- CARD 2A variables (optional) -----------------------
%--------------------------------------------------------------------------
% This card deals with the cloud models (ICLD>0) except for ICLD=11
if ICLD==18 || ICLD==19 % For cirrus cloud models
    CTHIK = 0; % Cirrus thickness in [km]. <=0 use cirrus statistics, >0 defined thickness
    CALT  = 0; % Altitude in [km]. <=0 use calculated value, >0 defined altitude
    CEXT  = 0; % Extinction coef. at 550 nm. <=0 use 0.14km^-2*CTHIK, >0 defined value
elseif ICLD>=1 && ICLD<=10 %Water/Ice cloud models
    CTHIK  = 0; % Vertical thickness in [km]. <=0 default value, >0 defined thickness
    CALT   = -9; % Vertical altitude in [km]. <0 default value, >=0 defined altitude
    CEXT   = 0; % Cloud liquid vapor water droplet and ice particle vertical extinction. <=0 do not scale, >0 extinction coef. in [km^-1]
    NCRALT = 1; % Number of layer boundary altitudes.<3 default for ICLD, >=3 number of layer altitudes (CARD 2E1)
    NCRSPC = 1; % Wavelength entries. <2 default spectral data for ICLD, >=2 number of wavelengths to which spectral data is entered (CARD 2E2)
    CWAVLN = 0; % Reference wavelength for vertical extinction [0.2, 200] um, out of this range considers 0.55 um
    CCOLWD = -9; % Water droplet vertical column density in [km g/m^3]. <0 do not scale, >=0 input value
    CCOLIP = -9; % Ice particle cloud vertical column density in [km g/m^3]. <0 do not scale, >=0 input value
    CHUMID = 0; % Relative humidity at all boundary layers with positive rain rate of cloud density. >0 & <=105 input value, <=0 assume 100%, >105 do not alter H20 profil within the clouds
    ASYMWD = 0; % Henyey-Greenstein phase function asymmetry factor for scattering by cloud liquid water droplets. |ASYMWD|<1  phase function asymmetry function at all wavelenght, |ASYMWD|>=1 user-defined or model spectral asymmetry factors
    ASYMIP = 0; % Henyey-Greenstein phase function asymmetry factor for scattering by cloud ice particles. |ASYMIP|<1  phase function asymmetry function at all wavelenght, |ASYMIP|>=1 user-defined or model spectral asymmetry factors
end

%--------------------------------------------------------------------------
%--------------------- CARD 2B variables (optional) -----------------------
%--------------------------------------------------------------------------
% This card works with the Army Vertical Structure Algorightm (VSA)
if IVSA==1
    % 4 cases of cloud/fog depending on the values f the variables. See
    % MODTRAN documentation
    ZCVSA  = 0; % Cloud ceiling height [km]. >0 sets the input value, =0 unknown value that will be calculated by MODTRAN depending on the case, <0 no cloud ceiling
    ZTVSA  = 0; % Thickness of cloud/fog in [km]. >0 input value, =0 unknown default is 0.2 km
    ZINVSA = 0; % Height of inversion boundary in [km]. >0 input value, =0 unknown default is 2 km for cloud and 0.2 km for fog, <0 no inversion layer
end

%--------------------------------------------------------------------------
%--------------------- CARD 2C variables (optional) -----------------------
%--------------------------------------------------------------------------
% CARD 2C and CARD 2C1 allow to create a user-supplied atmosphere profile
% data (MODEL=0,7 or 8 || I_RD2C=1 on CARD 1).
if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1
    IRD1   = 0; % Controls reading of WMOL (see table 8 MODTRAN documentation) in CARD 2C2. No read (IRD1=0), read (IRD1=1)
    if isstruct(auxMODEL)
        % User-defined atmospheric model or No Gaseous Absorption. 
        % Atmospheric species, temperature and pressure are provided 
        % (JCHAR in CARD 2C1 must be specified)
        IRD1 = 1;
    end
    IRD2   = 0; % Controls reading of AHAZE, EQLWCZ... (CARD 2C3). No read (IRD2=0), read (IRD2=1)
    if H>0 && Z>0
        % User-defined aerosol vertical profiles. Atmospheric species,
        % temperature and pressure are the same as the default atmospheric 
        % MODEL (set JCHAR blank in CARD 2C1)
        IRD2 = 1; % IRD2=2 for advanced option (independent control of 4 regions)
    end
     % HMODEL: Name identification of new model atmosphere
    if isstruct(auxMODEL)
        HMODEL = 'New_Atmosphere_Name';
    elseif auxMODEL==0
        HMODEL = 'No_Gaseous_Absorp';
    else
        auxname = {'TRO','MLS','MLW','SAS','SAW','USS'};
        HMODEL = ['Default_',auxname{auxMODEL}];
    end
    REE    = 6371; % Earth radius in [km]. Only read if MODEL=8. Redundant with RO on CARD 3
    NMOLYC = 0; % Number of user-defined species with specified names, concentrations and molecular weights
    E_MASS = 0; % Planetary mass in Earth masses. If =0, then E_Mass is set to 1. Used in the hydrostatic equation when MODEL=8
    AIRMWT = 0; % Molecular weight of air at surface in g/mol. If =0, AIRMWT is set to AIR_MW=29.964 g/mol
end

%--------------------------------------------------------------------------
%--------------------- CARD 2CY variables (optional) ----------------------
%--------------------------------------------------------------------------
% Defines the names of the NMOLYC user-defined species. Read MODTRAN
% documentation
if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && NMOLYC>0 && MDEF==2
    for i=1:NMOLYC
        YNAME{i} = ['Specie',num2str(i)]; 
    end
end

% NOTE: Cards 2C1 through 2C3 are repeated ML times
%--------------------------------------------------------------------------
%--------------------- CARD 2C1 variables (optional) ----------------------
%--------------------------------------------------------------------------
if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IVSA==0
%     ZM      = [4:25,30:5:60,70,80,100]; % Altitude, in [km], of each layer boundary
%     ZM      = [linspace(0,3,ML-numel(ZM)),ZM];
    ZM    = [0:25,30:5:60,70,80,100]; % Altitude, in [km], of each layer boundary
    P     = zeros(size(ZM)); % Pressure of layer boundary
    T     = zeros(size(ZM)); % Temperature of layer boundary
    WMOL1 = zeros(size(ZM)); % Water vapor density (see table 8 MODTRAN documentation)
    WMOL2 = zeros(size(ZM)); % Carbon dioxide density (see table 8 MODTRAN documentation)
    WMOL3 = zeros(size(ZM)); % Ozone density (see table 8 MODTRAN documentation)
    if isstruct(auxMODEL)
        % User-defined atmospheric model. Atmospheric species, temperature
        % and pressure are provided (JCHAR in CARD 2C1 must be specified)
        ZM = auxMODEL.ZM;
        P = auxMODEL.P;
        T = auxMODEL.T;
        WMOL1 = auxMODEL.H2O; % Water vapor density (H2O)
        WMOL2 = auxMODEL.CO2; % Carbon dioxide density (CO2)
        WMOL3 = auxMODEL.O3; % Ozone density (O3)
        % JCHAR, JCHARX and JCHARY control variables for selection of units
        % for primary profile inputs (P, T and molecular constituents). See
        % table 8 MODTRAN documentation
        % JCHAR1 and JCHAR2 can override P and T if selected to default
        % MODEL or M1 (check!!!)
        % JCHAR1 selects presure in [mb] (A), [atm] (B), to default MODEL 
        % value ('1-6') or default to M1 (CARD 1) (blank):
        JCHAR{1} = repmat({'A'},size(ZM));
        % JCHAR2 selects temperature in [K] (A), [C] (B), to difference in 
        % [K] from M1 profile, to default MODEL value ('1-6') or default to
        % M1 (CARD 1) (blank):
        JCHAR{2} = repmat({'A'},size(ZM));
        % JCHAR3-14 selects units of gas concentration (A-H, see MODTRAN
        % manual), to default MODEL value ('1-6') or default to M2-6 and 
        % MDEF (CARD 1) (blank): 
        for i=3:14
            JCHAR{i} = repmat({'A'},size(ZM));
        end
        % Single unit selection control input for the entire set of CFCs 
        % and cross-section molecules:
        JCHARX  = repmat({'2'},size(ZM));
        % Single unit selection control input for the entire set of 
        % user-defined molecules
        JCHARY  = repmat({'1'},size(ZM)); 
    elseif auxMODEL==0
        % OPTION NOT USED!! (kept for eventual future use)
        % No Gaseous Absorption atmospheric model. Default temperature and 
        % pressure and zero concentration of species provided
        % JCHAR, JCHARX and JCHARY control variables for selection of units
        % for primary profile inputs (P, T and molecular constituents). See
        % table 8 MODTRAN documentation
        JCHAR{1} = repmat({' '},size(ZM));
        JCHAR{2} = repmat({' '},size(ZM));
        for i=3:14
            JCHAR{i} = repmat({'A'},size(ZM));
        end
        JCHARX  = repmat({'A'},size(ZM));
        JCHARY  = repmat({'A'},size(ZM));
    elseif auxMODEL>0
        % Default atmosphere. Atmospheric species, temperature and pressure
        % are the same as the default atmospheric MODEL (set JCHAR blank in
        % CARD 2C1)
        % JCHAR, JCHARX and JCHARY control variables for selection of units
        % for primary profile inputs (P, T and molecular constituents). See
        % table 8 MODTRAN documentation
        JCHAR{1} = repmat({' '},size(ZM));
        JCHAR{2} = repmat({' '},size(ZM));
        for i=3:14
            JCHAR{i} = repmat({' '},size(ZM));
        end
        JCHARX  = repmat({num2str(auxMODEL)},size(ZM));
        JCHARY  = repmat({'1'},size(ZM)); 
    end
    ML = numel(ZM); % Number of atmospheric levels to be inserted (max. LAYDIM=126). See ZM in CARD 2C1!!
end

%--------------------------------------------------------------------------
%--------------------- CARD 2C2 variables (optional) ----------------------
%--------------------------------------------------------------------------
if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD1==1
    WMOL4   = zeros(size(ZM)); % Nitrous oxide density (N2O) (see table 8 MODTRAN documentation)
    WMOL5   = zeros(size(ZM)); % Carbon monoxide density (CO) (see table 8 MODTRAN documentation)
    WMOL6   = zeros(size(ZM)); % Methane density (CH4) (see table 8 MODTRAN documentation)
    WMOL7   = zeros(size(ZM)); % Oxygen density (O2) (see table 8 MODTRAN documentation)
    WMOL8   = zeros(size(ZM)); % Nitric oxide density (NO) (see table 8 MODTRAN documentation)
    WMOL9   = zeros(size(ZM)); % Sulfur dioxide density (SO2) (see table 8 MODTRAN documentation)
    WMOL10  = zeros(size(ZM)); % Nitrogen dioxide density (NO2) (see table 8 MODTRAN documentation)
    WMOL11  = zeros(size(ZM)); % Ammonia density (NH3) (see table 8 MODTRAN documentation)
    WMOL12  = zeros(size(ZM)); % Nitric acid density (HNO3) (see table 8 MODTRAN documentation)
    if isstruct(auxMODEL)
        WMOL4   = auxMODEL.N2O; % Nitrous oxide density (N2O)
        WMOL5   = auxMODEL.CO; % Carbon monoxide density (CO)
        WMOL6   = auxMODEL.CH4; % Methane density (CH4)
        WMOL7   = auxMODEL.O2; % Oxygen density (O2)
        WMOL8   = auxMODEL.NO; % Nitric oxide density (NO)
        WMOL9   = auxMODEL.SO2; % Sulfur dioxide density (SO2)
        WMOL10  = auxMODEL.NO2; % Nitrogen dioxide density (NO2)
        WMOL11  = auxMODEL.NH3; % Ammonia density (NH3)
        WMOL12  = auxMODEL.HNO3; % Nitric acid density (HNO3)
    end
end

%--------------------------------------------------------------------------
%-------------------- CARD 2C2X variables (optional) ----------------------
%--------------------------------------------------------------------------
if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD1==1 && MDEF==2
    % REVIEW!!! (read user-input profile data, e.g. text file)
    % Heavy molecular specie density (see table 9 MODTRAN documentation)
    WMOLX1  = WMOLX1; % CCl3F
    WMOLX2  = WMOLX2; % CCl2F2
    WMOLX3  = WMOLX3; % CClF3
    WMOLX4  = WMOLX4; % CF4
    WMOLX5  = WMOLX5; % CHClF2
    WMOLX6  = WMOLX6; % C2Cl3F3
    WMOLX7  = WMOLX7; % C2Cl2F4
    WMOLX8  = WMOLX8; % C2ClF5
    WMOLX9  = WMOLX9; % ClONO2
    WMOLX10 = WMOLX10; % HNO4
    WMOLX11 = WMOLX11; % CHCl2F
    WMOLX12 = WMOLX12; % CCl4
    WMOLX13 = WMOLX13; % N2O5
end

%--------------------------------------------------------------------------
%-------------------- CARD 2C2Y variables (optional) ----------------------
%--------------------------------------------------------------------------
% Repeat NMOLYC times at each altitude ZM
if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD1==1 && NMOLYC>0
    % REVIEW!!! (read user-input profile data, e.g. text file). There will
    % be NMOLYC user-defined species
    WMOLY = 1; % User-defined specie density 
end

%--------------------------------------------------------------------------
%--------------------- CARD 2C3 variables (optional) ----------------------
%--------------------------------------------------------------------------
% For user-defined aerosol/cloud/rain models
if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD2==1
    % Aerosol/cloud scaling factor (similar to visible extinction 
    % coefficient) at altitude ZM. The model implemented is a decreasing
    % exponential with a user-defined scale height (Z [km]) in the boundary 
    % layer from 0 km to a user-defined top altitude (H [km]).
    AHAZE = zeros(size(ZM));
    AHAZE(ZM<=H) = exp(-ZM(ZM<=H)/Z);
    % The vertical distribution is scaled so that the total column AOT 
    % (integral of extiction coefficient) has the value VIS at 550 nm.
    % Troposheric stratospheric and mesospheric aerosol distribution is 
    % kept to the default MODTRAN types.
    auxh = [1:25,30:5:60,70,80,100];
    % Extinction coef. at 1-4 km from fitting MODTRAN profiles at VIS:
    if any(abs(auxMODEL)==[0,1,2,4,6,7,8]) % SUMMER profiles
        auxEXTC_1km = 0.2107*abs(VIS)^2+0.2874*abs(VIS)-0.011; if abs(VIS)>0.35, auxEXTC_1km=0.8011*abs(VIS)-0.1643; end
        auxEXTC_2km = 0.2091*abs(VIS)-0.008; if abs(VIS)>0.35, auxEXTC_2km=6.21e-2; end
        auxEXTC_3km = -0.0607*abs(VIS)^2+0.139*abs(VIS)-0.0058; if abs(VIS)>0.35, auxEXTC_3km=3.46e-2; end
        auxEXTC_4km = 0.0481*abs(VIS)+0.0024; if abs(VIS)>0.35, auxEXTC_4km=1.85e-2; end
        auxEXTC = [9.31e-3,7.71e-3,6.23e-3,3.37e-3,1.82e-3,1.14e-3,7.99e-4,6.41e-4,5.17e-4,4.42e-4,3.95e-4,3.82e-4,4.25e-4,5.2e-4,5.81e-4,5.89e-4,5.02e-4,4.2e-4,3e-4,1.98e-4,1.31e-4,3.32e-5,4.3e-6,1.67e-6,8e-7,4.2e-7,2.207e-7,1.159e-7,3.2e-8,5.795e-9,1.9e-10];
    elseif any(abs(auxMODEL)==[3,5]) % WINTER profile
        auxEXTC_1km = 0.4569*abs(VIS)^2+0.2314*abs(VIS)-0.0013; if abs(VIS)>0.35, auxEXTC_1km=0.8032*abs(VIS)-0.142; end
        auxEXTC_2km = 0.2054*abs(VIS)-0.003; if abs(VIS)>0.35, auxEXTC_2km=6.21e-2; end
        auxEXTC_3km = -0.116*abs(VIS)^2+0.1341*abs(VIS)-0.0044; if abs(VIS)>0.35, auxEXTC_3km=2.72e-2; end
        auxEXTC_4km = 0.0317*abs(VIS)+0.002; if abs(VIS)>0.35, auxEXTC_4km=1.20e-2; end
        auxEXTC = [4.85e-3,3.54e-3,2.31e-3,1.41e-3,9.8e-4,7.87e-4,7.14e-4,6.64e-4,6.23e-4,6.45e-4,6.43e-4,6.41e-4,6e-4,5.62e-4,4.91e-4,4.23e-4,3.52e-4,2.95e-4,2.42e-4,1.9e-4,1.5e-4,3.32e-5,4.3e-6,1.67e-6,8e-7,4.2e-7,2.207e-7,1.159e-7,3.2e-8,5.795e-9,1.9e-10];
    end
    auxEXTC = interp1(auxh,[auxEXTC_1km,auxEXTC_2km,auxEXTC_3km,auxEXTC_4km,auxEXTC],ZM(ZM>=1)); auxZM = ZM(ZM>=1);
    auxVIS  = trapz(auxZM(auxZM>H),auxEXTC(auxZM>H)); % Default AOT from H to 100 km
    scale = (abs(VIS)-auxVIS)/trapz(ZM(ZM<=H),AHAZE(ZM<=H));
    AHAZE(ZM<=H) = scale*AHAZE(ZM<=H);
    % Equivalent liquid water content in [g/m^3] at altitude ZM for 
    % aerosol/cloud/fog models. Omitted if AHAZE>0
    EQLWCZ = zeros(size(ZM));
    RRATZ  = zeros(size(ZM)); % Rain rate in [mm/hr] at altitude ZM
    % Only one of IHA1, ICLD1 or IVUL1 is allowed (?):
    % Aerosol model extinction and meteorological range control for 
    % altitude ZM. See MODTRAN documentation. We assume that only boundary 
    % and troposphere have aerosol model extinction:
    IHA1  = zeros(size(ZM));
    IHA1(ZM<=H) = IHAZE; % Boundary layer (0-H km). H=2 km by default in MODTRAN
    IHA1(ZM>H & ZM<=10) = 6; % Troposphere (H-10 km). MODTRAN default
    
    ICLD1 = zeros(size(ZM)); % Cloud extinction control for altitude ZM. See MODTRAN documentation
    % Stratospheric aerosol profile and extinction control for altitude ZM.
    % See MODTRAN documentation. We assume that only stratosphere and 
    % mesossphere have stratospheric aeorol extinction:
    IVUL1 = zeros(size(ZM));
    IVUL1(ZM>10 & ZM<=30) = IVULCN;
    % If AHAZE and EQLWCZ are zero, the default profile is loaded from IHA1,
    % ICLD1 and IVUL1
    ISEA1 = ISEASN*ones(size(ZM)); % Aerosol season control for the altitude ZM. See MODTRAN documentation
    ICHR  = zeros(size(ZM)); % Indicates boundary change between 2 or more adjacent used defined aerosol/cloud regions at altitude ZM
elseif (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD2==2
    % AHAZE (1 to 4) are the aerosol/cloud scaling factor (similar to 
    % visible extinction coefficient) at altitude ZM for each atmospheric
    % region (boundary layer, troposphere, stratosphere and mesosphere)
    AHAZE1 = zeros(size(ZM)); AHAZE2=AHAZE1; AHAZE3=AHAZE1; AHAZE4=AHAZE1;
    % Boundary layer (0-H km). H=2 km by default in MODTRAN
    % Here the model implemented is a decreasing exponential with a 
    % user-defined scale height (Z [km]) from 0 km to a user-defined top 
    % altitude (H [km]). The vertical distribution is scaled for a total 
    % AOT (integral of extiction coefficient) in the boundary layer of 0.20
    % at 550 nm.
    AHAZE1(ZM<=H) = exp(-ZM(ZM<=H)/Z);
    scale = 0.20/trapz(ZM(ZM<=H),AHAZE1(ZM<=H));
    AHAZE1(ZM<=H) = scale*AHAZE1(ZM<=H);
        
	RRATZ  = zeros(size(ZM)); % Rain rate in [mm/hr] at altitude ZM
    
    % Troposheric, stratospheric and mesospheric aerosol distribution are 
    % kept to the default MODTRAN values.
%     AHAZE2 = 0.1; % Troposphere (H-10 km)
%     AHAZE3 = 0.1; % Stratosphere (10-30 km)
%     AHAZE4 = 0.1; % Mesossphere (30-100 km)
end

%--------------------------------------------------------------------------
%--------------------- CARD 2D variables (optional) -----------------------
%--------------------------------------------------------------------------
if (IHAZE==7 || ICLD==11) && strcmp(ARUSS,'USS') && SSA==-1
    % IREGi specifies in which of the four altitude regions a user-defined
    % aerosol/cloud model is used. 0=use default values, 1=read extinction,
    % absorption and asymmetry parameter
    IREG1 = numel(aeraux.VARSPC); IREG2 = 0; IREG3 = 0; IREG4 = 0;
    % NOTE1: Only boundary layer aerosols (IREG1) are modified!!
elseif (IHAZE==7 || ICLD==11) && strcmp(ARUSS,'USS') && SSA>=0
    IREG1 = 26; % Default MODTRAN5 wavelength grid until 9.2 um (Read manual)
    IREG2 = 0; IREG3 = 0; IREG4 = 0;
end

% NOTE: CARD 2D1 and 2D2 are read sequentially once for each of the four
% aerosol refions for which IREGi=1
%--------------------------------------------------------------------------
%----------------- CARD 2D1 & 2D2 variables (optional) --------------------
%--------------------------------------------------------------------------
for N=1:4 % For each IREG(N)
    % CARD 2D1:
    if (IHAZE==7 || ICLD==11) && eval(['IREG',num2str(N)])>=1
        AWCCON{N} = 0; % Conversion factor in [km g/m^3] from extinction coef. ([km^-1]) to equivalent liquid water content [g/m^3]. Ony for microwave fog extinction!
        AERNAM{N} = ['Aerosol-Cloud region Name',num2str(N)]; % Name of an aerosol or cloud region
    elseif (IHAZE==7 || ICLD==11) && eval(['IREG',num2str(N)])==0
        % Values not used
        AWCCON{N} = 0; % Conversion factor in [km g/m^3] from extinction coef. ([km^-1]) to equivalent liquid water content [g/m^3]. Ony for microwave fog extinction!
        AERNAM{N} = ['Aerosol-Cloud region Name',num2str(N)]; % Name of an aerosol or cloud region
    end
    
    % CARD 2D2:
    if (IHAZE==7 || ICLD==11) && eval(['IREG',num2str(N)])>=1
        if SSA==-1 % User-defined or OPAC model
            % VARSPC = wavelenght grid (in [um]) at which the aerosol/cloud 
            % coefficients are read:
            VARSPC{N} = aeraux.VARSPC;

            % EXTC and ABSC are the aerosol/cloud extinction and absorption 
            % coefficients normalized so that EXTC is 1 km^-1 at 550 nm.
            % ASYM is the aerosol/cloud asymmetry parameter. If CARD 3A1 is
            % to be read in, IPH must be set =2 to insure that these factors
            % will not be overwritten
            if ~isempty(aeraux.RH) % Only defined for OPAC
                EXTC{N} = interp1(aeraux.RH,aeraux.EXTC',RH(N),'linear','extrap');        
                ABSC{N} = interp1(aeraux.RH,aeraux.ABSC',RH(N),'linear','extrap');
                
                ASYM{N} = interp1(aeraux.RH,aeraux.ASYM',RH(N),'linear','extrap');
            else % From user-defined aerosols (ATK): no RH dependent
                 EXTC{N} = aeraux.EXTC'; ABSC{N} = aeraux.ABSC';
                 ASYM{N} = aeraux.ASYM';
            end
        elseif SSA>=0 % User-input single scattering albedo (spectrally constant)
            VARSPC{N} = [0.2 0.3 0.3371 0.55 0.6943 1.06 1.536 2 2.25 2.5 ...
                     2.7 3 3.3923 3.75 4.5 5 5.5 6 6.2 6.5 7.2 7.9 8.2 ...
                     8.7 9 9.2]; % In [um]

            % EXTC and ABSC are the aerosol/cloud extinction and absorption 
            % coefficients normalized so that EXTC is 1 km^-1 at 550 nm:
            EXTC{N} = (0.55./VARSPC{N}).^ASTMX;        
            ABSC{N} = (1-SSA)*EXTC{N};
            % With the input Single Scattering Absorption option, the  
            % asymmetry parameter ASYM is neglected. Instead, values are
            % overwritten with the use of the Henyey-Greenstein option
            % (IPH=0)
            ASYM{N} = 0.8*ones(size(VARSPC{N}));
        end
    elseif (IHAZE==7 || ICLD==11) && eval(['IREG',num2str(N)])==0
        % Values not used
        VARSPC{N} = [0.2 0.3 0.3371 0.55 0.6943 1.06 1.536 2 2.25 2.5 ...
                     2.7 3 3.3923 3.75 4.5 5 5.5 6 6.2 6.5 7.2 7.9 8.2 ...
                     8.7 9 9.2 10 10.591 11 11.5 12.5 14.8 15 16.4 17.2 ...
                     18.5 21.3 25 30 40 50 60 80 100 150 200 300]; % In [um]
        EXTC{N} = zeros(size(VARSPC{N}));
        ABSC{N} = zeros(size(VARSPC{N}));
        ASYM{N} = zeros(size(VARSPC{N}));
    end
end


%--------------------------------------------------------------------------
%--------------------- CARD 2E1 variables (optional) ----------------------
%--------------------------------------------------------------------------
% User-defined cloud parameters
% Repeat NCRALT times (REVIEW implementation)
if (ICLD>=1 && ICLD<=10) && NCRALT>=3
    ZCLD   = 1; % Altitude above the ground level of layer boundary for user-defined cloud/rain profile [km]
    CLD    = 1; % Liquid water droplet density at altitude ZCLD, [g/m^3]
    CLDICE = 1; % Ice particle density at altitude ZCLD, [g/m^3]
    RR     = 1; % Rain rate at altitude ZCLD, [mm/hr]
end

%--------------------------------------------------------------------------
%--------------------- CARD 2E2 variables (optional) ----------------------
%--------------------------------------------------------------------------
% User-defined cloud parameters
% Repeat NCRSPC times (REVIEW implementation)
if (ICLD>=1 && ICLD<=10) && NCRSPC>=2
    WAVLEN = 0.2; % Wavelength, [um], in the spectral grid
    EXTC6  = 1; % Liquid water droplet extinction coef. [km^-1 m^3/g] at WAVLEN
    ABSC6  = 1; % If possitive: liquid water droplet absorption coef. [km^-1 m^3/g] at WAVLEN. If negative: Liquid water droplet scattering albedo - 1 at WAVLEN
    ASYM6  = 1; % Liquid water droplet Henyey-Greenstein scattering phase function asymmetry factor at WAVLEN
    EXTC7  = 1; % Ice particle extinction coef. [km^-1 m^3/g] at WAVLEN
    ABSC7  = 1; % If possitive: Ice particle absorption coef. [km^-1 m^3/g] at WAVLEN. If negative: Ice particle scattering albedo - 1 at WAVLEN
    ASYM7  = 1; % Ice particle Henyey-Greenstein scattering phase function asymmetry factor at WAVLEN
elseif (ICLD>=1 && ICLD<=10) && NCRSPC==1
    CFILE  = 'DATA/Macke.dat'; % Cloud spectral data full path file name
    CLDTYP = 'water_cloud_type'; % Must be included in the CFILE
    CIRTYP = 'ice_cloud_type'; % Must be included in the CFILE
end


%--------------------------------------------------------------------------
%-------------------- CARD 3 variables (obligatory) -----------------------
%--------------------------------------------------------------------------
% Line Of Sight (LOS) geometry
if IEMSCT == 0 || IEMSCT == 1 || IEMSCT == 2 || IEMSCT == 4
    % Observer height [km]
    if isempty(flag) || strcmp(flag,'toa')
        H1 = H1;
    elseif strcmp(flag,'boa')
        H1 = GNDALT;
    end
    H2      = 0; % Final altitude (ITYPE=2) or Tangent height (ITYPE=3) in [km]
    ANGLE   = 180-ANGLE; % Initial zenith angle [deg] as measured from H1 (180=nadir view)
    RANGE   = 0; % Path length [km]
    BETA    = 0; % Earth center angle [deg] subtended by H1 and H2
    RO      = 0; % Earth radius at latitude of observation. If RO=0 then the value is set according to MODEL (see tabe 11 MODTRAN documentation)
    LENN    = 0; % Switch between long path extending through the tangent height (LENN=1) or short (LENN=0)
    PHI     = 0; % Zenith angle at H2 towards H1
    CKRANGE = 0; % Full slant range path boundary intersections up to the 1st exceeding CKRANGE
    % Not necessary to specify all. Depending on ITYPE (see MODTRAN documentation)
elseif IEMSCT == 3
    H1     = 0; % Observer altitude [km]
    H2     = 100; % Tangent heigh of path to sun or moon [km] 
    ANGLE  = PARM2; % Apparent solar or lunar zenith angle at H1 [deg]
    IDAY   = IDAY; % Day of the year, used to correct for variation in the Earth-Sun distance
    RO     = 0; % Radius of the Earth (default according to MODEL)
    ISOURC = 0; % Extra-terrestrial light source (0=Sun ; 1=Moon)
    ANGLEM = 0; % Phase angle of moon in [deg] (0=full moon, 90=half moon, 180=no moon). Only if ISOURC=1
   % Not necessary to specify all. Depending on ITYPE (see MODTRAN documentation)
end

%--------------------------------------------------------------------------
%--------------------- CARD 3A1 variables (optional) ----------------------
%--------------------------------------------------------------------------
% Solar/Lunar scattering geometry
if IEMSCT==2 || IEMSCT==4
    IPARM = 12; % Method for specifying the solar/lunar geometry (see MODTRAN documentation)
    % Phase Function option:
    if G==10
        IPH = 2;           % Mie-generated internal database (=2)
    elseif G>=-1&&G<=1
        IPH = 0;           % Spectrally independent Henyey-Greenstein (=0)
    end
    if IHAZE==7 && SSA==-1
        if ~isempty(aeraux.F)
            IPH = 1;          % User-supplied (OPAC) aerosol phase function (=1)
        end
    end
    IDAY = 93; % Day of the year (1-365) to specify Earth-Sun distance. If IPARM=1 it also specifies the Sun's location in the sky
    ISOURC = 0; % Extra-terrestrial light source (0=Sun ; 1=Moon)
end

%--------------------------------------------------------------------------
%--------------------- CARD 3A2 variables (optional) ----------------------
%--------------------------------------------------------------------------
if IEMSCT==2 || IEMSCT==4
    % Definitions of PARM1-4 are determined according to the value of IPARM
    % (see MODTRAN documentation)
    PARM1  = PARM1;
    PARM2  = PARM2;
    PARM3  = 0;
    PARM4  = 0;
    TIME   = 0; % Greenwich time in decimal hours. Used with IPARM=1 or 11
    PSIPO  = 0; %Path azimuth ([deg] east of north, i.e. North = 0deg, East=90deg)
    ANGLEM = 0; % Phase angle of moon in [deg] (0=full moon, 90=half moon, 180=no moon). Only if ISOURC=1
    G      = G; % Asymmetry factor for use with Henyey-Greenstein phase function (only if IPH=0). +1 for complete forward scattering, 0 isotropic and -1 for total backscattering
end

%--------------------------------------------------------------------------
%--------------------- CARD 3B1 variables (optional) ----------------------
%--------------------------------------------------------------------------
% User-defined scattering phase functions
if (IEMSCT==2 || IEMSCT==4) && IPH==1
    NANGLS = numel(aeraux.ANGF); % Number of angles for the user-defined phase functions
    NWLF   = IREG2; % Phase function no wavelength dependence (=0) or specified on a wavelength grid with NWLF>0 points
end

%--------------------------------------------------------------------------
%--------------------- CARD 3B2 variables (optional) ----------------------
%--------------------------------------------------------------------------
% User-defined wavelength-independent scattering phase functions
if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF==0
    ANGF = aeraux.ANGF;% Scattering angle in decimal degrees
    for N=1:4 % For each IREG(N)
        % NOTE: F must be provided for ALL aerosol region!!
        F = zeros(4,NANGLS);
        
        % REVISAR LAS UNIDADES DE F EN OPAC VS MODTRAN!!!!
        
        % F(N,I) is the unit normalized user-defined aerosol scattering
        % phase function in [sr^-1] evaluated at the aerosol region N (1 to
        % 4) and the scattering angle ANGF(I):
        F(N,:) = interp1(aeraux.RH,squeeze(mean(aeraux.F,2))',RH(N));
        % The altitude regions of these 4 layers may be overridden by the
        % parameters IHA1, ICLD1 or IVUL1 on CARD 2C3
    end
end

%--------------------------------------------------------------------------
%--------------------- CARD 3C1 variables (optional) ----------------------
%--------------------------------------------------------------------------
% User-defined wavelength-dependent scattering phase functions
% Repeated NANGLS time (REVIEW implementation!!)
if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
    ANGF = aeraux.ANGF;% Scattering angle in decimal degrees
end

%--------------------------------------------------------------------------
%--------------------- CARD 3C2 variables (optional) ----------------------
%--------------------------------------------------------------------------
% User-defined wavelength-dependent scattering phase functions
% Repeated NWLF times (REVIEW implementation!!)
if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
    WLF = VARSPC{2}; % Read wavelengths [um]
    % NOTE: Same as VARSPC in IREG2!!
end

%--------------------------------------------------------------------------
%----------------- CARD 3C3 to 3C6 variables (optional) -------------------
%--------------------------------------------------------------------------
% User-defined wavelength-dependent scattering phase functions
% Repeated NWLF times (REVIEW implementation!!)
if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
    F = zeros(4,NANGLS,NWLF);
    for N=1:4 % For each IREG(N)
        % NOTE: F must be provided for ALL aerosol region!!
        
        % REVISAR LAS UNIDADES DE F EN OPAC VS MODTRAN!!!!
        
        % F(N,I,J) is the unit normalized user-defined aerosol  scattering
        % phase function in [sr^-1] evaluated at the aerosol region N (1 to
        % 4), scattering angle ANGF(I) and wavelength WLF(J):
        F(N,:,:) = interp1(aeraux.RH,permute(aeraux.F,[3 1 2]),RH(N));    
    end
end

%--------------------------------------------------------------------------
%-------------------- CARD 4 variables (obligatory) -----------------------
%--------------------------------------------------------------------------
% Spectral range and resolution
% V1 Initial frequency in [cm^-1] or wavelengths ([um] or [nm]) defined via
% FLAGS(1:1)
% BMNAME
v2 = fliplr(str2num(SpeConf.MaxLambda.values)); % In [nm] from ALG toolbox
V1 = 1e7./v2; % Converted into [cm^-1]
% V2 Final frequency in [cm^-1] or wavelengths ([um] or [nm]) defined via
% FLAGS(1:1)
v1 = fliplr(str2num(SpeConf.MinLambda.values)); % In [nm] from ALG toolbox
V2 = 1e7./v1; % Converted into [cm^-1]
if numel(V2)~=1
    % Update wavelengths if spectral ranges are overlapped
    for i=2:numel(V1)
        auxBMNAME = strrep(BMNAME,'p1','.1');
        if V1(i)==V2(i-1) && str2num(auxBMNAME{i}(1:2))>=str2num(auxBMNAME{i-1}(1:2)) 
            V1(i) = V1(i) + str2num(auxBMNAME{i}(1:2));
        elseif V1(i)==V2(i-1) && str2num(auxBMNAME{i}(1:2))<str2num(auxBMNAME{i-1}(1:2)) 
            V2(i-1) = V2(i-1) - str2num(auxBMNAME{i-1}(1:2));
        end
    end
end    
% Back to [nm]: auxV2 = 1e7./V1; V1 = 1e7./V2; V2 = auxV2;

% DV Frequency (or wavelength) increment used for spectral outputs. DV
% applies to all outputs when using a default slit function, i.e.
% FLAGS(1:4)=' ', othwerwise DV is applied to tape7.scn and pltout.scn, and
% the frequency increment for the other files (tape6, tape7, tape8 and pltout)
% is set to the calculation bin size (0.1 cm^-1, 1 cm^-1 or 15 cm^-1). The
% recommended value is DV=FWHM/2
DV = 0.1;%0.5;
% FWHM of the slit function. FLAGS(1:1) specifies the units. Maximum of 50
% times the calculation bin size. A minimum of double the bin size will
% insure proper sampling. No convolution is performed if FWHM = the bin
% size and default slit function is selected
FWHM = 0.1;%0.3
% YFLAG selects the output value (Transmittances=T or Radiances=R) in the
% pltout (rootname.plt) and pltout.scn (rootname.psc) files. If YFLAG=R and
% IEMSCT=3 on CARD 1 the output is irradiance.
YFLAG = 'R';
% XFLAG controls the units of the output files pltout and pltout.scn.
% Spectral frequency in wavenumbers (=W), spectral wavelength in microns
% (=M) or spectral wavelength in nanometers (=N).
XFLAG = 'N';
% DLIMIT is a character string used i pltout and pltout.csn to separate
% output from repeat (sequential) MODTRAN runs
DLIMIT = ' ';
% FLAGS is a string of 7 characters. If FLAGS(1:2) (the first 2 characters)
% are both blank, the default slit function is used and FLAGS(3:7) are
% ignored.
% FLAG(1:1)
%   1) ' ' = default spectral units in wavenumbers
%   2) W = spectral units in wavenumbers
%   3) M = spectral units in microns
%   4) N = spectral units in nanometers
% FLAGS(2:2)
%   1) ' ' = Default slit function (triangular)
%   2) T = Triangular slit function
%   3) R = Rectangular slit function
%   4) G = Gaussian slit function
%   5) S = Sinc slit function
%   6) C = Sinc^2 slit function
%   7) H = Hamming slit function
%   2) U = User-supplied function
% FLAGS(3:3)
%   1) A = FWHM is absolute
%   2) R = FWHM is percent, i.e., FWHM=100 dv/v=100 dl/l
% FLAGS(4:4)
%   1) ' ' = Degrade only total radiance and transmittance
%   2) A = Degrade all radiance and transmittance components
% FLAGS(5:5)
%   1) S = Save non-degraded results for degrading later
%   2) ' ' = Do not save current results
% FLAGS(6:6)
%   1) R = Use saved results for degrading with the current slit function
%   2) ' ' = Do not use saved results
% FLAGS(7:7)
%   1) T = Write a specflux (rootname.flx) file. Use no more than 80

%   characters per line in spectral flux table
%   2) F = Write specflux file. For each spectral point, all flux values
%   are on a single line
%   3) ' ' = Do not write a spectral flux table
FLAGS = '       '; % FLAGS = 'NRAA   '; 
% MLFLX is the number of atmospheric levels for which spectral fluxed
% (FLAGS(7:7) ='T' or 'F') are output starting from the ground. TOA value
% is also output. If MLFLX=0 all spectral flux values are output at all
% atmospheric levels
MLFLX = 0;
% VRFRAC=' ' (index of refraction profile for spherical refraction
% performed at central spectral frequency value of input bandpass. VRFRAC>0
% (Spectral frequency [cm^-1] at which index of refraction profile is
% calculated for spherical refraction. VRFRAC<0 (No spherical refraction)
VRFRAC = 0;

%--------------------------------------------------------------------------
%---------------------- CARD 4A variables (optional) ----------------------
%--------------------------------------------------------------------------
% Ground surface characterization: Modeling adjacency effects by providing
% an option to decouple reflectance properties of the image pixel (H2)

% surface and the ground surface used in multiple scattering models.
if strcmp(SURREF,'B') || strcmp(SURREF,'L')
    NSURF  =  1; % Use reflectance properties of the image pixel for the 'area averaged' ground surface (=1) or define reflectance properties of the area averaged ground surface (=2)
    AATEMP = 0; % >0 Area-averaged ground surface temperature [K] if NSURF=2 (not used if NSURF=1). <=0 Set the temperature to TPTEMP for LOS interescing th Earth or atmospheric temperature profile
    DH2O   = 0; % Liquid water thickness in [mm]
    MLTRFL = 'F'; % Embedded surface moisture attenuation model (F), Surface water layer model (T)
end

%--------------------------------------------------------------------------
%--------------------- CARD 4B1 variables (optional) ----------------------
%--------------------------------------------------------------------------
if strcmp(SURREF,'B')
    % Name or number for the associated BRDF parametrization (see MODTRAN
    % documentation)
    CBRDF = 'Hapke'; 
end

%--------------------------------------------------------------------------
%--------------------- CARD 4B2 variables (optional) ----------------------
%--------------------------------------------------------------------------
if strcmp(SURREF,'B')
    NWVSRF = 3; % Number of BRDF spectral grid poins. If NWVSRF=1 the BRDF is spectrally independent-
    SURFZN = 0; % Zenith angle [deg] of the surface normal. Currently only a value of 0 is supported
    SURFAZ = 0; % True azimut angle of the image pixel surface normal (0 for North, 90 for East, 180 for South and 270 for East (?). This value is currently not used
end

%--------------------------------------------------------------------------
%--------------------- CARD 4B3 variables (optional) ----------------------
%--------------------------------------------------------------------------
% CARD 4B3 is repeated NWVSRF times. At each wavelength (in increasing order)
% we should set the parameters of the BRDF model
if strcmp(SURREF,'B')
    WVSURF = 0.2; % BRDF spectral wavelength in [um]
    % Most of the models take 4 parameters. REVIEW to use the other models
    % (see MODTRAN documentation)
    PARAMS1 = 0; % Value of the parameter P1
    PARAMS2 = 0; % Value of the parameter P2
    PARAMS3 = 0; % Value of the parameter P3
    PARAMS4 = 0; % Value of the parameter P4
end

%--------------------------------------------------------------------------
%--------------------- CARD 4L1 variables (optional) ----------------------
%--------------------------------------------------------------------------
if strcmp(SURREF,'L')
    % Name of the spectral albedo data file
    SALBFL = 'DATA/spec_alb.dat'; 
end

%--------------------------------------------------------------------------
%--------------------- CARD 4L2 variables (optional) ----------------------
%--------------------------------------------------------------------------
if strcmp(SURREF,'L')
    % Number or name of spectral albedo curvein the SALBFL file
    CSALB = '86'; 
end


%--------------------------------------------------------------------------
%---------------------- CARD 5 variables (obligatory) ---------------------
%--------------------------------------------------------------------------
% Repeat run option
% IRPT Mode of execution (repeat or not).
%  0  = Stop program
%  -1 = Read full set of new data cards
%  -3 = Read new LOS (and solar/lunar geometry) (CARD 3, 3A...) and surface
%  (CARD 4A...) inputs
%  -4 = Read new spectral and surface (CARD 4...) inputs (=4)
Nbands = numel(str2num(SpeConf.MinLambda.values)); % Number of spectral bands
if Nbands==1
    IRPT = 0;
else
    IRPT = [-1*ones(1,Nbands-1),0];
end



%% Write variables to to generate tape5 file:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%                    WRITE VARIABLES ON TAPE5 FILE                    %%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Open file with write permission
fid = fopen(file,'w');

%%%%%%%%%%%%%% Repeat Nbands times
for i=1:Nbands

    %----------------- CARD 1, CARD 1A, CARD 1A1, CARD 1A2, CARD 1A3,
    %----------------- CARD 1A4, CARD 1A5, CARD 1A6, CARD 1A7, CARD 1B

    % Write CARD 1 (obligatory):
    if ischar(SURREF)==1
        fprintf(fid,'%c%c%c%c%1i%5i%5i%5i%5i%5i%5i%5i%5i%5i%5i%5i %4i%8.3f%7s\n',...
                MODTRN, SPEED, BINARY, LYMOLC, MODEL, ITYPE, IEMSCT, IMULT, M1,...
                M2, M3, M4, M5, M6, MDEF, I_RD2C, NOPRNT, TPTEMP, SURREF);
    elseif strcmp(class(SURREF),'double')==1 % For SURREF>=0
        fprintf(fid,'%c%c%c%c%1i%5i%5i%5i%5i%5i%5i%5i%5i%5i%5i%5i %4i%8.3f%7.5f\n',...
                MODTRN, SPEED, BINARY, LYMOLC, MODEL, ITYPE, IEMSCT, IMULT, M1,...
                M2, M3, M4, M5, M6, MDEF, I_RD2C, NOPRNT, TPTEMP, SURREF);
    end

    % Write CARD 1A (obligatory):
    auxC_PROF = num2str(C_PROF);
    if SFWHM==0
        fprintf(fid,'%c%c%c%3i%4i%10.3f%-2s%8.6f%-2s%8.6f%c%c %c %c %c %c%10.3f%c%9.3f%10.3f%10.3f%10.3f%10i\n',DIS,...
               DISAZM, DISALB, NSTR, SFWHM, CO2MX, H2OSTRunit, H2OSTRvalue, O3STRunit, O3STRvalue, auxC_PROF,...
               LSUNFL, LBMNAM, LFLTNM, H2OAER, CDTDIR, SOLCON, CDASTM, ASTMC, ASTMX, ASTMO, AERRH, NSSALB);
    else
        auxSFWHM = sprintf('%4.2f',SFWHM);
        auxSFWHM = auxSFWHM(:,1:4);
        fprintf(fid,'%c%c%c%3i%4s%10.3f%-2s%8.6f%-2s%8.6f%c%c %c %c %c %c%10.3f%c%9.3f%10.3f%10.3f%10.3f%10i\n',DIS,...
               DISAZM, DISALB, NSTR, auxSFWHM, CO2MX, H2OSTRunit, H2OSTRvalue, O3STRunit, O3STRvalue, auxC_PROF,...
               LSUNFL, LBMNAM, LFLTNM, H2OAER, CDTDIR, SOLCON, CDASTM, ASTMC, ASTMX, ASTMO, AERRH, NSSALB);
    end

    % Write CARD 1A1 (optional):
    if LSUNFL=='T'
        fprintf(fid,'%-256s\n',USRSUN);
    end

    % Write CARD 1A2 (optional):
    if LBMNAM=='T' || LBMNAM=='4' || LBMNAM=='2'
        if iscell(BMNAME)
            fprintf(fid,'%-256s\n',BMNAME{i});
        else
            fprintf(fid,'%-256s\n',BMNAME);
        end
    end

    % Write CARD 1A3 (optional):
    if LFLTNM=='T'
        fprintf(fid,'%-256s\n',FILTNM);
    end

    % Write CARD 1A4 (optional):
    if CDTDIR=='T'
        fprintf(fid,'%-256s\n',DATDIR);
    end

    %Write CARD 1A5 (optional):
    if C_PROF==1 || C_PROF==3 || C_PROF==5 || C_PROF==7
        fprintf(fid,'%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f\n',S_UMIX1, S_UMIX2, S_UMIX3,...
               S_UMIX4, S_UMIX5, S_UMIX6, S_UMIX7, S_UMIX8, S_UMIX9, S_UMIX10);
    end

    % Write CARD 1A6 (optional):
    if C_PROF==2 || C_PROF==3 || C_PROF==6 || C_PROF==7
        fprintf(fid,'%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f\n',S_XSEC1, S_XSEC2, S_XSEC3,...
               S_XSEC4, S_XSEC5, S_XSEC6, S_XSEC7, S_XSEC8, S_XSEC9, S_XSEC10, S_XSEC11, S_XSEC12, S_XSEC13);
    end

    % Write CARD 1A7 (optional):
    if C_PROF==4 || C_PROF==5 || C_PROF==6 || C_PROF==7
        fprintf(fid,'%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f%5.3f\n',...
               S_TRAC1, S_TRAC2, S_TRAC3, S_TRAC4, S_TRAC5, S_TRAC6, S_TRAC7, S_TRAC8, S_TRAC9,...
               S_TRAC10, S_TRAC11, S_TRAC12, S_TRAC13, S_TRAC14,S_TRAC15, S_TRAC16);
    end

    % Write CARD 1B (optional):
    if NSSALB > 0
        fprintf(fid,'%10.3f\n',AWAVLN, ASSALB); %Repeated NSSALB times (REVIEW!!!)
       % MIRAR FUNCTION repmat!!!
    elseif NSSALB < 0 && ASTMX > 0
        fprintf(fid,'%10.2f%10.2f\n',ACOALB, RHASYM);
    end


    %----------------- CARD 2, CARD 2A+, CARD 2A, CARD 2B, CARD 2C, CARD 2CY,
    %----------------- CARD 2C1, CARD 2C2, CARD 2C2X, CARD 2C2Y, CARD 2C3,

    %----------------- CARD 2D, CARD 2D1, CARD 2D2, CARD 2E1 and CARD 2E2

    % Write CARD 2 (obligatory):
    fprintf(fid,'%2s%3i%c%4i%3s%2i%5i%5i%5i%10.3f%10.3f%10.3f%10.3f%10.3f\n',APLUS, IHAZE, CNOVAM, ISEASN,...
            ARUSS, IVULCN, ICSTL, ICLD, IVSA, VIS, WSS, WHH, RAINRT, GNDALT);

    % Write CARD 2A+ (optional):
    if strcmp(APLUS,'A+')
        fprintf(fid,' %9.3f %9.3f %9.3f                     %9.3f %9.3f %9.3f\n',...
               ZAER11, ZAER12, SCALE1, ZAER21, ZAER22, SCALE2);
        fprintf(fid,' %9.3f %9.3f %9.3f                     %9.3f %9.3f %9.3f\n',...
               ZAER31, ZAER32, SCALE3, ZAER41, ZAER42, SCALE4);
    end

    % Write CARD 2A (optional):
    if ICLD==18 || ICLD==19 % For cirrus cloud models
        fprintf(fid,'%8.3f%8.3f%8.3f\n',CTHIK, CALT, CEXT);
    elseif ICLD>=1 && ICLD<=10 %Water/Ice cloud models
        fprintf(fid,'%8.3f%8.3f%8.3f%4i%4i%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f\n',CTHIK, CALT, CEXT,...
               NCRALT, NCRSPC, CWAVLN, CCOLWD, CCOLIP, CHUMID, ASYMWD, ASYMIP);
    end

    % Write CARD 2B (optional):
    if IVSA==1
        fprintf(fid,'%10.3f%10.3f%10.3f\n',ZCVSA, ZTVSA, ZINVSA);
    end
    
    % Write CARD 2C (optional):
    if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1
        fprintf(fid,'%5i%5i%5i%20s%10.3f%5i%10.3f%10.3f\n',ML, IRD1, IRD2, HMODEL,...
               REE, NMOLYC, E_MASS, AIRMWT);
    
        % Write CARD 2CY (optional)
        if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && NMOLYC>0 && MDEF==2
            for I=1:NMOLYC
                fprintf(fid,'%10s\n',YNAME{I}); % Repeat NMOLYC times (REVIEW!!!)
            end
        end
        
        for I=1:ML % Repeated ML atmospheric layers (CARD 2C)
            % Write CARD 2C1 (optional)
            if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IVSA==0
                format2C1 = '%10.3f%10.3f%10.3f%10.3f%10.3f% 10.3E%c%c%c%c%c%c%c%c%c%c%c%c%c%c %c%c\n';
                if P(I)<1
                    format2C1 = '%10.3f% 10.3E%10.3f%10.3f%10.3f% 10.3E%c%c%c%c%c%c%c%c%c%c%c%c%c%c %c%c\n';
                end

                fprintf(fid,'%s',...
                    sprintf(format2C1,...
                    ZM(I), P(I), T(I), WMOL1(I), WMOL2(I), WMOL3(I),...
                    JCHAR{1}{I},  JCHAR{2}{I},  JCHAR{3}{I},  JCHAR{4}{I},...
                    JCHAR{5}{I},  JCHAR{6}{I},  JCHAR{7}{I},  JCHAR{8}{I},...
                    JCHAR{9}{I},  JCHAR{10}{I}, JCHAR{11}{I}, JCHAR{12}{I},...
                    JCHAR{13}{I}, JCHAR{14}{I}, JCHARX{I}, JCHARY{I}));
%                 fprintf(fid,'%s',...
%                         sprintf('%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f%c%c%c%c%c%c%c%c%c%c%c%c%c%c %c%c\n',...
%                         ZM(I), P(I), T(I), WMOL1(I), WMOL2(I), WMOL3(I),...
%                         JCHAR{1}{I},  JCHAR{2}{I},  JCHAR{3}{I},  JCHAR{4}{I},...
%                         JCHAR{5}{I},  JCHAR{6}{I},  JCHAR{7}{I},  JCHAR{8}{I},...
%                         JCHAR{9}{I},  JCHAR{10}{I}, JCHAR{11}{I}, JCHAR{12}{I},...
%                         JCHAR{13}{I}, JCHAR{14}{I}, JCHARX{I}, JCHARY{I}));
            end
            
            % Write CARD 2C2 (optional):
            if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD1==1
                fprintf(fid,'%s',...
                    sprintf('% 10.3E% 10.3E% 10.3E%10.3f% 10.3E% 10.3E% 10.3E% 10.3E\n% 10.3E\n',...
                    WMOL4(I), WMOL5(I), WMOL6(I), WMOL7(I), WMOL8(I), WMOL9(I), WMOL10(I), WMOL11(I), WMOL12(I)));
            end
            
            % Write CARD 2C2X (optional):
            if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD1==1 && MDEF==2
                % REVIEW FORMAT!!!
                fprintf(fid,'%s',strrep(sprintf('% 10.3E% 10.3E% 10.3E% 10.3E% 10.3E% 10.3E% 10.3E% 10.3E\n% 10.3E% 10.3E% 10.3E% 10.3E% 10.3E\n',...
                    WMOLX1, WMOLX2, WMOLX3, WMOLX4, WMOLX5, WMOLX6, WMOLX7,...
                    WMOLX8, WMOLX9, WMOLX10, WMOLX11, WMOLX12, WMOLX13),'E+0','E+'));
            end
            
            % Write CARD 2C2Y (optional):
            if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD1==1 && NMOLYC>0
                % REVIEW FORMAT!!! 
                %             fprintf(fid,'',);
            end
            % Write CARD 2C3 (optional)
            if (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD2==1
                fprintf(fid,'          %10.3f%10.3f%10.3f%5i%5i%5i%5i%5i\n',...
                    AHAZE(I), EQLWCZ(I), RRATZ(I), IHA1(I), ICLD1(I),...
                    IVUL1(I), ISEA1(I), ICHR(I));
            elseif (MODEL==0 || MODEL==7 || MODEL==8) && I_RD2C==1 && IRD2==2
                fprintf(fid,'          %10.3f          %10.3f%10.3f%10.3f%10.3f\n',...
                    AHAZE1(I), RRATZ(I), AHAZE2(I), AHAZE3(I), AHAZE4(I));
            end
        end
    end

    % Write CARD 2D (optional)
    if IHAZE==7 || ICLD==11
        fprintf(fid,'%5i%5i%5i%5i\n',IREG1, IREG2, IREG3, IREG4);
    end

    %Note: CARDS 2D1 and 2D2 are repeated on each region where IREG(N)=1 (CARD
    %2D)
    for N=1:4 % For each IREG(N)
        if (IHAZE==7 || ICLD==11) && eval(['IREG',num2str(N)])>=1
            % Write CARD 2D1 (optional):
%             fprintf(fid,'%s\n',strrep(sprintf('% 10.3E%-72s',AWCCON{N}, AERNAM{N}),'E+0','e+'));
            fprintf(fid,'%s\n',sprintf('% 10.3E%-72s',AWCCON{N}, AERNAM{N}));
            % Write CARD 2D2 (optional):
            for I=1:3:eval(['IREG',num2str(N)])
                if I+2<=eval(['IREG',num2str(N)])
                    % Print 3 sets of 4 numbers (VARSPC,EXTC,ABCS,ASYM):
                    fprintf(fid,'%6.2f%7.5f%7.5f%6.4f%6.2f%7.5f%7.5f%6.4f%6.2f%7.5f%7.5f%6.4f\n',...
                        VARSPC{N}(I),EXTC{N}(I),ABSC{N}(I),ASYM{N}(I),...
                        VARSPC{N}(I+1),EXTC{N}(I+1),ABSC{N}(I),ASYM{N}(I+1),...
                        VARSPC{N}(I+2),EXTC{N}(I+2),ABSC{N}(I+2),ASYM{N}(I+2));
                elseif eval(['IREG',num2str(N)])-I==1
                    % Only 2 sets of 4 numbers (VARSPC,EXTC,ABCS,ASYM):
                    fprintf(fid,'%6.2f%7.5f%7.5f%6.4f%6.2f%7.5f%7.5f%6.4f\n',...
                        VARSPC{N}(I),EXTC{N}(I),ABSC{N}(I),ASYM{N}(I),...
                        VARSPC{N}(I+1),EXTC{N}(I+1),ABSC{N}(I),ASYM{N}(I+1));
                elseif eval(['IREG',num2str(N)])-I==0
                    % Only 1 sets of 4 numbers (VARSPC,EXTC,ABCS,ASYM):
                    fprintf(fid,'%6.2f%7.5f%7.5f%6.4f\n',...
                        VARSPC{N}(I),EXTC{N}(I),ABSC{N}(I),ASYM{N}(I));
                end
            end
        end
    end

    % Write CARD 2E1 (optional):
    if (ICLD>=1 && ICLD<=10) && NCRALT>=3
        %Note: Repeat CARD 2E1 NCRALT (on CARD 2A) times.
        fprintf(fid,'%10.5f%10.5f%10.5f%10.5f\n',ZCLD, CLD, CLDICE, RR);
    end

    % Write CARD 2E2 (optional):
    %Note: Repeat CARD 2E2 NCRSPC (on CARD 2A) times (REVIEW implementation!!!)
    if (ICLD>=1 && ICLD<=10) && NCRSPC>=2
        fprintf(fid,'%10.5f%10.5f%10.5f%10.5f%10.5f%10.5f%10.5f\n',WAVELEN, EXTC6,...
               ABSC6, ASYM6, EXTC7, ABSC7, ASYM7);
    elseif (ICLD>=1 && ICLD<=10) && NCRSPC==1
        fprintf(fid,'%-256s\n%-256s\n%-256s\n',CFILE, CLDTYP, CIRTYP);
    end


    %----------------- CARD 3, CARD 3A1, CARD 3A2, CARD 3B1, CARD 3B2 and
    %----------------- CARD 3C1-3C6

    % Write CARD 3 (obligatory):
    if IEMSCT == 0 || IEMSCT == 1 || IEMSCT == 2 || IEMSCT == 4
        fprintf(fid,'%10.6f%10.6f%10.6f%10.5f%10.6f%10.6f%5i     %10.6f%10.6f\n',...
               H1, H2, ANGLE, RANGE, BETA, RO, LENN, PHI, CKRANGE);
    elseif IEMSCT == 3
        fprintf(fid,'%10.3f%10.3f%10.3f%5i     %10.3f%5i%10.3f\n',H1, H2, ANGLE,...
               IDAY, RO, ISOURC, ANGLEM);
    end

    % Write CARD 3A1 (optional):
    if IEMSCT==2 || IEMSCT==4
        fprintf(fid,'%5i%5i%5i%5i\n',IPARM, IPH, IDAY, ISOURC);
    end

    % Write CARD 3A2 (optional):
    if IEMSCT==2 || IEMSCT==4
        fprintf(fid,'%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f\n',PARM1,...
               PARM2, PARM3, PARM4, TIME, PSIPO, ANGLEM, G);
    end

    % Write CARD 3B1 (optional):
    if (IEMSCT==2 || IEMSCT==4) && IPH==1
        fprintf(fid,'%5i%5i\n',NANGLS, NWLF);
    end

    % Write CARD 3B2 (optional):
    if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF==0
        for I=1:NANGLS % Card repeated NANGLS times
            fprintf(fid,'%s',strrep(sprintf('% 10.3E% 10.3E% 10.3E% 10.3E% 10.3E\n',...
                    ANGF(I), F(1,I), F(2,I), F(3,I), F(4,I)),'E+0','E+'));
        end
    end

    % Write CARD 3C1 to CARD 3C6 (REVIEW!!!) CONTINUAR AQUI
    if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
        % Write CARD 3C1:
        for I=1:NANGLS % Card repeated NANGLS times
            fprintf(fid,strrep(sprintf('% 9.3E% 9.3E% 9.3E% 9.3E% 9.3E% 9.3E% 9.3E% 9.3E\n',...
                ANGF(I), F(1,I), F(2,I), F(3,I), F(4,I)),'E+0','E+'))
        end
        % Write CARD 3C2:
        % Write CARD 3C3 to 2C6:
    end
%%
%--------------------- CARD 3C1 variables (optional) ----------------------
% User-defined wavelength-dependent scattering phase functions
% Repeated NANGLS time (REVIEW implementation!!)
if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
    ANGF = aeraux.ANGF;% Scattering angle in decimal degrees
end

%--------------------------------------------------------------------------
%--------------------- CARD 3C2 variables (optional) ----------------------
%--------------------------------------------------------------------------
% User-defined wavelength-dependent scattering phase functions
% Repeated NWLF times (REVIEW implementation!!)
if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
    WLF = VARSPC{2}; % Read wavelengths [um]
    % NOTE: Same as VARSPC in IREG2!!
end

%--------------------------------------------------------------------------
%----------------- CARD 3C3 to 3C6 variables (optional) -------------------
%--------------------------------------------------------------------------
% User-defined wavelength-dependent scattering phase functions
% Repeated NWLF times (REVIEW implementation!!)
if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
    F = zeros(4,NANGLS,NWLF);
    for N=1:4 % For each IREG(N)
        % NOTE: F must be provided for ALL aerosol region!!
        
        % REVISAR LA INTERPOLACION EN RH PARA EL CWV CORRESPONDIENTE!!!!
        % REVISAR LAS UNIDADES DE F EN OPAC VS MODTRAN!!!!
        
        % F(N,I,J) is the unit normalized user-defined aerosol  scattering
        % phase function in [sr^-1] evaluated at the aerosol region N (1 to
        % 4), scattering angle ANGF(I) and wavelength WLF(J):
        F(N,:,:) = interp1(aeraux.RH,permute(aeraux.F,[3 1 2]),50);    
    end
end
%%
    if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
        % REVIEW format!!!
    end

    if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
        % REVIEW format!!!
    end

    if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
        % REVIEW format!!!
    end

    if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
        % REVIEW format!!!
    end

    if (IEMSCT==2 || IEMSCT==4) && IPH==1 && NWLF>0
        % REVIEW format!!!
    end


    %----------------- CARD 4, CARD 4A, CARD 4B1, CARD 4B2, CARD 4B3, CARD 4L1
    %----------------- and CARD 4L2

    % Write CARD 4 (obligatory):
    auxV1 = sprintf('%10.5f',V1(i)); auxV2 = sprintf('%10.5f',V2(i));
    auxDV = sprintf('%10.5f',DV); auxFWHM = sprintf('%10.5f',FWHM);
    auxV1 = auxV1(:,1:10); auxV2 = auxV2(:,1:10); auxDV = auxDV(:,1:10);
    auxFWHM = auxFWHM(:,1:10);
    fprintf(fid,'%10s%10s%10s%10s%c%c%8s%7s%3i%10.3f\n',auxV1, auxV2, auxDV, auxFWHM,...
            YFLAG, XFLAG, DLIMIT, FLAGS, MLFLX, VRFRAC);

    % Write CARD 4A (optional):
    if strcmp(SURREF,'B') || strcmp(SURREF,'L')
        fprintf(fid,'%1i%9.0f%9.0f%c\n',NSURF, AATEMP, DH2O, MLTRFL);
    end

    % Write CARD 4B1 (optional):
    if strcmp(SURREF,'B')
        fprintf(fid,'%-80c\n',CBRDF);
    end

    % Write CARD 4B2 (optional):
    if strcmp(SURREF,'B')
        fprintf(fid,'%i%4.0f%4.0f\n',NWVSRF, SURFZN, SURFAZ);
    end

    % Write CARD 4B3 (optional):
    if strcmp(SURREF,'B')
        fprintf(fid,'%8.4f %8.4f %8.4f %8.4f %8.4f\n',WVSURF, PARAMS1, PARAMS2, PARAMS3, PARAMS4);
    end
    %Note: Repeat NWVSRF (on CARD 4B2) times.

    % Write CARD 4L1 (optional, if SURREF='LAMBER'):
    if strcmp(SURREF,'L')
        fprintf(fid,'%-256s\n',SALBFL);
    end

    % Write CARD 4L2 (optional, if SURREF='LAMBER'):
    if strcmp(SURREF,'L')
        fprintf(fid,'%-80s\n',CSALB);
    end

    %----------------- CARD 5
    % Write CARD 5 (obligatory):
    if IRPT(i)==0
        fprintf(fid,'%5i',IRPT(i));
    else
        fprintf(fid,'%5i\n',IRPT(i));
    end

%%%%%%%%%%%%%% Close loop for Nbands times
end


% Close tape5 file:
fclose(fid);
clear aux

end