function [wvl,Lp0,Edir0,Edif0,S,tdir,tdif,I0] = Read_acd(alg,acdfile,SZA)
%READ_TAPE7 Read MODTRAN5 output tape7 file
%   [WVL,TDIR,DRCT_RFLT,GRND_RFLT,SOL_SCAT] = READ_ACD(ACDFILE) reads
%   MODTRAN5 output .acd file (ACDFILE) and the corresponding tape7 file
%   and generates the following output variables:
%          wvl  : Wavelength in [nm]
%          Lp0  : Atmospheric path radiance (in [mW/m2/sr/nm])
%          Edir : At-surface direct solar irradiance (in [mW/m2/nm])
%          Edif : At-surface diffuse solar irradiance (in [mW/m2/nm])
%          S    : Spherical Albedo (in [-])
%          tdir : Target-to-sensor direct transmittance (in [-])
%          tdif : Target-to-sensor diffuse transmittance (in [-])
%          I0   : Extraterrestrial solar irradiance (in mW/m2/nm)
%
%   Based on MODTRAN5 special DISORT option for atmospheric correction, 
%   selected through the DISALB='T' parameter.
%
%   See also: LUTGEN_PROCESS, READ_TAPE7

%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.00 $  $Date: 2018/03/16 $

%% Open File:
fid   = fopen(acdfile);
tline = fgetl(fid);

%% Scan header:
% Scan the acdfile to detect the number of header lines. Header in t7file
% ends with a line string containing the word "[CM-1]"
headerlines = 0;
while ischar(tline) && ~feof(fid)
    headerlines = headerlines+1;
    if strfind(tline,'[CM-1]')
        [~,count] = sscanf(tline,'%s');
        header = textscan(tline,'%s',16,'Delimiter','');
        break        
    end
    tline = fgetl(fid);
end

tline = fgetl(fid);

%% Get data from .tp7 file:
[~,~,~,~,Lp0,I0] = alg.readRTMoutput(strrep(acdfile,'.acd','.tp7'));

%% Get data from .acd file:
data = [];
tline = fgetl(fid);
while ~feof(fid) || ~isnumeric(tline)
    if ~isempty(tline)
        aux = textscan(tline,'%f','delimiter','\t'); aux = aux{1}';
        data(end+1,:) = aux;
        tline = fgetl(fid);
    else
        for i=1:headerlines, tline = fgetl(fid); end
        tline = fgetl(fid);
    end
end
fclose(fid);

N = max(unique(data(:,2))); % Total number of monotonic subintervals
freq = unique(data(:,1));
n = numel(freq); % Total number of wavelengths
aux = zeros(n,5);
for i=1:n
    ind = data(:,1)==freq(i);
    k            = data(ind,3);    % K-binning weights
    aux(n-i+1,:) = k'*data(ind,4:8);
end
Edif0 = fliplr(I0).*aux(:,1); % At-surface diffuse solar irradiance
Edir0 = fliplr(I0).*aux(:,2); % At-surface direct solar irradiance
tdif  = cosd(SZA)*aux(:,3); % Target-to-sensor diffuse transmittance
tdir  = aux(:,4); % Target-to-sensor direct transmittance
S     = aux(:,5); % Spherical albedo

wvl = flipud(1e7./freq); % Wavelength in [nm]

% END of function
end
