function [wvl,varargout] = readRTMoutput(tp7file)
%READRTMOUTPUT Read MODTRAN5 output tape7 file
%   [WVL,TDIR,DRCT_RFLT,GRND_RFLT,SOL_SCAT] = READRTMOUTPUT(TP7FILE) reads
%   MODTRAN6 output tape7 file (TP7FILE) in the radiance mode and 
%   generates the following output variables:
%   WVL       - Wavelength (in [nm]) at which the output data is given
%   TDIR      - Target to Sensor direct transmittance (in [-])
%   DRCT_REFL - Direct ground reflected radiance (in [mW/m^2/sr/nm])
%   GRND_RFLT - Total ground reflected radiance (in [mW/m^2/sr/nm])
%   SOL_SCAT  - Solar scattered radiance (i.e. path radiance) (in [mW/m^2/sr/nm])
%   EXTIRR    - Extraterrestrial solar irradiance (in mW/m2/nm)
%
%   [WVL,TTOT,TGAS] = READRTMOUTPUT(TP7FILE) reads MODTRAN6 output tape7 
%   file (TP7FILE) in the transmittance mode and  generates the following 
%   output target-to-sensor direct transmittances:
%   WVL       - Wavelength (in [nm]) at which the output data is given
%   TTOT      - Total transmittance (in [-])
%   TGAS      - Gaseous transmittances (in [-]), in the following order:
%               H2O transmittance (TH2O), O3 transmittance (TO3), N2 
%               transmittance (TN2), Aerosol+Clouds transmittance (TAER),
%               HNO3 transmittance (THNO3), CO2 transmittance (TCO2), CO 
%               transmittance (TCO), CH4 transmittance (TCH4), N2O 
%               transmittance (TN2O), O2 transmittance (TO2), NH3 
%               transmittance (TNH3), NO transmittance (TNO), NO2 
%               transmittance (TNO2), SO2 transmittance (TSO2)
%
%   Based on L. Guanter et al., On the application of the MODTRAN4 
%   atmospheric radiative transfer code to optical remote sensing, 
%   International Journal of Remote Sensing, vol. 30, no. 6, pp. 14071424,
%   2009
%   and
%   on W. Verhoef and H. Bach, Simulation of Sentinel-3 images by
%   four-stream surfaceatmosphere radiative transfer modeling in the 
%   optical and thermal domains, Remote Sensing of Environment, vol. 120,
%   pp. 197207, 2012.
%
%   See also: LUTGEN_PROCESS, TP7_TO_LUT, TP7_TO_LUT2

%   Copyright 2019 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 2.01 $  $Date: 2019/05/20 $

%% Open File:
fid   = fopen(tp7file);
tline = fgetl(fid);

%% Fast read method:
data = [];
while ~feof(fid)
    if strfind(tline,'FREQ')
        % Next line starts the data:
        tline = fgetl(fid);
        % Read first line to know how many columns we have:
        aux = textscan(tline,'%f','delimiter','\t'); aux = aux{1}';
        if isempty(aux)
            % We are in the tp7 corresponding to transmittance mode
            % therefor we need to skip one more line
            tline = fgetl(fid);
            aux = textscan(tline,'%f','delimiter','\t'); aux = aux{1}';
        end
        nCols = numel(aux);
        % Then read entire N sequential lines:
        aux = [aux;cell2mat(textscan(fid,repmat('%f\t',1,nCols),N))];
        data = [data;aux];
        if data(end,1)==-9999, data(end,:)=[];end
    elseif strfind(tline,'RN') % TO REIVEW!! make it more robust the detection of the line with spectral information
        % Read min/max wavelength range and resolution:
        aux = textscan(tline,'%f','delimiter','\t'); aux = aux{1}';
        % Calculate number of sequential lines N to be read:
        N   = (aux(2)-aux(1))/aux(3);
    end
    tline = fgetl(fid);    
end

%% Reorganize data (lower-to-higher wavelengths) and close file:
data = flipud(data);
fclose(fid);

%% Conversion to [mW/m2/sr/nm] instead of [W/cm2/sr/cm^-1]:

% Wavelength in [nm]:
wvl = 1e7./data(:,1);

% Only for method Radiance with Scattering: 
% Radiances in [mW/m2/sr/nm]:
sizeData = size(data);
if sizeData(2)==16
    % Total transmittance:
    varargout{1} = data(:,2);
    % Solar Scatter Radiance:
    varargout{4} = 1e7*(data(:,6).*data(:,1))./wvl; % =Path Radiance
    % Total Ground Reflected Radiance:
    varargout{3} = 1e7*(data(:,8).*data(:,1))./wvl;
    % Direct Ground Reflected Radiance:
    varargout{2} = 1e7*(data(:,9).*data(:,1))./wvl;
    % Extraterrestrial Solar Irradiance:
    varargout{5} = 1e7*(data(:,15).*data(:,1))./wvl;
    
    % Surface Emision:
    varargout{6}   = 1e7*(data(:,5).*data(:,1))./wvl;
    % Path thermal Radiance:
    varargout{7}   = 1e7*(data(:,3).*data(:,1))./wvl;
    % Total radiance:
    varargout{8}   = 1e7*(data(:,10).*data(:,1))./wvl;

elseif sizeData(2)==15
    % Total transmittance:
    varargout{1} = data(:,2);
    % Solar Scatter Radiance:
    SOL_SCAT = 1e7*(data(:,5).*data(:,1))./wvl; % =Path Radiance
    SOL_SCAT(SOL_SCAT<0 | isnan(SOL_SCAT)) = 0;
    varargout{4} = SOL_SCAT;
    % Total Ground Reflected Radiance:
    GRND_RFLT = 1e7*(data(:,7).*data(:,1))./wvl;
    GRND_RFLT(GRND_RFLT<0 | isnan(GRND_RFLT)) = 0;
    varargout{3} = GRND_RFLT;
    % Direct Ground Reflected Radiance:
    DRCT_RFLT = 1e7*(data(:,8).*data(:,1))./wvl;
    DRCT_RFLT(DRCT_RFLT<0 | isnan(DRCT_RFLT)) = 0;
    varargout{2} = DRCT_RFLT;
    % Extraterrestrial Solar Irradiance:
    varargout{5} = 1e7*(data(:,14).*data(:,1))./wvl;
    
    % Surface Emision:
    varargout{6}   = 1e7*(data(:,4).*data(:,1))./wvl;
    % Path thermal Radiance:
    varargout{7}   = 1e7*(data(:,3).*data(:,1))./wvl;
    % Total radiance:
    varargout{8}   = 1e7*(data(:,9).*data(:,1))./wvl;
    
else % Trasmittance mode
    % Total transmittance:
    varargout{1} = data(:,2);
    % Gas transmittances:
    Tgas = ones(numel(wvl),14);
    % H2O transmittance     % O3 transmittance      % N2 transmittance
    Tgas(:,1) = data(:,3);  Tgas(:,2) = data(:,5);  Tgas(:,3) = data(:,6);
    % Aerosol transmittance % HNO3 transmittance    % CO2 transmittance
    Tgas(:,4) = data(:,9);  Tgas(:,5) = data(:,10); Tgas(:,6) = data(:,13);
    % CO transmittance      % CH4 transmittance     % N2O transmittance
    Tgas(:,7) = data(:,14); Tgas(:,8) = data(:,15); Tgas(:,9) = data(:,16);   
    % O2 transmittance       % NH3 transmittance       % NO transmittance
    Tgas(:,10) = data(:,17); Tgas(:,11) = data(:,18);  Tgas(:,12) = data(:,19);
    % NO2 transmittance      % SO2 transmittance
    Tgas(:,13) = data(:,20); Tgas(:,14) = data(:,21);
    
    varargout{2} = Tgas;
end

% END of function
end
