function [lambda,Lp0,Edir0,Edif0,S,tdir,tdif,I0] = tp7_to_LUT2(alg,files,rhos,SZA)
% TP7_TO_LUT2 processes the MODTRAN output tape7 files (same atmosphere and
% different contant reflectivities and geometric conditions) in order to 
% obtain the atmospheric variables independent from surface properties.
%
% INPUTs: files = cell array with the input tape7 files
%         RHOS = vector of constant reflectivities associated with the
%                simulations in files
%         SZA = Sun Zenith Angle
%
% OUTPUTs: lambda = Wavelength in [nm]
%          Lp0  : Atmospheric path radiance (in [mW/m2/sr/nm])
%          Edir : At-surface direct solar irradiance (in [mW/m2/nm])
%          Edif : At-surface diffuse solar irradiance (in [mW/m2/nm])
%          S    : Spherical Albedo (in [-])
%          tdir : Target-to-sensor direct transmittance (in [-])
%          tdif : Target-to-sensor diffuse transmittance (in [-])
%          I0   : Extraterrestrial solar irradiance (in mW/m2/nm)
%
%   Based on W. Verhoef and H. Bach, Simulation of Sentinel-3 images by
%   four-stream surfaceatmosphere radiative transfer modeling in the 
%   optical and thermal domains, Remote Sensing of Environment, vol. 120,
%   pp. 197207, 2012.
%
%   See also: LUTGEN_PROCESS, READ_TAPE7

%   Copyright 2019 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es, m.neus.sabater@uv.es
%   $Revision: 1.00 $  $Date: 2019/05/19 $


% Initial values:
u_s = cosd(SZA); % Cosinus of the solar zenit angle in [deg]

% Recover parameters from the input files:
[lambda,tdir,~,~,~,I0,~,~,Lp0] = alg.readRTMoutput(files{1});
[lambda,~,GSUN05,GRFL05,~,I0,SFEM05] = alg.readRTMoutput(files{2});
[lambda,TRANB,~,GRFL1,~,I0,~,~,Tot_RAD1_B] = alg.readRTMoutput(files{3});
[lambda,tdir,GSUN,~,PATH1T,I0] = alg.readRTMoutput(files{4});

tdir(isnan(tdir) | isinf(tdir) | tdir<0) = 0;
I0(I0<0 | isnan(I0) | isinf(I0)) = 0;

t1      = I0.*u_s./pi; % Solar Extraterrestrial Irradiance

% Spherical albedo
S = (GRFL1-2.*GRFL05)./(GRFL1-GRFL05-SFEM05); S(isnan(S) | isinf(S) | S<0) = 0;

% Solar extraterrestrial irradiance directly transmited at ground level:
% tss.*I0.cos(SZA)
% Derived from BOA
t4 = GSUN05./(0.5*TRANB.*t1); 
t4 = GSUN05./(0.5*t1);
Edir0 = pi*t4.*t1/u_s;

% Solar extraterrestrial irradiance scattered by the atmosphere and 
% diffusily transmited at ground level: tsd.*Eo.cos(SZA)
% Derived from BOA
t5 = Tot_RAD1_B.*(1-S)./t1 - t4;  t5(isnan(t5) | isinf(t5) | t5<0)= 0;
Edif0 = pi*t5.*t1;

% Diffuse transmitance from target to the observer
tdif = (PATH1T-Lp0).*(1-S)./(t1.*(t5+t4)); 
tdif(isnan(tdif) | isinf(tdif) | tdif<0)= 0;

% END of function
end