function writeRTMinput(file,rho,varnames,inputval,SpeConf,flag)
% WRITERTMINPUT Create MODTRAN6 input json file
%   WRITERTMINPUT(FILE,RHO,VARNAMES,INPUTVAL,SPECONF) creates, according to 
%   MODTRAN documentation, a MODTRAN's json file (FILE) with a spectrally 
%   constant reflectance RHO in the spectral range and configuration 
%   deusefined in the input SPECONF structure. MODTRAN configuration is based 
%   on the input variables and associated values stored in the cell arrays 
%   VARNAMES (strings) and INPUTVAL (numeric values).
%
%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 0.01 $  $Date: 2018/06/17 $

%% Input variables to be used by MODTRAN:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%            INPUT VARIABLES TO BE USED BY MODTRAN              %%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numel(varnames)~=numel(inputval)
    error('Number of variable names and input values are not coincident.')
end

% Associates input values to each variable name:
for i=1:numel(varnames)
    eval([varnames{i},'=inputval{',num2str(i),'};'])
end

%--------------------------------------------------------------------------
% NOTE: Read MODTRAN6 Documentation for further information on the variables
% and JSON format.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%-------------------------------- RTOPTIONS -------------------------------
%--------------------------------------------------------------------------
% IEMSCT selects the radiation transport mode of execution (see MODTRAN
% documentation): 0-4
IEMSCT = IEMSCT;
% MODTRN model to be used: 0 or RT_MODTRAN (statistical band model),
% 1 or RT_CORRK_SLOW (slow correlated-k, 33 k values, only for radiance modes), 
% 2 or RT_CORRK_FAST (fast correlated-k, 17 k values)
% 3 or RT_MODTRAN_POLAR (polarimetric MODTRAN, unavailable)
% or 4 or RT_LINE_BY_LINE (line-by-line)
MODTRN = MODTRN;
if IEMSCT==0
    % Transmittance mode is only compatible with MODTRN band model type
    MODTRN = 0;
end
% LYMOLC: Includes auxiliary trace species (T=true) or doesn't include them
% (F=false)
LYMOLC = LYMOLC;
if strcmp(LYMOLC,'T')
    LYMOLC = true;
elseif strcmp(LYMOLC,'F')
    LYMOLC = false;
end
% T_BEST: Use slow and precise Voigt line transmittance integration (true).
% This painstakingly slow option should only be selected if the user 
% suspects that there is a problem with MODTRANs standard Voigt line 
% transmittance calculation. Transmittance differences of significance
% (>0.0001). Otherwise, T_BEST=false.
T_BEST = false;
% NSTR selects the number of streams used by DISORT (2, 4, 8, 16, 32).
% Usually take NSTR=8 but set to 32 if more precission is requiered
NSTR = NSTR;
% IMULT adds Multiple Scattering (MS):
% 0 or RT_NO_MULTIPLE_SCATTER (for no MS),
% 1 or RT_DISORT (for DISORT MS at path final/tangent point for limb)
% 2 or RT_DISORT_AT_OBS (same as before but at sensor)
% 3 or RT_ISAACS_2STREAMS (for Isaac's 2-streams MS at path final/tangent 
% point for limb)
% 4 or RT_ISAACS_2STREAMS_AT_OBS (same as before but at sensor)
% 5 or RT_ISAACS_SCALED (same as RT_ISAACS_2STREAMS but scaled to DISORT)
% 6 or RT_ISAACS_SCALED_AT_OBS (same as before but at sensor)
IMULT = 1;
if     NSTR==0,  IMULT = 3; 
elseif NSTR<0, NSTR = abs(NSTR); IMULT = 5;
end
% DISALB='T' will calculate the spectral spherical albedo of the atmosphere
% and diffuse transmittance for the LOS and Sun-to-Ground paths. DISALB
% option is only available with DISORT (IMULT=1).
DISALB = DISALB;
if strcmp(DISALB,'T')
    DISALB = true; IMULT = 1;
elseif strcmp(DISALB,'F')
    DISALB = false;
elseif strcmp(DISALB,'W')
    % Interrogation technique of Verhoef et al. 2012
    DISALB = false;
end
% NLBL: Number of spectral points in each 0.1 cm-1 bin at which LBL
% calculations are performed; default is 100 corresponding to 0.001 cm-1
% spacing. 
NLBL = NLBL;
% SOLCON sets the scale factor for the TOA Solar irradiance on the input
% file. <0 selects the scale factor, 0 uses the default TOA solar
% irradiance and >0 assigns the input spectrally integrated TOA solar 
% irradiance in W/m^2.
SOLCON = 0;

%--------------------------------------------------------------------------
%-------------------------------- ATMOSPHERE ------------------------------
%--------------------------------------------------------------------------
% MODEL: selects on geographical-seasonal atmospheres or specify to use a
% user-defined or radiosonde data (read MODTRAN documentation): 1 to 8
MODEL = MODEL; auxMODEL = MODEL;
if isstruct(MODEL)
    % User-defined atmospheric model (pressure, temperature and gas absorptions)
    MODEL = 7; %ATM_USER_ALT_PROFILE
elseif MODEL==0
    % No Gaseous Absorption atmosphere (In MODTRAN6 user manual, MODEL=0 is
    % for single-altitude meteorological data but we don't use this option
    % in ALG). We will scale gas concentration to 0 to remove absorptions.
    % For Rayleigh and temperature emission, we consider the Mid-Latitude
    % Summer atmosphere (review if other atmospheres might be needed)
    MODEL  = 2; %Mid-Latitude Summer
    C_PROF = 7;
end 

% H: boundary layer aerosols top height [km].
% Z: boundary layer aerosols scale height [km].
% Note: H and Z are user-defined variables and not originally in MODTRAN6
H = H; Z = Z;
if H>0 || Z>0 % User-defined aerosol vertical distribution
    MODEL=7;
    H = abs(H);
    Z = abs(Z);
end 
% ITYPE indicates the line-of-sight path: in our case the option will be 3 
%(vertical or slant path to space or ground)
ITYPE = 3;
% M1, M2, M3, M4, M5, M6 and MDEF modify user-specified altitude profiles
% for temperature, pressure and default molecular gases and heavy
% molecules: 1-6 (for M1, M2, M3, M4, M5 and M6)
% MDEF=0 include the default CO2, O2, NO, SO2, NO2, NH3 and HNO3 profiles
% MDEF=1 also include the heavy molecules with their default profiles, 
% namely the 9 chlorofluorocarbons (CFCs) and ClONO2, HNO4, CCl4 and N2O5, 
% as listed in Table 3.8 (see MODTRAN6 manual), in addition to the MDEF = 0
% profiles.
% MDEF=2 profiles for the heavy molecules will be defined via input arrays
% If MODEL is 1 to 6, then M1 to M6 and MDEF are assigned to the selected 
% MODEL is used for all the atmospheric species, temperature and pressure
M1 = 0; M2 = 0; M3 = 0; M4 = 0; M5 = 0; M6 = 0;
MDEF = 0;
if MODEL==7 && isstruct(auxMODEL)
    % User-defined atmospheric model. Atmospheric species, temperature and 
    % pressure are user-defined
    M1 = 0; M2 = 0; M3 = 0; M4 = 0; M5 = 0; M6 = 0;
    MDEF = -1; % check this value since it was defined in MODTRAN5 but not MODTRAN6
% elseif MODEL==7 && auxMODEL>0
%     % Only user-defined aerosol vertical profiles are introduced.
%     % Atmospheric species, temperature and pressure are the same as the 
%     % default atmospheric MODEL (set JCHAR blank in CARD 2C1)
%     M1 = auxMODEL; M2 = auxMODEL; M3 = auxMODEL;
%     M4 = auxMODEL; M5 = auxMODEL; M6 = auxMODEL;
%     MDEF = 0;
end
% M2_RHC=TRUE uses the H2O relative humidity profile of M2-selected model 
% (instead of its mixing ratio). This RH profile values will be used 
% regardless of the chosen temperature profile. Depending on the 
% temperature profile, the water concentration will generally differ from 
% the model selected by M2.
M2_RHC = false;
if M2_RHC==0
    M2_RHC = false;
elseif M2_RHC==1
    M2_RHC = true;
end
% HMODEL is a character string of length 20 that can be used to label the 
% atmosphere or calculation (not used in ALG, thus not included in the
% MODTRAN6 input files)
if MODEL==7 
    % NLAYERS is the number of layers (actually, layer boundaries or levels)
    % in the custom (i.e., MODEL=7) atmosphere.
    if isstruct(auxMODEL)
        NLAYERS = numel(auxMODEL.ZM);
    else
        NLAYERS = 36; % Based on MODTRAN5 value. Check !!
    end
    % NPROF is the number of profiles entries in PROFILES
    NPROF = 1;
    % TYPE identifies the type of atmospheric constituent:
    TYPE = 0; % Altitude
    % UNITS of each atmospheric constituent in TYPE:
    % 0 - Altitude in km
    UNITS = 0;
    % Altitude profile, in [km], of each layer boundary 
    if isstruct(auxMODEL)
        PROFILE(:,1) = auxMODEL.ZM;
    else
        PROFILE(:,1) = [0:25,30:5:60,70,80,100];
    end
    if isstruct(auxMODEL)
        % User-defined atmospheric model. Atmospheric species, temperature 
        % and pressure are provided in PROFILES
        % NPROF: REVIEW value!! In this case we include all the gas profiles 
        % and maybe it is not necessary except in very specific cases. On the 
        % other hand, we can input all the gas species, leaving by most of them
        % to a default atmospheric model (e.g. MLS).
        NPROF = NPROF + 14; 
        % TYPE identifies the type of atmospheric constituent:
        % 1, 2, 3, 4, 5 - Pressure, Temperature, H2O, CO2, O3
        % 6, 7, 8, 9, 10, 11, 12, 13 - N2O, CO, CH4, O2, NO, SO2, NO2, NH3
        % 14, 15 ,16 ,17, 18, 19 - HNO3, CCl3F, CCl2F2, CClF3, CF4, CHClF2
        % 20, 21, 22, 23, 24 - C2Cl3F3, C2Cl2F4, C2ClF5, ClONO2, HNO4
        % 25, 26, 27 - CHCl2F, CCl4, N2O5
        TYPE = [TYPE,1:14];
        % UNITS of each atmospheric constituent in TYPE:
        % 1 - Temperature in K
        % 2 - Temperature in C
        % 3 - Temperature difference from M1 profile
        % 4 - Dew point in K
        % 5 - Dew point in C
        % 6 - Total or partial pressure in mbars
        % 7 - Total or partial pressure in atm
        % 8 - Volume mixing ratio in ppmv
        % 9 - Density in molecules/cm3
        % 10 - Mass mixing ratio in g/kg
        % 11 - Mass density in g/m3
        % 12 - Relative humidity in %
        UNITS = [UNITS 6 1 8*ones(1,12)];
        if isstruct(auxMODEL)
            % TYPE=1 to 14-->PROFILE
            PROFILE(:,2) = auxMODEL.P;
            PROFILE(:,3) = auxMODEL.T;
            PROFILE(:,4) = auxMODEL.H2O;
            PROFILE(:,5) = auxMODEL.CO2;
            PROFILE(:,6) = auxMODEL.O3;
            PROFILE(:,7) = auxMODEL.N2O;
            PROFILE(:,8) = auxMODEL.CO;
            PROFILE(:,9) = auxMODEL.CH4;
            PROFILE(:,10) = auxMODEL.O2;
            PROFILE(:,11) = auxMODEL.NO;
            PROFILE(:,12) = auxMODEL.SO2;
            PROFILE(:,13) = auxMODEL.NO2;
            PROFILE(:,14) = auxMODEL.NH3;
            PROFILE(:,15) = auxMODEL.HNO3;
            % TYPE=15 to 27 --> default to MLS!!
%             for i=16:28
%                 PROFILE(:,i) = 'default';
%                 PRO_MASK{i} = -1*ones(size(PROFILE(:,1)));
%             end
        end
        
    end
    if H>0 || Z>0
        % User-defined aerosol vertical profiles are introduced.
        % Aerosol/cloud scaling factor (similar to visible extinction 
        % coefficient) at altitude ZM. The model implemented is a decreasing
        % exponential with a user-defined scale height (Z [km]) in the boundary 
        % layer from 0 km to a user-defined top altitude (H [km]).
        NPROF = NPROF + 1;
        ZM = PROFILE(:,1);
        TYPE = [TYPE 28];
        UNITS = [UNITS -1];
        AHAZE = zeros(size(ZM));
        AHAZE(ZM<=H) = exp(-ZM(ZM<=H)/Z);
        % The vertical distribution is scaled so that the total column AOT 
        % (integral of extiction coefficient) has the value VIS at 550 nm.
        % Troposheric stratospheric and mesospheric aerosol distribution is 
        % kept to the default MODTRAN types.
        auxh = [1:25,30:5:60,70,80,100];
        % Extinction coef. at 1-4 km from fitting MODTRAN profiles at VIS:
        if any(abs(auxMODEL)==[0,1,2,4,6,7,8]) % SUMMER profiles
            auxEXTC_1km = 0.2107*abs(VIS)^2+0.2874*abs(VIS)-0.011; if abs(VIS)>0.35, auxEXTC_1km=0.8011*abs(VIS)-0.1643; end
            auxEXTC_2km = 0.2091*abs(VIS)-0.008; if abs(VIS)>0.35, auxEXTC_2km=6.21e-2; end
            auxEXTC_3km = -0.0607*abs(VIS)^2+0.139*abs(VIS)-0.0058; if abs(VIS)>0.35, auxEXTC_3km=3.46e-2; end
            auxEXTC_4km = 0.0481*abs(VIS)+0.0024; if abs(VIS)>0.35, auxEXTC_4km=1.85e-2; end
            auxEXTC = [9.31e-3,7.71e-3,6.23e-3,3.37e-3,1.82e-3,1.14e-3,7.99e-4,6.41e-4,5.17e-4,4.42e-4,3.95e-4,3.82e-4,4.25e-4,5.2e-4,5.81e-4,5.89e-4,5.02e-4,4.2e-4,3e-4,1.98e-4,1.31e-4,3.32e-5,4.3e-6,1.67e-6,8e-7,4.2e-7,2.207e-7,1.159e-7,3.2e-8,5.795e-9,1.9e-10];
        elseif any(abs(auxMODEL)==[3,5]) % WINTER profile
            auxEXTC_1km = 0.4569*abs(VIS)^2+0.2314*abs(VIS)-0.0013; if abs(VIS)>0.35, auxEXTC_1km=0.8032*abs(VIS)-0.142; end
            auxEXTC_2km = 0.2054*abs(VIS)-0.003; if abs(VIS)>0.35, auxEXTC_2km=6.21e-2; end
            auxEXTC_3km = -0.116*abs(VIS)^2+0.1341*abs(VIS)-0.0044; if abs(VIS)>0.35, auxEXTC_3km=2.72e-2; end
            auxEXTC_4km = 0.0317*abs(VIS)+0.002; if abs(VIS)>0.35, auxEXTC_4km=1.20e-2; end
            auxEXTC = [4.85e-3,3.54e-3,2.31e-3,1.41e-3,9.8e-4,7.87e-4,7.14e-4,6.64e-4,6.23e-4,6.45e-4,6.43e-4,6.41e-4,6e-4,5.62e-4,4.91e-4,4.23e-4,3.52e-4,2.95e-4,2.42e-4,1.9e-4,1.5e-4,3.32e-5,4.3e-6,1.67e-6,8e-7,4.2e-7,2.207e-7,1.159e-7,3.2e-8,5.795e-9,1.9e-10];
        end
        auxEXTC = interp1(auxh,[auxEXTC_1km,auxEXTC_2km,auxEXTC_3km,auxEXTC_4km,auxEXTC],ZM(ZM>=1)); auxZM = ZM(ZM>=1);
        auxVIS  = trapz(auxZM(auxZM>H),auxEXTC(auxZM>H)); % Default AOT from H to 100 km
        scale = (abs(VIS)-auxVIS)/trapz(ZM(ZM<=H),AHAZE(ZM<=H));
        AHAZE(ZM<=H) = scale*AHAZE(ZM<=H);
        PROFILE(:,2) = AHAZE;
    end
end
% CO2MX introduces the CO2 mix in ppmv. Set to 0 for using the default 400 
% ppmv CO2 concentration (circa 2017 nominal value).
CO2MX = CO2MX;
if ~isstruct(auxMODEL) && auxMODEL==0 % No Gaseous Absorption
    CO2MX = 0.001;% In practice it's nearly no CO2
end
% H2OSTR vertical water vapor column. This variable is separated in two for
% input in MATLAB. The first selects the character for selecting the units
% ('g' for g/cm^2 or 'a' for ATM-cm). Then the value is introduced. If the
% first charecter is blank ' ', the value is a scale factor on the default
% water vapor column.
if H2OSTR<0
    H2OUNIT = ' '; % Scale factor
elseif H2OSTR==0
    H2OUNIT = 'g'; % g/cm2
    H2OSTR  = 0.00001; % In practice it's nearly no H2O
else
    H2OUNIT = 'g'; % g/cm2
end
H2OSTR = abs(H2OSTR);
if ~isstruct(auxMODEL) && auxMODEL==0 % No Gaseous Absorption
    H2OUNIT = 'g';
    H2OSTR = 0;
end  
% O3STR vertical ozone column. Similar to H2OSTR.
if O3STR<0
	O3UNIT = ' '; % Scale factor
	O3STR = 1; % Automatically sets to default value
elseif O3STR==0
    O3UNIT = ' '; % Scale factor
else
    O3UNIT = 'a'; % atm-cm
end
O3STR = abs(O3STR);
if ~isstruct(auxMODEL) && auxMODEL==0 % No Gaseous Absorption
    O3UNIT = 'a';
    O3STR = 0;
end
% C_PROF selects the scale factors for the molecular species (uniformely
% mixed, cross-section and trace gases) (see MODTRAN6 documentation)
C_PROF = C_PROF;
% S_UMIX scale factors for profiles of 10 ordered uniformly mixed (UMIX9
% molecular species [Sea level mixing ratios (ppmv) listed in brackets]
% N2O[0.32], CO[0.15], CH4[1.7], O2[2.09x10^5], NO[0.0003], SO2[0.0003], 
% NO2[0.000023], NH3[0.0005], HNO3[0.00005] and N2[7.81x10^5].
if C_PROF==1 || C_PROF==3 || C_PROF==5 || C_PROF==7
    % To be updated in a future version of ALG
    S_UMIX = [1 1 1 1 1 1 1 1 1 1];
    if auxMODEL==0, S_UMIX = 0*S_UMIX; end
end
% S_XSECS scale factors for profiles of the 15 ordered cross-section (X) 
% species [Sea level mixing ratios (ppmv) listed in brackets]: 
% F11[0.00014], F12[0.00024], F13[1x10^12], F14[1x10^12], F22[0.00006], 
% F113[0.000019], F114[0.000012], F115[1x10^12], ClONO2[5.75x10^6],
% HNO4[4.34x10^7], CHCl2F[1x10^12], CCl4[0.00013], N2O5[2.42x10^10], 
% H2[0.58] and He[5.2].
if C_PROF==2 || C_PROF==3 || C_PROF==6 || C_PROF==7
    % To be updated in a future version of ALG
    S_XSECS = [1 1 1 1 1 1 1 1 1 1 1 1 1];
    if auxMODEL==0, S_XSEC = 0*S_XSECS; end
end
% S_TRAC scale factors for profiles of the 16 ordered trace (Y) molecules 
% [Sea level mixing ratios (ppmv) listed in brackets]: OH[4.4x10^8], 
% HF[1x10^8], HCl[0.001], HBr[1.7x10^6], HI[3x10^6], ClO[1x10^8], 
% OCS[0.0006], H2CO[0.0024], HOCl[7.7x10^6], N2[7.81x10^5], HCN[0.00017],
% CH3Cl[0.0007], H2O2[0.0002], C2H2[0.0003], C2H6[0.002], and PH3[1x10^14]
if C_PROF==4 || C_PROF==5 || C_PROF==6 || C_PROF==7
    % To be updated in a future version of ALG
    S_TRAC = [1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1]; 
    if auxMODEL==0, S_TRAC = 0*S_TRAC; end
end
% AERRH relative humidity [%] that defines the RH-dependent boundary layer 
% aerosol optical (>0). If <=0 then RH for boundary layer aerosols determine
% from aerosol density weighted average
AERRH = 0;
% AYRANG (true): Range profile data read from yrange.asc or <ROOTNAME>.rng.
% This file specifies the temperature and the density of all local species 
% as a function of slant range from the sensor (see MODTRAN6 User Manual
% for further information)
% AYRANG (false): No local chemical (gas) plume range profile data
AYRANG = false;
% Planetary mass in Earth units (0.5 means half as massive as the Earth). 
% Used to solve the hydrostatic equation when modeling extra-terrestrial 
% planets [only invoked when MODEL is ATM_USER_PRESS_PROFILE (=8)].
if MODEL==8
    E_MASS = 0;
end
% Molecular weight of air at surface in g/mol. If =0, AIRMWT is set to 
% AIR_MW=29.9644 g/mol
AIRMWT = 0;

%--------------------------------------------------------------------------
%--------------------------------- AEROSOLS -------------------------------
%--------------------------------------------------------------------------
% IHAZE Selects the type of extinction and a default meteorological range 
% (visibility) for the boundary-layer (nominally, 0 to 3 km altitude) 
% aerosol model. See MODTRAN6 documentation
IHAZE = IHAZE;
if isstruct(IHAZE) % IHAZE contains data from OPAC or user-defined aerosol:
    aeraux = IHAZE;
    IHAZE   = 7;
    if strcmp(aeraux.type,'OPAC') % OPAC aerosol
        % RH: average relative humidity of the boundary and troposphere layers
        % for the selected atmosphere model and input columnar water vapor,
        % weighted by exp(-h[km]/8[km]):
        RH_boundary = [74.4 66.7 71.5 72 73.7 48.9 67.8]; % Default average RH for boundary layer and MODEL
        RH_troposphere = [41.8 38.1 44.5 52.7 52.3 47.9 46.2]; % Default average RH for troposphere and MODEL
        if strcmp(H2OUNIT,'g')
            auxCWV = [4.115, 2.922, 0.852, 2.081, 0.416, 1.416]; % Default CWV [g/cm2] for MODEL
            auxCWV = [auxCWV,mean(auxCWV)];
            scale = auxCWV(MODEL)/H2OSTR;
        else
            scale = H2OSTR;
        end
        RH = scale*[RH_boundary(MODEL), RH_troposphere(MODEL)]; RH(RH>100)=100;
    elseif strcmp(aeraux.type,'ATK') % ATK aerosol
    	aeraux.VARSPC = aeraux.data(:,1);
        aeraux.EXTC = aeraux.data(:,2);
        aeraux.ASYM = aeraux.data(:,4);
        aeraux.ABSC = (1-aeraux.data(:,3)).*aeraux.data(:,2); 
        aeraux.RH   = [];
        aeraux.ANGF = []; 
        aeraux.F    = [];
    end
elseif IHAZE>=3.1 && IHAZE<=3.9 % Navy maritime aerosols
    % ICSTL measure of the continental influence, only used with Navy 
    % Maritime aerosols. 1 = Open Ocean,..., 10 = Strong continental 
    % influence
    ICSTL = num2str(IHAZE); ICSTL = str2num(ICSTL(3));
    IHAZE = 3;
end
% VIS defines the surface meteorological range. >0 value in [km], =0 using
% the default range set by IHAZE or <0 for the negative of the 550nm
% vertical aerosol optical depth. If VIS<0, this is combined with ground 
% altitude, season, and volcanic aerosol inputs to determine the surface
% meteorological range
VIS = -1*VIS;
if VIS==0, IHAZE=0; end % No aerosols
% WSS specifies the wind speedm in [m/s], for use with the Navy Maritime
% (IHAZE=3) and Desert (IHAZE=10) aerosol models.
WSS = 0;
% WHH specifies the 24-hours average wind speed for use with the Navy
% Maritime aerosol model (IHAZE=3)
WHH = 0;
% Note: for Navy Aerosol Maritime model, if WSS=WHH=0, the default wind
% speed are set according to the atmospheric model (MODEL). See MODTRAN
% documentation (table 3.17).
% ICSTL is the air mass character (1 to 10). Default value is 3. See
% MODTRAN documentation
if IHAZE~=3, ICSTL = 3; end
% ISEASN selects the seasonal aerosol profile for the tropospheric (2-10 km)
% and stratospheric (10-30 km) aerosols. Also selects the tropospheric 
% aerosol extinction coefficients (nominally, 2-10 km). Default values 
% defined by the ATMOSPHERE JSONObject input MODEL
ISEASN = 0;
% IVULCN Selects both the profile and the type of extinction for the
% stratospheric aerosols. It also selects appropriate transition profiles 
% above the stratosphere to 100 km. Meteoric dust extinction coefficients
% are always used for altitudes from 30 to 100 km. The various options are:
% Background stratospheric (0), moderate aged volcanic (2), high fresh
% volcanic (3), high aged volcanic (4), moderate frech volcanic (5),
% moderate brackground stratospheric volcanic (6), high background
% stratospheric volcanic (7), extreme fresh volcanic (8)
IVULCN = 0;
% HGPF Asymmetry factor for use with Henyey-Greenstein phase function when
% IPH equals 0: +1 for complete forward scattering, 0 for isotropic or 
% symmetric scattering, and 1 for complete back scattering.
HGPF = G;
% IPH Selects the phase function option: Spectrally independent 
% Henyey-Greenstein (0), user-supplied aerosol phase function (1) or 
% uses Mie-generated internal database of aerosol phase functions (2)
if HGPF==10
    IPH = 2; % Mie-generated internal database (=2)
elseif HGPF>=-1 && HGPF<=1
	IPH = 0; % Spectrally independent Henyey-Greenstein (=0)
end
if IHAZE==7 && SSA==-1
	if ~isempty(aeraux.F)
        IPH = 1; % User-supplied (OPAC) aerosol phase function (=1)
    end
end
% ----------- Advanced Aerosol Options
% H2OAER modify, TRUE, (or not, FALSE) the aerosol optical properties to 
% reflect changes on the H2OSTR water vapor column.
H2OAER = H2OAER;
if strcmp(H2OAER,'T')
    H2OAER = true;
elseif strcmp(H2OAER,'F')
    H2OAER = false;
end
if ~isstruct(auxMODEL) && auxMODEL==0 % No Gaseous Absorption
    H2OAER = false; % Try not to affect aerosol properties when removing H2O (check!) 
end
% CNOVAM = TRUE allows introducing the Navy Ocean Vertical Aerosol Model
% (See chapter 10 for instructions)
CNOVAM = false;
% ARUSS Allows introducing user-defined aerosol optical properties (USS) or
% vertical profiles of aerosol spectral extinction, spectral absorption and
% spectral phase functions to be read in (SAP)
% default
ARUSS = 'DEFAULT';
if IHAZE==7, ARUSS = 'USS'; end
% IVSA selects the use of Army Vertical Structure Algorithm for aerosols in
% the boundary layer. This option cannot be used in conjunction with
% auxiliary species. The following cases are simulated:
% (1) Cloud/fog at the surface; increasing extinction with height from 
%     cloud/fog base to top. Use case (2) or (2') below the cloud and case 
%     (1) inside it. VIS<=0.5 km and ZCVSA<=0.
% (2) Hazy/light fog; increasing extinction with height up to the cloud 
%     base 0.5 km<=VIS<=10 km and ZCVSA>=0.
% (2') Clear/hazy; increasing extinction with height, but less so than case
%      (2), up to the cloud base. VIS>10 km and ZCVSA>=0.
% (3) No cloud ceiling but a radiation fog or an inversion or boundary layer
%     present; decreasing extinction with height up to height of fog or 
%     layer. ZCVSA<0 and ZINVSA>=0.
% (4) No cloud ceiling or inversion layer; constant extinction with height.
%     ZCVSA<0 and ZINVSA<0.
% Not used (IVSA=false), VSA (IVSA=true).
IVSA = false;
% ZCVSA Cloud ceiling height for VSA (km). If 0, ceiling height is 
% calculated by code for case (2) or 1.8 km for case (2'). If <0, no cloud
% ceiling (cases (3) and (4))
ZCVSA = 0;
% ZTVSA Cloud thickness for VSA (km). Uses default thickness of 0.2 km if
% ZTVSA=0
ZTVSA = 0; 
% ZINVSA Height of inversion layer (km). If 0, uses default inversion
% height of 2 km or 0.2 km for fog. If <0, No inversion layer (case (4), if
% ZCVSA < 0).
ZINVSA = 0;
% ----------- Aerosol Angstrom Exponent Input Option
% CDASTM perturbs the boundary layer and tropospheric aerosol reference
% spectral extinction data ('T') or only perturbs the boundary layer
% aerosol reference spectral extinction ('B'). Omit or 'F' to use Angstrom 
% Law description for boundary layer and tropospheric aerosol extinction 
% spectral extinction coefficients
CDASTM = 'F';
% ASTMX Amstrong law exponent for the boundary layer nd the tropospheric
% aerosol. Use the parameters provided for hte laters determines by CDASTM
ASTMX = ASTMX;
% ASTMC Amstrong law coefficient
ASTMC = 1;
% ASTMO Amstrong law offset not used with perturbation method
ASTMO = 0;
% ----------- Flexible Aerosol Options
% The SSALB structure defines the user-defined aerosol spectral single 
% scattering albedo. This is applied to the boundary layer region unless 
% CDASTM equals "t", "T", "d" or "D". In that case, it also applies to the 
% tropospheric aerosol region.
% The SSALB structure contains: NSSALB, AWAVLN, ASSALB, ACOALB and RHASYM
% NSSALB selects the number of aerosol single scattering albedo spectral 
% grid points (>0). If =0 then use reference value and if <0 then aerosol
% single scattering coalbedo is read in COALB and relative humidity RHASYM.
SSALB.NSSALB = 0;
SSA = SSA;
if SSA>=0
    SSALB.NSSALB = 1;
    % AWAVLN Wavelengths for boundary layer (and tropospheric) aerosol 
    % single scattering albedo in monotonically increasing order [um]
    SSALB.AWAVLN = 0.55;
    % ASSALB Boundary layer (and tropospheric) aerosol spectral single 
    % scattering albedo values
    SSALB.ASSALB = SSA;
    if SSALB.NSSALB<0
        % ACOALB Aerosol single scattering co-albedo (1  albedo) scaling 
        % factor. ACOALB may exceed 1, but the spectral co-albedo at each 
        % grid point is restricted to not exceed 1
        SSALB.ACOALB =1-SSA;
        % RHASYM Relative humidity used to define the aerosol asymmetry 
        % factor [%].
        SSALB.RHASYM = 0;
    end
end
% APLUS allows to vertically translate or scale user-defined aerosol
% optical profiles. ' ' = default, 'A+' = introduce vertical profiles using
% the REGALT structure
APLUS = ' ';
if strcmp(APLUS,'A+')
    % Base/bottom altitude of Region 1 (boundary layer) aerosol [km].
    REGALT.ZAER1(1) = 0;
    % Top of stretched/compressed Region 1 (boundary layer) aerosol [km].
    % If ZARER1(2)=ZAER1(1) then default ZAER1(1) and ZAER1(2) values, 0.0 
    % and 3.0 km, respectively are used. If <0, default aerosol layer 
    % thickness retained, ZAER1(2) = ZAER1(1) + 3.0 km
    REGALT.ZAER1(2) = 0;
    % Scale Region 1 atmospheric level aerosol densities by SCALE1.
    REGALT.SCALE1   = 0;
    % Base/bottom altitude of Region 2 (tropospheric layer) aerosol [km].
    REGALT.ZAER2(1) = 0;
    % Top of stretched/compressed Region 2 (tropospheric layer) aerosol 
    % [km]. If ZARER2(2)=ZAER2(1) then default ZAER2(1) and ZAER2(2) 
    % values, 2.0 and 11.0 km, respectively are used. If <0, default 
    % aerosol layer thickness retained, ZAER2(2) = ZAER2(1) + 9.0 km.
    REGALT.ZAER2(2) = 0;
    % Scale Region 2 atmospheric level aerosol densities by SCALE2.
    REGALT.SCALE2   = 0;
    % Base/bottom altitude of Region 3 (stratospheric layer) aerosol [km].
    REGALT.ZAER3(1) = 0;
    % Top of stretched/compressed Region 3 (stratospheric layer) aerosol 
    % [km]. If ZARER3(2)=ZAER3(1) then default ZAER3(1) and ZAER3(2) 
    % values, 10.0 and 35.0 km, respectively are used. If <0, default 
    % aerosol layer thickness retained, ZAER3(2) = ZAER3(1) + 25.0 km.
    REGALT.ZAER3(2) = 0;
    % Scale Region 3 atmospheric level aerosol densities by SCALE3.
    REGALT.SCALE3   = 0;
    % Base/bottom altitude of Region 4 (volcanic layer) aerosol [km].
    REGALT.ZAER4(1) = 0;
    % Top of stretched/compressed Region 4 (volcanic layer) aerosol [km].
    % If ZARER4(2)=ZAER4(1) then default ZAER4(1) and ZAER4(2) values, 
    % 30.0 and 100.0 km, respectively are used. If <0, default aerosol 
    % layer thickness retained, ZAER4(2) = ZAER4(1) + 70.0 km.
    REGALT.ZAER4(2) = 0;
    % Scale Region 4 atmospheric level aerosol densities by SCALE4.
    REGALT.SCALE4   = 0;
end
% PHASEFN allows to input a user-defined phase function when IPH=1
% The PHASEFN structure contains: NANGLS, NWLF, ANGF, WLF and AERPF
% NANGLS selects the number of scattering phase function grid points (>0).
if IPH==1
    PHASEFN.NANGLS = numel(aeraux.ANGF);
    % NWLF selected the number of scaterring phase function spectral wavelength
    % grid points (>0)
    PHASEFN.NWLF = numel(aeraux.VARSPC);
    % ANGF contains monotonically increasing scaterring angle array (decial
    % degrees) with NANGLS elements
    PHASEFN.ANGF = aeraux.ANGF;
    % WLF contains monotonically increasing wavelength array (um) with NWLF
    % elements
    PHASEFN.WLF = aeraux.VARSPC;
    % AERPF contains unit-normalized scattering phase function values (1/sr).
    % The 1D array is ordered with [(PF for 4 regions) over the angles] over
    % the spectral grid
    PHASEFN.AERPF = aeraux.F; % REVISAR!!
end
% IREGSPC[4] Required input for user-defined aerosols and clouds, i.e. if
% IHAZE=7; if ARUSS='USS'; or if ICLD=CLOUD_USER_DEFINED. The 4 members are
% defined for the 4 aerosol altitude regions.
if IHAZE==7
    % IREG The number of the aerosol region for which data is being defined
    % In ALG we only modify aerosol properties in the boundary layer
    % (IREG=1)
   IREGSPC(1).IREG=1;
   IREGSPC(2).IREG=0;
   IREGSPC(3).IREG=0;
   IREGSPC(4).IREG=0;
   % AWCCON Conversion factor (km g / m3) from extinction coefficient 
   % (km-1) to equivalent liquid water content (g/m3), only for fog 
   % extinction at microwave wavelengths (> 287.9?m, wavenumber
   % < 37.4 cm-1). It is numerically equal to the equivalent liquid water
   % content corresponding to extinction coefficient of 1.0 km-1, at
   % wavelength of 0.55 um.
   IREGSPC(1).AWCCON = 0;
   % AERNAM Name for IREG aerosol region.
   IREGSPC(1).AERNAM = 'User-defined aerosol model';
   % NARSPC Number of aerosol spectral grid points. If NARSPC=0 then use
   % the 47 default spectral grid points (see MODTRAN6 manual)
   IREGSPC(1).NARSPC = numel(aeraux.VARSPC);
   % VARSPC User-supplied wavelengths (um). If ommited or NARSPC=0 then use
   % the 47 built-in values; cannot be omitted if NARSPC exceeds zero. 
   IREGSPC(1).VARSPC = aeraux.VARSPC;
   % EXTC and ABSC are the NARSPC extinction and absorption coefficients 
   % [km-1], normalized so that value at lambda=0.55 um is 1/km.
   % ASYM are the NARSPC asymmetry parameters. IPH must be set to 2 to
   % insure that the asymmetry factor will not be overwritten
   if ~isempty(aeraux.RH) % Only defined for OPAC
       IREGSPC(1).EXTC = interp1(aeraux.RH,aeraux.EXTC',RH(N),'linear','extrap');        
       IREGSPC(1).ABSC = interp1(aeraux.RH,aeraux.ABSC',RH(N),'linear','extrap');
       IREGSPC(1).ASYM = interp1(aeraux.RH,aeraux.ASYM',RH(N),'linear','extrap');
   else % From user-defined aerosols (ATK): no RH dependent
       IREGSPC(1).EXTC = aeraux.EXTC';
       IREGSPC(1).ABSC = aeraux.ABSC';
       IREGSPC(1).ASYM = aeraux.ASYM';
   end
end

%--------------------------------------------------------------------------
%--------------------------------- CLOUDS ---------------------------------
%--------------------------------------------------------------------------
% ICLD specifies the cloud/rain models to be used among:
% No cloud or precipitation (0), Cumulus (1), Altostratus (2), Stratus (3),
% Stratus/stratocumulus (4), Nimbostratus (5)
% 6 - 2 mm/hr ground drizzle (stratus cloud and 0.86 mm/h at 1km)
% 7 - 5 mm/hr ground light rain (nimbostratus cloud and 2.6 mm/h at 0.66km)
% 8 - 12.5 mm/hr ground moderate rain (nimbostratus cloud and 6 mm/h at 0.66km)
% 9 - 25 mm/hr ground heavy rain (cumulus cloud and 0.2 mm/h at 3km)
% 10 - 75 mm/hr ground extreme rain (cumulus cloud and 1 mm/h at 3km)
% 11 - Read in user-defined cloud extinction coefficient, absorption 
%      coefficient, and asymmetry factor data. Triggers reading data for up
%      to 4 altitude regions (Option retained for backward compatibility; 
%      other input options afford greater flexibility in specifying 
%      user-defined clouds
% 18 - Standard Cirrus model (64 ?m mode radius for ice particles
% 19 - Sub-visual Cirrus model (4 ?m mode radius for ice particles).
ICLD = 0;
% RAINRT is the rain rate in [mm/hr]. The default value is 0 for no rain, 
% otherwise constant rate from ground to top of cloud when cloud is present
% and from ground to 6km when no cloud is defined.
RAINRT = 0;
if ICLD>=1 && ICLD<=10
    % CTHIK User-defined thickness. If CTHICK<=0 then use default value 
    % (see MODTRAN6 manual)
    CTHIK = 0;
    % CALT Cloud base altitude relative to ground level. If CALT<=0 then
    % use default value (see MODTRAN6 manual)
    CALT = 0;
    % CWAVLN Reference wavelength for defining cloud vertical extinction 
    % (from 0.2 to 200 um). Otherwise uses default wavelength (0.55 um)
    CWAVLN = 0;
    % CEXT Cloud water particle extinction coefficient [km-1] at CWAVLN. 
    % CEXT>0 scales the extinction (and absorption) coefficient curves. The
    % ratio of the input optical depth (CEXT*CTHIK) to the optical depth 
    % calculated from the product of column density and extinction 
    % coefficient at CWAVLN, summed for both liquid water droplets and ice
    % particles, is determined. The extinction and absorption coefficients
    % at all frequencies are multiplied by this ratio
    % CEXT<=0 Do not scale extinction coefficients. See MODTRAN6 manual for
    % default cloud extinctions at 0.55 um for each of the five MODTRAN 
    % liquid water droplet model clouds.
    CEXT = 0;
    % CCOLWD Cloud liquid water droplet vertical column density [km g/m3]. 
    % CCOLWD<=0 Use default value (no scaling of densities).
    CCOLWD = 0;
    % CCOLIP Cloud ice particle vertical column density [km g/m3]. If both 
    % CCOLIP and CCOLWD are zero, there is no scaling for either water or 
    % ice amount. CCOLIP<=0 use default value (no scaling of densities).
    CCOLIP = 0;
    % CHUMID from 0 to 105 is the cloud/rain relative humidity(RH) [%] at 
    % all layer boundaries with positive rain rate or positive cloud 
    % density; 5% supersaturation is permitted. If >105 then the relative
    % humidity profile defined in the absence of the cloud is not modified
    % due to the presence of cloud. If <0 Assume 100% relative humidity at
    % cloud/rain layer boundaries.
    CHUMID = -1;
    % ASYMWD from -1 to +1 is the water droplet Henyey-Greenstein 
    % wavelength-independent scattering phase function asymmetry factor. 
    % Use this value and ignore any spectral asymmetry input. Otherwise use
    % user-defined or model spectral asymmetry factors for scattering by 
    % cloud liquid water droplets.
    ASYMWD = 0;
    % ASYMIP from -1 to +1 is the ice particle Henyey-Greenstein 
    % wavelength-independent scattering phase function asymmetry factor. 
    % Otherwise use user-defined or model (standard cirrus) spectral 
    % asymmetry factors for scattering by cloud ice particles.
    ASYMIP = 0;
elseif ICLD==18 || ICLD==19
    CTHICK = 0;
    % CALT User-defined cirrus cloud base altitude relative to sea level 
    % (km). If CALT<=0 then use default value for cirrus cloud base altitude
    CALT = 0;
    % CEXT User-defined cirrus cloud extinction coefficient (km-1) at 
    % 0.55 um. If CEXT<=0 use 0.14 km-2 * CTHIK for cirrus cloud extinction
    % coefficient (km-1) at 0.55 um
    CEXT = 0;
end
% CLDALT For user-defined cloud and rain profiles for ICLD equals 1-10
if ICLD>=1 && ICLD<=10
    % Number of user-defined cloud and rain profile altitudes. If 0 then no
    % user-defined cloud or rain profiles
    CLDALT.NCRALT = 0;
    % ZPCLD If MODEL does not equal ATM USER PRESS PROFILE: ZPCLD array 
    % contains monotonically increasing above ground level (AGL) altitudes 
    % [km]. The first altitude level can be 0., i.e., the ground; this is 
    % necessary if rain is to be modeled at the ground or if the cloud is 
    % to sit on the ground. The highest altitude cannot exceed the top of
    % atmosphere (TOA). If MODEL equals ATM USER PRESS PROFILE: ZPCLD
    % contains a monotonically decreasing array of AGL pressure levels 
    % [mbar].
    CLDALT.ZPCLD = 0; % REVIEW!
    % CLD Liquid water droplet density profile [g/m3]. The densities are
    % modeled to vary linearly between altitudes. The entire CLD array is
    % scaled if CCOLWD is non-negative
    CLDALT.CLD = 0;
    % CLDICE Ice particle density profile [g/m3]. The densities are modeled
    % to vary linearly between altitudes. The entire CLDICE array is scaled
    % if CCOLIP is non-negative.
    CLDALT.CLDICE = 0;
    % RR Rain rate array [mm/hr] profile. If a rain rate is entered through
    % RAINRT, that constant rain rate supersedes this array of values. 
    % Thus, if a user-defined rain rate profile is desired, RAINRT must be 
    % omitted
    CLDALT.RR = 0;
end
% CLDSPC For user-defined cloud optical properties for ICLD equals 1  10.
if ICLD>=1 && ICLD<=10
    % NCRSPC If =0 no cloud user-defined spectral data. If =1, cloud data
    % entered from external file with CFILE. If >1, number of spectral grid
    % points in used-defined cloud optical data
    CLDSPC.NCRSPC = 0;
    % WAVLEN Monotonically increasing spectral grid of wavelengths in um. 
    % If a positive vertical cloud extinction, CEXT is input, the reference
    % wavelength, CWAVLN (Table 3.34) or 0.55 um, must be within the range 
    % of wavelength
    CLDSPC.WAVLEN = 0.55;
    % EXTC6 Liquid water droplet extinction coefficients [km-1 m3 / g]. If
    % omitted, then wavelength-interpolated extinction coefficients from
    % cloud model ICLD.
    % ABSC6 if >0 liquid water droplet absorption coefficients [km-1 m3/g].
    % If between -1 and 0, liqueid zqter droplet scattering albedos minus
    % 1. If <-1 or ABSC6(I) > EXTC6(I), then ABSC6(I) is calculated by 
    % first determining the default absorption to extinction ratio for
    % cloud model ICLD, and then multiplying EXTC6(I) by this ratio, i.e., 
    % the liquid water model cloud single scatter albedo is used to 
    % determine the absorption coefficient
    CLDSPC.ABSC6 = 0.5; % REVIEW!!
    % ASYM6 between -1 and +1 Henyey-Greenstein scattering phase function
    % asymmetry factors for liquid water droplets. Otherwise assigned the 
    % wavelength-interpolated value from cloud model ICLD
    CLDSPC.ASYM6 = 0;
    % EXTC7, ABSC7 and ASYM7. Same as above but for ice particles
    CLDSPC.ABSC7 = 0.5; % REVIEW!!
    CLDSPC.ASYM7 = 0;
    % CFILE If NCRSPC = 1, cloud spectral data full path file name
    if CLDSPC.NCRSPC==1, CLDSPC.CFILE='filename'; end
    % CLDTYP If NCRSPC = 1, name of first cloud from CFILE (usually a water
    % cloud)
    if CLDSPC.NCRSPC==1, CLDSPC.CLDTYP='water_cloud_name'; end
    % CIRTYP If NCRSPC = 1, name of second cloud from CFILE (usually an ice
    % cloud)
    if CLDSPC.NCRSPC==1, CLDSPC.CIRTYP='ice_cloud_name'; end
end

%--------------------------------------------------------------------------
%--------------------------------- GEOMETRY -------------------------------
%--------------------------------------------------------------------------
% ----------- Line of sight geometry
% ITYPE Geometric type for the atmospheric line-of-sight (LOS) path, as
% follows: (1) Horizontal (constant-pressure) path, i.e., flat Earth
% constant altitude path; (2) Vertical or slant path between two altitudes;
% (3) User-defined Line-Of-Sight (LOS) (see MODTRAN6 manual)
ITYPE = 3;
% H1ALT Sensor/observer altitude [km].
if isempty(flag) || strcmp(flag,'toa')
    H1ALT = H1ALT;
elseif strcmp(flag,'boa')
	H1ALT = GNDALT;
end
% H2ALT Tangent height for ITYPE = 3 limb paths; otherwise, path final 
% altitude [km].
H2ALT = 0;
% OBSZEN Sensor/observer zenith angle (0-180), measured at H1ALT [].
OBSZEN = 180-OBSZEN;
% For Earth-surface pointing (H2ALT=0), only H1ALT and OBSZEN are used

% HRANGE LOS path length [km]. Not used if ITYPE=3
HRANGE = 0;
% BETA Earth center angle subtended by LOS (0-180) [].
BETA = 0;
% LENN (0) Short path, used when there is an ambiguity or (1) Long path
% passing through a tangent height.
LENN = 0;
% BCKZEN Zenith angle at final altitude (H2ALT) for reverse path towards
% H1ALT (0-180) [].
BCKZEN = 0;
% The MLOS JSONarray defines a LOSGEOMETRY structure for each LOS after the
% first, with associated types, defaults, descriptions and units.
% (we are not using it in ALG)
% ----------- Additional geometry parameters
% RAD_E Radius of the Earth. Omit or 0 to set by MODEL selection (6378.39 
% km for tropical, 6353.91 km for sub-arctic, 6371.23 km for others)
RAD_E = 0;
% CKRANG Slant range for k-distribution output (when CKPRNT is TRUE), 
% interpreted as follows: If 0 then Output k-dependent data only at full 
% slant range (minimum output), if >0 then output k-dependent data at all
% path boundary intersections up to the first one exceeding CKRANG and at 
% the total path slant range, if <0 Output k-dependent data at all path 
% boundary intersections (maximum output)
CKRANG = 0;
% IDAY Day of the year, used to correct for variation in the Earth-to-Sun 
% distance and to define solar position when ephemeris data is to used.
% Omit or 0 to chooses IDAY = 93 as the day of year, the average 
% Earth-to-Sun distance.
IDAY = 0;
% ----------- Illumination geometry
% IPARM Solar/lunar geometry specification parameters for solar/lunar 
% scattering. 
IPARM = 12;
% PARM1 depends on IPARM. If IPARM=0 or 1 then PARM1 is the sensor/observer
% latitude (-90 to 90 deg, North of Equator). If IPARM=2, then PARM1 is the
% sensor/observer relative solar azimuth (0-360 deg). If IPARM=10 or 11 
% then PARM1 is the final altitude latitude (-90 to 90 deg, North of 
% Equator). If IPARM=12 then PARM1 is the final altitude relative solar
% azimuth ()-360 deg
PARM1 = PARM1;
% PARM2 depends on IPARM. If IPARM=0 or 1 then PARM2 is the sensor/observer
% longitude (0-360 deg, West of Greenwich). If IPARM=2 then PARM2 is the
% sensor/observer solar zenith angle (0-180 deg). If IPARM=10 and 11 then
% PARM2 is the final altitude longitude (0-360 deg, West of Greenwich). If
% IPARM=12 then PARM2 is the final altitude solar zenith angle (0-180 deg)
PARM2 = PARM2;
% PARM3 is the extraterrestrial source (sun or moon) latitude in deg North.
% Only used if IPARM is 0 or 10
PARM3 = 0;
% PARM4 is the extraterrestrial source (sun or moon) longitude in deg West
% of Greenwich. Only used if IPARM is 0 or 10
PARM4 = 0;
% GMTIME is the Universal Time in decimal hours (0 to 24), i.e., 8:45 a.m. 
% is 8.75, 5:20 p.m. is 17.33, etc. Only used if IPARM is 1 or 11
GMTIME = 10.5;
% TRUEAZ is the true path azimuth in deg East of North (0 to 360). Only 
% used if IPARM is 0, 1 (from sensor to final altitude), 10 or 11 (from 
% final altitude to sensor)
TRUEAZ = 0;
% ANGLEM Phase angle of the moon (0-180 deg, 0 for full moon and 90 for 
% half moon), required for IEMSCT = RT_LUNAR_...; Not available if IPARM is
% 1 or 11
ANGLEM = 0;

%--------------------------------------------------------------------------
%--------------------------------- SURFACE --------------------------------
%--------------------------------------------------------------------------
% SURFTYPE indicated the method to describe ground surface. The following
% options are available: (0) spectrally constant Lambertian reflectance,
% (1) Lambertian spectral reflectance from external file, (2) Lambertian
% spectral reflectance from JSON input file (not yet implemented) or (3)
% Spectral bi-directional reflectance distribution function (BRDF)
SURFTYPE = 0;
% SURREF Constant surface reflectance (0 to 1) if SURFTYPE=0
SURREF = rho;
% NSURF: (1) Imaged-pixel and area-averaged surfaces modeled as the same, a
% single surface type. (2) Imaged-pixel and area-averaged surfaces modeled 
% as distinct, two surface types
NSURF = 1;
% TPTEMP surface temperature with the following options:
% If <=0 then no surface emission unless H2ALT is the ground altitude. If
% the path intersects the Earth, code uses the temperature of the first 
% atmospheric level (the surface air temperature) as the boundary
% temperature. If AATEMP (the next entry) is omitted and the path 
% intersects the Earth, the temperature of the first atmospheric level is 
% also used as the lower boundary temperature for the multiple scattering
% models.
% If>0 then it is the boundary temperature of image pixel (i.e., at final
% altitude), used in radiance mode for slant paths that intersect the Earth
% OR terminate at a gray boundary such as a cloud or a target. If AATEMP is
% omitted and the LOS intersects the Earth, TPTEMP is also used as the
% lower boundary temperature for the multiple scattering models 
TPTEMP = 0;
% AATEMP indicates the area-averaged ground surface temperature. Two
% options are possible:
% If <=0 the AATMP, used to define the lower boundary condition in the 
% multiple  scattering algorithms, is set to the value of TPTEMP.
% If >0 then this is the value of the area-averaged ground surface 
% temperature (K) used to define the lower boundary condition in the 
% thermal multiple scattering calculations.
AATEMP = 0;
% WIDERP defines the ground from which forward scattered photons are 
% emitted or reflected: modeled with the area-averaged surface (false) or 
% modeled with the imaged-pixel surface (true)
WIDERP = false;
% GNDALT surface altitude. If 0, then surface at sea level, but set to the 
% first profile altitude when user-defined profile (radiosonde) data is
% used, i.e. if MODEL is ATM_USER_ALT_PROFILE. If <=6 is the altitude in km
% which may be negative. The baseline 0 to 6 km aerosol profiles are 
% compressed (or stretched) based on input GNDALT.
GNDALT = GNDALT;
% DH2O sets the liquid water layer thickness (in mm) used to model a
% shallow "pool" of water on surface (not functioning propertly at this
% time). Omit with 0
DH2O = 0;
% MLTRFL uses the embedded surface moisture attenuation model (false) or
% invokes a surface water layer model (true)
MLTRFL = false;
% SALBFL uses the default spectral albedo file, DATA/spec_alb.dat (omit) or
% sets the name of the user-supplied spectral albedo data file, which must
% conform to the format described in the default file (currently, not a 
% MODTRAN6 GUI option)
% SURFP and SURFA are structures describing respectively the imaged-pixel 
% and area-averaged Lambertian or BRDF surfaces. Only working when 
% SURFTYPE=1 (Lambertian) or 3 (BRDF). The structure contains the following
% elements:
% CSALB. Name or number of the Lambertian model (see MODTRAN6 user manual)
SURFP.CSALB = 1;
SURFA.CSALB = 1;
% SURFZN indicates the zenith angle of the surface normal, currently the 
% only allowed value is 0.
SURFP.SURFZN = 0;
SURFA.SURFZN = 0;
% SURFAZ sets the true azimuth angle of the image pixel surface normal in 
%  [0 for N, 90 for E, 180 for S, and 270 for W], currently not used.
SURFP.SURFAZ = 0;
SURFA.SURFAZ = 0;
% CBRDF indicates the BRDF models (see MODTRAN6 User manual for details).
% Omit is surface is Lambertian
% NWVSRF Number of surface spectral grid points for BRDF definition. Ommit
% if surface is Lambertian
% WVSURF Monotonically increasing spectral wavelength grid in um for BRDF.
% Omit if surface is Lambertian
% PBRDF1 to PBRDF7 are BRDF parameters at WVSURF wavelengths. Each requires
% input of the number of spectrally-dependent parameters listed in the 
% third column of Table 3.47 (see MODTRAN6 User manual). Omit if surface if
% Lambertian
% UDSALB User-defined spectral Lambertian, currently not implemented
% SURFNLOS=0 then multiple Line-of-Sight (MLOS) option not being used. If 
% SURFNLOS>0 then Surfaces for multiple lines-of-sight will be defined
% using SURFLOS structure.
SURFNLOS = 0;

%--------------------------------------------------------------------------
%--------------------------------- SPECTRAL -------------------------------
%--------------------------------------------------------------------------
% LBMNAM If 'F', the 1.0 cm-1 band model data file is used unless the 
% line-by-line option has been selected; in that case, a 0.1 cm-1 band
% model line tail data is used. If LBMNAM equals 'T', BMNAME defines the 
% band model data file root name.
LBMNAM = 'T';
% BMNAME Band model file root name input if LBMNAM is 'T'. The five 
% available root names for files in the DATA/ directory are "01_2013",
% "05_2013", "15_2013", "p1_2013"and "LBL2013". The first four of these are
% the 1.0 (default), 5.0, 15.0 and 0.1 cm-1 band models, respectively.
% "LBL2013t.asc" and "LBL2013t.bin" contain 0.1 cm-1 line tail ASCII and
% binary data, respectively, for the MODTRAN line-by-line algorithm.
if strcmp(LBMNAM,'T')
    aux = str2cell(SpeConf.BMNAME.values(2:end-1));
    if iscell(aux), BMNAME = fliplr(aux); else BMNAME = aux; end
end
% V1 Spectral bandpass lowerbound (cm-1, um or nm)
v2 = fliplr(str2num(SpeConf.MaxLambda.values)); % In [nm] from ALG toolbox
V1 = 1e7./v2; % Converted into [cm^-1]
% V2 Spectral bandpass upperbound (cm-1, um or nm)
v1 = fliplr(str2num(SpeConf.MinLambda.values)); % In [nm] from ALG toolbox
V2 = 1e7./v1; % Converted into [cm^-1]
auxBMNAME = strrep(BMNAME,'p1','.1');
if numel(V2)~=1
    % Update wavelengths if spectral ranges are overlapped
    for i=2:numel(V1)
        if V1(i)==V2(i-1) && str2num(auxBMNAME{i}(1:2))>=str2num(auxBMNAME{i-1}(1:2)) 
            V1(i) = V1(i) + str2num(auxBMNAME{i}(1:2));
        elseif V1(i)==V2(i-1) && str2num(auxBMNAME{i}(1:2))<str2num(auxBMNAME{i-1}(1:2)) 
            V2(i-1) = V2(i-1) - str2num(auxBMNAME{i-1}(1:2));
        end
    end
end 
% DV Frequency (cm-1) or wavelenght (um or nm) increment used for spectral
% outputs. DV applies to all spectral output files when using the default
% slit function (i.e. when FLAGS[0:3] is blank). Otherwise, DV is only 
% applied to the.7sc, .psc, .flx, *_scan.csv, *_scan.txt and *_scan.sli 
% output files. The frequency increment of other files (such as .tp6, .tp7,
% .tp8, .plt, .acd, .csv, .txt, .flx, *_flux.csv, and *_flux.sli) is set to
% the band model spectral resolution, 0.1, 1.0, 5.0 or 15.0 cm-1. The API 
% spectral output is a replica of the *.csv output. To avoid under-sampling
% of output spectra, DV should not exceed FWHM. The recommended value for 
% DV is FWHM/2, i.e., Nyquist sampling.
if numel(V2)==1
    DV = str2num(auxBMNAME(1:2));
else
    for i=1:numel(V2)
        DV(i) = str2num(auxBMNAME{i}(1:2));
    end
end
% FWHM Slit function Full Width at Half Maximum (FWHM) frequency (cm-1) or
% wavelength (um or nm). For band model calculations, FWHM must not exceed 
% 50 times the calculation bin size (0.1, 1, 5 or 15 cm-1). The type of 
% slit function is defined by FLAGS[1]. A minimum of twice the bin size 
% will insure proper sampling (Nyquist sampling). No convolution is 
% performed if FWHM equals the band model bin size and the default 
% triangular slit function (FLAGS[1] set to blank), is selected.
FWHM = DV;
% YFLAG Transmittances ("T") or radiances ("R") in output files .plt and .psc
YFLAG = 'R';
% XFLAG Abscissa spectral unit in plot output file .psc if its generated 
% ("W" = wavenumber, "M" = microns or "N" = nanometers); otherwise, it is 
% the abscissa spectral unit in plot output file .plt. For "W", the 
% radiance unit is W cm-2 sr-1 / cm-1 and the irradiance unit is 
% W cm-2 / cm-1; for "M", the radiance unit is W cm-2 sr-1/um and the 
% irradiance unit is W/cm2/um; for "N", the radiance unit is 
% uW cm-2 sr-1 / nm and the irradiance unit is uW cm-2 sr-1 / nm
XFLAG = 'N'; 
% DLIMIT Delimiter used to separate .plt and .psc output from repeat 
% (sequential) runs
DLIMIT = ' ';
% FLAGS[0:6] (7-element string)
% FLAGS[0] Defines the spectral units for input parameters V1, V2, DV and 
% FWHM and for output files .7sc, .psc, *_scan.csv, *_scan.txt, .flx and 
% *_scan.sli. 'W' or blank (default) for spectral frequency bandpass in 
% wavenumbers. Radiances are in W cm-2 sr-1 / cm-1. 'M' Spectral wavelength
% bandpass in microns. Radiances are in W cm-2 sr-1 / um. And 'N' Spectral
% wavelength bandpass in nanometers. Radiances are in uW cm-2 sr-1 / nm
% FLAGS[1] Selects the slit function as described by the following 
% functional forms (see MODTRAN6 user manual for details): 'T' or blank for
% triangular, 'R' for rectangular, 'G' for gaussian, 'S' for Sinc, 'C' for
% Sinc2, 'H' for Hamming or 'U' for user-supplied (currently unavailable)
% FLAGS[2] specifies whether the FWHM is absolute ('A') or relative ('R'),
% in which case FWHM=100dv/v=100dLambda/Lambda
% FLAGS[3] blank to degrade only total radiance and transmittance or 'A' to
% degrate all output file components
% FLAGS[4] Should always be blank; the original option is obsolete
% FLAGS[5] Should always be blank; the original option is obsolete.
% FLAGS[6] blank for not outputting the spectral flux table. 'T' to write a
% .flx file using no more than 80 characters per line in spectral flux
% table (i.e., include line feeds for each spectral point). These files can
% be quite large unless input MLFLX (see below) is used to limit the number
% of atmospheric levels (altitudes). The output data is spectrally-gridded 
% based on the input value of DV. 'F' to write a .flx file with all 
% altitude dependent flux values for each spectral point on a single line 
% (i.e., there are no line feeds). These files can be quite large unless 
% input MLFLX (see below) is used to limit the number of atmospheric levels
% (altitudes). Some FORTRAN compilers limit the number of characters per 
% line; this input can cause this limit to be exceeded.
FLAGS = 'WTA    ';
% MLFLX If FLAGS[6] is not blank, write spectral flux data to .flx at every
% altitude level (If MLFLX exceeds 0, values are only written for the first
% MLFLX altitude levels and for the top-of-atmosphere)
if ~strcmp(FLAGS(7),' '), MLFLX = 0; end
% VRFRAC Reference spectral frequency for refraction calculation (cm-1) if 
% VRFRAC exceeds zero; the bandpass central frequency is used if it equals 
% zero
VRFRAC = 0;
% SFWHM If positive, the TOA solar irradiance is degraded to this 
% full-width at half-maximum (FWHM) using a triangular slit function. The
% following options are possible:
% If 0, then 0.1 cm-1 resolution band model uses the 
% SUNp1med2irradwnNormt.asc data. Other resolution band models use 
% SUN01med2irradwnNormt.dat degraded to 5 cm-1 spectral resolution.
% If >0 the FWHM in wavenumbers of the triangular scanning function used to
% smooth TOA solar irradiance data.
% If <0 Spectral resolution set to |SFWHM| with spectrally convolved data
% output to a local data file. The data file is given the name 
% s0_xxxx.dat where xxxx is int(SFWHM) unless a 0.1 cm-1 band model 
% calculation is performed, in which case, xxxx is int(10?|SFWHM|). If 
% repeat runs are performed with SFWHM unchanged, only the last output file
% is saved.
SFWHM = 0;
% LSUNFL. If 'F' Top-of-atmosphere (TOA) solar irradiance file dependent on
% the band model resolution (SUNxxmed2irradwnNormt.dat for xx being 15, 05,
% 01 or p1 respectively for 15, 5, 1 and 0.1 cm-1). If  'T' User specifies
% top-of-atmosphere (TOA) solar irradiance file name in USRSUN.
% If '1' to ' D', specific solar irradiance file (see MODTRAN6 user manual)
if LSUNFL>=1
    LSUNFL = num2str(LSUNFL);
else
    % Only for options A, B, C, D (LSUNFL < in ALG)
    auxLSUNFL = {'A','B','C','D'};
    LSUNFL = auxLSUNFL{abs(LSUNFL)};
end
% USRSUN Name of solar irradiance file input if LSUNFL is 'T'.
if strcmp(LSUNFL,'T')
    URSSUN = 'sun_irradiance_file.txt';
end
% FILTNM User-supplied filter function name. Sample spectral response
% function data are provided in the DATA/ directory for the AIRS, ASTER, 
% AVIRIS, GOES, HyspIRI, LANDSAT-5, LANDSAT-7, LANDSAT-8, MODIS,
% CERES-PFM, ITRES-TASI and the Yankee MSR-7 spectrometers.
FILTNM = ' ';
% CH2OCM Mlawer, Tobin, Clough, Kneizys, Davies (MT-CKD) 2.5 water
% continuum [set to 0 for Clough, Kneizys, Davies (CKD) 2.4 water continuum
CH2OCM = '1';

%--------------------------------------------------------------------------
%------------------------------- FILEOPTIONS ------------------------------
%--------------------------------------------------------------------------
% NOFILE. (0) generates all legacy output files (.tp6, .tp7, .tp8, .flx, 
% etc.). (1) only generates the <ROOTNAME>.tp6 legacy output file. (2)
% generates no legacy output files.
NOFILE = 0;
% BINARY false to output files .tp7, .tp8 and .plt in an ASCII format or
% true to generate these files in a binary format. ASCII equivalent files
% are generated from the binary files by running the auxiliary program
% bin2as_mod6, located in the bin directory
BINARY = false;
% CKPRNT false to no output of cumulative path transmittances and radiances
% for Ck or LBL. True: If the Correlated-k option is on, k-distribution
% dependent cumulative path transmittances and radiances are written to 
% .t_k and .r_k output files, respectively. The data is also written to 
% output files *_highres.csv, *_highres.csv and *_highres.sli if CSVPRNT 
% and SLIPRNT are set appropriately. If the LBL option is on, the same 
% files are created and contain the high spectral resolution line-by-line 
% data instead of the k-dependent data. For either Ck or LBL, if CKRANG is 
% used, the k-dependent data is output as a function of slant range (see 
% input instructions for CKRANG in Table 3.40 of the Geometry section for 
% further details). If the BINARY output option is selected, the data is 
% written to files *_b.t_k and *_b.r_k instead of .t_k and .r_k, 
% respectively. Binary files can be translated to ASCII files by running 
% the utility bin2as_mod5_k located in the bin directory.
CKPRNT = false;
% NOPRNT: (0) for normal content fo .tp6. (1) to create .tp6 but omit model
% atmosphere profiles (unless special options such as user-defined profiles
% is selected). (2) to create .tp6 but omit spectral data and model 
% atmosphere profiles. (3) to delete .tp6 at end of processing if run is 
% successful. (-1) to Create additional .tp8 output, including 
% line-of-sight spectral fluxes in radiation mode when multiple scattering 
% on. These values are generated at the band model spectral bin resolution,
% 0.1, 1.0, 5.0 or 15.0 cm-1. Spectral flux values convolved with the 
% instrument slit function are output to .flx if FLAGS[6] is not left 
% blank. Be warned that setting NOPRNT to 1 for long paths (e.g., ground
% to space) over a large spectral range (e.g., 0.4 to 0.7 um) will generate
% large .tp8 files. (-2) to generate spectral cooling rate data in a .clr 
% file. .tp8 output is also generated
NOPRNT = NOPRNT;
% DATDIR: Omit to use directory specified by environment variable 
% MODTRAN_DATA. Otherwise specified the string path to directory containing
% the MODTRAN data files, e.g. for a band model file whose name has been
% explicitly input.
% FLROOT Root name of MODTRAN output files, e.g., the string that replaces 
% <ROOTNAME> in .tp6. In the MODTRAN6 GUI, this is the Case Control Name 
% (TRAIN01, see User Manual).
% CSVPRNT File root name and extension for CSVtype output, default is " ".
% If the extension is .txt, tab delimited files are produced; if the 
% extension is .csv, output files are produce in a comma separated values
% format
% SLIPRNT File root name for ENVI spectral library output, default is " ".
% JSONPRNT File root name for JSON formatted output (currently unavailable)


%--------------------------------------------------------------------------
%---------------------- CARD 5 variables (obligatory) ---------------------
%--------------------------------------------------------------------------
% Repeat run option
% IRPT Mode of execution (repeat or not).
%  0  = Stop program
%  -1 = Read full set of new data cards
%  -3 = Read new LOS (and solar/lunar geometry) (CARD 3, 3A...) and surface
%  (CARD 4A...) inputs
%  -4 = Read new spectral and surface (CARD 4...) inputs (=4)
Nbands = numel(str2num(SpeConf.MinLambda.values)); % Number of spectral bands
if Nbands==1
    IRPT = 0;
else
    IRPT = [-1*ones(1,Nbands-1),0];
end


%% Write variables to to generate json file:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%                    WRITE VARIABLES ON JSON FILE                     %%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[~,fileName,~] = fileparts(file);
% Open file with write permission
fid = fopen(file,'w');
fprintf(fid,'{\n');
fprintf(fid,'\t"MODTRAN": [ {\n');

%%%%%%%%%%%%%% Repeat Nbands times
for i=1:Nbands
    % Open JSON MODTRANINPUT
    clear txt
    fprintf(fid,'\t"MODTRANINPUT":{\n');
    txt.NAME        = fileName;
    txt.DESCRIPTION = ['Case_',fileName,'_Band',num2str(i)];
    txt.CASE        = i-1;
    if i>1
        txt.CASETEMPLATE = 0;
    end
    txt          = jsonencode(txt);
    txt([1,end]) = [];
    txt = ['\t\t',txt];
    txt = strrep(txt,',',',\n\t\t');
    if i>1
        txt=strrep(txt,'CASETEMPLATE','CASE TEMPLATE');
    end
    fprintf(fid,[txt,',\n\t\t']);

    if i==1
        %----------------- RTOPTIONS
        clear txt
        % Print only if values are not those by default (see MODTRAN6 manual)
        if IEMSCT~=0, txt.RTOPTIONS.IEMSCT = IEMSCT; end
        if MODTRN~=0, txt.RTOPTIONS.MODTRN = MODTRN; end
        if LYMOLC,    txt.RTOPTIONS.LYMOLC = LYMOLC; end
        if T_BEST,    txt.RTOPTIONS.T_BEST = T_BEST; end
        if IMULT~=0,  txt.RTOPTIONS.IMULT  = IMULT; end
        if DISALB,    txt.RTOPTIONS.DISALB = DISALB; end
        if NSTR~=8,   txt.RTOPTIONS.NSTR   = NSTR; end
        if NLBL~=100, txt.RTOPTIONS.NLBL   = NLBL; end
        if SOLCON~=0, txt.RTOPTIONS.SOLCON = SOLCON; end
        if exist('txt','var')
            txt          = jsonencode(txt);
            txt([1,end]) = [];
            idx = strfind(txt,'"');
            txt = [txt(1:idx(3)-1),'\n\t\t\t',txt(idx(3):end)];
            txt = strrep(txt,',',',\n\t\t\t');
            fprintf(fid,txt);
            % Close RTOPTIONS:
            fprintf(fid,',\n\t\t');
        end

        %----------------- ATMOSPHERE
        clear txt
        % Print only if values are not those by default (see MODTRAN6 manual)
        if MODEL~=2, txt.ATMOSPHERE.MODEL = MODEL; end
        if M1~=0,    txt.ATMOSPHERE.M1 = M1; end
        if M2~=0,    txt.ATMOSPHERE.M2 = M2; end
        if M3~=0,    txt.ATMOSPHERE.M3 = M3; end
        if M4~=0,    txt.ATMOSPHERE.M4 = M4; end
        if M5~=0,    txt.ATMOSPHERE.M5 = M5; end
        if M6~=0,    txt.ATMOSPHERE.M6 = M6; end
        if M2_RHC,   txt.ATMOSPHERE.M2_RHC = M2_RHC; end
        if MDEF~=0,  txt.ATMOSPHERE.MDEF   = MDEF;   end
        if MODEL==7
            txt.ATMOSPHERE.NLAYERS = NLAYERS; 
            txt.ATMOSPHERE.NPROF   = NPROF;
            for j=1:NPROF
                txt.ATMOSPHERE.PROFILES(j).TYPE = TYPE(j);
                txt.ATMOSPHERE.PROFILES(j).UNITS = UNITS(j);
                if j==1
                    txt.ATMOSPHERE.PROFILES(j).PROFILE = PROFILE(:,1);
                end
                if isstruct(auxMODEL)
                    if ~isstring(PROFILE(:,j))
                        txt.ATMOSPHERE.PROFILES(j).PROFILE = PROFILE(:,j);
                    else
                        txt.ATMOSPHERE.PROFILES(j).PRO_MASK = PRO_MASK{j};
                    end
                end
                if H>0 || Z>0
                    txt.ATMOSPHERE.PROFILES(j).PROFILE = PROFILE(:,j);
                end
            end
        end
        if CO2MX~=0 && CO2MX~=400, txt.ATMOSPHERE.CO2MX = CO2MX; end
        if (H2OSTR>=0 && strcmp(H2OUNIT,'g')) || (H2OSTR>0 && strcmp(H2OUNIT,' '))
            txt.ATMOSPHERE.H2OSTR = H2OSTR;
            txt.ATMOSPHERE.H2OUNIT = H2OUNIT;
        end
        if (O3STR>=0 && strcmp(O3UNIT,'a')) || (O3STR>0 && strcmp(O3UNIT,' '))
            txt.ATMOSPHERE.O3STR = O3STR;
            txt.ATMOSPHERE.O3UNIT = O3UNIT;
        end
        if C_PROF~=0
            txt.ATMOSPHERE.C_PROF = C_PROF;
            if C_PROF==1 || C_PROF==3 || C_PROF==5 || C_PROF==7
                txt.ATMOSPHERE.S_UMIX = S_UMIX;
            end
            if C_PROF==2 || C_PROF==3 || C_PROF==6 || C_PROF==7
                txt.ATMOSPHERE.S_XSEC = S_XSEC;
            end
            if C_PROF==4 || C_PROF==5 || C_PROF==6 || C_PROF==7
                txt.ATMOSPHERE.S_TRAC = S_TRAC;
            end
        end
        if AERRH>0,  txt.ATMOSPHERE.AERRH  = AERRH; end
        if AYRANG,   txt.ATMOSPHERE.AYRANG = AYRANG; end
        if MODEL==8
            if E_MASS>0, txt.ATMOSPHERE.E_MASS = E_MASS; end
        end
        if AIRMWT>0, txt.ATMOSPHERE.AIRMWT = AIRMWT; end

        if exist('txt','var')
            txt          = jsonencode(txt);
            txt([1,end]) = [];
            idx = strfind(txt,'"');
            txt = [txt(1:idx(3)-1),'\n\t\t\t',txt(idx(3):end)];
            txt = strrep(txt,',',',\n\t\t\t');
            fprintf(fid,txt);
            % Close ATMOSPHERE:
            fprintf(fid,',\n\t\t');
        end

        %----------------- AEROSOLS
        clear txt
        % Print only if values are not those by default (see MODTRAN6 manual)
        if IHAZE~=0, txt.AEROSOLS.IHAZE = IHAZE; end
        txt.AEROSOLS.VIS = VIS;
        if WSS~=0, txt.AEROSOLS.WSS = WSS; end
        if WHH~=0, txt.AEROSOLS.WHH = WHH; end
        if IHAZE==3, txt.AEROSOLS.ICSTL = ICSTL; end
        if ISEASN~=0, txt.AEROSOLS.ISEASN = ISEASN; end
        if IVULCN~=0, txt.AEROSOLS.IVULCN = IVULCN; end
        if IPH~=2, txt.AEROSOLS.IPH = IPH; end
        if IPH==0, txt.AEROSOLS.HGPF = HGPF; end
        if H2OAER, txt.AEROSOLS.H2OAER = H2OAER; end
        if CNOVAM, txt.AEROSOLS.CNOVAM = CNOVAM; end
        if ~strcmp(ARUSS,'DEFAULT'), txt.AEROSOLS.ARUSS = ARUSS; end
        if IVSA
            txt.AEROSOLS.IVSA = IVSA;
            txt.AEROSOLS.ZCVSA = ZCVSA;
            txt.AEROSOLS.ZTVSA = ZTVSA;
            txt.AEROSOLS.ZINVSA = ZINVSA;
        end
        if ~strcmp(CDASTM,'F'), txt.AEROSOLS.CDASTM = CDASTM; end
        if ASTMX~=0
            txt.AEROSOLS.ASTMX = ASTMX;
            txt.AEROSOLS.ASTMC = ASTMC;
            txt.AEROSOLS.ASTMO = ASTMO;
        end
        if SSA>=0
            txt.AEROSOLS.SSALB.NSSALB = SSALB.NSSALB;
            txt.AEROSOLS.SSALB.AWAVLN = SSALB.AWAVLN;
            txt.AEROSOLS.SSALB.ASSALB = SSALB.ASSALB;
            if SSALB.NSSALB<0
                txt.AEROSOLS.SSALB.ACOALB = SSALB.ACOALB;
                txt.AEROSOLS.SSALB.RHASYM = SSALB.RHASYM;
            end
        end
        if strcmp(APLUS,'A+')
            txt.AEROSOLS.APLUS = APLUS;
            txt.AEROSOLS.REGALT.ZAER1 = REGALT.ZAER1;
            txt.AEROSOLS.REGALT.SCALE1 = REGALT.SCALE1;
            txt.AEROSOLS.REGALT.ZAER2  = REGALT.ZAER2;
            txt.AEROSOLS.REGALT.SCALE2 = REGALT.SCALE2;
            txt.AEROSOLS.REGALT.ZAER3  = REGALT.ZAER3;
            txt.AEROSOLS.REGALT.SCALE3 = REGALT.SCALE3;
            txt.AEROSOLS.REGALT.ZAER4  = REGALT.ZAER4;
            txt.AEROSOLS.REGALT.SCALE4 = REGALT.SCALE4;
        end
        if IPH==1
            txt.AEROSOLS.PHASEFN.NANGLS = PHASEFN.NANGLS;
            txt.AEROSOLS.PHASEFN.NWLF   = PHASEFN.NWLF;
            txt.AEROSOLS.PHASEFN.ANGF   = PHASEFN.ANGF;
            txt.AEROSOLS.PHASEFN.WLF    = PHASEFN.WLF;
            txt.AEROSOLS.PHASEFN.AERPF  = PHASEFN.AERPF;
        end
        if IHAZE==7
            for j=1:4
                txt.AEROSOLS.IREGSPC.IREG = IREGSPC(j).IREG;
                if j==1
                    txt.AEROSOLS.IREGSPC.AWCCON = IREGSPC(j).AWCCON;
                    txt.AEROSOLS.IREGSPC.AERNAM = IREGSPC(j).AERNAM;
                    txt.AEROSOLS.IREGSPC.NARSPC = IREGSPC(j).NARSPC;
                    txt.AEROSOLS.IREGSPC.VARSPC = IREGSPC(j).VARSPC;
                    txt.AEROSOLS.IREGSPC.EXTC = IREGSPC(j).EXTC;
                    txt.AEROSOLS.IREGSPC.ABSC = IREGSPC(j).ABSC;
                    txt.AEROSOLS.IREGSPC.ASYM = IREGSPC(j).ASYM;                
                end
            end
        end
        if ICLD~=0
            txt.AEROSOLS.ICLD = ICLD;
            if ICLD>=1 && ICLD<=10
                if CTHIK~=0
                    txt.AEROSOLS.CTHICK = CTHICK;
                    txt.AEROSOLS.CALT = CALT;
                    txt.AEROSOLS.CWAVLN = CWAVLN;
                    txt.AEROSOLS.CEXT = CEXT;
                    txt.AEROSOLS.CCOLWD = CCOLWD;
                    txt.AEROSOLS.CCOLIP = CCOLIP;
                    txt.AEROSOLS.CHUMID = CHUMID;
                    txt.AEROSOLS.ASYMWD = ASYMWD;
                    txt.AEROSOLS.ASYMIP = ASYMIP;
                end
                if CLDALT.NCRALT~=0
                    txt.AEROSOLS.CLDALT.NCRALT = CLDALT.NCRALT;
                    txt.AEROSOLS.CLDALT.ZPCLD = CLDALT.ZPCLD;
                    txt.AEROSOLS.CLDALT.CLD = CLDALT.CLD;
                    txt.AEROSOLS.CLDALT.CLDICE = CLDALT.CLDICE;
                    txt.AEROSOLS.CLDALT.RR = CLDALT.RR;
                end
                if CLDSPC.NCRSPC~=0
                    txt.AEROSOLS.CLDSPC.NCRSPC = CLDSPC.NCRSPC;
                    txt.AEROSOLS.CLDSPC.WAVLEN = CLDSPC.WAVLEN;
                    txt.AEROSOLS.CLDSPC.ABSC6 = CLDSPC.ABSC6;
                    txt.AEROSOLS.CLDSPC.ASYM6 = CLDSPC.ASYM6;
                    txt.AEROSOLS.CLDSPC.ABSC7 = CLDSPC.ABSC7;
                    txt.AEROSOLS.CLDSPC.ASYM7 = CLDSPC.ASYM7;
                    if CLDSPC.NCRSPC==1
                        txt.AEROSOLS.CLDSPC.CFILE = CLDSPC.CFILE;
                        txt.AEROSOLS.CLDSPC.CLDTYP = CLDSPC.CLDTYP;
                        txt.AEROSOLS.CLDSPC.CIRTYP = CLDSPC.CIRTYP;
                    end
                end
            elseif ICLD==18 || ICLD==19
                if CTHIK~=0
                    txt.AEROSOLS.CTHICK = CTHICK;
                    txt.AEROSOLS.CALT = CALT;
                    txt.AEROSOLS.CEXT = CEXT;
                end
            end

        end
        if RAINRT~=0, txt.AEROSOLS.RAINRT = RAINRT; end

        if exist('txt','var')
            txt          = jsonencode(txt);
            txt([1,end]) = [];
            idx = strfind(txt,'"');
            txt = [txt(1:idx(3)-1),'\n\t\t\t',txt(idx(3):end)];
            txt = strrep(txt,',',',\n\t\t\t');
            fprintf(fid,txt);
            % Close AEROSOLS:
            fprintf(fid,',\n\t\t');
        end

        %----------------- GEOMETRY
        clear txt
        % Print only if values are not those by default (see MODTRAN6 manual)
        if ITYPE~=1, txt.GEOMETRY.ITYPE = ITYPE; end
        if ITYPE==2
            if BCKZEN>0
                if HRANGE>0
                    txt.GEOMETRY.H2ALT = H2ALT;
                    txt.GEOMETRY.HRANGE = HRANGE;
                    txt.GEOMETRY.BCKZEN = BCKZEN;
                elseif H2ALT>H1ALT && BCKZEN>0
                    txt.GEOMETRY.H1ALT = H1ALT;
                    txt.GEOMETRY.H2ALT = H2ALT;
                    txt.GEOMETRY.LENN = LENN;
                    txt.GEOMETRY.BCKZEN = BCKZEN;
                else
                    txt.GEOMETRY.H1ALT = H1ALT;
                    txt.GEOMETRY.H2ALT = H2ALT;
                    txt.GEOMETRY.BCKZEN = BCKZEN;
                end
            elseif BETA>0
                txt.GEOMETRY.H1ALT = H1ALT;
                txt.GEOMETRY.H2ALT = H2ALT;
                txt.GEOMETRY.BETA = BETA;
            elseif HRANGE>0
                if OBSZEN>0
                    txt.GEOMETRY.H1ALT = H1ALT;
                    txt.GEOMETRY.OBSZEN = OBSZEN;
                    txt.GEOMETRY.HRANGE = HRANGE;
                else
                    txt.GEOMETRY.H1ALT = H1ALT;
                    txt.GEOMETRY.H2ALT = H2ALT;
                    txt.GEOMETRY.HRANGE = HRANGE;
                end
            elseif H1ALT>H2ALT && OBSZEN>90
                txt.GEOMETRY.H1ALT = H1ALT;
                txt.GEOMETRY.H2ALT = H2ALT;
                txt.GEOMETRY.LENN = LENN;
                txt.GEOMETRY.OBSZEN = OBSZEN;
            else
                txt.GEOMETRY.H1ALT = H1ALT;
                txt.GEOMETRY.H2ALT = H2ALT;
                txt.GEOMETRY.OBSZEN = OBSZEN;
            end
        elseif ITYPE==3
            if BCKZEN>0
                txt.GEOMETRY.H1ALT = H1ALT;
                txt.GEOMETRY.H2ALT = H2ALT;
                txt.GEOMETRY.LENN = LENN;
                txt.GEOMETRY.OBSZEN = OBSZEN;
            else
                txt.GEOMETRY.H1ALT = H1ALT;
                txt.GEOMETRY.H2ALT = H2ALT;
                txt.GEOMETRY.OBSZEN = OBSZEN;
            end
        end
        if RAD_E~=0, txt.GEOMETRY.RAD_E = RAD_E; end
        if CKRANG~=0, txt.GEOMETRY.CKRANG = CKRANG; end
        if IDAY~=0, txt.GEOMETRY.IDAY = IDAY; end
        txt.GEOMETRY.IPARM = IPARM;
        txt.GEOMETRY.PARM1 = PARM1;
        txt.GEOMETRY.PARM2 = PARM2;
        if IPARM==0 || IPARM==10
            txt.GEOMETRY.PARM3 = PARM3;
            txt.GEOMETRY.PARM4 = PARM4;
            txt.GEOMETRY.TRUEAZ = TRUEAZ;
            if IEMSCT==4 || IEMSCT==5
                txt.GEOMETRY.ANGLEM = ANGLEM;
            end
        elseif IPARM==1 || IPARM==11
            txt.GEOMETRY.GMTIME = GMTIME;
            txt.GEOMETRY.TRUEAZ = TRUEAZ;
        elseif IPARM==2 && (IEMSCT==4 || IEMSCT==5)
            txt.GEOMETRY.ANGLEM = ANGLEM;
        elseif IPARM==12 && (IEMSCT==4 || IEMSCT==5)
            txt.GEOMETRY.ANGLEM = ANGLEM;
        end

        if exist('txt','var')
            txt          = jsonencode(txt);
            txt([1,end]) = [];
            idx = strfind(txt,'"');
            txt = [txt(1:idx(3)-1),'\n\t\t\t',txt(idx(3):end)];
            txt = strrep(txt,',',',\n\t\t\t');
            fprintf(fid,txt);
            % Close GEOMETRY:
            fprintf(fid,',\n\t\t');
        end

        %----------------- SURFACE
        clear txt
        % Print only if values are not those by default (see MODTRAN6 manual)    
        if SURFTYPE~=0, txt.SURFACE.SURFTYPE = SURFTYPE; end
        if SURFTYPE==0, txt.SURFACE.SURREF = SURREF; end
        txt.SURFACE.NSURF = NSURF;
        if TPTEMP~=0, txt.SURFACE.TPTEMP = TPTEMP; end
        if AATEMP~=0, txt.SURFACE.AATEMP = AATEMP; end
        if WIDERP, txt.SURFACE.WIDERP = WIDERP; end
        txt.SURFACE.GNDALT = GNDALT;
        if DH2O~=0, txt.SURFACE.DH2O = DH2O; end
        if MLTRFL, txt.SURFACE.MLTRFL = MLTRFL; end
        if SURFTYPE==1 || SURFTYPE==3
            txt.SURFACE.SURFP.CSALB = SURFP.CSALB;
            txt.SURFACE.SURFA.CSALB = SURFA.CSALB;
            txt.SURFACE.SURFP.SURFZN = SURFP.SURFZN;
            txt.SURFACE.SURFA.SURFZN = SURFA.SURFZN;
            if SURFTYPE==3
                txt.SURFACE.SURFP.CBRDF = SURFP.CBRDF;
                txt.SURFACE.SURFA.CBRDF = SURFA.CBRDF;
                txt.SURFACE.SURFP.NWVSRF = SURFP.NWVSRF;
                txt.SURFACE.SURFA.NWVSRF = SURFA.NWVSRF;
                txt.SURFACE.SURFP.PBRDF1 = SURFP.PBRDF1;
                txt.SURFACE.SURFA.PBRDF1 = SURFA.PBRDF1;
                txt.SURFACE.SURFP.PBRDF2 = SURFP.PBRDF2;
                txt.SURFACE.SURFA.PBRDF2 = SURFA.PBRDF2;
                txt.SURFACE.SURFP.PBRDF3 = SURFP.PBRDF3;
                txt.SURFACE.SURFA.PBRDF3 = SURFA.PBRDF3;
                txt.SURFACE.SURFP.PBRDF4 = SURFP.PBRDF4;
                txt.SURFACE.SURFA.PBRDF4 = SURFA.PBRDF4;
                txt.SURFACE.SURFP.PBRDF5 = SURFP.PBRDF5;
                txt.SURFACE.SURFA.PBRDF5 = SURFA.PBRDF5;
                txt.SURFACE.SURFP.PBRDF6 = SURFP.PBRDF6;
                txt.SURFACE.SURFA.PBRDF6 = SURFA.PBRDF6;
                txt.SURFACE.SURFP.PBRDF7 = SURFP.PBRDF7;
                txt.SURFACE.SURFA.PBRDF7 = SURFA.PBRDF7;
            end
        end
        txt.SURFACE.SURFNLOS = SURFNLOS;

        if exist('txt','var')
            txt          = jsonencode(txt);
            txt([1,end]) = [];
            idx = strfind(txt,'"');
            txt = [txt(1:idx(3)-1),'\n\t\t\t',txt(idx(3):end)];
            txt = strrep(txt,',',',\n\t\t\t');
            fprintf(fid,txt);
            % Close SURFACE:
            fprintf(fid,',\n\t\t');
        end

        %----------------- SPECTRAL
        clear txt
        % Print only if values are not those by default (see MODTRAN6 manual) 
        txt.SPECTRAL.LBMNAM = LBMNAM;
        if strcmp(LBMNAM,'T')
            if iscell(BMNAME)
                txt.SPECTRAL.BMNAME = BMNAME{i};
            else
                txt.SPECTRAL.BMNAME = BMNAME;
            end
        end
        txt.SPECTRAL.V1 = V1(i);
        txt.SPECTRAL.V2 = V2(i);
        txt.SPECTRAL.DV = DV(i);
        txt.SPECTRAL.FWHM = FWHM(i);
        if ~strcmp(YFLAG,'T'), txt.SPECTRAL.YFLAG = YFLAG; end
        if ~strcmp(XFLAG,'W'), txt.SPECTRAL.XFLAG = XFLAG; end
        if ~strcmp(DLIMIT,' '), txt.SPECTRAL.DLIMIT = DLIMIT; end
        txt.SPECTRAL.FLAGS = FLAGS;
        if ~strcmp(FLAGS(7),' '), txt.SPECTRAL.MLFLX = MLFLX; end
        if VRFRAC~=0, txt.SPECTRAL.VRFRAC = VRFRAC; end
        if SFWHM~=0, txt.SPECTRAL.SFWHM = SFWHM; end
        if VRFRAC~=0, txt.SPECTRAL.VRFRAC = VRFRAC; end
        txt.SPECTRAL.LSUNFL = LSUNFL;
        if strcmp(LSUNFL,'T'), txt.SPECTRAL.URSSUN = URSSUN; end
        if ~strcmp(FILTNM,' '), txt.SPECTRAL.FILTNM = FILTNM; end
        if ~strcmp(CH2OCM,'1'), txt.SPECTRAL.CH2OCM = CH2OCM; end

        if exist('txt','var')
            txt          = jsonencode(txt);
            txt([1,end]) = [];
            idx = strfind(txt,'"');
            txt = [txt(1:idx(3)-1),'\n\t\t\t',txt(idx(3):end)];
            txt = strrep(txt,',',',\n\t\t\t');
            fprintf(fid,txt);
            % Close SPECTRAL:
            fprintf(fid,',\n\t\t');
        end

        %----------------- FILEOPTIONS
        clear txt
        % Print only if values are not those by default (see MODTRAN6 manual)     

        if NOFILE~=0, txt.FILEOPTIONS.NOFILE = NOFILE; end
        if BINARY,txt.FILEOPTIONS.BINARY = BINARY; end
        if CKPRNT,txt.FILEOPTIONS.CKPRNT = CKPRNT; end
        txt.FILEOPTIONS.NOPRNT = NOPRNT;

        if exist('txt','var')
            txt          = jsonencode(txt);
            txt([1,end]) = [];
            idx = strfind(txt,'"');
            txt = [txt(1:idx(3)-1),'\n\t\t\t',txt(idx(3):end)];
            txt = strrep(txt,',',',\n\t\t\t');
            fprintf(fid,txt);
            % Close FILEOPTIONS:
            fprintf(fid,'\n\t\t');
        end
    else
        % Print only spectral information
        %----------------- SPECTRAL
        clear txt
        % Print only if values are not those by default (see MODTRAN6 manual) 
        if strcmp(LBMNAM,'T')
            if iscell(BMNAME)
                txt.SPECTRAL.BMNAME = BMNAME{i};
            else
                txt.SPECTRAL.BMNAME = BMNAME;
            end
        end
        txt.SPECTRAL.V1 = V1(i);
        txt.SPECTRAL.V2 = V2(i);
        txt.SPECTRAL.DV = DV(i);
        txt.SPECTRAL.FWHM = FWHM(i);
        
        if exist('txt','var')
            txt          = jsonencode(txt);
            txt([1,end]) = [];
            idx = strfind(txt,'"');
            txt = [txt(1:idx(3)-1),'\n\t\t\t',txt(idx(3):end)];
            txt = strrep(txt,',',',\n\t\t\t');
            fprintf(fid,txt);
            % Close SPECTRAL:
            fprintf(fid,'\n\t\t');
        end
    end
    
% Close JSON MODTRANINPUT:
fprintf(fid,'\t\t}\n');    
if i~=Nbands
    fprintf(fid,'\t},\n');
    fprintf(fid,'\t{\n');
else
    fprintf(fid,'\t} ]\n');
end
%%%%%%%%%%%%%% Close loop for Nbands times
end

fprintf(fid,'}');
% Close json file:
fclose(fid);
clear aux

end