function [lambda,Lp0,Edir0,Edif0,S,T1,T2] = out_to_LUT(alg,files,rho,SZA)
% OUT_TO_LUT processes the libRadtran output .out files (same atmosphere 
% and different geometric conditions) in order to obtain the atmospheric 
% variables independent from surface properties.
%
% INPUTs: FILES = cell array containing the following 5 .out files needed 
%                 for the interrogation technique:
%                 file{1} - at TOA with rho=0
%                 file{2} - at BOA with rho=0
%                 file{3} - at BOA with RHO(1)~=0
%                 file{4} - at TOA with RHO(2)~=0
%         RHO = vector of constant reflectance values used for each of the 
%               .out files in FILES(3-5)
%         SZA = Sun Zenith Angle
%
% OUTPUTs: lambda = Wavelength in [nm]
%          Lp0  : Atmospheric path radiance (in [mW/m2/sr/nm])
%          Edir : At-surface direct solar irradiance (in [mW/m2/nm])
%          Edif : At-surface diffuse solar irradiance (in [mW/m2/nm])
%          S    : Spherical Albedo (in [-])
%          tdir : Target-to-sensor direct transmittance (in [-])
%          tdif : Target-to-sensor diffuse transmittance (in [-])
%          Ttot : Target-to-sensor total transmittance (in [-])
%
%   Based on internal communication with C. Emde (27/02/2019).
%
%   See also: LUTGEN_PROCESS, READRTMOUTPUT

%   Copyright 2019 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.00 $  $Date: 2019/04/19 $


% Initial values:
u_s = cosd(SZA); % Cosinus of the solar zenith angle in [deg]

% Recover parameters from the three files:

% Path radiance is recovered from files{1}, where simulation was carried
% out at-sensor altitude with a black (rho=0) surface. Therefore, at-sensor
% radiance is directly the TOA radiance.
[lambda,~,~,~,Lp0] = alg.readRTMoutput(files{1});
Lp0(isnan(Lp0) | isinf(Lp0) | Lp0<0) = 0;

% At-surface direct and diffuse irradiances are directly given on through
% the downwards direct/diffuse irradiances when the sensor altitude is
% placed at surface altitude. For that we use files{2}
[lambda,Edir0,Edif0,~,~] = alg.readRTMoutput(files{2});
Edir0 = Edir0/u_s;
Edir0(isnan(Edir0) | isinf(Edir0) | Edir0<0) = 0;
Edif0(isnan(Edif0) | isinf(Edif0) | Edif0<0) = 0;

% Spherical albedo can be determined from the at-sensor radiance at BOA
% given that, with this geometric configuration, path radiance is 0 and
% target-to-sensor transmittances (both direct and diffuse) are 1:
[~,~,~,~,Laux] = alg.readRTMoutput(files{3});
Etot = Edir0*u_s + Edif0;
S = (Laux - Etot*rho(1)/pi)./(Laux*rho(1));
S(isnan(S) | isinf(S) | S<0 | S>1) = 0;

if ~contains(files{4},'recip')
    % For an airborne sensor, libRadtran outputs does not allow to separate
    % between the direct and diffuse components of the target-to-sensor
    % transmittance. Accordingly, only the total target-to-sensor 
    % transmittance is calculated from one at-sensor simulation, knowing 
    % the path radiance, total at-surface irradiance and spherical albedo 
    % that have been previously calculated
    [~,~,~,~,Laux] = alg.readRTMoutput(files{4});
    Ttot = pi*(Laux - Lp0).*(1-S*rho(2))./(Etot*rho(2));
    Ttot(isnan(Ttot) | isinf(Ttot) | Ttot<0 | Ttot>1) = 0;
    T1 = Ttot;
else
    % For a sensor at TOA, we can determine the target-to-sensor direct and
    % diffuse transmittances from a TOC sensor simulation using the reciprocity
    % principle (i.e. that sun-to-target path is the same as target-to-sensor):
    [~,tdir,tdif,~,~] = alg.readRTMoutput(files{4});
    tdir(isnan(tdir) | isinf(tdir) | tdir<0 | tdir>1) = 0;
    tdif(isnan(tdif) | isinf(tdif) | tdif<0 | tdif>1) = 0;
    T1 = tdir;
    T2 = tdif;
end

% END of function
end