function writeRTMinput(file,rho,varnames,inputval,SpeConf,flag)
% WRITERTMINPUT Create libRadtran input tape5 file
%   WRITERTMINPUT(FILE,RHO,VARNAMES,INPUTVAL,SPECONF,FLAG) creates,
%   according to libRadtran documentation, a libRadtran's input file (FILE)
%   with a spectrally constant reflectance RHO in the spectral range and 
%   configuration defined in the input SPECONF structure. LibRadtran 
%   configuration is based on the input variables and associated values
%   stored in the cell arrays VARNAMES (strings) and INPUTVAL (numeric
%   values). An optional flag FLAG is set to determine whether simulations
%   are carried out at sensor level (FLAG='toa' or FLAG=[]), 
%   bottom-of-atmosphere (FLAG='boa'), using the reciprocity principe
%   (FLAG='recip') or to calculate the solar irradiance (FLAG='irr'). FLAG 
%   is mainly used to perform a libRadtran interrogation technique and 
%   derive atmospheric transfer functions.
%
%   Copyright 2019 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.00 $  $Date: 2019/04/14 $

%% Input variables to be used by libRadtran:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%           INPUT VARIABLES TO BE USED BY LIBRADTRAN            %%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numel(varnames)~=numel(inputval)
    error('Number of variable names and input values are not coincident.')
end

% Associates input values to each variable name:
for i=1:numel(varnames)
    eval([varnames{i},'=inputval{',num2str(i),'};'])
end

%--------------------------------------------------------------------------
% NOTE: Read libRadtran Documentation for further information on the 
% variables and input file format.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%----------- STEP 1: Wavelength grid and band parameterization ------------
%--------------------------------------------------------------------------
% v1 and v2 define the minimum and maximum wavelengths [nm]  of the output 
% libRadtran data:
% User-defined min/max wavelengths
Nbands = numel(str2num(SpeConf.MinLambda.values)); % Number of spectral bands
v2 = fliplr(str2num(SpeConf.MaxLambda.values)); % In [nm] from ALG toolbox
V1 = 1e7./v2; % Converted into [cm^-1]
v1 = fliplr(str2num(SpeConf.MinLambda.values)); % In [nm] from ALG toolbox
V2 = 1e7./v1; % Converted into [cm^-1]
% Spectral sampling: fine (1 cm-1), medium (5 cm-1) and coarse (15 cm-1)
SSI = str2cell(SpeConf.SSI.values(2:end-1)); auxSSI=zeros(size(v1));
for i=1:numel(v1)
	if numel(v1)==1, aux = {SSI}; else, aux=SSI; end
	if strcmp(aux{i},'fine')
		auxSSI(i) = 1; % in cm-1
	elseif strcmp(aux{i},'medium')
		auxSSI(i) = 5; % in cm-1
	elseif strcmp(aux{i},'coarse')
		auxSSI(i) = 15; % in cm-1
	end
end
	
if numel(V2)~=1
    % Update wavelengths if spectral ranges are overlapped
    for i=2:numel(V1)
        if V1(i)==V2(i-1) && auxSSI(i)>=auxSSI(i-1) 
            V1(i) = V1(i) + auxSSI(i);
        elseif V1(i)==V2(i-1) && auxSSI(i)<auxSSI(i-1) 
            V2(i-1) = V2(i-1) - auxSSI(i-1);
        end
    end
end 
% Back to [nm]:
v2 = fliplr(1e7./V1); v1 = fliplr(1e7./V2);

% With wavelength_grid_file pseudo-spectral calculations can be performed,
% meaning that you can calculate radiation at any wavelength you want, but
% it is always a weighted mean calculated from radiative transfer 
% calculations at so-called representative wavelengths of the belonging band
% If you select the wavelengths too close, you will see the steps in your
% spectrum
% Since a single file cannot combine different wavelength intervals with
% different resolutions and source type (solar/termal), the idea will be to
% create Nbands libRadtran files, each with the corresponding wavelength grid,
% resolution and source. In addition, in order to include the a wavelength
% value at 550 nm (so that we can modify the AOT@550 in all spectral bands),
% we will use wavelength_grid_file. This file will contain the nominal
% wavelengths use by reptran (fine,medium,coarse).
wavelength_grid_file = cell(1,Nbands);
for i=1:Nbands
	% Check if 550 nm is within range
	if v1(i)<=550 && v2(i)>=550
		% Since 550 nm is included in the interval, the default grid will be
		% used
		wavelength_grid_file{i} = [];
	else
		% We must expand the wavelength interval in the internal
        % calculation grid so that 550 nm within the range. To do so, we
        % make the grid by steps of SSI in the min-max range. We add a 
        % wavelength at 300 nm and 550.555 so that in the post-processing
        % we remove it.
		% WARNING if interested in simulation in the UV range!!!
		% NOTE: ideally we should extract the nominal grid used in reptran so
		% that we don't modify the internal grid <--- REVIEW!!!
        filepath = fileparts(file);
		wavelength_grid_file{i} = [filepath,'/wvlGrid_',num2str(i),'.txt'];
		if ~exist(wavelength_grid_file{i},'file')
			wvlGrid = (1e7/v2(i)):auxSSI(i):(1e7/v1(i)); % In cm-1
			wvlGrid = sort([300, 550.555, fliplr(1e7./wvlGrid)]); % Back to nm
			% Print in wavelength_grid_file
			fid = fopen(wavelength_grid_file{i},'w');
			fprintf(fid, '%.3f\n', wvlGrid');
			fclose(fid);
		end
	end
end
% The problem with the approach above is that we are printing the wavelength
% grid everytime we write a libRadtran input. Since all input files will have 
% the same grid this is an important overhead in the calculation. We should
% better precalculate the grid before writing the libRadtran input files
% (or check if the to-be-created file has already been calculated)

% Set the wavelengths to be selected
wavelength_index = [];

% File with the center wavelengths and the wavelength band intervals to be 
% used for calculations in the thermal range
thermal_bands_file = [];
% Specify constant bandwidth in cm-1 for thermal calculations
thermal_bandwidth = 1;


%--------------------------------------------------------------------------
%-------------------- STEP 2: Geometrical Conditions ----------------------
%--------------------------------------------------------------------------
% Solar Zenith Angle (in deg):
SZA = SZA;
if strcmp(flag,'recip')
    % By reciprocity principle (for 1D atmospheres!), the sun-to-target
    % transmittance is the same as the target-to-sensor transmittance if
    % SZA=VZA. This will allow us to calculate the direct/diffuse
    % target-to-sensor transmittance (only when the sensor is at TOA)
    SZA = VZA;
end
% Solar Zenith Angle file. This option is useful if you want to simulate an
% instrument which scans so slowly that the solar zenith angle may change
% significantly during the wavelength
sza_file = []; 
% phi0 is the Solar Azimuth Angle (in deg):
phi0 = 0;
% DOY: Day Of Year (1 to 365) to specify Earth-Sun distance.
DOY = 93; % DOY=93 corresponding to 1 AU
% GNDALT: Ground altitude (in m):
GNDALT = GNDALT;
% Sensor altitude over sea level (in km):
if isempty(flag) || strcmp(flag,'toa') || strcmp(flag,'irr')
    zout_sea = ZOUT;
elseif strcmp(flag,'boa') || strcmp(flag,'recip')
    zout_sea = GNDALT;
end
% VZA: Viewing Zenith Angle (in deg). 0<VZA<90 means downwards looking (i.e.
% satellite) and VZA<0 means upwards looking (e.g. for CIMEL)
VZA = VZA;
% RAA: Relative Azimuth Angle (in deg)
RAA = RAA;
% Latitude and longitude coordinates of location. Not used in ALG since
% illumination conditions and atmosphere type is selected by the user
latitude = []; longitude = [];
% Specify time of simulation. Read libRadtran user manual for further 
% information. Not used in ALG since illumination conditions and atmosphere
% type is selected by the user
time = [0 0 0 0 0 0];
% Interpolate optical properties over the altitude grid
zout_interpolate = 0;

%--------------------------------------------------------------------------
%--------------------- STEP 3: Atmospherical Model ------------------------
%--------------------------------------------------------------------------
% Set the radiation source type. By default, libRadtran computes the direct
% /diffuse downward and upward solar irradiance. For simulating thermal 
% radiance use 'thermal'. Notice that thermal and solar cannot be calculated
% together and thus it will be necessary to run twice libRadtran in these
% cases (i.e. for calculations around 3 um)
source = source;

% Earth radius (in km). Needed by all spherical geometry solvers
earth_radius = 6370;
% MODEL: selects on geographical-seasonal atmospheres or specify to use a
% user-defined (read libRadtran documentation): 1 to 6
MODEL = MODEL; auxMODEL = MODEL;
if isstruct(MODEL) || MODEL~=0 
    if isstruct(MODEL) % MODEL contains user-defined atmospheric profile
		% In libRadtran, users can input atmospheric vertical profiles
		% using various alternative/complementary options:
		% atmosphere_file --> spefifies vertical profiles of pressure
		% temperature, air density, O3, O2, H2O, CO2 and NO2
		% mol_file --> specifies density profiles of trace gases (O3, NO2,
		% H2O, BRO, OCLO, HCHO, CO2, SO2, O2, O4, CH4, N2O, CO, N2)
		% radiosonde --> spefifies vertical profiles of pressure, temperature,
		% O3, N2O, H2O, BRO, OCLO, HCHO, CO2, SO2 and O4
		% Accordingly, radiosonde offers more versatility except for a few 
		% gases, which can only be defined with mol_file. In order to avoid
		% missing data, we set the atmosphere_file to midlatitude_summer
		atmosphere_file = 'midlatitude_summer';
		radiosonde   = MODEL.file;
		radiosonde_levels_only = 0;
		mol_file.O2  = MODEL.O2file;
		mol_file.CH4 = MODEL.CH4file;
		mol_file.N2O = MODEL.N2Ofile;
		mol_file.CO  = MODEL.COfile;
		mol_file.N2  = MODEL.N2file;
		MODEL = [];
    end
	% Default trace gases content:
	mol_modify.O3   = O3;   % O3 column concentration (in DU)
	mol_modify.O2   = O2;   % O2 column concentration (in DU)
	mol_modify.H2O  = H2O;  % H2O column concentration (in g/cm2)
	mol_modify.CO2  = CO2;  % CO2 mixing ratio (in DU)
	mol_modify.NO2  = NO2;  % NO2 column concentration (in DU)
	mol_modify.BRO  = BRO;   % BrO column concentration (in DU)
	mol_modify.OCLO = OCLO; % OCLO column concentration (in DU)
	mol_modify.HCHO = HCHO; % HCHO column concentration (in DU)
	mol_modify.O4   = O4;   % O4 column concentration (in DU)
	mol_modify.SO2  = SO2;  % SO2 column concentration (in DU)
	mol_modify.CH4  = CH4;  % CH4 column concentration (in DU)
	mol_modify.N2O  = N2O;  % N2O column concentration (in DU)
	mol_modify.CO   = CO;   % CO column concentration (in DU)
	mol_modify.N2   = N2;   % N2 column concentration (in DU)
elseif MODEL==0
	auxMODEL = MODEL;
    % No Gaseous Absorption atmosphere
    MODEL = 2; % Mid-latitude summer by default
	mol_modify.O3   = 0; mol_modify.O2   = 0; mol_modify.H2O  = 0;
	mol_modify.CO2  = 0; mol_modify.NO2  = 0; mol_modify.BRO  = 0;
	mol_modify.OCLO = 0; mol_modify.HCHO = 0; mol_modify.O4   = 0;
	mol_modify.SO2  = 0; mol_modify.CH4  = 0;  mol_modify.N2O = 0;
	mol_modify.CO   = 0; mol_modify.N2   = 0;
	
	ck_lowtran_absorption.O4  = 'off'; ck_lowtran_absorption.N2  = 'off';
	ck_lowtran_absorption.CO  = 'off'; ck_lowtran_absorption.SO2 = 'off';
	ck_lowtran_absorption.NH3 = 'off'; ck_lowtran_absorption.NO  = 'off';
	ck_lowtran_absorption.HNO = 'off';
	
end
% NOTE: In ALG, the default trace gases content is modified with libRadtran
% mol_modify option. An alternative (not used in ALG) is mixing_ratio, 
% where the quantity is specified in ppm

% Atmospheric gases cross sections file:
crs_file.O3   = [];   
crs_file.O2   = [];  
crs_file.H2O  = [];
crs_file.CO2  = []; 
crs_file.NO2  = [];
crs_file.BRO  = []; 
crs_file.OCLO = []; 
crs_file.HCHO = [];
crs_file.O4   = [];
crs_file.SO2  = [];
crs_file.CH4  = [];
crs_file.N2O  = [];
crs_file.CO   = [];
crs_file.N2   = [];
% LibRadtran library of cross-section models
crs_model.rayleigh = 'default';
crs_model.o3 = 'default';
crs_model.no2 = 'default';

% Molecular scattering or absorption optical depth files. See libRadtran
% user manual for further information about the content and format of these
% files
mol_tau_file.sca = [];
mol_tau_file.abs = [];

% Surface pressure at user-defined altitude in hPa
pressure = pressure;

%--------------------------------------------------------------------------
%------------------------ STEP 4: Aerosol Model ---------------------------
%--------------------------------------------------------------------------
% Boundary layer aerosol type: Rural (1), maritime (4), urban (5) and 
% tropospheric (6), OPAC (-1)
IHAZE = IHAZE;
if isstruct(IHAZE) % IHAZE contains data from OPAC or user-defined aerosol:
    % Review how to implement user-defined aerosols with ALG
	% check libRadtran user manual: aerosol_file explicit
    if strcmp(IHAZE.type,'OPAC')
        % Predefined OPAC aerosol models:
        aux = {'continental_clean','continental_average',...
               'continental_polluted','urban','desert',...
               'maritime_clean','maritime_polluted','maritime_tropical',...
               'arctic','antarctic'};
        aerosol_species_file = aux{IHAZE.ID};
    end
end
% Tropospheric layer aerosol type: Background (1), moderate volcanic (2), 
% high volcanic (3), extreme volcanic (4)
IVULCAN = IVULCAN; 
% Aerosol Optical Thickness. aerosol_set_tau_at_wvl needs that the selected
% wavelength is included in the simulated wavelengths in the output file.
% In ALG, we fix this wavelength to 550 nm
AOT = AOT;
% Modify aerosol optical depth by Angstrom law given the Angstrom exponent
ALPHA = ALPHA;
% Modify aerosol optical depth by Kind-Byrne formula (see libRadtran manual
% for further information). For the moment, this option is not available in
% ALG.
aerosol_king_byrne = [];

% Modify aerosol optical properties: gg (asymmetry factor), ssa (single% scattering albedo), tau (wavelength-independent cloud optical thickness)
% In ALG, these user-input values are spectrally-independent
aerosol_modify.gg.value = G;
aerosol_modify.ssa.value = SSA;
aerosol_modify.tau.value = [];

%--------------------------------------------------------------------------
%------------------------ STEP 5: Clouds Model ---------------------------
%--------------------------------------------------------------------------
% Location of file defining water cloud properties. LibRadtran accepts 3 
% different types of water cloud files (see libRadtran user manual for
% further details):
% 1D: one-dimensional profile where liquid water content and droplet size
% are given at the defined altitudes.
% ipa_files: defines the weights of a list of separate 1D water cloud files
% moments: A  way  to  specify  water  cloud  extinction  coefficient, % single  scattering albedo, and scattering phase function for each layer
% via a list of separate files
% NOTE: In ALG v2.0, only the 1D type is used.
wc_file = [];
% Location of file defining ice cloud properties. See wv_file for further 
% details. % NOTE: In ALG v2.0, only the 1D type is used.
ic_file = [];

% Modify water cloud properties: gg (asymmetry factor), ssa (single scattering
% albedo), tau (wavelength-independent cloud optical thickness)
wc_modify.gg.value = [];
wc_modify.gg.type  = 'set'; % or 'value'
wc_modify.ssa.value = [];
wc_modify.ssa.type  = 'set'; % or 'value'
wc_modify.tau.value = [];
wc_modify.tau.type  = 'set'; % or 'value'
% wc_properties. Define how liquid water content and effective radius are 
% translated to optical properties. Options: 'hu', 'echam4' and 'mie'
% See libRadtran manual for further information
wc_properties = 'hu';

% Modify ice cloud properties (same as in wc_modify)
ic_modify.gg.value = [];
ic_modify.gg.type  = 'set'; % or 'value'
ic_modify.ssa.value = [];
ic_modify.ssa.type  = 'set'; % or 'value'
ic_modify.tau.value = [];
ic_modify.tau.type  = 'set'; % or 'value'
% ic_properties. Define how ice  water content and effective radius are 
% translated to optical properties. Options: 'fu', 'echam4' and 'key',
% 'yang', 'baum', 'baum_v36', 'hey', 'yang2013' and 'raytracing'
% See libRadtran manual for further information
ic_properties = 'fu';
% Specify ice cloud properties and definitions (see libRadtran manual for
% further information)
% ic_fu reff_def sets the definition of the effective radius used in the 
% calculations. Two options are available: 'on' for Fu parameterization or
% 'off' for default parameterization
ic_fu.reff_def = 'off';
% ic_fu deltascaling (only valid if reff_def='on') delta-scales optical 
% properties. Two options: 'on' and 'off'
ic_fu.deltascaling = 'off';
% ic_habit_: Ice crystal habit when ic_properties are key/yang/hey or 
% yang2013 (ic_habit_yang2013). See libRadtran user manual for further
% information.
ic_habit = []; ic_habit_yang2013 = [];
if strcmp(ic_properties,'key') || strcmp(ic_properties,'yang') || strcmp(ic_properties,'hey')
	% Options are: solid-column, hollow-column, rough-aggregate, 
	% rosette-4, rosette-6, plate, droxtal, dendrite and spheroid.
	% rosette-4, dendrite and spheroid are only valid in 'hey'
	ic_habit = 'rosette-6';
elseif strcmp(ic_properties,'yang2013')
	% Options are: column_8elements, droxtal, hollow_bullet_rosette,	% hollow_column, plate, plate_10elements,
	ic_habit_yang2013.type = 'column_8elements';
	ic_habit_yang2013.roughness = 'moderate'; % smooth, moderate or severe
end
% Ice crystal properties for geometrical optics ray tracing
ic_raytracing_file = [];

% Cloud fraction profile file
cloud_fraction_file = [];
% Cloud overlap assumption. Options are: rand, maxrand, max or off. See% libRadtran user manual for further information
cloud_overlap = 'off';
% Fraction of horizontal sky covered by clouds. Only cloud_fraction_file
% is empty and if Ice and/or Water cloud files are defined.
cloudcover = 100;

%--------------------------------------------------------------------------
%-------------------- STEP 6: Surface Properties --------------------------
%--------------------------------------------------------------------------
% In ALG, the input rho has different meanings:
% If 0<rho<1rho ==> spectrally-constant lambertian reflectance
% If rho<10 ==> albedo IGBP library (see libRadtran user manual)
% If rho = struct ==> albedo file (see libRadtran user manual)
if isstruct(rho)
	albedo_file = rho.file;
elseif rho>=0 && rho<=1 
	albedo = rho;
elseif rho>10
	albedo_library = rho-10;
end 

% Wind speed for ocean BRDF (in m/s). Only available with rte_solver mystic
bpdf_tsang_u10 = [];
% AMBRALS (Ross-Li) 3-parameters BRDF. May be combined with rte_solver 
% mystic, disort and fdisort2
brdf_ambrals = [];
% Turn on hot spot correction factor when activating AMBRALS BRDF. May be
% combined with rte_solver mystic and disort
brdf_ambrals_hotspot = 0;
% Spectrally-dependeng AMBRALS (Ross-Li) BDRF parameterization file. May be
% combined with rte_solver mystic and disort
brdf_ambrals_file = [];
% Cox and Munk (1954a,b) ocean BRDF properties.
brdf_cam.pcl  = []; % Pigment concentration (in mg/m-3). Default is 0.01
brdf_cam.sal  = []; % Salinity (in permille, 0.1% = part-per-thousand). Default is 34.3
brdf_cam.u10  = []; % Wind speed (in m/s). Minimum of 1 m/s
% Only for Mystic solver
brdf_cam.uphi = []; % Wind direction ( in def). Default is 0 (i.e. from south). 90=from west
% Wavelength-independent Hapke BRDF values.
brdf_hapke.w  = []; % Single scattering albedo of surface
brdf_hapke.B0 = []; % Hot spot factor
brdf_hapke.h  = []; % Surface roughness parameter
% Spectrally-dependeng Hapke BDRF parameterization file. May be combined 
% with rte_solver mystic and disort
brdf_hapke_file = [];
% Wavelength independent RPV values. See brdf_rpv_file
brdf_rpv.k = [];
brdf_rpv.rho0 = [];
brdf_rpv.theta = [];
brdf_rpv.sigma = [];
brdf_rpv.t1 = [];
brdf_rpv.t2 = [];
% Apply a constant scaling facto  for the RPV BRDF. Only used by rte_solver
% disort, fdisort2 and mystic
brdf_rpv.scale = [];
% 4 to 7 column file, containing the Rahman, Pinty and Verstraete (RPV)% BRDF file. Only supported with solvers disort, fdisort2 and mystic
brdf_rpv_file = [];

% Wavelength-independent fluorescence emission. Currently only works with
% DISORT. Must be used together with "source solar file". Units are the same
% as those used in the solar source
fluorescence = 0;
% Wavelength-dependent fluorescence emission file. See comments above. In 
% addition, if raman is not set, wavelength_grid_file must be specified 
% with the same resolution as the "source solar file" and the first value
% must be specified by wavelength.
fluorescence_file = [];

% Surface temperature. Used for thermal infrared calculations. If not 
% specified, the temperature of the lowest atmospheric level is used as
% surface temperature
sur_temperature = [];

% Surface type map. See libRadtran for further information. Does not seem
% relevant for ALG applications so not implemented in the input file
% printing part of the code
surface_type_map = [];

%--------------------------------------------------------------------------
%--------------------------- STEP 7: Advanced -----------------------------
%--------------------------------------------------------------------------
% atm_z_grid: vertical resolution of the atmosphere_file data is changed to
% the levels (in km above sea surface) given as argument
atm_z_grid = [];
% Radiative Transfer Equation solver. Options are: disort (1), twostr (2),
% fdisort1 (3), fdisort2 (4), sdisort (5), spsdisort (6), polradtran (7),
% ftwostr (8), rodents (9), sslidar (10), sos (11), montecarlo (12),
% mystic (13), tzs (14), sss (15), null (16)
rte_solver = rte_solver;
% Parameterization for simulating molecular absorptions. Options are:
% reptran, reptran_channel, crs, kato, kato2, kato2andwandji, fu, avhrr_kratz
% lowtran and sbdart
mol_abs_param = mol_abs_param;
% Location of representative wavelengths file. Modifies default representative
% wavelengths when using reptran
reptran_file = [];

% DISORT solver advanced configuration:
% Delta-M Scaling
deltam = deltam;
% DISORT intensity correction: phase/Nakajima (1, default) or Legendre
% moments (2). Only applicable if rte_solver is disort or fdisort2
disort_intcor = disort_intcor;
% Number of streams. Only if rte_solver is disort, fdisort1 or fdisort2
NSTR = NSTR;
if ~(rte_solver==1 || rte_solver==3 || rte_solver==3)
	NSTR = 0;
end
% Activate raman scattering. Only available with disort
raman = raman;
if raman==1
	rte_solver = 1;
end
% Speficy sdisort values. Only valid with sdisort rte solver
sdisort.nscat   = 2; % Order of scattering (1=single, 2=multiple)
sdisort.nrefrac = 0; % Include refaction (0=none, 1=fast, 2=slow)
% Specifies that isotropic illumination is used at top-boundary instead of 
% beam source. Useful for those who want to calculate the reflectance for a
% homogeneous or inhomogeneous atmosphere. The intensity is still set by
% source solar file. Only works with disort and twostr
isotropic_source_toa = isotropic_source_toa;
% Activate or desactivate pseudospherical simulation (only if DISORT or
% Two Streams):
pseudospherical = logical(pseudospherical);
if ~(rte_solver==1 || rte_solver==2)
	pseudospherical = false;
end

% POLRADTRAN solver advanced configuration. See libradtran for further
% information
polradtran.aziorder = []; % Order of Fourier azimuth series
polradtran.nstokes  = 2; % Number of Stokes parameters. 1=no polarization
polradtran.src_code = 1; % Radiation source. None (0, Solar (1), Thermal (2), Both (3)
% Initial layer thickness for doubling; governs accuracy, 10E-5 should be
% adequate. Do not go beyond half the real precision, i.e. 10e-8 for REAL*8.
% Default 1.e-05
polradtran_max_delta_tau = 1e-5;
if polradtran_max_delta_tau>1e-8, polradtran_max_delta_tau=1e-8; end
% Type of quadrature used in polradtran
polradtran_quad_type = 'G';

% MYSTIC solver advanced configuration:
% Backward tracing photos. See libRadtran documentation for further 
% information:
mc_backward = 'off';
% Specify output to be calculated using backward Monte Carlo and its units
mc_backward_output.edir = 'off'; % Direct horizontal irradiance
mc_backward_output.edn = 'off'; % Diffuse downward irradiance
mc_backward_output.eup = 'off'; % Diffuse upward irradiance
mc_backward_output.exp = 'off'; % Diffuse irradiance in positive x direction
mc_backward_output.exn = 'off'; % Diffuse irradiance in negative x direction
mc_backward_output.eyp = 'off'; % Diffuse irradiance in positive y direction
mc_backward_output.eyn = 'off'; % Diffuse irradiance in negative y direction
mc_backward_output.act = 'off'; % Actinic flux
mc_backward_output.abs = 'off'; % Absorption
mc_backward_output.emis = 'off'; % Emission
mc_backward_output.heat = 'off'; % Heating rates (abs+emis)
% mc_backward_writeback. Write the distribution of photos contributing to the
% results in a .bac file.
mc_backward_writeback = 0;
% MYSTIC 3D output filename
mc_basename = 'mc';
% Box-airmass-factor calculation
mc_boxairmass = 0;
% Activate backscattering. Only works with mc_polarisation
mc_coherent_backscatter.on = 0;
mc_coherent_backscatter.option1 = 0;
mc_coherent_backscatter.option2 = 0;
% Calculate MYSTIC radiances via escape probabilities;  slows down the % tracing but usually speeds up the computation considerably since it 
% reduces noise. Switched on per default since it should basically be used
% always when calculating radiances
mc_escape = 1;
% Total number of photons to be traced by the Monte Carlo solver, MYSTIC
mc_photons = 100000;
% Distribution of photons over wavelength bands; to be used with
% mol_abs_param
mc_photons_file = [];
% Activate polarisation
mc_polarisation = mc_polarisation;
% Define opening angle for all-sky simulations
mc_rad_alpha = 0;
% Random seed number for repetition of simulations
mc_randomseed = [];
% Calculate spectrum with high spectral resolution using an importance
% sampling method
mc_spectral_is = mc_spectral_is;
% Activate spherical geometry in Mystic
mc_spherical = mc_spherical;
% Switch on surface reflct always option. See libRadtran user manual for 
% further information
mc_surface_reflectalways = 0;
% Variance Reduction Optical Options Method (VROOM)
mc_vroom = mc_vroom;

% SSLIDAR solver advanced configuration:
% Set single scattering lidar parameters
sslidar.area 	 = 1;   % Area of single scattering lidar (in m2)
sslidar.E0       = 0.1; % Laser pulse energy (in Joule)
sslidar.eff      = 0.5; % Lidar efficiency
sslidar.position = 0;   % Lidar position (in km)
sslidar.range 	 = 0.1; % Lidar range bin width (in km)
% Set number of lidar range bins
sslidar_nranges = 100;
% Turn polarisation measurements for lidar
sslidar_polarisation = 0;

% SOS solver
sos_nscat = 20; % Number of successive orders of scattering
% TZA solver advanced configuration:
% Define cloud top height of "blackbody clouds"
tzs_cloud_top_height = [];

% Calculation of heating rates. Options are layer_cd; local or layer_fd. 
% See libRadtran user manual for further information
heating_rate = [];
% Print informative messages:
quiet = quiet;
% If specified abundances of informative messages are output to stderr
verbose = false;
% Rayleight depolarization factor (see libRadtran for further information)
% By default the depolarization is calculated using the expressions from
% Bodhaine et al. (1999)
rayleigh_depol = [];
% Reverse atmosphere
reverse_atmosphere = 0;

%--------------------------------------------------------------------------
%--------------------------- STEP 8: Post-processing ----------------------
%--------------------------------------------------------------------------
% Spectral response function
filter_function_file = [];
% Output file name (not needed if libRadtran is run with a command line 
% selecting the output file)
output_file = [];
% Output file format
output_format = 'ascii';
% Process output from uvspec. Options are: sum (for sum over wavelengths),
% integrate (for integration over wavelengts), per_nm (for output in 
% xW/m2.nm), per_cm-1 (as before but using cm-1 instead of nm), per_band
% (output in xW/m2 given at each correlated-k or reptran band) and nono
% (default; output spectral information) 
output_process = 'none';
% Convert radiances / irradiances to equivalent output quantity. Options 
% are: brightness (to brightness temperatures), reflectivity (where 
% radiances and irradiances are normalized by the extraterrestrial flux and
% reflectances and transmittances are provided) or transmittance (similar as
% before but withouth correcting by the Sun-Earth distance). See user manual
% for further information
output_quantity = 'default';
if strcmp(flag,'recip')
    output_quantity = 'reflectivity';
end
% output_user allows to select the output quantities to be printed by
% libRadtran in the .out libRadtran file
if strcmp(flag,'irr')
    selectedOut = 'edir';
else
    selectedOut = 'edir edn eup uu';
end
% Specify output levels in pressure coordinates (in hPa)
pressure_out = [];
% Print disort info
print_disort_info = 0;
% Slit function for performing convolution of output spectrum
slit_function_file = [];
% Spline interpolation of the calculated spectrum. See libRadtran for further
% information
spline = [0 0 0];
spline_file = [];


%% Write variables to to generate libRadtran input file:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%              WRITE VARIABLES ON LIBRADTRAN INPUT FILE               %%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%--------------------------------------------------------------------------
% NOTE: Read libRadtran Documentation for further information on the variable
% and input file format.
%--------------------------------------------------------------------------

%%%%%%%%%%%%%% Repeat Nbands times
for i=1:Nbands
    % Each spectral band requires a new libRadtran input file
    % Open file with write permission
    aux = strfind(file,'.inp');
    fid = fopen([file(1:aux-1),'_band',num2str(i),file(aux:end)], 'w');
	
	%fprintf(fid,'data_files_path /home/flex/libRadtran/data \n');

	%--------------------------------------------------------------------------
    %----------- STEP 1: Wavelength grid and band parameterization ------------
    %--------------------------------------------------------------------------
	if isempty(wavelength_grid_file{i})
		fprintf(fid, 'wavelength %.3f %.3f\n', v1(i), v2(i));
	else
		fprintf(fid,'wavelength_grid_file "%s"\n',wavelength_grid_file{i});
	end
	% File with the center wavelengths and the wavelength band intervals to be 
	% used for calculations in the thermal range
	if ~isempty(thermal_bands_file) && v1(i)>=8
		fprintf(fid,'thermal_bands_file\n');
	end
	% Constant bandwidth in cm-1 for thermal calculations
	if thermal_bandwidth~=1 && v1(i)>=8
		fprintf(fid,'thermal_bandwidth %.3f\n',thermal_bandwidth);
	end
	% Set the wavelengths to be selected
	if ~isempty(wavelength_index)
		fprintf(fid,['wavelength_index ',repmat('%i ',1,6),'\n'],wavelength_index);
	end 

    %--------------------------------------------------------------------------
    %-------------------- STEP 2: Geometrical Conditions ----------------------
    %--------------------------------------------------------------------------
	if ~isempty(sza_file)
		fprintf(fid, 'sza_file %s\n',sza_file); % Solar Zenith Angle file
	else
		fprintf(fid, 'sza %.2f\n',SZA); % Solar Zenith Angle (in deg)
	end
	fprintf(fid, 'phi0 %.2f\n',phi0); % Solar Azimuth Angle (in deg)
    fprintf(fid, 'day_of_year %i\n',DOY); % Day Of Year (1 to 365)
	fprintf(fid, 'altitude %.3f\n', GNDALT); % Ground altitude (in m)
	if zout_sea>=100
		fprintf(fid, 'zout_sea TOA\n'); % Sensor altitude at TOA (satellite)
	else
		fprintf(fid, 'zout_sea %.3f\n', zout_sea); % Sensor altitude (in m)
	end
	% Interpolate optical properties over the altitude grid
	if zout_interpolate==1
		fprintf(fid,'zout_interpolate\n');
    end
	if VZA<0
		fprintf(fid, 'umu %.2f\n',-cosd(VZA));
	else
		fprintf(fid, 'umu %.2f\n',cosd(VZA));
	end
	% Since phi0=0, the Viewing Azimuth Angle is actually the Relative Azimuth Angle
	fprintf(fid, 'phi %.2f\n',RAA);
	% Latitude and longitude
	if ~isempty(latitude)
		fprintf(fid,'latitude %.3f\n',latitude);
	end
	if ~isempty(longitude)
		fprintf(fid,'longitude %.3f\n',longitude);
	end
	% Time of simulation
	if sum(time)~=0
		fprintf(fid,['time ',repmat('%i ',1,6),'\n'],time);
	end
	
    %--------------------------------------------------------------------------
    %--------------------- STEP 3: Atmospherical Model ------------------------
    %--------------------------------------------------------------------------
	if isnumeric(source) % Set radiance source
		if v1(i)>=8000 % Automatically set to thermal for wavelenghts >8 um
			if strcmp(mol_abs_param,'lowtran')
				fprintf(fid,'source thermal ../data/solar_flux/kurudz_0.1nm.dat');
			else
				fprintf(fid,'source thermal\n');
			end
		elseif v1(i)<=2500 % Automatically set to solar for wavelenghts <2.5 um
			if strcmp(mol_abs_param,'lowtran')
				fprintf(fid,'source solar ../data/solar_flux/kurudz_0.1nm.dat');
			else
				fprintf(fid,'source solar\n');
			end
		else
			% Let the user select
			aux = {'solar','thermal'};
			fprintf(fid,'source %s\n',aux{source});
		end
	elseif isstruct(source)
		fprintf(fid,'source %s %s\n',source.type,source.file,source.unit);
	end
	if earth_radius~=6370
		fprintf(fid,'earth_radius %.2f\n',earth_radius);
	end
	modelNames = {'tropical','midlatitude_summer','midlatitude_winter','subarctic_summer',...
				  'subarctic_winter','US-standard'};
	if ~isempty(MODEL)
		fprintf(fid, ['atmosphere_file ',modelNames{MODEL},'\n']);
    elseif ischar(MODEL) % MODEL contains user-defined atmospheric profile
		fprintf(fid, ['atmosphere_file ',atmosphere_file,'\n']);
		radioSpecies = 'O3 N2O H2O BRO OCLO HCHO CO2 SO2 O4';
		% radioUnits = 'to review'; DEFAULT
		fprintf(fid,'radiosonde %s %s %s\n',radiosonde,radioSpecies);
		if radiosonde_levels_only==1
			fprintf(fid,'radiosonde_levels_only\n');
		end
		% REVISAR UNIDADES
		mol_file.O2  = MODEL.O2file;
		mol_file.CH4 = MODEL.CH4file;
		mol_file.N2O = MODEL.N2Ofile;
		mol_file.CO  = MODEL.COfile;
		mol_file.N2  = MODEL.N2file;
		fn = fieldnames(mol_file);
		for j=1:numel(fn) % Insert files with vertical profiles of trace gases
			if ~iempty(mol_file.(fn{j}))
				fprintf(fid,'mol_file %s %s\n',fn{j},mol_file.(fn{j}));
				% Note: use default density units in particles/cm3
			end
		end
	end
	if auxMODEL==0 && strcmp(mol_abs_param,'lowtran')
		% No molecular gas absorption in LOWTRAN option
		if strcmp(mol_abs_param,'lowtran')
			fn = fieldnames(ck_lowtran_absorption);
			for j=1:numel(fn)
				fprintf(fid, ['ck_lowtran_absorption ',fn{j},' %s\n'],ck_lowtran_absorption.(fn{j}));
			end
		end
		fprintf(fid,'no_absorption mol\n');
		fprintf(fid,'no_scattering mol\n');
	end
	% Surface pressure:
	if pressure~=-1
		fprintf(fid,'pressure %.3f\n',pressure);
	end
	% Modifying default trace gases content:
	fn = fieldnames(mol_modify);
	for j=1:numel(fn)
		if mol_modify.(fn{j})~=-1 && ~strcmp(fn{j},'H2O') && ~strcmp(fn{j},'O3')
			fprintf(fid, ['mol_modify ',fn{j},' %.3f CM_2\n'],mol_modify.(fn{j}));
		elseif mol_modify.(fn{j})~=-1 && strcmp(fn{j},'O3')
			% Convert atm-cm (input in ALG) to DU (required by libRadtran)
			fprintf(fid, ['mol_modify ',fn{j},' %.3f DU\n'],mol_modify.(fn{j})*1000);
		elseif mol_modify.(fn{j})~=-1 && strcmp(fn{j},'H2O')
			% Convert gm/cm2 (input in ALG) to mm (required by libRadtran)
			fprintf(fid, ['mol_modify ',fn{j},' %.3f MM\n'],mol_modify.(fn{j})*10);
		end
	end
	% User-defined gas cross-section files
	fn = fieldnames(crs_file);
	for j=1:numel(fn)
		if ~isempty(crs_file.(fn{j}))
			fprintf(fid, ['crs_file ',fn{j},' %s\n'],crs_file.(fn{j}));
		end
	end
	% LibRadtran library of cross-section models
	fn = fieldnames(crs_model);
	for j=1:numel(fn)
		if ~strcmp(crs_model.(fn{j}),'default')
			fprintf(fid, ['crs_model ',fn{j},' %s\n'],crs_model.(fn{j}));
		end
	end
	% Molecular scattering or absorption optical depth files
	if ~isempty(mol_tau_file.sca)
		fprintf(fid,'mol_tau_file sca %s\n',mol_tau_file.sca);
	end
	if ~isempty(mol_tau_file.abs)
		fprintf(fid,'mol_tau_file abs %s\n',mol_tau_file.abs);
    end
	
	%--------------------------------------------------------------------------
    %------------------------ STEP 4: Aerosol Model ---------------------------
    %--------------------------------------------------------------------------
	% Boundary layer aerosol type
	if isstruct(IHAZE)
        if strcmp(IHAZE.type,'OPAC') % OPAC aerosols
            % Review how to implement other user-defined aerosolswith ALG
            fprintf(fid, 'aerosol_default\n');
            fprintf(fid, 'aerosol_species_file %s\n',aerosol_species_file);
            % fprintf(fid, 'aerosol_file explicit %s\n',IHAZE);
        end
	elseif ~IHAZE==0
		% Predefined aerosols
		fprintf(fid, 'aerosol_default\n');
		fprintf(fid, 'aerosol_haze %i\n',IHAZE);
	end
	% Tropospheric layer aerosol type
    if ~isstruct(IHAZE) && IHAZE~=0 && IVULCAN~=1
        fprintf(fid, 'aerosol_vulcan %i\n',IVULCAN);
    end
	if ischar(MODEL) || MODEL==1 || MODEL==3 ||  MODEL==5 ||  MODEL==6 
		fprintf(fid, 'aerosol_season %i\n',1);  % Spring-Summer vertical profile
	elseif MODEL==2 || MODEL==4
		fprintf(fid, 'aerosol_season %i\n',2);  % Autumn-Winter vertical profile
	end
	% Aerosol Optical Depth:
	fprintf(fid, 'aerosol_set_tau_at_wvl 550 %.3f\n',AOT);
	% Angstrom exponent:
	if ALPHA~=0
		BETA = AOT*(0.55^ALPHA);
		fprintf(fid,'aerosol_angstrom %.3f %.5f\n',ALPHA,BETA);
	end
	% king_byrne modification:
	if ~isempty(aerosol_king_byrne)
		fprintf(fid,'aerosol_king_byrne %.3f %.3f %.3f\n',...
				aerosol_king_byrne.ALPHA0,aerosol_king_byrne.ALPHA1,...
				aerosol_king_byrne.ALPHA2);
	end
	% Modification of aerosol properties:
	if aerosol_modify.gg.value~=10
		fprintf(fid,'aerosol_modify gg set %.3f\n',aerosol_modify.gg.value);
	end
	if aerosol_modify.ssa.value~=-1
		fprintf(fid,'aerosol_modify ssa set %.3f\n',aerosol_modify.ssa.value);
	end
	if ~isempty(aerosol_modify.tau.value)
		fprintf(fid,'aerosol_modify tau set %.3f\n',aerosol_modify.tau.value);
	end
	
	%--------------------------------------------------------------------------
    %---------------------------- STEP 5: Clouds ------------------------------
    %--------------------------------------------------------------------------
	% Water clouds:
	if ~isempty(wc_file)
		fprintf(fid,'wc_file 1D %s\n',wc_file);
	end
	fn = fieldnames(wc_modify);
	for j=1:numel(fn)
		if ~isempty(wc_modify.(fn{j}).value)
			fprintf(fid,['wc_modify',fn{j},' %s %.3f\n'],...
					wc_modify.(fn{j}).type,wc_modify.(fn{j}).value);
		end
	end
	if ~strcmp(wc_properties,'hu')
		fprintf(fid,'wc_properties %s\n',wc_properties);
	end
	% Ice clouds:
	if ~isempty(ic_file)
		fprintf(fid,'ic_file 1D %s\n',ic_file);
	end
	fn = fieldnames(ic_modify);
	for j=1:numel(fn)
		if ~isempty(ic_modify.(fn{j}).value)
			fprintf(fid,['ic_modify',fn{j},' %s %.3f\n'],...
					ic_modify.(fn{j}).type,ic_modify.(fn{j}).value);
		end
	end
	if ~strcmp(ic_properties,'fu')
		fprintf(fid,'ic_properties %s\n',ic_properties);
	end
	if ~strcmp(ic_fu.reff_def,'off')
		fprintf(fid,'c_fu reff_def %s\n',ic_fu.reff_def);
		fprintf(fid,'c_fu deltascaling %s\n',ic_fu.deltascaling);
	end
	if ~isempty(ic_habit)
		fprintf(fid,'ic_habit %s\n',ic_habit);
	elseif ~isempty(ic_habit_yang2013)
		fprintf(fid,'ic_habit_yang2013 %s %s\n',ic_habit_yang2013.type,ic_habit_yang2013.roughness);
	end
	% Cloud fraction
	if ~isempty(cloud_fraction_file)
		fprintf(fid,'cloud_fraction_file %s\n',cloud_fraction_file);
	end
	% Cloud overlap
	if ~strcmp(cloud_overlap,'off')
		fprintf(fid,'cloud_overlap %s\n',cloud_overlap);
	end
	% Fraction of horizontal sky covered by clouds. Only cloud_fraction_file is
	% empty and if Ice and/or Water cloud files are defined.
	if isempty(cloud_fraction_file) && (~isempty(wc_file) || ~isempty(ic_file))
		if ~isempty(wc_file)
			fprintf(fid,'cloudcover wc %.3f\n',cloudcover);
		end
		if ~isempty(ic_file)
			fprintf(fid,'cloudcover ic %.3f\n',cloudcover);
		end
	end
	% Ice crystal properties for geometrical optics ray tracing
	if ~isempty(ic_raytracing_file) && rte_solver==13
		fprintf(fid,'ic_raytracing_file %s\n',ic_raytracing_file);
	end
	
    %--------------------------------------------------------------------------
    %-------------------- STEP 6: Surface Properties --------------------------
    %--------------------------------------------------------------------------
	% Reflectance
	if isstruct(rho)
		fprintf(fid, 'albedo_file %s\n', albedo_file);
	elseif rho>=0 && rho<=1 
		fprintf(fid, 'albedo %.3f\n', albedo);
	elseif rho>10
		fprintf(fid, 'albedo_library IGBP\n');
		fprintf(fid, 'brdf_rpv_type %i\n', albedo_library);
	end
	% BRDF models:
	if ~isempty(bpdf_tsang_u10) && rte_solver==13
		fprintf(fid, 'bpdf_tsang_u10 %.3f\n',bpdf_tsang_u10);
	end
	if ~isempty(brdf_ambrals) && (rte_solver==1 || rte_solver==4 || rte_solver==13)
		fprintf(fid, 'brdf_ambrals %.3f %.3f %.3f\n',...
				brdf_ambrals(1),brdf_ambrals(2),brdf_ambrals(3));
	end
	if ~isempty(brdf_ambrals) && brdf_ambrals_hotspot==1
		fprintf(fid, 'brdf_ambrals_hotspot\n');
	end
	if ~isempty(brdf_ambrals_file) && (rte_solver==1 || rte_solver==13)
		fprintf(fid, 'brdf_ambrals_file %s\n',brdf_ambrals_file);
	end
	fn = fieldnames(brdf_cam);
	for j=1:numel(fn)
		if ~isempty(brdf_cam.(fn{j}))
			if strcmp(fn{j},'uphi') && rte_solver==13
				fprintf(fid,['brdf_cam ',fn{j},' %.3f\n'],brdf_cam.(fn{j}));
			elseif ~strcmp(fn{j},'uphi')
				fprintf(fid,['brdf_cam ',fn{j},' %.3f\n'],brdf_cam.(fn{j}));
			end
		end
	end
	fn = fieldnames(brdf_hapke);
	for j=1:numel(fn)
		if ~isempty(brdf_hapke.(fn{j}))
			fprintf(fid,['brdf_hapke ',fn{j},' %.3f\n'],brdf_hapke.(fn{j}));
		end
	end
	if ~isempty(brdf_hapke_file) && (rte_solver==1 || rte_solver==13)
		fprintf(fid, 'brdf_hapke_file %s\n',brdf_hapke_file);
	end
	fn = fieldnames(brdf_rpv);
	for j=1:numel(fn)
		if ~isempty(brdf_rpv.(fn{j})) && ~isempty(brdf_rpv_file) && (rte_solver==1 || rte_solver==4 || rte_solver==13)
			fprintf(fid,['brdf_rpv ',fn{j},' %.3f\n'],brdf_rpv.(fn{j}));
		end
	end
	if ~isempty(brdf_rpv_file) && (rte_solver==1 || rte_solver==4 || rte_solver==13)
		fprintf(fid, 'brdf_rpv_file %s\n',brdf_rpv_file);
	end
	% Wavelength-independent fluorescence emission. Currently only works with
	% DISORT. Must be used together with "source solar file". Units are the same
	% as those used in the solar source
	if fluorescence~=0 && rte_solver==1 && ~isempty(source_solar_file) && isempty(fluorescence_file)
		fprintf(fid,'fluorescence %.3f\n',fluorescence);
	end
	% Wavelength-dependent fluorescence emission file. See comments above. In 
	% addition, if raman is not set, wavelength_grid_file must be specified 
	% with the same resolution as the "source solar file" and the first value
	% must be specified by wavelength.
	if ~isempty(fluorescence_file) && rte_solver==1 && ~isempty(source_solar_file)
		% REVIEW!!
		fprintf(fid,'fluorescence %.3f\n',fluorescence);
	end
	% Surface temperature
	if ~isempty(sur_temperature)
		fprintf(fid,'sur_temperature %.3f\n',sur_temperature);
	end
	     
	%--------------------------------------------------------------------------
    %--------------------------- STEP 7: Advanced -----------------------------
    %--------------------------------------------------------------------------
	if ~isempty(atm_z_grid)
		fprintf(fid,['atm_z_grid ',repmat('%d ',1,numel(atm_z_grid)),'\n'],...
				atm_z_grid);
	end
	% Set RTE solver:
	rte_options = {'disort','twostr','fdisort1','fdisort2','sdisort',...
				   'spsdisort','polradtran','ftwostr','rodents','sslidar',...
				   'sos','montecarlo','mystic','tzs','sss','null'};
	fprintf(fid,'rte_solver %s\n',rte_options{rte_solver});
	% By default, libRadtran uses the REPTRAN band parameterization with a 
	% spectral resolution of 15 cm-1 but other options are available:
	% If you need higher spectral resolution of the band data, use  
	% mol_abs_param reptran fine/medium. To swtich off spectral parameterizations
	% (userful for <205 nm), use mol_abs_param crs. For integrated shortwave or
	% longwave radiation, better use one of the folliwing correlated-k options:
	% mol_abs_param kato2 or mol_abs_param fu 
	if strcmp(mol_abs_param,'reptran')
        try
            fprintf(fid,'mol_abs_param %s %s\n',mol_abs_param,SSI{i});
        catch
             fprintf(fid,'mol_abs_param %s %s\n',mol_abs_param,SSI); 
        end
		if ~isempty(reptran_file)
			% User user-defined representative wavelengths
			fprintf(fid,'reptran_file %s\n',reptran_file);
		end
	else
		fprintf(fid,'mol_abs_param %s\n',mol_abs_param);
	end
	
	% Set raman scattering
	if raman==1
		fprintf(fid,'raman\n');
	end
	
	% Advanced disort options;
	% Delta-M scaling. Always ON if disort or fdisort2
	if (rte_solver==1 || rte_solver==4) && deltam==1
		fprintf(fid,'deltam on\n');
	end
	% DISORT intensity correction. Only if disort or fdisort2
	if (rte_solver==1 || rte_solver==4) && disort_intcor==2
		fprintf(fid,'disort_intcor moments\n');
	elseif (rte_solver==1 || rte_solver==4) && disort_intcor==3
		fprintf(fid,'disort_intcor off\n');
	end
	% Set number of DISORT streams:
	fprintf(fid,'number_of_streams %i\n',NSTR);
	% Speficy sdisort values. Only valid with sdisort rte solver
	if rte_solver==5
		if sdisort.nscat~=2
			fprintf(fid,'sdisort nscat %i\n',sdisort.nscat);
		end
		if sdisort.nrefrac~=0
			fprintf(fid,'sdisort nrefrac %i\n',sdisort.nrefrac);
		end
	end
	% Specify isotropic illumination
	if isotropic_source_toa==1 && (rte_solver==1 || rte_solver==2)
		fprintf(fid,'isotropic_source_toa\n');
	end
	% Set pseudospherical:
	if pseudospherical && (rte_solver==1 || rte_solver==2)
		fprintf(fid,'pseudospherical\n');
	end
	
	% POLRADTRAN solver advanced configuration. See libradtran for further
	% information
	if rte_solver==7
		if ~isempty(polradtran.aziorder) % Azimuth order
			fprintf(fid,'polradtran aziorder %i\n',polradtran.aziorder);
		end
		if polradtran.nstokes~=1 % Stokes vector
			fprintf(fid,'polradtran nstokes %i\n',polradtran.nstokes);
		end
		if polradtran.src_code~=1 % Radiance source
			fprintf(fid,'polradtran src_code %i\n',polradtran.src_code);
		end
		% Initial layer thickness for doubling
		if polradtran_max_delta_tau~=1e-5
			fprintf(fid,'polradtran_max_delta_tau %f\n',polradtran_max_delta_tau);
		end
		% Type of quadrature used
		if ~strcmp(polradtran_quad_type,'G')
			fprintf(fid,'polradtran_quad_type %c_n',polradtran_quad_type);
		end
	end
	
	% MYSTIC advanced configuration:
	if rte_solver==13
	% Backward ray tracing
		if ~strcmp(mc_backward,'off') 
			% REVIEW!!
			fprintf(fid,'mc_backward[%i %i] [%i %i]\n',ix_start,iy_start,ix_end,iy_end);
			fn = fieldnames(mc_backward_output);
			for j=1:numel(fn)
				if ~strcmp(mc_backward_output.(fn{j}),'off')
					fprintf(fid,'mc_backward_output %s\n',fn{j});
				end
			end
			if mc_backward_writeback==1
				fprintf(fid,'mc_backward_writeback\n');
			end
		end
		% MYSTIC 3D output filename
		if ~strcmp(mc_basename,'mc')
			fprintf(fid,'mc_basename %s\n',mc_basename);
		end
		% Box-airmass-factor calculation
		if mc_boxairmass==1
			fprintf(fid,'mc_boxairmass\n');
		end
		% mc_coherent_backscatter
		% Calculate MYSTIC radiances via escape probabilities;  
		if mc_escape==0
			fprintf(fid,'mc_escape off\n');
		end
		% Total number of photons to be traced by the Monte Carlo solver, MYSTIC
		fprintf(fid,'mc_photons %i\n',mc_photons);
		% Switch coherent backscattering:
		if mc_polarisation==1 && mc_coherent_backscatter.on==1
			fprintf(fid,'mc_coherent_backscatter %i %i\n',...
					mc_coherent_backscatter.option1,mc_coherent_backscatter.option2);
		end
		% Distribution of photons over wavelength bands
		if ~isempty(mc_photons_file)
			fprintf(fid,'mc_photons_file %s\n',mc_photons_file);
		end
		% Switch on polarisation
		if mc_polarisation~=0
		% Re-setting values to libRadtran user manual
			if mc_polarisation<=4, mc_polarisation = mc_polarisation-1;
			else,        mc_polarisation = -1*(mc_polarisation-4);
			end
			fprintf(fid,'mc_polarisation %i\n',mc_polarisation);
		end
		% Define opening angle for all-sky simulations
		if mc_rad_alpha==1
			fprintf(fid,'mc_rad_alpha\n');
		end
		% Random seed number for repetition of simulations
		if ~isempty(mc_randomseed)
			fprintf(fid,'mc_randomseed %i\n',mc_randomseed);
		end
		% Activate Importance Sampling method for high spectral resolution
		% simulations
		if mc_spectral_is==1
			fprintf(fid,'mc_spectral_is\n');
		end
		% Activate spherical geometry in Mystic
		if mc_spherical==1
			fprintf(fid,'mc_spherical 1D\n');
		end
		% Switch on surface reflct always option
		if mc_surface_reflectalways==1
			fprintf(fid,'mc_surface_reflectalways\n');
		end
		% Variance Reduction Optical Options Method (VROOM)
		if mc_vroom==1
			fprintf(fid,'mc_vroom\n');
		end
	end
	
	% SSLIDAR advanced configuration
	if rte_solver==10
		% Set single scattering lidar parameters
		if sslidar.area~=1
			% Area of single scattering lidar (in m2)
			fprintf(fid,'sslidar area %.2f\n',sslidar.area);
		end
		if sslidar.E0~=0.1
			% Laser pulse energy (in Joule)
			fprintf(fid,'sslidar E0 %.3f\n',sslidar.E0);
		end
		if sslidar.eff~=0.5
			% Lidar efficiency
			fprintf(fid,'sslidar eff %.3f\n',sslidar.eff);
		end
		if sslidar.position~=0
			% Lidar position (in km)
			fprintf(fid,'sslidar position %.3f\n',sslidar.position);
		end
		if sslidar.range~=0.1
			% Lidar range bin width
			fprintf(fid,'sslidar range %.3f\n',sslidar.range);
		end
		% Set number of lidar range bins
		if sslidar_nranges~=100
			fprintf(fid,'sslidar_nranges %i\n',sslidar_nranges);
		end
		% Turn polarisation measurements for lidar
		if sslidar_polarisation~=0
			fprintf(fid,'sslidar_polarisation\n');
		end
    end
    
    % SOS advanced configuration
	if rte_solver==11
        fprintf(fid,'sos_nscat %i\n',sos_nscat);
    end
	
	% TZS advanced configuration
	if rte_solver==14
		if ~isempty(tzs_cloud_top_height)
			fprintf(fid,'tzs_cloud_top_height %.3f\n',tzs_cloud_top_height);
		end
	end
	
	% Calculation of heating rates. Options are layer_cd; local or layer_fd. 
	% See libRadtran user manual for further information
	if ~isempty(heating_rate)
		fprintf(fid,'heating_rate %s\n',heating_rate);
	end
	% Rayleight depolarization factor
	if ~isempty(rayleigh_depol)
		fprintf(fid,'rayleigh_depol %.2f\n',rayleigh_depol);
	end
	% Reverse atmosphere
	if reverse_atmosphere==1
		fprintf(fid,'reverse_atmosphere\n');
	end
	
	
	% fprintf(fid,'source solar /home/flex/libRadtran/data/solar_flux/kurudz_0.1nm.dat\n');
	
	%--------------------------------------------------------------------------
    %--------------------------- STEP 8: Post-processing ----------------------
    %--------------------------------------------------------------------------
	% Spectral response function
	if ~isempty(filter_function_file)
		fprintf(fid,'filter_function_file %s normalize\n',filter_function_file);
	end
	% Output file
	if ~isempty(output_file)
		fprintf(fid,'output_file\n');
	end
	% Output file format
	if ~strcmp(output_format,'ascii')
		fprintf(fid,'output_format %s\n',output_format);
	end
	% Process output from uvspec:
	if ~strcmp(output_process,'none')
		fprintf(fid,'output_process %s\n',output_process);
	end
	% Output quantity to print:
	if ~strcmp(output_quantity,'default')
		fprintf(fid,'output_quantity %s\n',output_quantity);
	end
	% Define outputs to be printed (see libRadtran user manual for further 
	% information):
	fprintf(fid,['output_user lambda ',selectedOut,'\n']);
	% Specify output levels in pressure coordinates
	if ~isempty(pressure_out)
		fprintf(fid,['pressure_out ',repmat('%.f ',1,numel(pressure_out)),'\n'],...
				pressure_out);
	end
	% Print disort information:
	if print_disort_info~=0
		fprintf(fid,'print_disort_info %i\n',print_disort_info);
	end
	% Slit function for performing convolution of output spectrum
	if ~isempty(slit_function_file)
		fprintf(fid,'slit_function_file %s\n',slit_function_file);
	end
	% Spline interpolation of the calculated spectrum
	if ~isempty(spline_file)
		fprintf(fid,'spline_file %s\n',spline_file);
	elseif sum(spline)~=0
		fprintf(fid,'spline %.3f %.3f %.3f\n',spline(1),spline(2),spline(3));
	end
	
	
	if ~quiet
		fprintf(fid,'quiet\n');
	end
	if verbose
		fprintf(fid,'verbose\n');
	end
		
	% Close libRadtran input file:
    fclose(fid);

    %%%%%%%%%%%%%% Close loop for Nbands times
end

end

