function [wvl,Lp0,Tgas,Etot,Ttot,S,I0] = readRTMoutput(outfile)
%READRTMOUTPUT Read 6SV output tape7 file
%   [WVL,LP0,TGAS,ETOT,TTOT,S,I0,IQUV] = READRTMOUTPUT(OUTFILE) reads
%   6SV output file (OUTFILE) and generates the following output variables:
%   WVL       - Wavelength (in [nm]) at which the output data is given
%   LP0       - Path radiance (in [mW/m^2/sr/nm])
%   TGAS      - Total gas transmittance (in [-])
%   ETOT      - Total at-surface irradiance due to scattering (in [mW/m^2/nm])
%   TTOT      - Total upwards transmittance due to scattering (in [-])
%   S         - Spherical albedo (in [-])
%   I0        - Extraterrestrial Solar Irradiance (in [mW/m^2/nm])
%
%   Accordingly, the total Top-of-Atmosphere radiance (LTOA) is given by 
%   the following equation:
%   
%   LTOA = LP0 + (1/pi)*TGAS*ETOT*TTOT*RHO/(1-S*RHO)
%
%   where RHO is the lambertian surface reflectance.
%
%   See also: LUTGEN_PROCESS

%   Copyright 2018 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.02 $  $Date: 2018/02/28 $

%% Open File:
fid   = fopen(outfile);
tline = fgetl(fid);

%% Scan header:
% Scan the outfile to detect the number of header lines. Header in outfile
% ends with a line string containing the word "wave"
headerlines = 0;
while ischar(tline) && ~strcmp(tline,'-9999.')
    headerlines = headerlines+1;
    if strfind(tline,'wave')
        [~,count] = sscanf(tline,'%s');
        header = textscan(tline,'%s',16,'Delimiter','');
        break        
    end
    tline = fgetl(fid);
    
    if strfind(tline,'wl inf')
        % Read inferior and superior wavelengths:
        aux = strfind(tline,'wl inf');
        wl_inf = str2num(tline(aux+7:aux+13));
        aux = strfind(tline,'wl sup');
        wl_sup = str2num(tline(aux+7:aux+13));
        % Knowing that the sampling interval in 6SV is 2.5 nm we can
        % calculate the number of wavelengths (N):
        N = (wl_sup-wl_inf)/0.0025+1;  
    end
    if strfind(tline,'solar zenith angle')
        % Read solar zenith angle:
        aux = strfind(tline,'solar zenith angle');
        sza = str2num(tline(aux+21:aux+27)); % [deg]
    end
end

% Read two more lines to fully skip header
tline = fgetl(fid); tline = fgetl(fid);

%% Get atmospheric transfer functions data:
data = [];
for i=1:N % For each wavelength
    if i==N+1, keyboard, end
    tline = fgetl(fid);
    aux = textscan(tline(2:end-1),'%f','delimiter','\t'); aux = aux{1}';
    data(end+1,:) = aux;
end

fclose(fid);

%% Conversion to [mW/m2/sr/nm] from [W/m2/sr/um]:

% Wavelength in [nm]:
wvl = 1000*data(:,1);

% Total gas transmittance:
Tgas = data(:,2);

% Extraterrestrial Solar irradiance:
I0 = data(:,7);

% Total at-surface irradiance:
Etot = cosd(sza)*I0.*data(:,3);

% Total target-to-sensor transmittance:
Ttot = data(:,4);

% Spherical albedo:
S = data(:,5);

% Path radiance:
Lp0 = (cosd(sza)/pi)*I0.*Tgas.*data(:,6);

% END of function
end
