function writeRTMinput(file,rho,varnames,inputval,SpeConf)
% WRITERTMINPUT Create 6SV input file
%   WRITERTMINPUT(FILE,RHO,VARNAMES,INPUTVAL) creates, according to 6SV 
%   documentation, a 6SV input file (FILE) with a spectrally constant
%   reflectance RHO in the spectral range and configuration defined in the
%   input SPECONF structure. 6SV configuration is based on the input
%   variables and associated values stored in the cell arrays VARNAMES
%   (strings) and INPUTVAL (numeric values).
%
%   Copyright 2016 Image Processing Laboratory (IPL), Univ. of Valencia.
%   Contact: jorge.vicent@uv.es
%   $Revision: 1.00 $  $Date: 2017/04/20 $
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%            INPUT VARIABLES TO BE USED BY 6SV              %%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The 6S Code predicts the satellite signal from 0.25 to 4.0 um assuming 
% cloudless atmosphere. The main atmospheric effects (gaseous absortion by 
% water vapour, carbon dioxyde, oxygen and ozone; scattering by molecules 
% and aerosols) are taken into account.


if numel(varnames)~=numel(inputval)
    error('Number of variable names and input values are not coincident.')
end

% Associates input values to each variable name:
for i=1:numel(varnames)
    eval([varnames{i},'=inputval{',num2str(i),'};'])
end

filepath = fileparts(file);

%--------------------------------------------------------------------------
%--------------------- STEP 1: Geometric Conditions -----------------------
%--------------------------------------------------------------------------
% GeoOptions selects the geometric conditions from the available options:
% User's Data (0), Meteosat (1), Goes East (2), Goes West (3), AVHRR (PM 
% Noaa) (4), AVHRR (AM Noaa) (5), Hrv (6), TM (Landsat) (7)

GeoOptions = 0;

if GeoOptions == 0
    SZA = SZA; % Solar Zenith Angle
    VZA = VZA; % Visual Zenith Angle
    RAA = RAA; % Relative Azimuth Angle
elseif GeoOptions > 0 && GeoOptions < 4
    GMT    = GMT;    % GMT decimal hour
    Column = Column; % Selected Column number
    Line   = Line;   % Selected Line number
elseif GeoOptions > 3 && GeoOptions < 6
    GMT    = GMT;
    Column = Column;
    NodeLong = NodeLong; % Ascendant Node Longitude
    NodeHour = NodeHour; % Ascendant Node Hour
elseif GeoOptions == 6 || GeoOptions == 7
    GMT  = GMT;
    Long = Long; % Longitude -> coordinates of the scene center
    Lat  = Lat;  % Latitude -> coordinates of the scene center
    % Latitude must be > 0 for northern hemisphere and < 0 for southern.
    % Longitude must be > 0 for eastern hemisphere and < 0 for wertern.
end

% Month and Day for a Earth-Sun distance of 1 AU: 93 (of 365) i.e. April 3rd
Month = 4; Day   = 3;


%--------------------------------------------------------------------------
%-------------------- STEP 2: Atmospherical Model  ------------------------
%-------------------------------------------------------------------------
% MODEL defines the Atmospheric Profile: No Gaseous Absorption (0), Tropical
% (1), Mid-Latitude Summer (2), Mid-Latitude Winter (3), Subartic Summer 
% (4), Subartic Winter (5), US Standard 62 (6), User's data base profile 
% (7) and UW and UO3 (8)
MODEL = MODEL;

if MODEL == 8
    H2O = H2O; % Water vapour concentration (g/cm2)
    O3  = O3;  % Ozone concentration (amt-cm)
elseif MODEL < 0
    MODEL = 7;
    DataTable = zeros (5,34);
    % Enter radiosonde data on 34 levels (5 columns, 34 lines).
    % Altitude / pressure / temperature / h2o density / o3 density
end

% MAERO defines the Aerosol Model: No Aerosol (0), Continental (1), 
% Maritime (2), Urban (3), User's Components (4), Desert (5), Biomass 
% burning (6), Stratospheric (7), Multimodal (OPAC) (8) and Modified Gamma 
% Distribution (9)
MAERO = MAERO;
if isstruct(MAERO) % MAERO contains data from OPAC or user-defined aerosol:
    aeraux = MAERO;
    MAERO  = aeraux.modeltype;
    if strcmp(aeraux.type,'OPAC') % OPAC aerosol
        % Multimodal (log-normal) as used in OPAC and compatible with 6SV
    elseif strcmp(aeraux.type,'ATK') % ATK aerosol
        if MAERO==4 % 4 components
            DustLike     = aeraux.data(1);
            WaterSoluble = aeraux.data(2);
            Oceanic      = aeraux.data(3);
            Soot         = aeraux.data(4);
        elseif MAERO==8
            % Multimodal (log-normal)
            Rmin  = aeraux.data(1,1);
            Rmax  = aeraux.data(1,2);
            ncomp = (size(aeraux.data,1)-1)/3;
            Rmean = aeraux.data(2:3:end,1);
            Sigma = aeraux.data(2:3:end,2);
            Perc  = aeraux.data(2:3:end,3);
            nRe   = aeraux.data(3:3:end,:);
            nIm   = aeraux.data(4:3:end,:);
        elseif MAERO==12 % Aerosol Optical Properties ==> Load .mie file
            % Save optical properties into .mie file
            % NOTE: In the current version, phase function is calculated
            % with the Henyey-Greenstein approximation taking into account
            % the asymmery parameter!!
            mieFile = [filepath,filesep,aeraux.aername,'.mie'];
           if ~exist(mieFile)
               writeMieFile(mieFile,aeraux.data);
           end
        end
    end
end
% Print aerosol optical properties into .mie file
IAERP = IAERP;

% Aerosol Optical Thickness:
if MAERO ~= 0, AOT = AOT; end
    
%--------------------------------------------------------------------------
%-------------------- STEP 3: Target & Sensor Altitude --------------------
%--------------------------------------------------------------------------
% GNDALT defines the Target altitude [km] (0 km at sea level)
GNDALT = GNDALT;

% H1 defines the Sensor altitude [km], between 0 km (sea level ) and 99.9 
% km (satellite level)
H1 = H1;
if H1 > 99.9
    % 6S only exectutes the program for altitudes between 0 km and < 100 km
    H1 = 99.9; 
end

if H1 > 0 && H1 < 100
    H2O = H2O;
    O3 = O3;
    AOT = AOT;
end

%--------------------------------------------------------------------------
%-------------------- STEP 4: Spectral Conditions -------------------------
%--------------------------------------------------------------------------
% There are two possibilities (LambdaMode): either define your own spectral
% configuration (-2, -1, 0 or 1) or choose a code indicating the band of 
% one of the pre-defined satellites. We are interested in user-input
% spectral  configuration:
LambdaMode = -2;


% v1 and v2 define the minimum and maximum wavelengths [um]  of the output 
% 6SV data:
if LambdaMode == -2 || LambdaMode == 0
    % User-defined min/max wavelengths
    Nbands = numel(str2num(SpeConf.MinLambda.values)); % Number of spectral bands
    v1 = str2num(SpeConf.MinLambda.values);
    v2 = str2num(SpeConf.MaxLambda.values);
elseif LambdaMode == -1
    % Unique wavelength because of monochromatic mode
    v1 = str2num(SpeConf.MinLambda.values);
elseif LambdaMode == 1
    Nbands = numel(str2num(SpeConf.MinLambda.values)); % Number of spectral bands
    v1 = str2num(SpeConf.MinLambda.values);
    v2 = str2num(SpeConf.MaxLambda.values);
    fFunction = fFunction; % User's defined filter function
end
% If Lambdamode > 1, LambdaMode refers to the band of one of the 
% pre-defined satellite instruments


%--------------------------------------------------------------------------
%-------------------- STEP 5: Ground Reflectance --------------------------
%--------------------------------------------------------------------------
% There are two ground reflectance type (Homogeneous), you can choose a 
% Homogeneous Surface (0) or non Uniform Surface (1). We are only 
% interested in the output 6SV atmospheric transfer functions, and thus we
% choose an homogeneous surface:
Homogeneous = 0;

% Into Homogeneous Surface Option there are two more options: to take into
% account directional effects (Directional = 1) or not (Directional = 0).
% In our case, we choose the 'no directional effects' option:
Directional = 0;

if Homogeneous == 0 && Directional == 0 % Homogeneous Surface without directional effect
% Inside of this options there are 6 options to define the surface:
% Input constant value of ro (0); Input ro by step of 2.5 nm (1); Mean 
% spectral value of green vegetation (2); Mean spectral value of clear 
% water (3); Mean spectral value of sand (4); Mean spectral value of lake 
% water (5). As we are not interested in the output TOA radiance but only 
% on the atmospheric transfer functions, we fix the option (0).
    Ref = 0;
    if Ref == 0
        SurRef = rho; % Spectrally-constant reflectance (ro)
    elseif Ref == -1 % ???
        MinLambda = MinLambda;
        MaxLambda = MaxLambda;
        SurRef = SurRef;
    elseif Ref > 0
        Ref = Ref;
    end
end

if Homogeneous == 0 && Directional ~= 0 % Homogeneous with directional effect
    % Here there are many options to specify the brdf of the surface of the
    % actual solar illumination the user is considering as well as the brdf
    % for the sun which would be at an angle theta. 
    % (More information about the options in 6S web page)
end

if Homogeneous == 1 % Non Uniform Surface
    % Target Reflectance (TarRef). 6 options: Input constant value of ro 
    % (0); Input ro by step of 2.5 nm (1); Mean spectral value of green 
    % vegetation (2); Mean spectral value of clear water (3); Mean spectral
    % value of sand (4); Mean spectral value of lake water (5). By default 
    % we choose the 0 option.
    TarRef = TarRef;
    if TarRef == 0
        SurRef1 = SurRef1; % Spectrally-constant target reflectance (ro1)
    elseif TarRef == -1
        SurRef1 = SurRef1;
    elseif TarRef > 0
        TarRef = TarRef;
    end
    
    % Environment Reflectance (EnviRef). 6 options: Input constant value of ro 
    % (0); Input ro by step of 2.5 nm (1); Mean spectral value of green 
    % vegetation (2); Mean spectral value of clear water (3); Mean spectral
    % value of sand (4); Mean spectral value of lake water (5). By default 
    % we choose the 0 option.
    EnviRef = EnviRef;
    if EnviRef == 0
        SurRef2 = SurRef2; % Spectrally-constant target reflectance (ro2)
    elseif EnviRef == -1
        SurRef2 = SurRef2;
    elseif EnviRef > 0
        EnviRef = EnviRef;
    end
    
    Radius = Radius; % Target Radius in km
end

%--------------------------------------------------------------------------
%-------------------- STEP 6: Signal  -------------------------------------
%--------------------------------------------------------------------------
% In this section, the User can choose the atmospheric correction mode: No 
% atmospheric correction (-1); Atmospheric Correction with Lambertian
% assumption (0); and Atmospheric correction with BRDF (1). We are only
% interested into the atmospheric transfer functions and thus we fix Signal
% to -1:
Signal = -1;

if Signal ~= -1
    % To perform the atmospheric correction, the user must input the
    % satellite TOA radiance or reflectance:
    Radiance = Radiance; 
end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%                  WRITE VARIABLES ON 6SV INPUT FILE                  %%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%--------------------------------------------------------------------------
% NOTE: Read 6SV Documentation for further information on the variables and
% input file format.
%--------------------------------------------------------------------------

%%%%%%%%%%%%%% Repeat Nbands times
for i=1:Nbands
    % Each spectral band requires a new 6SV input file
    % Open file with write permission
    aux = strfind(file,'.in');
    fid = fopen([file(1:aux-1),'_band',num2str(i),file(aux:end)], 'w');


    %--------------------------------------------------------------------------
    %-------------------- STEP 1: Geometrical Conditions ----------------------
    %--------------------------------------------------------------------------

    fprintf(fid, '%i\n%5.2f 0.0 %5.2f %5.2f %i %i\n', GeoOptions, SZA, VZA, RAA, Month, Day);

    %--------------------------------------------------------------------------
    %-------------------- STEP 2: Atmospherical Model  ------------------------
    %-------------------------------------------------------------------------

    if MODEL == 7
        msgbox('Data Table Required','Error MODEL','error')
        %fprintf(fid, '%i\n Data Table Required', MODEL);
    elseif MODEL == 8
        auxH2O = H2O; if H2O == -1, auxH2O = 2.73; end 
        auxO3 = O3;   if O3 == -1, auxO3 = 0.315; end
        fprintf(fid, '%i\n%3.3f %3.3f\n', MODEL, auxH2O, auxO3);
    else
        fprintf(fid, '%i\n', MODEL);
    end

    fprintf(fid, '%i\n',MAERO);
    if MAERO == 0
        fprintf(fid, '-1\n');
    elseif MAERO == 4
        fprintf(fid, '%4.2f %4.2f %4.2f %4.2f\n0\n%6.4f\n', DustLike, WaterSoluble, Oceanic, Soot, AOT);
    elseif any(MAERO == [8,9,10])
        % 8 - Multimodal (Log Normal) Distribution
        % 9 - Modified Gamma Distribution
        % 10 - Junge Powe-law Distribution
        fprintf(fid, '%5.3f %5.3f %i\n', Rmin,Rmax,ncomp);
        for n=1:ncomp
            fprintf(fid, '%5.3f %5.3f %4.2e\n', Rmean(n),Sigma(n),Perc(n));
            fprintf(fid, '%5.3f ',nRe(n,:)); fprintf(fid, '\n');
            fprintf(fid, '%5.3f ',nIm(n,:));  fprintf(fid, '\n');
        end
        if IAERP==1 && ~exist([filepath,filesep,aeraux.aername,'.mie'])
            fprintf(fid, '%i\n', IAERP);
            fprintf(fid, '%s\n0\n',aeraux.aername);
        else
        	fprintf(fid, '0\n0\n');
        end
        fprintf(fid, '%6.4f\n',AOT);
    elseif MAERO == 11
        % OPTION TO BE IMPLEMENTED
        fprintf(fid, 'Data Table Required');
    elseif MAERO == 12
        fprintf(fid, '%s\n',mieFile);
        fprintf(fid, '%6.4f\n',AOT);
    else
        aername={'continental','maritime','urban','','desert','biomass','stratospheric'};
        if IAERP==1 && ~exist([filepath,filesep,aername{MAERO},'.mie'])
            fprintf(fid, '%i\n', IAERP);
            fprintf(fid, '%s\n0\n',aername{MAERO});
            fprintf(fid, '%6.4f\n',AOT);
        else
        	fprintf(fid, '0\n0\n%6.4f\n', AOT);
        end
    end
    %--------------------------------------------------------------------------
    %-------------------- STEP 3: Target & Sensor Altitude --------------------
    %--------------------------------------------------------------------------

    if GNDALT == 0
        fprintf(fid, '0\n');
    else
        fprintf(fid, '-%5.3f\n', GNDALT);
    end

    if H1 == 0
        fprintf(fid, '0\n');
    elseif H1==99.9
        fprintf(fid, '-%6.3f\n%3.3f %3.3f\n%6.4f\n', H1, H2O, O3, AOT);
	else
		% In this case, O3, H2O and AOT values between surface and sensor are
		% not known ==> use 6SV default profile (US62 and 2km exponential) for
		% scaling total column values ==> negative values on H2O, O3 and AOT
		fprintf(fid, '-%6.3f\n%3.3f %3.3f\n%6.4f\n', H1, -H2O, -O3, -AOT);
    end

    %--------------------------------------------------------------------------
    %-------------------- STEP 4: Spectral Conditions -------------------------
    %--------------------------------------------------------------------------

    fprintf(fid, '-2\n%5.3f %5.3f\n', v1(i), v2(i));

    %--------------------------------------------------------------------------
    %-------------------- STEP 5: Ground Reflectance --------------------------
    %--------------------------------------------------------------------------
    %SURREF means constant reflectance over the spectra

    fprintf(fid, '%i\n%i\n%i\n%5.3f\n', Homogeneous, Directional, Ref, SurRef);

    %--------------------------------------------------------------------------
    %-------------------- STEP 6: Signal  -------------------------------------
    %--------------------------------------------------------------------------

    fprintf(fid, '-1');

    % Close 6SV input file:
    fclose(fid);

    %%%%%%%%%%%%%% Close loop for Nbands times
end

end

%%% WRITE MIE FILE %%%
function writeMieFile(filename,data)

fid = fopen(filename,'w'); % Open .mie file with writting permission

% Scqttering angles. Hard-coded!!
theta = [180.00 178.29 176.07 173.84...
         171.61 169.37 167.14 164.90 162.67 160.43 158.20 155.96 153.72 151.49 149.25...
         147.02 144.78 142.55 140.31 138.07 135.84 133.60 131.37 129.13 126.89 124.66...
         122.42 120.19 117.95 115.71 113.48 111.24 109.01 106.77 104.53 102.30 100.06...
         97.83 95.59 93.35 91.12 90.00 88.88 86.65 84.41 82.17 79.94 77.70 75.47 73.23...
         70.99 68.76 66.52 64.29 62.05 59.81 57.58 55.34 53.11 50.87 48.63 46.40...
         44.16 41.93 39.69 37.45 35.22 32.98 30.75 28.51 26.28 24.04 21.80 19.57...
         17.33 15.10 12.86 10.63 8.39 6.16 3.93 1.71 0.00]; % Fixed scattering angle
nbmu = numel(theta);
% Header optical properties:
fprintf(fid,' %i\n',nbmu);
fprintf(fid,['   %5s ',repmat(' %10s ',1,5),' %10s\n'],'Wlgth','Nor_Ext_Co','Nor_Sca_Co','Sg_Sca_Alb','Asymm_Para','Extinct_Co','Scatter_Co');

% Printing optical properties:
nis = 153.8462;
for i=1:size(data,1)
    fprintf(fid,['  %8.4f',repmat('   %8.4f   ',1,4),repmat('  %10.4E',1,2),'\n'],...
        data(i,1),data(i,2),data(i,2)*data(i,3),data(i,3),data(i,4),data(i,2)/nis,data(i,2)*data(i,3)/nis);
end
fprintf(fid,'\n\n');

% Printing phase function
fprintf(fid,'                    Phase Function \n');
fprintf(fid,['   TETA ',repmat('   %6.4f  ',1,size(data,1)),'\n'],data(:,1)');
format = ['  %6.2f',repmat(' %10.4E',1,size(data,1)),'\n'];
for i=1:nbmu % First loop: intensity
    phi = ((1-data(:,4)'.^2)/(4*pi))./(1+data(:,4)'.^2-2*data(:,4)'*cosd(theta(i))).^1.5;
    fprintf(fid,format,[theta(i),4*pi*phi]);
end
% Note: simulated unpolariwzed aerosol phase function ==> Q and U components of Stoke vector =0
for i=1:nbmu % First loop: Q component of Stokes vector
    phi = ((1-data(:,4)'.^2)/(4*pi))./(1+data(:,4)'.^2-2*data(:,4)'*cosd(theta(i))).^1.5;
    fprintf(fid,format,[theta(i),zeros(1,size(data,1))]);
end
for i=1:nbmu % First loop: U component of Stokes vector
    phi = ((1-data(:,4)'.^2)/(4*pi))./(1+data(:,4)'.^2-2*data(:,4)'*cosd(theta(i))).^1.5;
    fprintf(fid,format,[theta(i),zeros(1,size(data,1))]);
end
% Close .mie file
fclose(fid);
end
